<?php

namespace App\Events;

use Illuminate\Broadcasting\Channel;
use Illuminate\Queue\SerializesModels;
use Illuminate\Broadcasting\PrivateChannel;
use Illuminate\Broadcasting\PresenceChannel;
use Illuminate\Foundation\Events\Dispatchable;
use Illuminate\Broadcasting\InteractsWithSockets;
use Illuminate\Contracts\Broadcasting\ShouldBroadcast;

class UpcomingAlert implements ShouldBroadcast
{
    use Dispatchable, InteractsWithSockets, SerializesModels;

    public $from_user;
    public $to_user_id;
    public $profile_picture;
    public $conversation;
    public $team_conversation;
    public $coid;
    public $first_name;
    public $last_name;
    public $created_at;
    public $channel_id;
    /**
     * Create a new event instance.
     *
     * @return void
     */
    public function __construct($from_user, $to_user_id, $profile_picture, $conversation, $team_conversation, $coid, $first_name, $last_name, $channel_id = '')
    {
        $this->from_user = $from_user;
        $this->to_user_id = $to_user_id;
        $this->profile_picture = $profile_picture;
        $this->conversation = $conversation;
        $this->team_conversation = $team_conversation;
        $this->coid = $coid;
        $this->first_name = $first_name;
        $this->last_name = $last_name;
        $this->channel_id = $channel_id;
        $this->created_at = date('g:i a', strtotime($conversation->created_at) + session('timezone_difference', 0));
    }

    /**
     * Get the channels the event should broadcast on.
     *
     * @return \Illuminate\Broadcasting\Channel|array
     */
    public function broadcastOn()
    {
        return new PrivateChannel('alerts'.$this->channel_id);
    }
}
