<?php

namespace App\Events;

use Illuminate\Broadcasting\Channel;
use Illuminate\Queue\SerializesModels;
use Illuminate\Broadcasting\PrivateChannel;
use Illuminate\Broadcasting\PresenceChannel;
use Illuminate\Foundation\Events\Dispatchable;
use Illuminate\Broadcasting\InteractsWithSockets;
use Illuminate\Contracts\Broadcasting\ShouldBroadcast;

class UpcomingChatMessage implements ShouldBroadcast
{
    use Dispatchable, InteractsWithSockets, SerializesModels;

    public $from_user;
    public $to_user_id;
    public $profile_picture;
    public $conversation;
    public $created_at;
    /**
     * Create a new event instance.
     *
     * @return void
     */
    public function __construct($from_user, $to_user_id, $profile_picture, $conversation)
    {
        $this->from_user = $from_user;
        $this->to_user_id = $to_user_id;
        $this->profile_picture = $profile_picture;
        $this->conversation = $conversation;
        $this->created_at = date('g:i a', strtotime($conversation->created_at) + session('timezone_difference', 0));
    }

    /**
     * Get the channels the event should broadcast on.
     *
     * @return \Illuminate\Broadcasting\Channel|array
     */
    public function broadcastOn()
    {
        if(strpos($this->to_user_id, "+") !== false)
        {
            //$new_to_user_id = 'customer_'.preg_replace("/[^0-9]/i", "", trim($this->to_user_id));
            $new_to_user_id = 'customer_'.preg_replace("/[^0-9]/i", "", trim($this->from_user->id));
        } else {
            $new_to_user_id = preg_replace("/[^0-9]/i", "", trim($this->from_user->id));
        }
        
        //$new_to_user_id = 'customer_'.$this->to_user_id;
        return new PrivateChannel('chat.'.$new_to_user_id);
    }
}

