<?php

namespace App\Http\Controllers;

use App\Assignment;
use App\CallerSchedule;
use App\UserDealer;
use App\CallType;
use App\AssignmentNote;
use App\User;
use App\AssignableCalltype;
use App\SentAssignmentEmail;
use App\Mail\GeneralEmailAlert;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Mail;
use DateTime;
use DateTimeZone;

class AssignmentController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {
        $today = date('Y-m-d');
        
        $dealers = DB::connection('mysql2')->table('callcenter_companies')
                                ->select('co_name', 'co_short', 'coid')
                                ->where('is_active', 1)
                                ->orderBy('co_name')
                                ->get();
        /*$dealer_names = array();
        foreach ($dealers as $key => $value) {
            $dealer_names[$value->coid] = $value->co_name;
        }*/

        $calltypes = CallType::orderBy('import_priority')->get();
        $calltypes_names = array();
        $call_in_q_days  = array();
        $import_priority  = array();
        foreach ($calltypes as $key => $value) {
            $call_in_q_days[$value->id]  = ( ! empty ( $value->call_in_q_days ) ) ? $value->call_in_q_days : 7;
            $calltypes_names[$value->id] = $value->name;
            $import_priority[$value->id] = ( ! empty( $value->import_priority ) )? $value->import_priority : 0;
        }
        
        $call_type_import_priority = [];
        $call_types_at_end = [];
        foreach ( $import_priority as $call_type => $value ) {
            if( ! empty( $value ) ) {
                $call_type_import_priority[] = $call_type;
            } else {
                $call_types_at_end[] = $call_type;
            }
        }
        $call_type_import_priority = array_merge( $call_type_import_priority, $call_types_at_end );
        
        $assigned_calls = array();
        $assigned_calls_totals = array();
        foreach ($dealers as $key => $value) {
            $db_table = 'livecall_'.$value->co_short;
            /*$livecalls_today = DB::connection('mysql2')->table($db_table)
                                    ->selectRaw('COUNT(id) as total_calls, livecall_next_calltype as call_type_id')
                                    ->where('remind', '>=', date('Y-m-d', strtotime('-7 days')))
                                    ->where('remind', '<=', date('Y-m-d'))
                                    ->whereRaw('livecall_count < livecall_times')
                                    ->where('livecall_showtocall', 1)
                                    ->where('livecall_next_calltype', '!=', 0)
                                    ->whereRaw('livecall_next_calltype IS NOT NULL')
                                    ->groupBy('livecall_next_calltype')
                                    ->get();*/

            $assignable_calltypes = AssignableCalltype::select('call_type_id')
                                    ->where('coid', $value->coid)
                                    ->get()
                                    ->pluck('call_type_id')
                                    ->toArray();

            if(count($assignable_calltypes) > 0) 
            {
                $livecalls_today = DB::connection('mysql2')->table($db_table)
                                    ->selectRaw('livecall_next_calltype as call_type_id, remind, livecall_next_calltype')
                                    ->where('remind', '>=', date('Y-m-d', strtotime('-90 days', strtotime($today))))
                                    ->where('remind', '<=', $today)
                                    ->whereRaw('livecall_count < livecall_times')
                                    ->where('livecall_showtocall', 1)
                                    ->where('livecall_next_calltype', '!=', 0)
                                    ->whereRaw('livecall_next_calltype IS NOT NULL')
                                    ->whereRaw("FIND_IN_SET(livecall_next_calltype, '".implode(',', $assignable_calltypes)."')")
                                    ->get();
            } else {
                $livecalls_today = array();
            }
            

            foreach ($livecalls_today as $call) {
                $time_of_call = strtotime( $call->remind );
                $check_time   = strtotime('-' . $call_in_q_days[ $call->livecall_next_calltype ] . ' days', strtotime( $today ) );
                if ( $time_of_call < $check_time ) {
                    continue;
                }

                if(!isset($assigned_calls_totals[$value->coid]))
                {
                    $assigned_calls_totals[$value->coid] = 0;
                }
                if(!isset($assigned_calls[$value->coid][$call->call_type_id]))
                {
                    $assigned_calls[$value->coid][$call->call_type_id] = 0;
                }
                $assigned_calls[$value->coid][$call->call_type_id]+= 1;
                $assigned_calls_totals[$value->coid]+= 1;
            }
        }
        
        $caller_assignments = array();
        $current_assignments = Assignment::selectRaw('COUNT(id) as total_assignments, user_id, coid, call_type_id')
                                            ->where('assigned_date', $today)
                                            ->whereNull('time_on_call')
                                            ->groupBy('user_id')
                                            ->groupBy('coid')
                                            ->groupBy('call_type_id')
                                            ->get();

        foreach ($current_assignments as $key => $value) {
            $caller_assignments[$value->user_id][$value->coid][$value->call_type_id] = $value->total_assignments;
        }                                  

        $available_caller_hours = array();
        $caller_hours = CallerSchedule::where('date_scheduled', $today)->get();
        foreach ($caller_hours as $key => $value) {
            $available_caller_hours[$value->user_id] = $value->available_hours;
        }

        $caller_access = UserDealer::join('user_calltypes', 'user_dealer.user_id', '=', 'user_calltypes.user_id')
                                        ->selectRaw('user_dealer.user_id as user_id, user_dealer.dealer_id as dealer_id, user_calltypes.call_type_id as call_type_id')
                                        ->get();

        $caller_access_array = array();
        foreach ($caller_access as $key => $value) {
            $caller_access_array[$value->user_id][$value->dealer_id][$value->call_type_id] = $value->call_type_id;
        }

        

        $caller_users = User::join('caller_schedules', 'users.id', '=', 'caller_schedules.user_id')
                                ->select('users.*', 'caller_schedules.available_hours as available_hours')
                                ->where('date_scheduled', $today)
                                ->where('user_type_id', 4)
                                ->get();

        $caller_calls_per_day = array();
        foreach ($caller_users as $key => $value) {
            $caller_calls_per_day[$value->id] = $value->calls_per_day_avg * $value->available_hours;
        }


        return view('pages.assignments', [
                                                'dealers' => $dealers,
                                                'caller_users' => $caller_users,
                                                'caller_calls_per_day' => $caller_calls_per_day,
                                                'calltypes_names' => $calltypes_names,
                                                'current_assignments' => $current_assignments,
                                                'current_date' => date('m/d/Y', strtotime($today)),
                                                'assigned_calls' => $assigned_calls,
                                                'assigned_calls_totals' => $assigned_calls_totals,
                                                'caller_assignments' => $caller_assignments,
                                                'available_caller_hours' => $available_caller_hours,
                                                'caller_access_array' => $caller_access_array,
                                                'import_priority' => $import_priority,
                                                'call_type_import_priority' => $call_type_import_priority,
                                            ]);
    }


    /**
     * Search the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function search(Request $request)
    {
        $today = date('Y-m-d', strtotime($request->current_date));
        
        $dealers = DB::connection('mysql2')->table('callcenter_companies')
                                ->select('co_name', 'co_short', 'coid')
                                ->where('is_active', 1)
                                ->orderBy('co_name')
                                ->get();
        /*$dealer_names = array();
        foreach ($dealers as $key => $value) {
            $dealer_names[$value->coid] = $value->co_name;
        }*/

        $calltypes = CallType::orderBy('import_priority')->get();
        $calltypes_names = array();
        $call_in_q_days  = array();
        $import_priority  = array();
        foreach ($calltypes as $key => $value) {
            $call_in_q_days[$value->id]  = ( ! empty ( $value->call_in_q_days ) ) ? $value->call_in_q_days : 7;
            $calltypes_names[$value->id] = $value->name;
            $import_priority[$value->id] = ( ! empty( $value->import_priority ) )? $value->import_priority : 0;
        }
        
        $call_type_import_priority = [];
        $call_types_at_end = [];
        foreach ( $import_priority as $call_type => $value ) {
            if( ! empty( $value ) ) {
                $call_type_import_priority[] = $call_type;
            } else {
                $call_types_at_end[] = $call_type;
            }
        }
        $call_type_import_priority = array_merge( $call_type_import_priority, $call_types_at_end );

        $assigned_calls = array();
        $assigned_calls_totals = array();
        foreach ($dealers as $key => $value) {
            $db_table = 'livecall_'.$value->co_short;
            /*$livecalls_today = DB::connection('mysql2')->table($db_table)
                                    ->selectRaw('COUNT(id) as total_calls, livecall_next_calltype as call_type_id')
                                    ->where('remind', '>=', date('Y-m-d', strtotime('-7 days')))
                                    ->where('remind', '<=', date('Y-m-d'))
                                    ->whereRaw('livecall_count < livecall_times')
                                    ->where('livecall_showtocall', 1)
                                    ->where('livecall_next_calltype', '!=', 0)
                                    ->whereRaw('livecall_next_calltype IS NOT NULL')
                                    ->groupBy('livecall_next_calltype')
                                    ->get();*/

            $assignable_calltypes = AssignableCalltype::select('call_type_id')
                                    ->where('coid', $value->coid)
                                    ->get()
                                    ->pluck('call_type_id')
                                    ->toArray();

            if(count($assignable_calltypes) > 0) 
            {
                $livecalls_today = DB::connection('mysql2')->table($db_table)
                                    ->selectRaw('livecall_next_calltype as call_type_id, remind, livecall_next_calltype')
                                    ->where('remind', '>=', date('Y-m-d', strtotime('-90 days', strtotime($today))))
                                    ->where('remind', '<=', $today)
                                    ->whereRaw('livecall_count < livecall_times')
                                    ->where('livecall_showtocall', 1)
                                    ->where('livecall_next_calltype', '!=', 0)
                                    ->whereRaw('livecall_next_calltype IS NOT NULL')
                                    ->whereRaw("FIND_IN_SET(livecall_next_calltype, '".implode(',', $assignable_calltypes)."')")
                                    ->get();
            } else {
                $livecalls_today = array();
            }
            

            foreach ($livecalls_today as $call) {
                $time_of_call = strtotime( $call->remind );
                $check_time   = strtotime('-' . $call_in_q_days[ $call->livecall_next_calltype ] . ' days', strtotime( $today ) );
                if ( $time_of_call < $check_time ) {
                    continue;
                }

                if(!isset($assigned_calls_totals[$value->coid]))
                {
                    $assigned_calls_totals[$value->coid] = 0;
                }
                if(!isset($assigned_calls[$value->coid][$call->call_type_id]))
                {
                    $assigned_calls[$value->coid][$call->call_type_id] = 0;
                }
                $assigned_calls[$value->coid][$call->call_type_id]+= 1;
                $assigned_calls_totals[$value->coid]+= 1;
            }
        }
        
        $caller_assignments = array();
        $current_assignments = new \stdClass();
        if($request->reload == 0)
        {
            /* $current_assignments = Assignment::selectRaw('COUNT(id) as total_assignments, user_id, coid, call_type_id')
                                                ->where('assigned_date', $today)
                                                ->groupBy('user_id')
                                                ->groupBy('coid')
                                                ->groupBy('call_type_id')
                                                ->get(); */

            $current_assignments = Assignment::selectRaw('COUNT(id) as total_assignments, user_id, coid, call_type_id')
                ->where('assigned_date', $today)
                ->whereNull('time_on_call')
                ->groupBy('user_id')
                ->groupBy('coid')
                ->groupBy('call_type_id')
                ->get();

            foreach ($current_assignments as $key => $value) {
                $caller_assignments[$value->user_id][$value->coid][$value->call_type_id] = $value->total_assignments;
            }                
            $total_assignments = $current_assignments->count();                  
        } else {
            $total_assignments = 0;
        }
        
        $available_caller_hours = array();
        $caller_hours = CallerSchedule::where('date_scheduled', $today)->get();
        foreach ($caller_hours as $key => $value) {
            $available_caller_hours[$value->user_id] = $value->available_hours;
        }

        $caller_access = UserDealer::join('user_calltypes', 'user_dealer.user_id', '=', 'user_calltypes.user_id')
                                        ->selectRaw('user_dealer.user_id as user_id, user_dealer.dealer_id as dealer_id, user_calltypes.call_type_id as call_type_id')
                                        ->get();

        $caller_access_array = array();
        foreach ($caller_access as $key => $value) {
            $caller_access_array[$value->user_id][$value->dealer_id][$value->call_type_id] = $value->call_type_id;
        }

        $caller_users = User::join('caller_schedules', 'users.id', '=', 'caller_schedules.user_id')
                                ->select('users.*', 'caller_schedules.available_hours as available_hours')
                                ->where('date_scheduled', $today)
                                ->where('user_type_id', 4)
                                ->get();

        $caller_calls_per_day = array();
        foreach ($caller_users as $key => $value) {
            $caller_calls_per_day[$value->id] = $value->calls_per_day_avg * $value->available_hours;
        }

        return view('partials.assignments_search', [
                                                'dealers' => $dealers,
                                                'caller_users' => $caller_users,
                                                'caller_calls_per_day' => $caller_calls_per_day,
                                                'calltypes_names' => $calltypes_names,
                                                'current_assignments' => $current_assignments,
                                                'current_date' => date('m/d/Y', strtotime($today)),
                                                'assigned_calls' => $assigned_calls,
                                                'assigned_calls_totals' => $assigned_calls_totals,
                                                'caller_assignments' => $caller_assignments,
                                                'available_caller_hours' => $available_caller_hours,
                                                'caller_access_array' => $caller_access_array,
                                                'total_assignments' => $total_assignments,
                                                'import_priority' => $import_priority,
                                                'call_type_import_priority' => $call_type_import_priority,
                                            ]);

    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        //
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        $assignment_date = $request->assignment_date;
        $dealers = DB::connection('mysql2')->table('callcenter_companies')
                                ->select('co_name', 'co_short', 'coid')
                                ->where('is_active', 1)
                                ->orderBy('co_name')
                                ->get();
        $co_shorts = array();
        foreach ($dealers as $key => $value) {
            $co_shorts[$value->coid] = $value->co_short;
        }

        $calltypes = CallType::all();
        $call_in_q_days  = array();
        foreach ($calltypes as $key => $value) {
            $call_in_q_days[$value->id]  = ! empty ( $value->call_in_q_days ) ? $value->call_in_q_days : 7;
        }

        $caller_ids = array();
        $i = 0;

        Assignment::where('assigned_date', $assignment_date)->delete();
        AssignmentNote::where('assigned_date', $assignment_date)->delete();
        $already_assigned = array();
        foreach ($request->caller_ids as $caller_id) 
        {
            $coid = $request->coids[$i];
            $call_type_id = $request->call_type_ids[$i];
            $total_calls = $request->total_calls[$i];
            $db_table = 'livecall_'.$co_shorts[$coid];
            $livecalls_ids = DB::connection('mysql2')->table($db_table)
                                    ->select('id')
                                    ->where('remind', '>=', date('Y-m-d', strtotime('-' . $call_in_q_days[$call_type_id] . ' days', strtotime($assignment_date))))
                                    ->where('remind', '<=', $assignment_date)
                                    ->whereRaw('livecall_count < livecall_times')
                                    ->where('livecall_showtocall', 1)
                                    ->where('livecall_next_calltype', $call_type_id)
                                    ->get();

            foreach ($livecalls_ids as $call) {
                if(!isset($already_assigned[$coid][$call->id]))
                {
                    $new_assignment = new Assignment;
                    $new_assignment->user_id = $caller_id;
                    $new_assignment->coid = $coid;
                    $new_assignment->call_type_id = $call_type_id;
                    $new_assignment->call_id = $call->id;
                    $new_assignment->assigned_date = $assignment_date;
                    $new_assignment->save();
                    $already_assigned[$coid][$call->id] = true;
                    $total_calls--;
                    if($total_calls == 0)
                    {
                        break;
                    }
                }
            }

            $i++;
        }

        $i = 0;
        foreach ($request->assignment_notes_caller as $caller_id) 
        {
            $new_assignment_note = new AssignmentNote;
            $new_assignment_note->user_id = $caller_id;
            $new_assignment_note->assignment_note = $request->assignment_notes[$i];
            $new_assignment_note->assigned_date = $request->assignment_date;

            $is_locked_id = 'is_locked_'.$caller_id;
            if(isset($request->{$is_locked_id}))
            {
                $new_assignment_note->is_locked = 1;
            } else {
                $new_assignment_note->is_locked = 0;
            }
            $new_assignment_note->save();
            $i++;
        }

        return response()->json([
            'inserted_id' => 1,
            'message' => 'Assignments saved.'
        ]);
    }

    /**
     * Display the specified resource.
     *
     * @param  \App\Assignment  $assignment
     * @return \Illuminate\Http\Response
     */
    public function show(Assignment $assignment)
    {
        //
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  \App\Assignment  $assignment
     * @return \Illuminate\Http\Response
     */
    public function edit(Assignment $assignment)
    {
        //
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  \App\Assignment  $assignment
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, Assignment $assignment)
    {
        //
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  \App\Assignment  $assignment
     * @return \Illuminate\Http\Response
     */
    public function destroy(Assignment $assignment)
    {
        //
    }

    /**
     * Gets the assigments notes for a specific date.
     *
     * @param  \App\Assignment  $assignment
     * @return \Illuminate\Http\Response
     */
    public function getAssignmentsNotes(Request $request)
    {
        $today = date('Y-m-d', strtotime($request->current_date));
        $assignment_notes = AssignmentNote::where('assigned_date', $today)->get();
        $caller_assignment_notes = array();
        $caller_assignment_locked = array();
        foreach ($assignment_notes as $key => $value) {
            $caller_assignment_notes[$value->user_id] = $value->assignment_note;
            $caller_assignment_locked[$value->user_id] = $value->is_locked;
        }

        $calltypes = CallType::all();
        $calltypes_array = array();
        foreach ($calltypes as $key => $value) {
            $calltypes_array[$value->id] = $value->name;
        }

        $dealers = DB::connection('mysql2')->table('callcenter_companies')
                                ->select('co_name', 'co_short', 'coid')
                                ->where('is_active', 1)
                                ->orderBy('co_name')
                                ->get();

        $dealers_array = array();
        foreach ($dealers as $key => $value) {
            $dealers_array[$value->coid] = $value->co_name;
        }

        $users = User::all();
        $users_array = array();
        foreach ($users as $key => $value) {
            $users_array[$value->id] = $value->first_name.' '.$value->last_name;
        }

        $assignments = json_decode($request->assignments, true);
        return view('dialogs.assignments_preview',[
                                            'assignment_date' => $today, 
                                            'assignments' => $assignments, 
                                            'caller_assignment_notes' => $caller_assignment_notes, 
                                            'caller_assignment_locked' => $caller_assignment_locked, 
                                            'calltypes_array' => $calltypes_array, 
                                            'dealers_array' => $dealers_array,
                                            'users_array' => $users_array,
                                        ]);
    }


    public function getAssignmentsPreview(Request $request)
    {
        $today = date('Y-m-d', strtotime($request->current_date));
        $assignment_notes = AssignmentNote::where('assigned_date', $today)->get();
        $caller_assignment_notes = array();
        $caller_assignment_locked = array();
        foreach ($assignment_notes as $key => $value) {
            $caller_assignment_notes[$value->user_id] = $value->assignment_note;
            $caller_assignment_locked[$value->user_id] = $value->is_locked;
        }

        $calltypes = CallType::all();
        $calltypes_array = array();
        foreach ($calltypes as $key => $value) {
            $calltypes_array[$value->id] = $value->name;
        }

        $dealers = DB::connection('mysql2')->table('callcenter_companies')
                                ->select('co_name', 'co_short', 'coid')
                                ->where('is_active', 1)
                                ->orderBy('co_name')
                                ->get();

        $dealers_array = array();
        foreach ($dealers as $key => $value) {
            $dealers_array[$value->coid] = $value->co_name;
        }

        $users = User::all();
        $users_array = array();
        foreach ($users as $key => $value) {
            $users_array[$value->id] = $value->first_name.' '.$value->last_name;
        }

        $last_assignment_email = SentAssignmentEmail::orderBy('sent', 'desc')->first();

        $assignments = json_decode($request->assignments, true);
        return view('partials.assignments_side_panel',[
                                            'assignment_date' => $today, 
                                            'assignments' => $assignments, 
                                            'caller_assignment_notes' => $caller_assignment_notes, 
                                            'caller_assignment_locked' => $caller_assignment_locked,
                                            'calltypes_array' => $calltypes_array, 
                                            'dealers_array' => $dealers_array,
                                            'users_array' => $users_array,
                                            'last_assignment_email' => $last_assignment_email,
                                        ]);
    }

    function sendAssignmentsEmail(Request $request)
    {
        $today  = date('Y-m-d', strtotime($request->current_date));
        $assignment_notes = AssignmentNote::where('assigned_date', $today)->get();
        if($assignment_notes->count() <= 0)
        {
            return response()->json([
                'message' => 'You need to save the assignments first!'
            ]);
        }
        $caller_assignment_notes = array();
        $caller_assignment_locked = array();
        foreach ($assignment_notes as $key => $value) {
            $caller_assignment_notes[$value->user_id] = $value->assignment_note;
            $caller_assignment_locked[$value->user_id] = $value->is_locked;
        }

        $calltypes = CallType::all();
        $calltypes_array = array();
        foreach ($calltypes as $key => $value) {
            $calltypes_array[$value->id] = $value->name;
        }

        $dealers = DB::connection('mysql2')->table('callcenter_companies')
                                ->select('co_name', 'co_short', 'coid')
                                ->where('is_active', 1)
                                ->orderBy('co_name')
                                ->get();

        $dealers_array = array();
        foreach ($dealers as $key => $value) {
            $dealers_array[$value->coid] = $value->co_name;
        }

        $users = User::all();
        $users_array = array();
        $user_emails = array();
        foreach ($users as $key => $value) {
            $users_array[$value->id] = $value->first_name.' '.$value->last_name;
            $user_emails[$value->id] = $value->email;
        }

        foreach ($caller_assignment_notes as $user_id => $assignment_note) {
            $caller_assignments = Assignment::selectRaw('COUNT(id) as call_count, coid, call_type_id')
                                            ->where('user_id', $user_id)
                                            ->where('assigned_date', $today)
                                            ->groupBy('coid')
                                            ->groupBy('call_type_id')
                                            ->get();
            $assigments_array = array();
            foreach ($caller_assignments as $key => $value) {
                $assigments_array[$value->coid][$value->call_type_id] = $value->call_count;
            }

            $mail_data = new \stdClass();
            $mail_data->subject = 'Call Assignments for: '.$request->current_date;
            $mail_data->view = 'general_email';
            $mail_data->email_css = '.label_td {
                                          font-weight: bold;
                                        }

                                        .value_td {
                                          
                                        }

                                        .footer_notes {
                                          font-size: 10px;
                                        }

                                        .align_top {
                                            vertical-align: baseline;
                                        }';
            
            $mail_data->email_content = view('email.caller_assignment')->with([
                                                                                'user_id' => $user_id,
                                                                                'assignment_note' => $assignment_note,
                                                                                'users_array' => $users_array,
                                                                                'assigments_array' => $assigments_array,
                                                                                'dealers_array' => $dealers_array,
                                                                                'calltypes_array' => $calltypes_array,
                                                                            ])
                                                                            ->render();
            Mail::to($user_emails[$user_id])->send(new GeneralEmailAlert($mail_data));
        }

        $local_time = new DateTime();
        $local_time->setTimezone(new DateTimeZone('America/New_York'));

        $sent_assignment_email = new SentAssignmentEmail;
        $sent_assignment_email->sent = $local_time->format('Y-m-d H:i:s');
        $sent_assignment_email->save();
        
        return response()->json([
            'message' => 'The assignments have been sent!'
        ]);
    }


    function getCallerAssignments(Request $request)
    {
        $today  = date('Y-m-d');
        $assignment_notes = AssignmentNote::where('assigned_date', $today)
                            ->where('user_id', Auth::id())
                            ->first();

        $dealers = DB::connection('mysql2')->table('callcenter_companies')
                                ->select('co_name', 'co_short', 'coid')
                                ->where('is_active', 1)
                                ->orderBy('co_name')
                                ->get();

        $dealers_array = array();
        $co_shorts = array();
        foreach ($dealers as $key => $value) {
            $dealers_array[$value->coid] = $value->co_name;
            $co_shorts[$value->coid] = $value->co_short;
        }

        $caller_assignments_coid = Assignment::select('coid')
                                            ->where('user_id', Auth::id())
                                            ->where('assigned_date', $today)
                                            ->groupBy('coid')
                                            ->get();
        $assignments_array = array();
        foreach ($caller_assignments_coid as $key => $value) {
            $db_table = 'livecall_'.$co_shorts[$value->coid];
            $caller_assignments = DB::table('zassfast_livedata.'.$db_table.' as livecall_table')
                                            ->select(
                                                        'livecall_table.id as id', 
                                                        'livecall_table.fname as fname', 
                                                        'livecall_table.lname as lname',
                                                        'livecall_table.vehicle_year as vehicle_year',
                                                        'livecall_table.vehicle_make as vehicle_make',
                                                        'livecall_table.vehicle_model as vehicle_model',
                                                        'livecall_table.remind as remind',
                                                        'livecall_table.vin as vin',
                                                        'livecall_table.cellphone as cellphone',
                                                        'livecall_table.phone as phone',
                                                        'livecall_table.best_phone as best_phone',
                                                        'livecall_table.global_dnc as global_dnc',
                                                        'livecall_table.dealer_dnc as dealer_dnc',
                                                        'zassfast_fullbdcvintracker.assignments.coid as coid',
                                                        'zassfast_fullbdcvintracker.assignments.call_type_id as call_type_id'
                                                    )
                                            ->join('zassfast_fullbdcvintracker.assignments as assignments', 'livecall_table.id', '=', 'assignments.call_id')
                                            ->where('assignments.user_id', Auth::id())
                                            ->where('assignments.assigned_date', $today)
                                            ->where('assignments.coid', $value->coid)
                                            ->whereRaw('livecall_table.livecall_next_calltype IS NOT NULL')
                                            ->where('livecall_table.livecall_showtocall', 1)
                                            ->whereRaw('livecall_table.livecall_count < livecall_table.livecall_times')
                                            ->orderBy('assignments.call_type_id')
                                            ->get();
            $assignments_array[$value->coid] = $caller_assignments;
        }

        /*$caller_assignments = Assignment::selectRaw('id, coid, call_type_id')
                                            ->where('user_id', Auth::id())
                                            ->where('assigned_date', $today)
                                            ->orderBy('coid')
                                            ->orderBy('call_type_id')
                                            ->get();*/
        
        $caller_assignment_notes = array();
        $caller_assignment_locked = array();
        
        $calltypes = CallType::all();
        $calltypes_array = array();
        foreach ($calltypes as $key => $value) {
            $calltypes_array[$value->id] = $value->name;
        }

        if(isset($request->redirected))
        {
            $redirected =  1;
        } else {
            $redirected = 0;
        }

        return view('pages.caller_assignments',[
                                            'redirected' => $redirected, 
                                            'assignment_date' => $today, 
                                            'assignments_array' => $assignments_array, 
                                            'assignment_notes' => $assignment_notes, 
                                            'calltypes_array' => $calltypes_array, 
                                            'dealers_array' => $dealers_array,
                                        ]);
    }


    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  \App\Company  $company
     * @return \Illuminate\Http\Response
     */
    public function agreeToAssignment(Request $request)
    {
        AssignmentNote::where('id', $request->assignment_id)
                ->update([
                    'approved' => 1,
                    'approved_date' => date('Y-m-d'),
                    ]
                );

        return response()->json([
            'inserted_id' => $request->assignment_id,
            'message' => 'Assignment updated.'
        ]);
    }


    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function assignHoursToCallerPanel(Request $request)
    {

        $callers = User::selectRaw("CONCAT(first_name, ' ', last_name) as name, id")
                            ->where('user_type_id', 4)
                            ->orderBy('first_name')
                            ->get();
        $commonFunctions = new CommonFunctionsController();
        $commonFunctions->modelToSelectOptions($callers,'name', 'id');
        $callers_options = $commonFunctions->select_options;

        return view('pages.caller_hour_assignments', [
                                                'callers_options' => $callers_options,
                                            ]);
    }


    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function getCallerHours(Request $request)
    {
        $start_date = date('Y-m-d', strtotime($request->start_date_search));
        $end_date = date('Y-m-d', strtotime($request->end_date_search));
        $caller_schedule = CallerSchedule::where('user_id', $request->caller_search)
                                    ->where('date_scheduled', '>=', $start_date)
                                    ->where('date_scheduled', '<=', $end_date)
                                    ->get();

        $monday = '';
        $tuesday = '';
        $wednesday = '';
        $thursday = '';
        $friday = '';
        $saturday = '';
        $sunday = '';

        foreach ($caller_schedule as $key => $value) {
            $day = date('w', strtotime($value->date_scheduled));
            switch($day) {
                case 1 : $monday = $value->available_hours; break;
                case 2 : $tuesday = $value->available_hours; break;
                case 3 : $wednesday = $value->available_hours; break;
                case 4 : $thursday = $value->available_hours; break;
                case 5 : $friday = $value->available_hours; break;
                case 6 : $saturday = $value->available_hours; break;
                case 0 : $sunday = $value->available_hours; break;
            }

            if($monday != "" && $tuesday != "" && $wednesday != "" && $thursday != "" && $friday != "" && $saturday != "" && $sunday != "") {
                break;
            }
        }

        return response()->json([
            'monday' => $monday,
            'tuesday' => $tuesday,
            'wednesday' => $wednesday,
            'thursday' => $thursday,
            'friday' => $friday,
            'saturday' => $saturday,
            'sunday' => $sunday,
        ]);
    }


    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function saveCallerHours(Request $request)
    {
        $start_date = date('Y-m-d', strtotime($request->start_date_search));
        $end_date = date('Y-m-d', strtotime($request->end_date_search));
        CallerSchedule::where('user_id', $request->caller_search)
                                    ->where('date_scheduled', '>=', $start_date)
                                    ->where('date_scheduled', '<=', $end_date)
                                    ->delete();

        $current_date = $start_date;              
        while(strtotime($current_date) <= strtotime($end_date)) {
            /*if($values!="") {
                $values.=",";
            }*/
            $day = date('w', strtotime($current_date));
            switch($day) {
                case 1 : $hours_available = $request->monday; break;
                case 2 : $hours_available = $request->tuesday; break;
                case 3 : $hours_available = $request->wednesday; break;
                case 4 : $hours_available = $request->thursday; break;
                case 5 : $hours_available = $request->friday; break;
                case 6 : $hours_available = $request->saturday; break;
                case 0 : $hours_available = $request->sunday; break;
            }
            
            $date_to_insert = date("Y-m-d", strtotime($current_date));
            /*if(isset($not_available_dates[$date_to_insert])) {
                $hours_available = -1;
            }*/
            $caller_schedule = new CallerSchedule;
            $caller_schedule->user_id = $request->caller_search;
            $caller_schedule->date_scheduled = $date_to_insert;
            $caller_schedule->available_hours = $hours_available;
            $caller_schedule->save();
            //$values.= "('$employee_uid', '$date_to_insert', '$hours_available')";
            $current_date = date('m/d/Y', strtotime('+1 day', strtotime($current_date)));
        }

        $caller = User::find($request->caller_search);
        return response()->json([
            'message' => 'Hours updated for '.$caller->first_name.' '.$caller->last_name.'!'
        ]);
    }

}
