<?php

namespace App\Http\Controllers;


use App\CallType;
use App\UserGroupDealer;
use App\CustomerStatus;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\DB;

class CallReportController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {


        if(Auth::user()->user_type->name == 'Admin') 
        {
            $dealers = DB::connection('mysql2')->table('callcenter_companies')
                                ->select('co_name', 'co_short', 'coid')
                                ->where('is_active', 1)
                                ->orderBy('co_name')
                                ->get();
        } else if(Auth::user()->user_type->name == 'Dealer') {
            $user_companies_array = array();
            foreach(Auth::user()->dealers as $company)
            {
              $user_companies_array[] = $company->dealer_id;
            }

            $dealers = DB::connection('mysql2')->table('callcenter_companies')
                                ->select('co_name', 'co_short', 'coid')
                                ->whereRaw("FIND_IN_SET(coid, '".implode(',', $user_companies_array)."')")
                                ->get();
        } else if(Auth::user()->user_type->name == 'Full BDC Caller') {
            $user_companies_array = array();
            foreach(Auth::user()->dealers as $company)
            {
              $user_companies_array[] = $company->dealer_id;
            }

            $dealers = DB::connection('mysql2')->table('callcenter_companies')
                                ->select('co_name', 'co_short', 'coid')
                                ->whereRaw("FIND_IN_SET(coid, '".implode(',', $user_companies_array)."')")
                                ->get();
        } else if(Auth::user()->user_type->name == 'Inbound Caller') {
            $user_companies_array = array();
            foreach(Auth::user()->dealers as $company)
            {
              $user_companies_array[] = $company->dealer_id;
            }

            $dealers = DB::connection('mysql2')->table('callcenter_companies')
                                ->select('co_name', 'co_short', 'coid')
                                ->whereRaw("FIND_IN_SET(coid, '".implode(',', $user_companies_array)."')")
                                ->get();
        } else if(Auth::user()->user_type->name == 'Dealer Caller') {
            $user_companies_array = array();
            foreach(Auth::user()->dealers as $company)
            {
              $user_companies_array[] = $company->dealer_id;
            }

            $dealers = DB::connection('mysql2')->table('callcenter_companies')
                                ->select('co_name', 'co_short', 'coid')
                                ->whereRaw("FIND_IN_SET(coid, '".implode(',', $user_companies_array)."')")
                                ->get();
        }

        if( Auth::user()->user_type->name != 'Admin' ) {
            $user_groups_ids = [];
            foreach (Auth::user()->user_groups as $user_group) {
                $user_groups_ids[] = $user_group->id;
            }
            $dealers_ids = UserGroupDealer::whereRaw("FIND_IN_SET(user_group_id, '".implode(',', $user_groups_ids)."')")->get();
            $dealers_ids_array = [];
            foreach ( $dealers_ids as $dealer_id ) {
                $dealers_ids_array[] = $dealer_id->dealer_id;
            }

            $user_groups_dealers_ids = array_unique( array_merge($user_companies_array, $dealers_ids_array) );

            $dealers = DB::connection('mysql2')->table('callcenter_companies')
                                ->select('co_name', 'co_short', 'coid')
                                ->whereRaw("FIND_IN_SET(coid, '".implode(',', $user_groups_dealers_ids)."')")
                                ->get();
                
        }

        $commonFunctions = new CommonFunctionsController();
        $commonFunctions->modelToSelectOptions($dealers,'co_name', 'coid');
        $dealers_options = $commonFunctions->select_options;

        $commonFunctions->modelToSelectOptions(CallType::all(),'name', 'id');
        $calltypes_options = $commonFunctions->select_options;

        $all_dealers = DB::connection('mysql2')->table('callcenter_companies')
                                ->select('coid', 'co_name', 'co_short')
                                ->orderBy('co_name')
                                ->get();
        $dealer_names = array();
        foreach ($all_dealers as $key => $value) {
        	$dealer_names[$value->coid] = $value->co_name;
        }

        return view('pages.call_report', [
    										'dealers_options' => $dealers_options,
    										'dealer_names' => $dealer_names,
    										'calltypes_options' => $calltypes_options,
										]);
    }


    /**
     * Display a list of the resource with pagination.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */

    public function pagination(Request $request)

    {
        $current_user_id = 0;
        $no_links = false;

        $raw_search = $this->getRawSearch($request);

        //dd($raw_search);

        if($request->record_type_search == "completed")
    	{
        	$search_date = 'lastupdate';
    	} else if($request->record_type_search == "pending"){
    		$search_date = 'remind';
    	} else if($request->record_type_search == "all"){
    		$search_date = 'dateadded';
    	}       

    	$company = DB::connection('mysql2')->table('callcenter_companies')
                                ->select('*')
                                ->where('coid', $request->coid_search)
                                ->first();

        $db_table = 'livecall_'.$company->co_short;

        if( ! empty( $raw_search ) ) {
            $call_records = DB::connection('mysql2')->table($db_table)
        										->select(
        													'id', 
        													'fname', 
        													'lname', 
        													'phone', 
        													'cellphone', 
        													'email', 
        													'vin', 
        													'service_date', 
        													'livecall_prev_calltype', 
        													'livecall_next_calltype', 
                                                            'caller_comments', 
                                                            'customer_status',
        													'lastupdate',
                                                            'global_dnc',
                                                            'dealer_dnc',
                                                            'dateadded',
        													'remind'
        												)
        										->whereRaw($raw_search)
        										->orderBy($search_date, 'desc')->paginate(25);
        } else {
            $call_records = DB::connection('mysql2')->table($db_table)
        										->select(
        													'id', 
        													'fname', 
        													'lname', 
        													'phone', 
        													'cellphone', 
        													'email', 
        													'vin', 
        													'service_date', 
        													'livecall_prev_calltype', 
        													'livecall_next_calltype', 
                                                            'caller_comments', 
                                                            'customer_status',
        													'lastupdate',
                                                            'global_dnc',
                                                            'dealer_dnc',
                                                            'dateadded',
        													'remind'
        												)
        										->orderBy($search_date, 'desc')->paginate(25);
        }
    	
        

        $call_types = CallType::all();
        $call_types_names = array();
        foreach ($call_types as $key => $value) {
        	$call_types_names[$value->id] = $value->name;
        }


        $customer_status = CustomerStatus::orderBy('name')->get();

        $customer_status_icons = array();
        $customer_status_names = array();

        foreach ($customer_status as $key => $value) {
            $customer_status_icons[$value->id] = $value->icon;
            $customer_status_name_parts = explode('-', $value->name);
            $customer_status_names[$value->id] = $customer_status_name_parts[0];
        }

        return view('lists.call_records_list', [
        										'call_records' => $call_records,
        										'call_types_names' => $call_types_names,
        										'record_type' => $request->record_type_search,
        										'company' => $company,
                                                'customer_status_icons' => $customer_status_icons,
                                                'customer_status_names' => $customer_status_names,
    										]);
    }



    public function getRawSearch($my_request) {
        $raw_search_array = array();

        if($my_request->record_type_search == "completed")
    	{
        	$raw_search_array[] = " customer_status != '' ";
        	$raw_search_array[] = " customer_status != 0 ";
        	$raw_search_array[] = " customer_status IS NOT NULL ";
        	//$raw_search_array[] = " livecall_status != '' ";
        	//$raw_search_array[] = " livecall_status != 0 ";
    	} else if($my_request->record_type_search == "pending"){
            $raw_search_array[] = " livecall_count < livecall_times ";
    		$raw_search_array[] = " livecall_showtocall = 1 ";
    		$raw_search_array[] = " global_dnc != 1 ";
    		$raw_search_array[] = " dealer_dnc != 1 ";
    	}

        if($my_request->call_type_id_search != 'all' && $my_request->call_type_id_search != '')
        {
        	if($my_request->record_type_search == "completed")
        	{
            	$raw_search_array[] = " livecall_prev_calltype = '".$my_request->call_type_id_search."' ";
        	} else if($my_request->record_type_search == "pending"){
        		$raw_search_array[] = " livecall_next_calltype = '".$my_request->call_type_id_search."' ";
        	} else if($my_request->record_type_search == "all"){
        		$raw_search_array[] = " ( livecall_next_calltype = '".$my_request->call_type_id_search."' OR livecall_prev_calltype = '".$my_request->call_type_id_search."' ) ";
        	}
        }

        if($my_request->start_date_search != '' && $my_request->end_date_search != '')
        {
        	if($my_request->record_type_search == "completed")
        	{
            	$start_date_search = date('Y-m-d 00:00:00', strtotime($my_request->start_date_search));
            	$end_date_search = date('Y-m-d 23:59:59', strtotime($my_request->end_date_search));
            	$raw_search_array[] = " (lastupdate >= '".$start_date_search."' AND lastupdate <= '".$end_date_search."')";
        	} else if($my_request->record_type_search == "pending") {
        		$start_date_search = date('Y-m-d', strtotime($my_request->start_date_search));
            	$end_date_search = date('Y-m-d', strtotime($my_request->end_date_search));
            	$raw_search_array[] = " (remind >= '".$start_date_search."' AND remind <= '".$end_date_search."')";
        	} else if($my_request->record_type_search == "all") {
            	$start_date_search = date('Y-m-d 00:00:00', strtotime($my_request->start_date_search));
            	$end_date_search = date('Y-m-d 23:59:59', strtotime($my_request->end_date_search));
            	$raw_search_array[] = " (dateadded >= '".$start_date_search."' AND dateadded <= '".$end_date_search."')";
        	}  
        }

        if(trim($my_request->value_search) != '')
        {
        	$raw_search_array[] = " (
                                        fname like '%".$my_request->value_search."%' OR
                                        lname like '%".$my_request->value_search."%' OR
                                        CONCAT(fname, ' ', lname) like '%".$my_request->value_search."%' OR
            							email like '%".$my_request->value_search."%' OR 
                                        phone like '%".$my_request->value_search."%' OR
                                        cellphone like '%".$my_request->value_search."%' OR
                                        vin like '%".$my_request->value_search."%'
                                    )";
        }

        $raw_search = implode(' AND ', $raw_search_array);

        return $raw_search;
    }
}
