<?php

namespace App\Http\Controllers;

use App\CallType;
use App\InboundCallType;
use App\InboundCallTypeScript;
use App\Company;
use App\CallTypeScript;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use PHPUnit\Util\Json;

class CallTypeController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {
        $call_types = DB::connection('mysql2')->table('callcenter_calltypes')
                                ->select('*')
                                ->orderBy('calltype_text')
                                ->get();

        foreach ( $call_types as $call_type ) {
            $dealer_scripts[ $call_type->cid ] = utf8_encode( $call_type->script_content );
        }
        
        foreach ( $call_types as $call_type ) {
            $call_times_values[ $call_type->cid ]                  = $call_type->livecall_times;
            $dealer_costs_values[ $call_type->cid ]['record_cost'] = $call_type->record_cost;
            $dealer_costs_values[ $call_type->cid ]['call_cost']   = $call_type->call_cost;
            $dealer_costs_values[ $call_type->cid ]['flat_cost']   = $call_type->flat_cost;
        }
        
        return view('pages.call_types', [
                                                        'call_types' => $call_types,
                                                        'dealer_scripts' => $dealer_scripts,
                                                        'dealer_costs_values' => $dealer_costs_values,
                                                        'call_times_values' => $call_times_values,
                                                    ]);
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        //
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        //
    }

    /**
     * Display the specified resource.
     *
     * @param  \App\CallType  $callType
     * @return \Illuminate\Http\Response
     */
    public function show(CallType $callType)
    {
        //
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  \App\CallType  $callType
     * @return \Illuminate\Http\Response
     */
    public function edit(CallType $callType)
    {
        //
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  \App\CallType  $callType
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, CallType $callType)
    {
        //
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  \App\CallType  $callType
     * @return \Illuminate\Http\Response
     */
    public function destroy(CallType $callType)
    {
        //
    }

    public function getScriptData(Request $request)
    {
        $call_type = CallType::find($request->call_type_id);
        $call_type_script = CallTypeScript::where('call_type_id', $request->call_type_id)
                                            ->where('coid', $request->coid)
                                            ->first();
        if(!$call_type_script)
        {
            $call_type_script = CallTypeScript::where('call_type_id', $request->call_type_id)
                                            ->where('coid', 0)
                                            ->first();
        }

        $company = DB::connection('mysql2')->table('callcenter_companies')
                                ->select('*')
                                ->where('coid', $request->coid)
                                ->first();

        if($request->transaction_type == 'call_records')
        {
            $db_table = 'livecall_'.$company->co_short;
            $call_record = DB::connection('mysql2')->table($db_table)
                                ->select('*')
                                ->where('id', $request->record_id)
                                ->first();
        } else if($request->transaction_type == 'leo')
        {
            $db_table = $company->co_short.'_master_vins';
            $call_record = DB::connection('mysql')->table($db_table)
                                ->select('*')
                                ->where('id', $request->record_id)
                                ->first();
        } else if($request->transaction_type == 'recall')
        {
            $db_table = 'recall_records';
            $call_record = DB::connection('mysql')->table($db_table)
                                ->select('*')
                                ->where('id', $request->record_id)
                                ->first();
        } else if($request->transaction_type == 'toyota_care')
        {
            $db_table = 'toyota_care_'.$company->co_short; 
            $call_record = DB::connection('mysql')->table($db_table)
                                ->select('*')
                                ->where('id', $request->record_id)
                                ->first();

        }

        $commonFunctions = new CommonFunctionsController();
        $script = $commonFunctions->removeTagsFromText($call_type_script->content, $company, $call_record, $ibData=array());

        return response()->json([
            'key' => $call_type->key,
            'name' => $call_type->name,
            'call_times' => $call_type->call_times,
            'coid' => $call_type_script->coid,
            'script' => $script
        ]);
    }

    public function getScriptDataRaw(Request $request)
    {
        $call_type = DB::connection('mysql2')->table('callcenter_calltypes')
                                ->select('*')
                                ->where('cid', $request->cid)
                                ->first();
        $coid = ! empty( $request->coid ) ? $request->coid : '';
        $call_type_script = DB::connection('mysql2')->table('callcenter_companies_callscripts')
            ->where('coid', $coid)
            ->where('csid', $request->cid)
            ->first();

        $script = ! empty( $call_type_script ) ? $call_type_script->script_content : $call_type->script_content;
        $script = utf8_encode( $script );

        return view('dialogs.call_script_edit', [
            'call_type' => $call_type,
            'coid'       => $request->coid,
            'cid'        => $request->cid,
            'script'     => $script,
        ]);
    }

    public function updateCallScript( Request $request ) 
    {
        $script_content = rawurldecode( $request->script_content );
        $coid           = ! empty( $request->coid ) ? $request->coid : '';
        $cid            = $request->cid;

        $call_type_script = DB::connection('mysql2')->table('callcenter_companies_callscripts')
            ->where('coid', $coid)
            ->where('csid', $cid)
            ->first();
        if ( $call_type_script ) {
            DB::connection('mysql2')->table('callcenter_companies_callscripts')
            ->where('csid', $cid)
            ->where('coid', $coid)
            ->update([
                'script_content' => $script_content,
            ]);
        } else {
            DB::connection('mysql2')->table('callcenter_companies_callscripts')
            ->insert([
                'csid'           => $cid,
                'coid'           => $coid,
                'script_content' => $script_content,
            ]);
        }

        // Make sure we update the global call type script content if the coid is empty
        if( empty( $coid ) ) {
            DB::connection('mysql2')->table('callcenter_calltypes')
                ->where('cid', $cid)
                ->update([
                    'script_content' => $script_content,
                ]);
        }

        $coid             = ! empty( $request->coid ) ? $request->coid : 0;
        $call_type_script = CallTypeScript::where('call_type_id', $request->call_type_id)
            ->where('coid', $coid)
            ->first();
        if ( $call_type_script ) {
            CallTypeScript::where('call_type_id', $cid)
            ->where('coid', $coid)
            ->update([
                'content' => $script_content,
            ]);
        } else {
            CallTypeScript::insert([
                'call_type_id'   => $cid,
                'coid'           => $coid,
                'content' => $script_content,
            ]);
        }

        return response()->json([
            'message' => 'Call script updated successfully',
        ]);
    }


    public function getInboundScriptData(Request $request)
    {
        $call_type = InboundCallType::find($request->inbound_call_type_id);
        $call_type_script = InboundCallTypeScript::where('inbound_call_type_id', $request->inbound_call_type_id)
                                            ->where('coid', $request->coid)
                                            ->first();
        if(!$call_type_script)
        {
            $call_type_script = InboundCallTypeScript::where('inbound_call_type_id', $request->inbound_call_type_id)
                                            ->where('coid', 0)
                                            ->first();
        }

        $company = DB::connection('mysql2')->table('callcenter_companies')
                                ->select('*')
                                ->where('coid', $request->coid)
                                ->first();

        $commonFunctions = new CommonFunctionsController();
        $script = $commonFunctions->removeTagsFromText($call_type_script->content, $company, $call_record=array(), $ibData=array());


        $inbound_reps = DB::connection('mysql2')
                                    ->table('callcenter_inbound_sales_person')
                                    ->select('callcenter_inbound_sales_person.*')
                                    ->join('callcenter_inbound_salesperson_department', 'callcenter_inbound_sales_person.id', '=', 'callcenter_inbound_salesperson_department.pid')
                                    ->where('callcenter_inbound_sales_person.coid', $request->coid)
                                    ->where('callcenter_inbound_sales_person.email', '!=', '')
                                    ->whereRaw('callcenter_inbound_sales_person.email IS NOT NULL')
                                    ->where('callcenter_inbound_sales_person.on_alert_list', 1)
                                    ->whereRaw('FIND_IN_SET(\''.$request->inbound_call_type_id.'\', callcenter_inbound_salesperson_department.alert_access)')
                                    ->orderBy('callcenter_inbound_sales_person.fname')
                                    ->get();
        $inbound_reps_html = '';

        foreach ($inbound_reps as $key => $value) {
            $inbound_reps_html.='<div class="custom-control custom-checkbox">
                                    <input type="checkbox" class="custom-control-input inbound_reps" id="inbound_rep_'.$value->id.'" name="send_alert_to[]" value="'.$value->email.'">
                                    <label class="custom-control-label" for="inbound_rep_'.$value->id.'">'.$value->fname.' '.$value->lname.'</label>
                                </div>';
        }

        return response()->json([
            'coid' => $call_type_script->coid,
            'script' => $script,
            'name' => $call_type->name,
            'inbound_reps_html' => $inbound_reps_html
        ]);
    }



    public function getIBScriptDataRaw(Request $request)
    {
        $call_type = DB::connection('mysql2')->table('callcenter_calltypes_inbound')
                                ->select('*')
                                ->where('cid', $request->cid)
                                ->first();
        $coid = ! empty( $request->coid ) ? $request->coid : '';
        $call_type_script = DB::connection('mysql2')->table('callcenter_calltypes_inbound_scripts')
            ->where('coid', $coid)
            ->where('cid', $request->cid)
            ->first();

        if( ! $call_type_script ) {
            $call_type_script = DB::connection('mysql2')->table('callcenter_calltypes_inbound_scripts')
            ->where('coid', 0)
            ->where('cid', $request->cid)
            ->first();
        }

        $script = ! empty( $call_type_script ) ? $call_type_script->script_content : '';
        $script = utf8_encode( $script );

        return view('dialogs.call_script_edit', [
            'call_type'  => $call_type,
            'coid'       => $request->coid,
            'cid'        => $request->cid,
            'script'     => $script,
            'is_inbound' => 1,
        ]);
    }

    public function updateIBCallScript( Request $request ) 
    {
        $script_content = rawurldecode( $request->script_content );
        $coid           = ! empty( $request->coid ) ? $request->coid : '';
        $cid            = $request->cid;

        $call_type_script = DB::connection('mysql2')->table('callcenter_calltypes_inbound_scripts')
            ->where('coid', $coid)
            ->where('cid', $cid)
            ->first();
        if ( $call_type_script ) {
            DB::connection('mysql2')->table('callcenter_calltypes_inbound_scripts')
            ->where('cid', $cid)
            ->where('coid', $coid)
            ->update([
                'script_content' => $script_content,
            ]);
        } else {
            DB::connection('mysql2')->table('callcenter_calltypes_inbound_scripts')
            ->insert([
                'cid'           => $cid,
                'coid'           => $coid,
                'script_content' => $script_content,
            ]);
        }

        // Make sure we update the global call type script content if the coid is empty
        if( empty( $coid ) ) {
            $call_type_script = DB::connection('mysql2')->table('callcenter_calltypes_inbound_scripts')
            ->where('coid', 0)
            ->where('cid', $cid)
            ->first();
            if ( $call_type_script ) {
                DB::connection('mysql2')->table('callcenter_calltypes_inbound_scripts')
                ->where('coid', 0)
                ->where('cid', $cid)
                ->update([
                    'script_content' => $script_content,
                ]);
            } else {
                DB::connection('mysql2')->table('callcenter_calltypes_inbound_scripts')
                ->insert([
                    'cid'           => $cid,
                    'coid'           => 0,
                    'script_content' => $script_content,
                ]);
            }
        }

        return response()->json([
            'message' => 'Call script updated successfully',
        ]);
    }

    public function updateCallTypeValue( Request $request ) {
        $update_value  = $request->update_value;
        $column = $request->column;
        $coid   = $request->coid;
        $cid    = $request->cid;

        if( $coid == 0 ) {
            DB::connection('mysql2')
                ->table('callcenter_calltypes')
                ->where('cid', $cid)
                ->update([
                    $column => $update_value,
                ]);
                
            CallType::where('id', $cid)
                ->update([
                    $column => $update_value,
                ]);
        }

        return response()->json([
            'message' => 'Call type value updated successfully',
        ]);
    }

    public function surveyQuestionsDialog(Request $request)
    {
        $call_type = DB::connection('mysql2')->table('callcenter_calltypes')
                                ->select('*')
                                ->where('cid', $request->cid)
                                ->first();
        $coid = ! empty( $request->coid ) ? $request->coid : 0;

        $questions = DB::connection('mysql2')->table('callcenter_survey_questions')
            ->where('coid', $coid)
            ->where('cid', $request->cid)
            ->get();

        return view('dialogs.survey_questions_dialog', [
            'call_type' => $call_type,
            'questions' => $questions,
            'coid'      => $coid,
            'cid'       => $request->cid,
        ]);
    }

    public function surveyQuestionsDelete(Request $request) {
        DB::connection('mysql2')->table('callcenter_survey_questions')
            ->where('id', $request->qid)
            ->delete();

        return response()->json([
            'qid' => $request->qid,
        ]);
    }

    public function surveyQuestionsInsert(Request $request) {
        $new_id = DB::connection('mysql2')->table('callcenter_survey_questions')
            ->insertGetId([
                'coid' => $request->coid,
                'cid' => $request->cid,
            ]);

        return response()->json([
            'qid' => $new_id,
        ]);
    }

    public function updateSurveyQuestions(Request $request) {
        $i = 0;
        foreach($request->ids as $id) {
            DB::connection('mysql2')->table('callcenter_survey_questions')
            ->where('id', $id)
            ->update([
                'question' => $request->questions[$i],
            ]);
            $i++;
        }

        return response()->json([
            'message' => 'Done!',
        ]);
    }
}