<?php

namespace App\Http\Controllers;

use DateTime;
use DatePeriod;
use DateInterval;
use App\CallerSchedule;
use App\ScheduleChangeRequest;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;

class CallerScheduleController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {
        //
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        //
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        //
    }

    /**
     * Display the specified resource.
     *
     * @param  \App\CallerSchedule  $callerSchedule
     * @return \Illuminate\Http\Response
     */
    public function show(CallerSchedule $callerSchedule)
    {
        //
    }


    public function events(Request $request)

    {
        $start_date_iso = new DateTime($request->start);
        $start_date = $start_date_iso->format('Y-m-d H:i:s'); 
        $end_date_iso = new DateTime($request->end);
        $end_date = $end_date_iso->format('Y-m-d H:i:s'); 
        if(Auth::user()->user_type->name == 'Admin' || Auth::user()->user_type->name == 'Dealer') 
        {
            $caller_schedules = CallerSchedule::where('user_id', $request->user_id)->
                                                whereRaw("date_scheduled BETWEEN '".$start_date."' AND '".$end_date."' ")
                                                ->get();
        } elseif(Auth::user()->user_type->name == 'Full BDC Caller' || Auth::user()->user_type->name == 'Dealer Caller' || Auth::user()->user_type->name == 'Inbound Caller') {
            $caller_schedules = CallerSchedule::where('user_id', Auth::id())->
                                                whereRaw("date_scheduled BETWEEN '".$start_date."' AND '".$end_date."' ")
                                                ->get();
        } 
        $half_hour_seconds = 1800;
        $event_array = array();
        foreach ($caller_schedules as $key => $value) {
            $start_date = date('Y-m-d 8:00:00', strtotime($value->date_scheduled));
            $start = strtotime($start_date);
            $end = $start + $half_hour_seconds;

            $new_start_date = new DateTime($start_date);
            $start_date_iso = $new_start_date->format(DateTime::ATOM);

            $new_end_date = new DateTime(date('Y-m-d H:i:s', $end));
            $end_date_iso = $new_end_date->format(DateTime::ATOM);
            
            $title = 'Hours: '.$value->available_hours;
            $description =  '<strong>Hours:</strong> '.$value->available_hours.
                            '<br><strong>Caller Notes:</strong> '.$value->caller_notes.
                            '<br><strong>Admin Notes:</strong> '.$value->admin_notes;
            $event_array[] = '{
                                  "title": "'.$title.'",
                                  "start": "'.$start_date_iso.'",
                                  "end": "'.$end_date_iso.'",
                                  "hours": "'.$value->available_hours.'",
                                  "description": "'.$description.'",
                                  "schedule_id": "'.$value->id.'"
                                }';
        }

        $events = '[
                    '.implode(',', $event_array).'
                  ]';
        return $events;
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  \App\CallerSchedule  $callerSchedule
     * @return \Illuminate\Http\Response
     */
    public function edit(Request $request, CallerSchedule $callerSchedule)
    {
        $caller_schedule = CallerSchedule::where('user_id', Auth::id())
                                            ->where('date_scheduled', date('Y-m-d', strtotime($request->search_date)))
                                            ->first();
        $date_to_schedule = date('m/d/Y', strtotime($request->search_date));
        $has_schedule_change_request = 0;
        if(!$caller_schedule)
        {
            $caller_schedule = new \stdClass();  
            $caller_schedule->id = 0;
            $caller_schedule->user_id =  Auth::id();
            $caller_schedule->date_scheduled = date('Y-m-d', strtotime($request->search_date));
            $caller_schedule->available_hours = 0;
            $caller_schedule->caller_notes = '';
            $caller_schedule->admin_notes = '';
        } else {
            $schedule_change_request = ScheduleChangeRequest::where('caller_schedule_id', $caller_schedule->id)
                                                                ->whereNull('resolution_id')
                                                                ->first();
            if($schedule_change_request)
            {
                $has_schedule_change_request = 1;
            }
        }



        return view('dialogs.edit_caller_schedule', [
                                                'caller_schedule' => $caller_schedule,
                                                'date_to_schedule' => $date_to_schedule,
                                                'has_schedule_change_request' => $has_schedule_change_request,
                                            ]);

    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  \App\CallerSchedule  $callerSchedule
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, CallerSchedule $callerSchedule)
    {
        $validatedData = $request->validate([
            'date_scheduled' => 'required|date',
            'available_hours' => 'required|numeric',
            'caller_notes' => 'max:5000|nullable',
            'admin_notes' => 'max:5000|nullable',
        ]);

        
        if($request->caller_schedule_id == 0)
        {
            $caller_schedule = new CallerSchedule;
            $caller_schedule->user_id = Auth::id();
            $caller_schedule->date_scheduled = date('Y-m-d', strtotime($request->date_scheduled));
            $caller_schedule->available_hours = $request->available_hours;
            $caller_schedule->caller_notes = $request->caller_notes;
            $caller_schedule->admin_notes = $request->admin_notes;
            $caller_schedule->save();
            $caller_schedule_id = $caller_schedule->id;
        } else {

            if(Auth::user()->user_type->name == 'Admin' || Auth::user()->user_type->name == 'Dealer') 
            {
                CallerSchedule::where('id', $request->caller_schedule_id)
                ->update(
                    $request->only( 'available_hours', 
                                    'caller_notes',
                                    'admin_notes',
                                )
                );
            } else {
                CallerSchedule::where('id', $request->caller_schedule_id)
                ->update(
                    $request->only( 'caller_notes'
                                )
                );
            }
            
            $caller_schedule_id = $request->caller_schedule_id;
        }

        return response()->json([
            'inserted_id' => $caller_schedule_id,
            'message' => 'Hours updated!'
        ]);
  
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  \App\CallerSchedule  $callerSchedule
     * @return \Illuminate\Http\Response
     */
    public function editRange(Request $request, CallerSchedule $callerSchedule)
    {
        return view('dialogs.edit_caller_schedule_range', []);
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  \App\CallerSchedule  $callerSchedule
     * @return \Illuminate\Http\Response
     */
    public function updateRange(Request $request, CallerSchedule $callerSchedule)
    {
        $validatedData = $request->validate([
            'start_date' => 'required|date',
            'end_date' => 'required|date',
            'available_hours' => 'required|numeric',
        ]);


        if(strtotime($request->start_date) > strtotime($request->end_date))
        {
            return response()->json([
                'message' => 'Invalid date range!'
            ], 500);
        }   

        $start_date = date('Y-m-d', strtotime($request->start_date));
        $end_date = date('Y-m-d', strtotime($request->end_date));

        $period = new DatePeriod(
             new DateTime($start_date),
             new DateInterval('P1D'),
             new DateTime($end_date.' +1 day')
        );

        foreach ($period as $key => $value) {
            $caller_schedule = CallerSchedule::where('user_id', Auth::id())->where('date_scheduled', $value->format('Y-m-d'))->first();
            if($caller_schedule)
            {
                if(Auth::user()->user_type->name == 'Admin' || Auth::user()->user_type->name == 'Dealer') 
                {
                    CallerSchedule::where('id', $caller_schedule->id)
                    ->update(
                        $request->only( 'available_hours'
                                    )
                    );
                }
            } else {
                $caller_schedule = new CallerSchedule;
                $caller_schedule->user_id = Auth::id();
                $caller_schedule->date_scheduled = $value->format('Y-m-d');
                $caller_schedule->available_hours = $request->available_hours;
                $caller_schedule->save();
                $caller_schedule_id = $caller_schedule->id;
            }    
        }

        return response()->json([
            'message' => 'Hours updated!'
        ]);
  
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  \App\CallerSchedule  $callerSchedule
     * @return \Illuminate\Http\Response
     */
    public function destroy(CallerSchedule $callerSchedule)
    {
        //
    }


    /**
     * Remove the specified resource from storage.
     *
     * @param  \App\CallerSchedule  $callerSchedule
     * @return \Illuminate\Http\Response
     */
    public function requestChangeDialog(Request $request, CallerSchedule $callerSchedule)
    {
        $caller_schedule = CallerSchedule::find($request->caller_schedule_id);
        return view('dialogs.schedule_change_request', [
                                                'caller_schedule' => $caller_schedule,
                                            ]);
    }


    /**
     * Remove the specified resource from storage.
     *
     * @param  \App\CallerSchedule  $callerSchedule
     * @return \Illuminate\Http\Response
     */
    public function requestChange(Request $request, CallerSchedule $callerSchedule)
    {
        $validatedData = $request->validate([
            'hours_from' => 'required|numeric',
            'hours_to' => 'required|numeric',
            'reason' => 'nullable|max:10000',
        ]);

        $schedule_change_request = new ScheduleChangeRequest;
        $schedule_change_request->caller_schedule_id = $request->caller_schedule_id;
        $schedule_change_request->hours_from = $request->hours_from;
        $schedule_change_request->hours_to = $request->hours_to;
        $schedule_change_request->requested_by = $request->caller_schedule_user_id;
        $schedule_change_request->reason = $request->reason;
        $schedule_change_request->save();

        return response()->json([
            'inserted_id' => $schedule_change_request->id,
            'message' => 'Change requested!.'
        ]);
    }




    
}
