<?php

namespace App\Http\Controllers;

use Illuminate\Support\Facades\DB;
use Illuminate\Http\Request;
use App\Events\UserAlert;
use App\CallType;
use App\CompanyCallTime;
use Illuminate\Support\Facades\Auth;

class CommonFunctionsController extends Controller
{
    public $select_options;
    public $new_date_format;
    public $letter;
    public $number_letter;

	/**
     * Show the form for editing the specified resource.
     *
     * @param  Eloquent Model $model
     * @param  key text identifier $text_key
     * @param  key value identifier $value_key
     * @return array of values $to_array 
     */
    public function modelToSelectOptions($model, $text_key, $value_key)
    {
    	$from_array = $model->toArray();
    	$to_array = array();
    	$i = 0;
    	foreach ($from_array as $new_array) 
    	{
    		foreach ($new_array as $key => $value) 
    		{
    			//dd($value);
    			if($key == $text_key) $to_array[$i]["text"] = $value;
    			if($key == $value_key) $to_array[$i]["value"] = $value;
    		}
    		$i++;
    	}
    	
    	$json_array = json_decode(json_encode($to_array), FALSE);
    	$this->select_options = $json_array;
    }


    public function getTimeOnCall($user_reference_id, $coid, $call_id)
    {
        $time_on_call = DB::connection('mysql4')->table('employee_assignment')
                                ->select('time_spent')
                                ->where('employee_uid', $user_reference_id)
                                ->where('companyid', $coid)
                                ->where('callid', $call_id)
                                ->first();     
        if($time_on_call)
        {
            $totaltime = $time_on_call->time_spent;
            $totalHours = $totaltime/3600;

            $currentMilliseconds = $totalHours*60*60*1000;

            $decTime = $totalHours;
            $hour = floor($decTime);
            $min = floor(60*($decTime-$hour));
            $sec = (60*($decTime-$hour))-(floor(60*($decTime-$hour)));
            $sec = floor(60*$sec);

            $currentTimer = $this->fillWithZeros($hour).":".$this->fillWithZeros($min).":".$this->fillWithZeros($sec);
        } else {
            $currentTimer = '00:00:00';
        }
        
        return $currentTimer;
    }

    public function fillWithZeros($t){
        if(strlen($t)==1) {
            return "0".$t;
        }
        else {
            return $t;
        }
    }

    public function convertDateFormat($current_date, $separator)
    {
        $good_format = true;
        if(strpos($current_date, $separator) === false)
        {
            $separator = '/';
        }
        if(strpos($current_date, $separator) === false)
        {
            $separator = '-';
        }
        if(strpos($current_date, $separator) === false)
        {
            $separator = '.';
        }
        if(strpos($current_date, $separator) === false)
        {
            $good_format = false;
        }
        if($good_format == true)
        {
            $date_parts = explode($separator, $current_date);
            $this->new_date_format = date('Y-m-d', strtotime($date_parts[1].$separator.$date_parts[0].$separator.$date_parts[2]));
        }
        
    }

    public function getNameFromNumber($num) {
        $numeric = $num % 26;
        $letter = chr(65 + $numeric);
        $num2 = intval($num / 26);
        if ($num2 > 0) {
            return getNameFromNumber($num2 - 1) . $letter;
        } else {
            $this->letter = $letter;
        }
    }

    public function getNumberFromName($letter) {
        $letters = array('A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M', 'N', 'O', 'P', 'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X', 'Y', 'Z' );
        $number_letter = array_search($letter, $letters);
        $this->number_letter = $number_letter;
    }


    public function getDealersModelAll()
    {

        $dealers = DB::connection('mysql2')->table('callcenter_companies')
                                ->select('co_name', 'coid')
                                ->where('is_active', 1)
                                ->orderBy('co_name')
                                ->get();

        return $dealers;
    }


    public function getDealersNames()
    {

        $dealers = DB::connection('mysql2')->table('callcenter_companies')
                                ->select('co_name', 'coid')
                                ->where('is_active', 1)
                                ->orderBy('co_name')
                                ->get();
        $dealer_names = array();
        foreach ($dealers as $key => $value) {
            $dealer_names[$value->coid] = $value->co_name;
        }

        return $dealer_names;
    }


    public function getDealersModel($search)
    {

        $dealers = DB::connection('mysql2')->table('callcenter_companies')
                                ->select('co_name', 'coid')
                                ->where('is_active', 1)
                                ->whereRaw($search)
                                ->orderBy('co_name')
                                ->get();

        return $dealers;
    }


    public function getDealersDepartments()
    {

        $dealer_departments = DB::connection('mysql2')->table('callcenter_companies_departments')
                                ->select('id', 'name')
                                ->orderBy('name')
                                ->get();

        return $dealer_departments;
    }


    public function getDealerOptions()
    {
        $dealers = $this->getDealersModel();
        $this->modelToSelectOptions($dealers,'co_name', 'coid');
        $dealers_options = $this->select_options;
        return $dealers_options;
    }

    public function checkSession(Request $request)
    {
        $alert_data = new \stdClass();
        $alert_data->type = 'logout';
        $channel_id = '.'.Auth::id();
        broadcast(new UserAlert($alert_data, $channel_id));

        $output = array(
         'session_status' => 'success'
        );
        return response()->json($output);
    }


    public function hoursToSecods($hour) { // $hour must be a string type: "HH:mm:ss"
        $parse = array();
        if (!preg_match ('#^(?<hours>[\d]{2}):(?<mins>[\d]{2}):(?<secs>[\d]{2})$#',$hour,$parse)) {
            // Throw error, exception, etc
            return "not valid";
        }
        return (int) $parse['hours'] * 3600 + (int) $parse['mins'] * 60 + (int) $parse['secs'];
    }


    public function removeTagsFromText($content="",$companyData=array(),$customerData=array(),$ibData=array()) 
    {
        $replacefieldscompany = 'bdc_name,co_name,co_phone,dealer_website,service_mgr_title,service_mgr_name,service_phone,service_email,sales_mgr_title,sales_mgr_name,sales_phone,sales_email,twilio_sms';
        $replacefieldscompany = explode(",", $replacefieldscompany);
        $replacefieldscustomer = 'notes,customer_id,first_name,last_name,fname,lname,phone,email,cellphone,vehicle_year,vehicle_make,vehicle_model,vin,ro_number,sales_rep,service_advisor,service_date,address,city,state,zip,sale_date,service_visit_notes,tc_exp_date,service_apt_date,service_apt_time,sale_apt_date,lead_date,caller_comments,salesrep';
        $replacefieldscustomer = explode(",", $replacefieldscustomer);
        $replacefieldsibform = 'appointment_date,appointment_time,appointment_with,sell_vehicle,vehicle_year_sell,vehicle_make_sell,vehicle_model_sell,bank_payment,current_balance,vehicle_condition,current_miles,time_on_call,resolution,billable_call,farmed_email_address,alert_sent_to,copy_alert_to,appointment_notes';
        $replacefieldsibform = explode(",", $replacefieldsibform);


        foreach($replacefieldsibform as $value) {
            if(isset($ibData->{$value}))
                $content = str_replace("[".$value."]", $ibData->{$value}, $content);
        }

        foreach($replacefieldscompany as $value) {
            if(isset($companyData->{$value}))
                $content = str_replace("[".$value."]", $companyData->{$value}, $content);
        }

        foreach($replacefieldscustomer as $value) {
            if(isset($customerData->{$value}))
            {
                if($value == "service_date" || $value == "sale_date" || $value == "tc_exp_date" || $value == "last_service_date" || $value == "service_apt_date" || $value == "sale_apt_date") {
                    $customerData->{$value} = ($customerData->{$value} != "" && $customerData->{$value} != "0000-00-00")? date("m/d/Y", strtotime($customerData->{$value})) : "";
                } 
                if($value == "notes")
                {
                    $content = str_replace("[caller_comments]", $customerData->{$value}, $content);
                } else if($value == "first_name")
                {
                    $content = str_replace("[fname]", $customerData->{$value}, $content);
                } else if($value == "last_name")
                {
                    $content = str_replace("[lname]", $customerData->{$value}, $content);
                } else if($value == "sales_rep")
                {
                    $content = str_replace("[salesrep]", $customerData->{$value}, $content);
                } else {
                    $content = str_replace("[".$value."]", $customerData->{$value}, $content);
                }     
            } else {
                $content = str_replace("[".$value."]", "", $content);
            }
        }
        
        return $content;
    }

    public function adfRemoveTags($content="", $values=array()) {
        $replacefields = 'caller_comments,customer_status,call_type';
        $replacefields = explode(",", $replacefields);
        foreach($replacefields as $value) {
            $content = str_replace("[".$value."]", $values[$value], $content);
        }
        return $content;
    }


    public function setRadioValue($value, $compare)
    {
        if($value ==  $compare)
        {
            return "checked";
        }
        return "";
    }

    public function setSelectValue($value, $compare)
    {
        if($value ==  $compare)
        {
            return "selected";
        }
        return "";
    }

    public function getDepartmentsSelectId($department_access = "", $selected="") 
    {
        if($department_access != "")
        {
            $companies = DB::connection('mysql2')
                            ->table('callcenter_companies_departments')
                            ->select('id', 'name')
                            ->whereRaw("id IN (".$department_access.")")
                            ->orderBy('name')
                            ->get();
        } else {
            $companies = DB::connection('mysql2')
                            ->table('callcenter_companies_departments')
                            ->select('id', 'name')
                            ->orderBy('name')
                            ->get();
        }
        
        $selectcontent = '';
        foreach($companies as $company) {
            $id = $company->id;
            $name = $company->name;
            $selectcontent.= '<option value="'.$id.'" '.$this->setSelectValue($id, $selected).'>'.$name.'</option>';
        }

        return $selectcontent;
    }

    public function getDepartmentsSelect($department_access = "", $selected="") 
    {

        if($department_access != "")
        {
            $companies = DB::connection('mysql2')
                            ->table('callcenter_companies_departments')
                            ->select('id', 'name')
                            ->whereRaw("id IN (".$department_access.")")
                            ->orderBy('name')
                            ->get();
        } else {
            $companies = DB::connection('mysql2')
                            ->table('callcenter_companies_departments')
                            ->select('id', 'name')
                            ->orderBy('name')
                            ->get();
        }

        $selectcontent = '';
        foreach($companies as $company) {
            $id = $company->id;
            $name = $company->name;
            $selectcontent.= '<option value="'.$id.'" '.$this->setSelectValue(strtolower($name), strtolower($selected)).'>'.$name.'</option>';
        }
        //$selectcontent.= '</select>';
        return $selectcontent;
    }

    public function getDepartmentsSelectOptions($department_access = "") 
    {

        if($department_access != "")
        {
            $companies = DB::connection('mysql2')
                            ->table('callcenter_companies_departments')
                            ->select('id', 'name')
                            ->whereRaw("id IN (".$department_access.")")
                            ->orderBy('name')
                            ->get();
        } else {
            $companies = DB::connection('mysql2')
                            ->table('callcenter_companies_departments')
                            ->select('id', 'name')
                            ->orderBy('name')
                            ->get();
        }

        $this->modelToSelectOptions($companies, 'name', 'id');
        $select_options = $this->select_options;
        return $select_options;
    }


    public function getCalltypesKeysText($wheresql="") {
        $callTypes = array();

        $result = DB::connection('mysql2')
                            ->table('callcenter_calltypes')
                            ->select('calltype_key', 'calltype_text')
                            ->orderBy('calltype_text')
                            ->get();
    
        foreach($result as $row) {
            $callTypes[$row->calltype_key] = $row->calltype_text;
        }
        
        return $callTypes;
    } 

    public function getCalltypesIdByKey($wheresql="") {
        $callTypes = [];

        $result = DB::connection('mysql2')
                            ->table('callcenter_calltypes')
                            ->select('calltype_key', 'cid')
                            ->get();
    
        foreach($result as $row) {
            $callTypes[$row->calltype_key] = $row->cid;
        }
        
        return $callTypes;
    } 

    public function getCalltypesKeyById($wheresql="") {
        $callTypes = [];

        $result = DB::connection('mysql2')
                            ->table('callcenter_calltypes')
                            ->select('calltype_key', 'cid')
                            ->get();
    
        foreach($result as $row) {
            $callTypes[$row->cid] = $row->calltype_key;
        }
        
        return $callTypes;
    } 

    public function getCalltypesSelect($wheresql="", $selectedvalue="") 
    {
        $callTypes = array();
        $callTypes_keys = array();
        $result = DB::connection('mysql2')
                            ->table('callcenter_calltypes')
                            ->select('cid', 'calltype_key', 'calltype_text')
                            ->orderBy('calltype_text')
                            ->get();
        foreach($result as $row) {
            $callTypes[$row->cid] = $row->calltype_text;
            $callTypes_keys[$row->cid] = $row->calltype_key;
        }
        //$selectcontent = '<select id="'.$id.'" name="'.$id.'" '.$onchange.' '.$class.'>';
        $selectcontent = '';
        //$selectcontent.= $firstvalue;
        foreach($callTypes as $key => $type) {
            if(is_array($selectedvalue))
            {
                $selected = ( in_array($key, $selectedvalue) || in_array($callTypes_keys[$key], $selectedvalue) )? "selected" : "";
            } else {
                $selected = ($selectedvalue==$key || $selectedvalue==$callTypes_keys[$key])? "selected" : "";
            }
            $selectcontent.= '<option value="'.$key.'" '.$selected.'>'.$type.'</option>';
        }
        //$selectcontent.= '</select>';
        return $selectcontent;
    }

    public function getCalltypesSelectOptions() 
    {
        $call_types = DB::connection('mysql2')
                            ->table('callcenter_calltypes')
                            ->select('cid', 'calltype_text')
                            ->orderBy('calltype_text')
                            ->get();
        $this->modelToSelectOptions($call_types, 'calltype_text', 'cid');
        $select_options = $this->select_options;
        return $select_options;
    }


    

    public function getReviewFileColumns() 
    {
        $column_names = array(  "ShowToCall", "PMA", "SaleDate", "TrueSaleDate", "LastServiceDate", 
                                "FirstName", "LastName", "Email", "CallType", "DealerDepartment", "Year",
                                "Make", "Model", "Vin", "HomePhone", "CellPhone", "BestPhone", "CustID",
                                "TradeYear", "TradeMake", "TradeModel", "TradeVin", "BusinessName", "SalesRep",
                                "ServiceAdv", "UrgentNote", "Address", "City", "ST", "ZipCode", "RO",
                                "LeadDate", "ToyotaCareExpDate", "ServiceAptDate", "ServiceAptTime",
                                "SalesAptDate", "SalesAptTime", "Source", "OriginalLeadSource", "SaleType",
                                "OilType", "VipFlag", "CallerNotes", "ServiceVisitNotes", "ServiceAptNotes",
                                "ScriptNotes", "Offer", "OfferExpires", "LastMiles", "EstimatedMiles",
                                "SaleAmount", "CallNotes", "BadReviewNote", "BadReview", "SalesGrossProfit",
                                "SalesFirstPaymentDate", "SalesLastPaymentDate", "SalesDealNumber", "SalesTerm",
                                "SalesManager", "SalesStockNumber", "FinanceManager", "ServiceTechnician",
                                "SalesSource","ServiceSource", "SalesStatus", "VehicleType", "PartsNumber",
                                "PartsDescription", "AutoAlertUrl", "AutoAlertScore", "VinSolutionsId",
                                "VehicleUrl", "DistanceFromDealer", "IntentToService", "IntentToServiceReason",
                                "SalesPayment", "PreferedContact", "ManfSource", "ManfCustomerType", 
                                "ManfCustStatus", "ToyotaCareStatus", "DateOfBirth", "FullName", 
                                "NameMultiple", "PhoneMultiple", "WorkPhone", "VehicleMultiple", "CompanyName", 
                                "ServiceInterval", "CustomerMixedData", "TradeMiles", "TradeOff", "TradeACV", "TradeCondition", "TradeAppraisal"

                            );
        return $column_names;
    }

    public function getReviewFileColumnsFiltered() 
    {
        $column_names = array(   
                                "livecall_showtocall" => "ShowToCall", 
                                "pma" => "PMA", 
                                "service_date" => "SaleDate", 
                                "sale_date" => "TrueSaleDate", 
                                "last_service_date" => "LastServiceDate", 
                                "fname" => "FirstName", 
                                "lname" => "LastName", 
                                "email" => "Email", 
                                "call_type" => "CallType", 
                                "dealer_department" => "DealerDepartment", 
                                "vehicle_year" => "Year",
                                "vehicle_make" => "Make", 
                                "vehicle_model" => "Model", 
                                "vin" => "Vin", 
                                "phone" => "HomePhone", 
                                "cellphone" => "CellPhone", 
                                "best_phone" => "BestPhone", 
                                "cust_id" => "CustID",
                                "trade_year" => "TradeYear", 
                                "trade_make" => "TradeMake", 
                                "trade_model" => "TradeModel", 
                                "trade_vin" => "TradeVin", 
                                "business_name" => "BusinessName", 
                                "salesrep" => "SalesRep",
                                "service_advisor" => "ServiceAdv", 
                                "urgent_note" => "UrgentNote", 
                                "address1" => "Address", 
                                "city" => "City", 
                                "state" => "ST", 
                                "zip" => "ZipCode", 
                                "RO" => "RO",
                                "lead_date" => "LeadDate", 
                                "toyota_care_exp_date" => "ToyotaCareExpDate", 
                                "service_apt_date" => "ServiceAptDate", 
                                "service_apt_time" => "ServiceAptTime",
                                "sales_apt_date" => "SalesAptDate", 
                                "sales_apt_time" => "SalesAptTime", 
                                "source" => "Source", 
                                "original_lead_source" => "OriginalLeadSource", 
                                "sale_type" => "SaleType",
                                "oil_type" => "OilType", 
                                "vip_flag" => "VipFlag", 
                                "caller_comments" => "CallerNotes", 
                                "service_visit_notes" => "ServiceVisitNotes", 
                                "service_apt_notes" => "ServiceAptNotes",
                                "script_notes" => "ScriptNotes", 
                                "offer" => "Offer", 
                                "offer_expires" => "OfferExpires", 
                                "last_miles" => "LastMiles", 
                                "estimated_miles" => "EstimatedMiles",
                                "sale_amount" => "SaleAmount", 
                                "call_notes" => "CallNotes", 
                                "bad_review_note" => "BadReviewNote", 
                                "bad_review" => "BadReview", 
                                "sales_gross_profit" => "SalesGrossProfit",
                                "sales_first_payment_date" => "SalesFirstPaymentDate", 
                                "sales_last_payment_date" => "SalesLastPaymentDate", 
                                "sales_deal_number" => "SalesDealNumber", 
                                "sales_term" => "SalesTerm",
                                "sales_manager" => "SalesManager", 
                                "sales_stock_number" => "SalesStockNumber", 
                                "finance_manager" => "FinanceManager", 
                                "service_technician" => "ServiceTechnician",
                                "sales_source" => "SalesSource",
                                "service_source" => "ServiceSource", 
                                "sales_status" => "SalesStatus", 
                                "vehicle_type" => "VehicleType", 
                                "parts_number" => "PartsNumber",
                                "part_description" => "PartsDescription", 
                                "auto_alert_url" => "AutoAlertUrl", 
                                "auto_alert_score" => "AutoAlertScore", 
                                "vin_solutions_id" => "VinSolutionsId",
                                "vehicle_url" => "VehicleUrl", 
                                "distance_from_dealer" => "DistanceFromDealer", 
                                "intent_to_service" => "IntentToService", 
                                "intent_to_service_reason" => "IntentToServiceReason",
                                "sales_payment" => "SalesPayment", 
                                "prefered_contact" => "PreferedContact", 
                                "manf_source" => "ManfSource", 
                                "manf_customer_type" => "ManfCustomerType", 
                                "manf_customer_status" => "ManfCustStatus", 
                                "toyota_care_status" => "ToyotaCareStatus", 
                                "date_of_birth" => "DateOfBirth",
                                "stc_flag" => "StcFlag",
                                "trade_miles" => "TradeMiles",
                                "trade_off" => "TradeOff",
                                "trade_acv" => "TradeACV",
                                "trade_condition" => "TradeCondition",
                                "trade_appraisal" => "TradeAppraisal"
                            );
        return $column_names;
    }

    public function phoneNumberFormat($phone)
    {
        $formated_phone = preg_replace("/(\d{3})(\d{3})(\d{4})/", "\\1-\\2-\\3", trim($phone));
        return $formated_phone;
    }

    public function getIconsUrls() {
        $icons = array();
        $result = DB::connection('mysql2')
                            ->table('callcenter_dealertalerts')
                            ->select('id', 'icon')
                            ->get();
        foreach($result as $row) {
            $icons[$row->id] = $row->icon;
        }
        return $icons;
    }

    public function getCostumerStatusTexts() {
        $ctexts = array();
        $cstatus = DB::connection('mysql2')
                            ->table('callcenter_dealertalerts')
                            ->get();

        foreach($cstatus as $status) {
            $id = $status->id;
            $type = $status->type;
            $ctexts[$id] = $type;
        }
        $ctexts[0] = '';
        return $ctexts;
    }

    public function getCallTypesTexts() 
    {
        $calltypesArray = array();
        $calltypes = DB::connection('mysql2')
                            ->table('callcenter_calltypes')
                            ->select('cid', 'calltype_text')
                            ->orderBy('calltype_text')
                            ->get();
        foreach($calltypes as $call) {
            $calltypesArray[$call->cid] = $call->calltype_text;
        }
        return $calltypesArray;
    }

    public function getCalltypesInfo() {
        $carray = array();
        $ctypes = DB::connection('mysql2')
                            ->table('callcenter_calltypes')
                            ->get();

        foreach($ctypes as $types) {
            $id = $types->cid;
            foreach ($types as $key => $value) {
                $carray[0][$key] = '';
                $carray[$id][$key] = $value;
            }
        }
        return $carray;
    }

    public function getGlobalDNC()
    {
        $gdnc = DB::connection('mysql2')
                    ->table("callcenter_dnc")
                    ->where('phone', '!=', '')
                    ->whereNotNull('phone')
                    ->get();
        $gdnc_array = [];
        foreach ($gdnc as $key => $value) {
            $gdnc_array[$value->phone] = true;
        }
        return $gdnc_array;
    }

    public function getDealerDNC($co_short)
    {
        $ddnc = DB::connection('mysql2')
                    ->table("dnc_".$co_short)
                    ->where('phone', '!=', '')
                    ->whereNotNull('phone')
                    ->get();
        $ddnc_array = [];
        foreach ($ddnc as $key => $value) {
            $ddnc_array[$value->phone] = true;
        }
        return $ddnc_array;
    }

    public function getDealerDNE($co_short)
    {
        $dne = DB::connection('mysql2')->table('dne_'.$co_short)
                            ->where('email', '!=', '')
                            ->whereNotNull('email')
                            ->groupBy('email')
                            ->get();
        $dne_array = [];
        foreach ($dne as $key => $value) {
            $dne_array[strtoupper($value->email)] = true;
        }
        return $dne_array;
    }

    public function getGhostedVins($coid)
    {
        $global_removed_vins = DB::connection('mysql2')->table('global_removed_vins')
                                ->select('vin')
                                ->where('coid', $coid)
                                ->where('vin', '!=', '')
                                ->whereNotNull('vin')
                                ->get();

        $removed_vins_array = [];
        foreach ($global_removed_vins as $key => $value) {
            $removed_vins_array[$value->vin] = $value->vin;
        }
        return $removed_vins_array;
    }

    public function getGhostedVinsCustomerStatus($coid)
    {
        $global_removed_vins = DB::connection('mysql2')->table('global_removed_vins')
                                ->select('vin', 'customer_status')
                                ->where('coid', $coid)
                                ->where('vin', '!=', '')
                                ->whereNotNull('vin')
                                ->get();

        $removed_vins_array = [];
        foreach ($global_removed_vins as $key => $value) {
            $removed_vins_array[$value->vin] = $value->customer_status;

        }
        return $removed_vins_array;
    }

    public function getCallTypesCallTimes($coid)
    {
        $call_times = [];
        $call_types = CallType::get();
        if($coid == '')
        {
            foreach ($call_types as $key => $value) {
                $call_times[$value->id] = $value->call_times;
            }
        } else {
            $dealer_call_times = CompanyCallTime::where('coid', $coid)->get();
            foreach ($dealer_call_times as $key => $value) {
                $call_times[$value->call_type_id] = $value->call_times;
            }
            foreach ($call_types as $key => $value) {
                if(!isset($call_times[$value->id]))
                {
                    $call_times[$value->id] = $value->call_times;
                }
            }
        }

        return $call_times;
    }

    public function sanitazeModel($model, $values_array)
    {
        if(!isset($model))
        {
            $model = new \stdClass();
        }
        foreach ($values_array as $key => $value) {
            if(!isset($model->{$value}))
            {
                $model->{$value} = '';
            }
        }
        return $model;
    }

    public function getCustomerUpdateLink($coid, $customer_data, $label)
    {
        $link = '<a href="https://login.fullbdc.com/ccdata22.php?coid='.$coid.'&vin='.$customer_data->vin.'&fname='.$customer_data->fname.'&phone='.$customer_data->phone.'&c=urfe&ctype='.$customer_data->livecall_next_calltype.'">'.$label.'</a>';
        return $link;
    }

    public function getRescheduleEmailLink($coid, $customer_data, $label) 
    {
        $link = '<a href="https://login.fullbdc.com/ccdata22.php?coid='.$coid.'&vin='.$customer_data->vin.'&fname='.$customer_data->fname.'&phone='.$customer_data->phone.'&c=rscbe&ctype='.$customer_data->livecall_next_calltype.'">'.$label.'</a>';
        return $link;
    }

    public function getCompleteCallEmailLink($coid, $customer_data, $label) 
    {
        $link = '<a href="https://login.fullbdc.com/ccdata23.php?coid='.$coid.'&vin='.$customer_data->vin.'&fname='.$customer_data->fname.'&phone='.$customer_data->phone.'&c=ccfe&ctype='.$customer_data->livecall_next_calltype.'">'.$label.'</a>';
        return $link;
    }

    public function unsubFooterEmail($company_data, $email) 
    {
        $content = '<br><br><br>
                    <table class="centeredtable">
                        <tr>
                            <td style="font-size: 9px">
                                To unsubscribe to '.$company_data->co_name.' '.$company_data->bdc_name.' E-Mails please click <a href="https://login.fullbdc.com/unsub.php?c=usp&coid='.$company_data->coid.'&email='.$email.'">here</a>. Be advised that doing so will prevent us from e-mailing you important information in the future to include recall notifications and appointment reminders.
                            </td>
                        </tr>
                    </table>';
        return $content;
    }

    public function setImgToCheckViewed($email, $subject, $date_sent, $coid) 
    {
        $subject = addslashes($subject);
        $img = '<br><br><br><img src="https://login.fullbdc.com/ccdata16.php?c=cve&email='.$email.'&subject='.$subject.'&date_sent='.$date_sent.'&coid='.$coid.'">';
        return $img;
    }

    public function getDealerBadReviews($co_short) 
    {
        $bad_reviews = DB::connection('mysql2')
            ->table('livecall_'.$co_short)
            ->select(
                'fname',
                'lname',
                'phone',
                'vin',
                'vehicle_year',
                'vehicle_make',
                'vehicle_model',
                'caller_comments',
                'bad_review_note',
                'salesrep',
                'service_advisor',
                'RO',
                'service_date'
                )
            ->where('bad_review', 1)
            ->groupBy('phone')
            ->groupBy('vin')
            ->get();
        $bad_review_data = [];
        foreach ($bad_reviews as $key => $value) {
            $bad_review_data[$value->phone][$value->vin]['fname'] = $value->fname;
            $bad_review_data[$value->phone][$value->vin]['lname'] = $value->lname;
            $bad_review_data[$value->phone][$value->vin]['phone'] = $value->phone;
            $bad_review_data[$value->phone][$value->vin]['vin'] = $value->vin;
            $bad_review_data[$value->phone][$value->vin]['vehicle_year'] = $value->vehicle_year;
            $bad_review_data[$value->phone][$value->vin]['vehicle_make'] = $value->vehicle_make;
            $bad_review_data[$value->phone][$value->vin]['vehicle_model'] = $value->vehicle_model;
            $bad_review_data[$value->phone][$value->vin]['caller_comments'] = $value->caller_comments;
            $bad_review_data[$value->phone][$value->vin]['bad_review_note'] = $value->bad_review_note;
            $bad_review_data[$value->phone][$value->vin]['salesrep'] = $value->salesrep;
            $bad_review_data[$value->phone][$value->vin]['service_advisor'] = $value->service_advisor;
            $bad_review_data[$value->phone][$value->vin]['RO'] = $value->RO;
            $bad_review_data[$value->phone][$value->vin]['service_date'] = $value->service_date;
        }

        return $bad_review_data;
    }

    public function getDealerAlertEmails($coid)
    {
        $dealer_alert_emails = DB::connection('mysql2')
                                ->table('callcenter_dealeralert_email')
                                ->select('*')
                                ->where('dealer_id', $coid)
                                ->get();

        $dealer_alert_emails_array = [];
        foreach ($dealer_alert_emails as $key => $value) {
            $dealer_alert_emails_array[$value->alert_id] = $value->email;
        }
        return $dealer_alert_emails_array;
    }

    public function seconds_to_hours($seconds) {
        $init = $seconds;
        $hours = floor($init / 3600);
        $minutes = floor(($init / 60) % 60);
        $seconds = $init % 60;
    
        $parts["hours"]   = $this->fillZeroes($hours);
        $parts["minutes"] = $this->fillZeroes($minutes);
        $parts["seconds"] = $this->fillZeroes($seconds);
        return $parts;
    }

    public function fillZeroes($t){
        if(strlen($t)==1) {
            return "0".$t;
        }
        else {
            return $t;
        }
    }
}
