<?php

namespace App\Http\Controllers;

use App\EmailHistory;
use App\Conversation;
use App\ConversationMessage;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\DB;

class CommunicationController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index( $type )
    {

        $last_messages[]   = [];
        $unseen_messages[] = [];
        $last_datetime[]   = [];

        if(Auth::user()->user_type->name == 'Admin') 
        {
            
            $dealers = DB::connection('mysql2')->table('callcenter_companies')
                                ->select('co_name', 'co_short', 'coid')
                                ->where('is_active', 1)
                                ->orderBy('co_name')
                                ->get();
            if( $type == 'sms' ) {
                $communications = Conversation::orderBy('created_at', 'desc')->paginate(25);

                foreach ($communications as $key => $value) {
                    $conversation = ConversationMessage::where('conversation_id', $value->id)->orderBy('created_at', 'desc')->first();
                    if($conversation)
                    {
                        $last_messages[$value->id] = $conversation->message;
                        $unseen_messages[$value->id] = $conversation->seen;
                        $last_datetime[$value->id] = $conversation->created_at;
                    }
                }
            } else {
                $communications = EmailHistory::orderBy('created_at', 'desc')->paginate(25);
            }
        } else if(Auth::user()->user_type->name == 'Dealer' || Auth::user()->user_type->name == 'Full BDC Caller') {
            $user_companies_array = array();
            foreach(Auth::user()->dealers as $company)
            {
              $user_companies_array[] = $company->dealer_id;
            }

            $dealers = DB::connection('mysql2')->table('callcenter_companies')
                                ->select('co_name', 'co_short', 'coid')
                                ->whereRaw("FIND_IN_SET(coid, '".implode(',', $user_companies_array)."')")
                                ->get();

            $communications = EmailHistory::whereRaw("FIND_IN_SET(coid, '".implode(',', $user_companies_array)."')")
                                            ->orderBy('created_at', 'desc')
                                            ->paginate(25);
        }

        $commonFunctions = new CommonFunctionsController();
        $commonFunctions->modelToSelectOptions($dealers,'co_name', 'coid');
        $dealers_options = $commonFunctions->select_options;

        $all_dealers = DB::connection('mysql2')->table('callcenter_companies')
                                ->select('coid', 'co_name', 'co_short')
                                ->orderBy('co_name')
                                ->get();
        $dealer_names = array();
        foreach ($all_dealers as $key => $value) {
        	$dealer_names[$value->coid] = $value->co_name;
        }

        $type = ( ! empty( $type ) )? $type : 'email';

        return view('pages.communication', [
        										'dealers_options' => $dealers_options,
        										'dealer_names' => $dealer_names,
        										'communication_type' => $type,
        										'communications' => $communications,
                                                'last_messages' => $last_messages,
                                                'unseen_messages' => $unseen_messages,
                                                'last_datetime' => $last_datetime
    										]);
    }


    /**
     * Display a list of the resource with pagination.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */

    public function pagination(Request $request)

    {
        $current_user_id = 0;
        $no_links = false;
        $last_messages = array();
        $unseen_messages = array();
        $last_datetime = array();

        $raw_search = $this->getRawSearch($request);

        //dd($raw_search);

        $all_dealers = DB::connection('mysql2')->table('callcenter_companies')
                                ->select('coid', 'co_name', 'co_short')
                                ->orderBy('co_name')
                                ->get();
        $dealer_names = array();
        foreach ($all_dealers as $key => $value) {
        	$dealer_names[$value->coid] = $value->co_name;
        }


        if($request->communication_type_search == 'email')
        {
        	if($raw_search != "")
        	{
        		$communications = EmailHistory::whereRaw($raw_search)->orderBy('created_at', 'desc')->paginate(25);
        	} else {
        		$communications = EmailHistory::orderBy('created_at', 'desc')->paginate(25);
        	}
        } elseif($request->communication_type_search == 'sms') {
            if($request->communication_status_search != 'all')
            {
                if($raw_search != "")
                {
                    $communications = Conversation::join('conversation_messages', 'conversations.id', '=', 'conversation_messages.conversation_id')
                                                    ->select('conversations.*')
                                                    ->whereRaw($raw_search)
                                                    ->where('seen', $request->communication_status_search)
                                                    ->orderBy('conversations.created_at', 'desc')
                                                    ->groupBy('conversation_id')
                                                    ->paginate(25);
                } else {
                    $communications = Conversation::join('conversation_messages', 'conversations.id', '=', 'conversation_messages.conversation_id')
                                                    ->select('conversations.*')
                                                    ->where('seen', $request->communication_status_search)
                                                    ->orderBy('conversations.created_at', 'desc')
                                                    ->groupBy('conversation_id')
                                                    ->paginate(25);
                }
            } else {
                if($raw_search != "")
                {
                    $communications = Conversation::whereRaw($raw_search)->orderBy('created_at', 'desc')->paginate(25);
                } else {
                    $communications = Conversation::orderBy('created_at', 'desc')->paginate(25);
                }
            }
        	
        	foreach ($communications as $key => $value) {
        		$conversation = ConversationMessage::where('conversation_id', $value->id)->orderBy('created_at', 'desc')->first();
        		if($conversation)
        		{
        			$last_messages[$value->id] = $conversation->message;
                    $unseen_messages[$value->id] = $conversation->seen;
                    $last_datetime[$value->id] = $conversation->created_at;
        		}
        	}
        	
        }

        return view('lists.communication_list', [
        										'dealer_names' => $dealer_names,
        										'communication_type' => $request->communication_type_search,
        										'communications' => $communications,
        										'last_messages' => $last_messages,
                                                'unseen_messages' => $unseen_messages,
                                                'last_datetime' => $last_datetime
    										]);
    }



    public function getRawSearch($my_request) {
        $raw_search_array = array();
        if($my_request->coid_search != 'all' && $my_request->coid_search != '')
        {
            $raw_search_array[] = " coid = '".$my_request->coid_search."' ";
        }

        if($my_request->start_date_search != '' && $my_request->end_date_search != '')
        {
            $start_date_search = date('Y-m-d 00:00:00', strtotime($my_request->start_date_search));
            $end_date_search = date('Y-m-d 23:59:59', strtotime($my_request->end_date_search));
            $raw_search_array[] = " (created_at >= '".$start_date_search."' AND created_at <= '".$end_date_search."')";
        }

        if(trim($my_request->value_search) != '')
        {
        	if($my_request->communication_type_search == 'email')
        	{
	            $raw_search_array[] = " (
	            							email like '%".$my_request->value_search."%' OR 
	                                        subject like '%".$my_request->value_search."%' 
	                                    )";
        	} elseif($my_request->communication_type_search == 'sms') {
                $user_identifier = preg_replace("/[^0-9]/i", "", $my_request->value_search);
        		$raw_search_array[] = " (
	            							user_identifier like '%".$user_identifier."%'
	                                    )";
        	}

        }

        $raw_search = implode(' AND ', $raw_search_array);

        return $raw_search;
    }
}
