<?php

namespace App\Http\Controllers;

use App\Company;
use App\AssignableCalltype;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;

class CompanyController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {
        //
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        //
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        $messages = [
            'co_name.required' => 'Your company name is required.',
            'main_contact_name.required' => 'required|max:191',
            'main_contact_phone.regex' => 'We need a valid main contact phone number.',
            'main_contact_email.email' => "We need a valid main contact email address",
            'email.email' => 'We need a valid company email address',
            'phone.regex' => 'We need a valid company phone number.',
        ];

        $validatedData = $request->validate([
            'co_name' => 'required|max:191',
            'main_contact_name' => 'required|max:191',
            'main_contact_phone' => 'required|regex:/^[0-9\-\(\)\/\+\s]*$/|max:50',
            'main_contact_email' => "required|email|max:191",
            'address' => 'required|max:191',
            'city' => 'required|max:191',
            'state' => 'required|max:191',
            'zip' => 'required|max:191',
            'email' => 'required|email|max:191',
            'phone' => 'required|regex:/^[0-9\-\(\)\/\+\s]*$/|max:50',
        ], $messages);

        $company = new Company;
        $company->co_name = $request->co_name;
        $company->main_contact_name = $request->main_contact_name;
        $company->main_contact_phone = $request->main_contact_phone;
        $company->main_contact_email = $request->main_contact_email;
        $company->sales_mgr_name = $request->sales_mgr_name;
        $company->service_mgr_name = $request->service_mgr_name;
        $company->address = $request->address;
        $company->city = $request->city;
        $company->state = $request->state;
        $company->zip = $request->zip;
        $company->email = $request->email;
        $company->phone = $request->phone;
        $company->sales_phone = $request->sales_phone;
        $company->sales_email = $request->sales_email;
        $company->service_phone = $request->service_phone;
        $company->service_email = $request->service_email;
        $company->parts_phone = $request->parts_phone;
        $company->website = $request->website;
        $company->facebook_link = $request->facebook_link;
        $company->save();

        return response()->json([
            'inserted_id' => $company->id,
            'message' => 'Company created.'
        ]);
    }

    /**
     * Display the specified resource.
     *
     * @param  \App\Company  $company
     * @return \Illuminate\Http\Response
     */
    public function show(Company $company)
    {
        //
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  \App\Company  $company
     * @return \Illuminate\Http\Response
     */
    public function edit(Request $request, Company $company)
    {
        $company = DB::connection('mysql2')->table('callcenter_companies')
                                ->select('*')
                                ->where('coid', $request->company_id)
                                ->first();

        $assignable_calltypes = AssignableCalltype::where('coid', $request->company_id)->get();

        $assignable_calltypes_array = array();
        foreach ($assignable_calltypes as $key => $value) {
            $assignable_calltypes_array[] = $value->call_type_id;
        }

        $call_types = DB::connection('mysql2')->table('callcenter_calltypes')
                                ->select('*')
                                ->orderBy('calltype_text')
                                ->get();

        $call_scripts = DB::connection('mysql2')->table('callcenter_companies_callscripts')
            ->where('coid', $request->company_id)
            ->get();
        
        $dealer_scripts = [];
        foreach( $call_scripts as $script ) {
            $dealer_scripts[ $script->csid ] = utf8_encode( $script->script_content );
        }

        foreach ( $call_types as $call_type ) {
            $dealer_scripts[ $call_type->cid ] = $dealer_scripts[ $call_type->cid ] ?? $call_type->script_content;
        }

        $dealer_costs = DB::connection('mysql2')
            ->table('dealer_costs')
            ->where('did', $request->company_id)
            ->get();
        
        $dealer_costs_values = [];
        foreach ( $dealer_costs as $costs ) {
            $dealer_costs_values[ $costs->cid ]['record_cost'] = $costs->record_cost;
            $dealer_costs_values[ $costs->cid ]['call_cost']   = $costs->call_cost;
            $dealer_costs_values[ $costs->cid ]['flat_cost']   = $costs->flat_cost;
        }

        $caller_ids = DB::connection('mysql2')
            ->table('callcenter_calltypes_callerid')
            ->where('coid', $request->company_id)
            ->get();
        
        $caller_ids_values = [];
        foreach ( $caller_ids as $caller_id ) {
            $caller_ids_values[ $caller_id->ctid ] = $caller_id->caller_id;
        }

        $call_times = DB::connection('mysql2')
            ->table('callcenter_dealer_calltimes')
            ->where('coid', $request->company_id)
            ->get();

        $call_times_values = [];
        foreach ( $call_times as $call_time ) {
            $call_times_values[ $call_time->callid ] = $call_time->livecall_times;
        }

        foreach ( $call_types as $call_type ) {
            $caller_ids_values[ $call_type->cid ] = $caller_ids_values[ $call_type->cid ] ?? '';
            $call_times_values[ $call_type->cid ] = $call_times_values[ $call_type->cid ] ?? $call_type->livecall_times;
            $dealer_costs_values[ $call_type->cid ]['record_cost'] = $dealer_costs_values[ $call_type->cid ]['record_cost'] ?? $call_type->record_cost;
            $dealer_costs_values[ $call_type->cid ]['call_cost']   = $dealer_costs_values[ $call_type->cid ]['call_cost'] ?? $call_type->call_cost;
            $dealer_costs_values[ $call_type->cid ]['flat_cost']   = $dealer_costs_values[ $call_type->cid ]['flat_cost'] ?? $call_type->flat_cost;
        }

        // INBOUND.
        
        $ib_call_types = DB::connection('mysql2')
            ->table('callcenter_calltypes_inbound')
            ->orderBy('calltype_text')
            ->get();

        $ib_call_scripts = DB::connection('mysql2')
            ->table('callcenter_calltypes_inbound_scripts')
            ->where('coid', $request->company_id)
            ->get();
        
        if( ! $ib_call_scripts ) {
            $ib_call_scripts = DB::connection('mysql2')
            ->table('callcenter_calltypes_inbound_scripts')
            ->where('coid', 0)
            ->get();
        }

        $ib_scripts_array        = [];
        $ib_call_is_active_array = [];
        foreach ( $ib_call_scripts as $script ) {
            $cid                             = $script->cid;
            $ib_scripts_array[ $cid ]        = $script->script_content;
            $ib_call_is_active_array[ $cid ] = $script->is_active;
        }

        $ib_costs = DB::connection('mysql2')
            ->table('callcenter_inbound_costs')
            ->where('coid', $request->company_id)
            ->get();
        
        if( ! $ib_costs ) {
            $ib_call_scripts = DB::connection('mysql2')
            ->table('callcenter_inbound_costs')
            ->where('coid', 0)
            ->get();
        }

        $ib_minute_cost_array  = [];
        $ib_monthly_cost_array = [];
        $ib_max_time_array     = [];
        foreach ( $ib_costs as $cost ) {
            $cid                           = $cost->cid;
            $ib_minute_cost_array[ $cid ]  = $cost->minute_cost;
            $ib_monthly_cost_array[ $cid ] = $cost->monthly_cost;
            $ib_max_time_array[ $cid ]     = $cost->max_time;
        }

        foreach ( $ib_call_types as $ib_calltype ) {
            $ib_scripts_array[ $ib_calltype->cid ]        = $ib_scripts_array[ $ib_calltype->cid ] ?? 'No content set';
            $ib_call_is_active_array[ $ib_calltype->cid ] = $ib_call_is_active_array[ $ib_calltype->cid ] ?? 0;
            $ib_minute_cost_array[ $ib_calltype->cid ]    = $ib_minute_cost_array[ $ib_calltype->cid ] ?? '';
            $ib_monthly_cost_array[ $ib_calltype->cid ]   = $ib_monthly_cost_array[ $ib_calltype->cid ] ?? '';
            $ib_max_time_array[ $ib_calltype->cid ]       = $ib_max_time_array[ $ib_calltype->cid ] ?? '';
        }


        $dealer_alerts_emails = DB::connection('mysql2')
            ->table('callcenter_dealeralert_email')
            ->where('dealer_id', $request->company_id)
            ->get();

        $alerts_emails_list = [];
        foreach ( $dealer_alerts_emails as $dealer_alerts_email ) {
            $alert_id                         = $dealer_alerts_email->alert_id;
            $alerts_emails_list[ $alert_id ]  = $dealer_alerts_email->email;
        }

        $dealer_departments = DB::connection('mysql2')
            ->table('callcenter_companies_departments')
            ->get();

        foreach ( $dealer_departments as $dealer_department ) {
            if ( empty( $alerts_emails_list[ $dealer_department->id ] ) ) {
                $alerts_emails_list[ $dealer_department->id ] = '';
            }
        }

        return view('dialogs.add_company_dialog', [
                                                        'company' => $company, 
                                                        'assignable_calltypes_array' => $assignable_calltypes_array,
                                                        'call_types' => $call_types,
                                                        'dealer_scripts' => $dealer_scripts,
                                                        'dealer_costs_values' => $dealer_costs_values,
                                                        'caller_ids_values' => $caller_ids_values,
                                                        'call_times_values' => $call_times_values,
                                                        'ib_call_types' => $ib_call_types,
                                                        'ib_scripts_array' => $ib_scripts_array,
                                                        'ib_call_is_active_array' => $ib_call_is_active_array,
                                                        'ib_minute_cost_array' => $ib_minute_cost_array,
                                                        'ib_monthly_cost_array' => $ib_monthly_cost_array,
                                                        'ib_max_time_array' => $ib_max_time_array,
                                                        'alerts_emails_list' => $alerts_emails_list,
                                                        'dealer_departments' => $dealer_departments,
                                                    ]);
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  \App\Company  $company
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, Company $company)
    {
        AssignableCalltype::where('coid', $request->company_id)->delete();
        if (isset($request->assignable_calltypes))
        {
            foreach ($request->assignable_calltypes as $key => $value) {
                $assignable_calltype = new AssignableCalltype;
                $assignable_calltype->coid = $request->company_id;
                $assignable_calltype->call_type_id = $value;
                $assignable_calltype->save();
            }
        }

        $index = 0;
        foreach ($request->ib_call_types_ids as $ib_calltype_id) {
            $ib_callscript = DB::connection('mysql2')
                    ->table('callcenter_calltypes_inbound_scripts')
                    ->where('coid', $request->company_id)
                    ->where('cid', $ib_calltype_id)
                    ->first();

            if ( $ib_callscript ) {
                DB::connection('mysql2')
                    ->table('callcenter_calltypes_inbound_scripts')
                    ->where('coid', $request->company_id)
                    ->where('cid', $ib_calltype_id)
                    ->update( 
                        [
                            'is_active' => ! empty($request->{'ib_call_type_active_'.$ib_calltype_id}) ? 1 : 0,
                        ] 
                    );
            } else {
                DB::connection('mysql2')
                    ->table('callcenter_calltypes_inbound_scripts')
                    ->insertGetId([
                        'coid' => $request->company_id, 
                        'cid'  => $ib_calltype_id,
                        'script_content'  => '',
                        'is_active' => ! empty($request->{'ib_call_type_active_'.$ib_calltype_id}) ? 1 : 0,
                    ]);
            }
            $index++;
        }
        
        $dealer_alerts_emails = DB::connection('mysql2')
            ->table('callcenter_dealeralert_email')
            ->where('dealer_id', $request->company_id)
            ->get();

        $alerts_emails_list = [];
        foreach ( $dealer_alerts_emails as $dealer_alerts_email ) {
            $alert_id                         = $dealer_alerts_email->alert_id;
            $alerts_emails_list[ $alert_id ]  = $dealer_alerts_email->email;
        }

        foreach ( $request->dealer_alerts_list as $key => $alert_list ) {
            $new_alert_list = ! empty( $alert_list ) ? $alert_list : '';
            if ( ! empty( $alerts_emails_list[ $request->dealer_alerts_list_ids[ $key ] ] ) ) {
                DB::connection('mysql2')
                    ->table('callcenter_dealeralert_email')
                    ->where('dealer_id', $request->company_id)
                    ->where('alert_id', $request->dealer_alerts_list_ids[ $key ])
                    ->update( 
                        [
                            'email' => $new_alert_list,
                        ] 
                    );
            } else {
                DB::connection('mysql2')
                    ->table('callcenter_dealeralert_email')
                    ->insertGetId([
                        'dealer_id' => $request->company_id, 
                        'alert_id'  => $request->dealer_alerts_list_ids[ $key ], 
                        'email'     => $new_alert_list,
                    ]);
            }
        }

        DB::connection('mysql2')->table('callcenter_companies')
                                ->where('coid', $request->company_id)
                                ->update([
                                    'service_mgr_name' => $request->service_mgr_name,
                                    'service_mgr_title' => $request->service_mgr_title,
                                    'service_phone' => $request->service_phone,
                                    'parts_phone' => $request->parts_phone,
                                    'service_email' => $request->service_email,
                                    'service_advisors' => $request->service_advisors,
                                    'service_apt_notify' => $request->service_apt_notify,
                                    'avg_ro_amount' => $request->avg_ro_amount,
                                    'current_service_csi' => $request->current_service_csi,
                                    'sales_mgr_name' => $request->sales_mgr_name,
                                    'sales_mgr_title' => $request->sales_mgr_title,
                                    'sales_phone' => $request->sales_phone,
                                    'sales_email' => $request->sales_email,
                                    'sales_reps' => $request->sales_reps,
                                    'sales_apt_notify' => $request->sales_apt_notify,
                                    'current_sales_csi' => $request->current_sales_csi,
                                ]);

        return response()->json([
            'inserted_id' => $request->company_id,
            'message' => 'Dealer updated.'
        ]);
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  \App\Company  $company
     * @return \Illuminate\Http\Response
     */
    public function destroy(Company $company)
    {
        //
    }


    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  \App\Company  $company
     * @return \Illuminate\Http\Response
     */
    public function activeAutomatedReminder(Request $request, Company $company)
    {
        DB::connection('mysql2')->table('callcenter_companies')
                                ->select('*')
                                ->where('coid', $request->coid)
                                ->update([
                                    'automated_reminder' => $request->active
                                ]);

        return response()->json([
            'updated_id' => $request->coid,
            'message' => 'Company updated.'
        ]);
    }
}
