<?php

namespace App\Http\Controllers;

use App\Conversation;
use App\User;
use App\UserDealer;
use App\TwilioPhone;
use Illuminate\Support\Facades\Auth;
use App\ConversationMessage;
use App\SmsOptin;
use App\Events\UpcomingChatMessage;
use App\Events\UpcomingTeamChatMessage;
use App\Events\UpcomingAlert;
use Twilio\Rest\Client;
use Illuminate\Support\Facades\DB;
use Illuminate\Http\Request;
use DateTime;
use DateTimeZone;

class ConversationController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {
        //
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        //
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {

        if($request->team_conversation == 0)
        {
            $conversation = Conversation::where('coid', $request->chat_coid)
                                    ->where('user_identifier', $request->to_user_id)
                                    ->first();

            if($conversation)
            {
                $conversation_id = $conversation->id;
            } else {
                $conversation = new Conversation;
                $conversation->coid = $request->chat_coid;
                $conversation->user_identifier = $request->to_user_id;
                $conversation->save();
                $conversation_id = $conversation->id;
            }


            $phone_to_send = $request->to_user_id;
            if(strpos($phone_to_send,"+1") === false) {
                $phone_to_send = "+1".$phone_to_send;
            }
            $phone_to_send = preg_replace("/[^0-9]/i", "", $phone_to_send);
            if(strpos($phone_to_send,"+") === false) {
                $phone_to_send = "+".$phone_to_send;
            }

            $recipients = $phone_to_send;
            $created_at_label = '';
            $twilio_sms_sent = false;

            $company_twilio_phone = TwilioPhone::where('coid', $request->chat_coid)->first();
            if($company_twilio_phone)
            {
                $twilio_phone = $company_twilio_phone->phone;
            } else {
                $twilio_phone = null;
            }

            if($this->sendSmsMessage($request->chat_message, $recipients, $twilio_phone)){
                
                $new_conversation = new ConversationMessage;
                $new_conversation->conversation_id = $conversation_id;
                $new_conversation->from = $request->from_user_id;
                $new_conversation->to = $request->to_user_id;
                $new_conversation->message = $request->chat_message;
                $new_conversation->seen = 1;
                $new_conversation->date_seen = date('Y-m-d H:i:s');
                $new_conversation->save();

                $date_start = new DateTime();
                $date_start->setTimezone(new DateTimeZone('America/New_York'));
                $new_conversation_fullbdc_id = DB::connection('mysql2')->table('callcenter_sms_conversations')->insertGetId([
                            'sent_from' => $twilio_phone, 
                            'sent_to' => $request->to_user_id, 
                            'seen' => 1, 
                            'date_sent' => $date_start->format('Y-m-d H:i:s'),
                            'content' => $request->chat_message,
                        ]);

                if($request->is_optin_msg == 1)
                {
                    $optin_sms_sent_fbv = SmsOptin::where('sent_from', 'like', '%'.$twilio_phone.'%')
                                            ->where('sent_to', 'like', '%'.$request->to_user_id.'%')
                                            ->first();

                    if(!$optin_sms_sent_fbv)
                    {
                        $optin_sms_sent = new SmsOptin;
                        $optin_sms_sent->sent_from = $twilio_phone;
                        $optin_sms_sent->sent_to = $request->to_user_id;
                        $optin_sms_sent->save();
                    } 

                    $optin_sms_sentfb = DB::connection('mysql2')->table('optin_sms_sent')
                                ->select('*')
                                ->where('sent_from', 'like', '%'.$twilio_phone.'%')
                                ->where('sent_to', 'like', '%'.$request->to_user_id.'%')
                                ->first();

                    if(!$optin_sms_sentfb)
                    {
                        $optin_sms_sent_id = DB::connection('mysql2')->table('optin_sms_sent')->insertGetId([
                            'sent_from' => $twilio_phone, 
                            'sent_to' => $request->to_user_id, 
                            'sent_date' => date('Y-m-d H:i:s'),
                            'approved_date' => '000-00-00 00:00:00',
                            'canceled_date' => '000-00-00 00:00:00',
                        ]);
                    }
                }

                $twilio_sms_sent = true;
            }
        
        } else {
            $conversation_id = 0;

            $new_conversation = new ConversationMessage;
            $new_conversation->from = $request->from_user_id;
            $new_conversation->to = $request->to_user_id;
            $new_conversation->message = $request->chat_message;
            $new_conversation->save();

            $twilio_sms_sent = false;
        }

        
        

        /*if(Auth::user()->picture_id != "") {
            $profile_pic = Picture::find(Auth::user()->picture_id);
            $profile_picture = $profile_pic->filename;
        } else {
            $profile_picture = "default.jpg";
        }*/

        $profile_picture = "default.jpg";
        
        if(isset($request->no_events))
        {
            if($request->team_conversation == 0)
            {
                if($twilio_sms_sent==true)
                {
                    broadcast(new UpcomingChatMessage(Auth::user(), $new_conversation->to, $profile_picture, $new_conversation))->toOthers();
                    broadcast(new UpcomingAlert(
                                                    Auth::user(), 
                                                    $new_conversation->to, 
                                                    $profile_picture, 
                                                    $new_conversation, 
                                                    $request->team_conversation, 
                                                    $request->chat_coid,
                                                    $request->chat_first_name,
                                                    $request->chat_last_name,
                                                )
                            )->toOthers();
                }
            } else {
                broadcast(new UpcomingTeamChatMessage(Auth::user(), $new_conversation->to, $profile_picture, $new_conversation))->toOthers();
                broadcast(new UpcomingAlert(
                                                    Auth::user(), 
                                                    $new_conversation->to, 
                                                    $profile_picture, 
                                                    $new_conversation, 
                                                    $request->team_conversation, 
                                                    $request->chat_coid,
                                                    $request->chat_first_name,
                                                    $request->chat_last_name,
                                                    '.'.$new_conversation->to,
                                                )
                            )->toOthers();
            }
        }
        
        //event(new UpcomingContactRequest(Auth::user(), $new_conversation->to_user_id, 'chat_message'));

        $created_at_label = date('g:i a', strtotime($new_conversation->created_at) + session('timezone_difference', 0));

        return response()->json([
            'conversation' => $new_conversation,
            'created_at_label' => $created_at_label
        ]);
    }


    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function storeFromTwilio(Request $request)
    {
        $from = (strpos($_REQUEST['From'], '+1') === false)? '+1'.$_REQUEST['From'] : $_REQUEST['From'];
        $to = (strpos($_REQUEST['To'], '+1') === false)? '+1'.$_REQUEST['To'] : $_REQUEST['To'];
        $body = $_REQUEST['Body'];

        $company = TwilioPhone::select('*')->where('phone', 'like', '%'.$to.'%')->first();

        if( ! $company ) {
            $company = DB::connection('mysql2')->table('callcenter_companies')
                                ->select('co_short', 'coid')
                                ->where('twilio_sms', 'like', '%'.$to.'%')
                                ->first();
            if( $company ) {
                $twilio_phone_company = TwilioPhone::select('*')->where('coid', $company->coid)->first();
                $to = $twilio_phone_company->phone;
            }
        }
        
        $conversation = Conversation::where('coid', $company->coid)
                                    ->where('user_identifier', $from)
                                    ->first();

        if($conversation)
        {
            $conversation_id = $conversation->id;
        } else {
            $conversation = new Conversation;
            $conversation->coid = $company->coid;
            $conversation->user_identifier = $from;
            $conversation->save();
            $conversation_id = $conversation->id;
        }

        $new_conversation = new ConversationMessage;
        $new_conversation->conversation_id = $conversation_id;
        $new_conversation->from = $from;
        $new_conversation->to = $to;
        $new_conversation->message = $body;
        $new_conversation->save();

        $date_start = new DateTime();
        $date_start->setTimezone(new DateTimeZone('America/New_York'));
        $new_conversation_fullbdc_id = DB::connection('mysql2')->table('callcenter_sms_conversations')->insertGetId([
                            'sent_from' => $from, 
                            'sent_to' => $to, 
                            'date_sent' => $date_start->format('Y-m-d H:i:s'),
                            'content' => $body,
                        ]);

        $profile_picture = "default.jpg";

        $dealer = DB::connection('mysql2')->table('callcenter_companies')
                                ->select('co_short', 'coid')
                                ->where('coid', $company->coid)
                                ->first();

        $db_table = 'livecall_'.$dealer->co_short;
        $search_phone = str_replace("+1", "", $new_conversation->from);
        $customer = DB::connection('mysql2')->table($db_table)
                                ->select('*')
                                ->where('phone', $search_phone)
                                ->orWhere('cellphone', $search_phone)
                                ->orWhere('best_phone', $search_phone)
                                ->first();
        if(!$customer)
        {
            $customer = new \stdClass();
            $customer->fname = '';
            $customer->lname = '';
        }
        $from_user = new \stdClass();
        $from_user->id = $from;
        $from_user->first_name = $customer->fname;
        $from_user->last_name = $customer->lname;

        $optin_sms_sent_fbv = SmsOptin::where('sent_from', 'like', '%'.$to.'%')
                                    ->where('sent_to', 'like', '%'.$from.'%')
                                    ->first();

        $optin_sms_sentfb = DB::connection('mysql2')->table('optin_sms_sent')
                        ->select('*')
                        ->where('sent_from', 'like', '%'.$to.'%')
                        ->where('sent_to', 'like', '%'.$from.'%')
                        ->first();

        if(strtolower($body) != "stop" && strtolower($body) != "cancel")
        {
            if(!$optin_sms_sent_fbv)
            {
                $optin_sms_sent = new SmsOptin;
                $optin_sms_sent->sent_from = $to;
                $optin_sms_sent->sent_to = $from;
                $optin_sms_sent->approved = 1;
                $optin_sms_sent->approved_date = date('Y-m-d H:i:s');
                $optin_sms_sent->save();
            } else {
                if($optin_sms_sent_fbv->approved != 1)
                {
                    SmsOptin::where('id', $optin_sms_sent_fbv->id)
                                    ->update([
                                        'approved' => 1,
                                        'approved_date' => date('Y-m-d H:i:s'),
                                        'canceled' => 0,
                                        'canceled_date' => null,
                                    ]);
                    
                    // Automatically set to 'seen' if the response is a opt in 'yes' response.
                    if( strtolower( trim( $body ) ) == "yes" ) {
                        ConversationMessage::where( 'id', $new_conversation->id )
                            ->update([
                                'seen' => 1,
                                'date_seen' => $date_start->format('Y-m-d H:i:s'),
                            ]);
                    }
                }
            } 

            if(!$optin_sms_sentfb)
            {
                $optin_sms_sent_id = DB::connection('mysql2')->table('optin_sms_sent')->insertGetId([
                            'sent_from' => $to, 
                            'sent_to' => $from, 
                            'sent_date' => date('Y-m-d H:i:s'),
                            'approved' => 1,
                            'approved_date' => date('Y-m-d H:i:s'),
                            'canceled_date' => '000-00-00 00:00:00',
                        ]);
            } else {
                if($optin_sms_sentfb->approved != 1)
                {
                    DB::connection('mysql2')->table('optin_sms_sent')->where('uid', $optin_sms_sentfb->uid)
                                        ->update([
                                            'approved' => 1,
                                            'approved_date' => date('Y-m-d H:i:s'),
                                            'canceled' => 0,
                                            'canceled_date' => '000-00-00 00:00:00',
                                        ]);
                }
            }
        } else if(strtolower($body) == "stop" || strtolower($body) == "cancel") {
            if(!$optin_sms_sent_fbv)
            {
                $optin_sms_sent = new SmsOptin;
                $optin_sms_sent->sent_from = $to;
                $optin_sms_sent->sent_to = $from;
                $optin_sms_sent->canceled = 1;
                $optin_sms_sent->canceled_date = date('Y-m-d H:i:s');
                $optin_sms_sent->save();
            } else {
                if($optin_sms_sent_fbv->canceled != 1)
                {
                    SmsOptin::where('id', $optin_sms_sent_fbv->id)
                                    ->update([
                                        'approved' => 0,
                                        'approved_date' => null,
                                        'canceled' => 1,
                                        'canceled_date' => date('Y-m-d H:i:s'),
                                    ]);
                }
            } 

            if(!$optin_sms_sentfb)
            {
                $optin_sms_sent_id = DB::connection('mysql2')->table('optin_sms_sent')->insertGetId([
                            'sent_from' => $to, 
                            'sent_to' => $from, 
                            'sent_date' => date('Y-m-d H:i:s'),
                            'canceled' => 1,
                            'approved_date' => '000-00-00 00:00:00',
                            'canceled_date' => date('Y-m-d H:i:s'),
                        ]);
            } else {
                if($optin_sms_sentfb->canceled != 1)
                {
                    DB::connection('mysql2')->table('optin_sms_sent')->where('uid', $optin_sms_sentfb->uid)
                                        ->update([
                                            'approved' => 0,
                                            'approved_date' => '000-00-00 00:00:00',
                                            'canceled' => 1,
                                            'canceled_date' => date('Y-m-d H:i:s'),
                                        ]);
                }
            }
        }

        broadcast(new UpcomingChatMessage($from_user, $new_conversation->to, $profile_picture, $new_conversation));
        broadcast(new UpcomingAlert(
                                        $from_user, 
                                        $new_conversation->from, 
                                        $profile_picture, 
                                        $new_conversation, 
                                        $team_conversation = 0, 
                                        $company->coid,
                                        $from_user->first_name,
                                        $from_user->last_name
                                    )
                );

        $created_at_label = date('g:i a', strtotime($new_conversation->created_at));

        return response()->json([
            'conversation' => $new_conversation,
            'created_at_label' => $created_at_label
        ]);
    }


    /**
     * Sends sms to user using Twilio's programmable sms client
     * @param String $message Body of sms
     * @param Number $recipients Number of recipient
     */
    private function sendSmsMessage($message, $recipients, $sent_from)
    {
        $account_sid = getenv("TWILIO_SID");
        $auth_token = getenv("TWILIO_AUTH_TOKEN");
        $twilio_number = ($sent_from == null)? getenv("TWILIO_NUMBER") : $sent_from;
        $client = new Client($account_sid, $auth_token);
        $client->messages->create($recipients, ['from' => $twilio_number, 'body' => $message]);
        return true;
    }

    /**
     * Display the specified resource.
     *
     * @param  \App\Conversation  $conversation
     * @return \Illuminate\Http\Response
     */
    public function show(Conversation $conversation)
    {
        //
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  \App\Conversation  $conversation
     * @return \Illuminate\Http\Response
     */
    public function edit(Conversation $conversation)
    {
        //
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  \App\Conversation  $conversation
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, Conversation $conversation)
    {
        $message_coid = $request->coid;
        $message_id = $request->message_id;

        ConversationMessage::where('id', $message_id)->update([
                                                                'seen' => 1
                                                            ]);
        return response()->json([
            'updated_id' => $message_id,
        ]);
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  \App\Conversation  $conversation
     * @return \Illuminate\Http\Response
     */
    public function destroy(Conversation $conversation)
    {
        //
    }



    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function conversationWindow(Request $request)
    {
        //$to_user = User::find($request->to_id);
        /*if($to_user->picture_id != "") {
            $profile_pic = Picture::find($to_user->picture_id);
            $profile_picture = $profile_pic->filename;
        } else {
            $profile_picture = "default.jpg";
        }*/
        $profile_picture = "default.jpg";
        $already_optin = false;
        $optin_already_sent = false;
        $needs_optin = false;
        $company = new \stdClass();
        $company->co_name = 'your vehicle dealer';
        
        // Conversation::where('from_user_id', $request->to_id)
        //   ->where('to_user_id', Auth::id())
        //   ->update(['seen' => 1, 'date_seen' => date('Y-m-d H:i:s')]);

        if(isset($request->team_conversation) && $request->team_conversation == 1)
        {
            $to_user = User::find($request->to_id);
            $conversations_count = ConversationMessage::where(function($query) use($request){
                                            $query->where('from', '=', Auth::id())
                                            ->where('to', '=', $request->to_id);
                                        })->orWhere(function($query) use($request){
                                            $query->where('from', '=', $request->to_id)
                                            ->where('to', '=', Auth::id());
                                        })
                                        ->count();
            $conversations = ConversationMessage::where(function($query) use($request){
                                                $query->where('from', '=', Auth::id())
                                                ->where('to', '=', $request->to_id);
                                            })->orWhere(function($query) use($request){
                                                $query->where('from', '=', $request->to_id)
                                                ->where('to', '=', Auth::id());
                                            })
                                            ->orderBy('id', 'desc')
                                            ->paginate(15);
            ConversationMessage::where('from', $request->to_id)
              ->where('to', Auth::id())
              ->update(['seen' => 1, 'date_seen' => date('Y-m-d H:i:s')]);

        } else {
            $to_user = new \stdClass();
            $to_user->id = $request->to_id;
            $to_user->first_name = $request->first_name;
            $to_user->last_name = $request->last_name;

            $conversation = Conversation::where(function($query) use($to_user){
                                                $query->where('user_identifier', $to_user->id)
                                                ->orWhere('user_identifier', 'like', '%'.$to_user->id.'%');
                                            })
                                            ->where('coid', $request->coid)->first();

            if($conversation) {
                $conversations_count = ConversationMessage::where('conversation_id', $conversation->id)->count();
                ConversationMessage::where('conversation_id', $conversation->id)
                                        ->update([
                                                    'seen' => 1,
                                                    'date_seen' => date('Y-m-d H:i:s')
                                                ]);
                $conversations = ConversationMessage::where('conversation_id', $conversation->id)
                                            ->orderBy('id', 'desc')
                                            ->paginate(15);
            } else {
                $conversations_count = 0;
                $conversations = ConversationMessage::where('id', null)->paginate(15);
            }


            $company = DB::connection('mysql2')->table('callcenter_companies')
                                ->select('*')
                                ->where('coid', $request->coid)
                                ->first();

            $company_twilio_phone = TwilioPhone::select('*')->where('coid', $request->coid)->first();


            $optin_sms_sent = DB::connection('mysql2')->table('optin_sms_sent')
                                ->select('*')
                                ->where(function($query) use($company, $company_twilio_phone) {
                                    if($company_twilio_phone)
                                    {
                                        $query->where('sent_from', 'like', '%'.$company->twilio_sms.'%')
                                        ->orWhere('sent_from', 'like', '%'.$company_twilio_phone->phone.'%');
                                    } else {
                                        $query->where('sent_from', 'like', '%'.$company->twilio_sms.'%');
                                    }
                                })
                                ->where('sent_to', 'like', '%'.$request->to_id.'%')
                                ->first();

            if(!$optin_sms_sent)
            {
                $optin_sms_sent = SmsOptin::where(function($query) use($company, $company_twilio_phone) {
                                                if($company_twilio_phone)
                                                {
                                                    $query->where('sent_from', 'like', '%'.$company->twilio_sms.'%')
                                                    ->orWhere('sent_from', 'like', '%'.$company_twilio_phone->phone.'%');
                                                } else {
                                                    $query->where('sent_from', 'like', '%'.$company->twilio_sms.'%');
                                                }
                                            })
                                            ->where('sent_to', 'like', '%'.$request->to_id.'%')
                                            ->first();
            }

            if($optin_sms_sent)
            {
                $already_optin = $optin_sms_sent->approved;
                $optin_already_sent = true;
                if($optin_sms_sent->approved != 1)
                {
                    $needs_optin = true; 
                }
            } else {
                $needs_optin = true;
            }

            DB::connection('mysql2')->table('callcenter_sms_conversations')
                                    ->where(function($query) use($company, $company_twilio_phone){
                                        if(($company->twilio_sms != '' && $company->twilio_sms != null) && ($company_twilio_phone->phone != '' && $company_twilio_phone->phone != null))
                                        {
                                            $query->where('sent_from', 'like', '%'.$company->twilio_sms.'%')
                                                ->orWhere('sent_to', 'like', '%'.$company->twilio_sms.'%')
                                                ->orWhere('sent_from', 'like', '%'.$company_twilio_phone->phone.'%')
                                                ->orWhere('sent_to', 'like', '%'.$company_twilio_phone->phone.'%');
                                        } else if(($company->twilio_sms != '' && $company->twilio_sms != null)) {
                                            $query->where('sent_from', 'like', '%'.$company->twilio_sms.'%')
                                                ->orWhere('sent_to', 'like', '%'.$company->twilio_sms.'%');
                                        } else if(($company_twilio_phone->phone != '' && $company_twilio_phone->phone != null)) {
                                            $query->where('sent_from', 'like', '%'.$company_twilio_phone->phone.'%')
                                                ->orWhere('sent_to', 'like', '%'.$company_twilio_phone->phone.'%');
                                        }
                                    })
                                    ->where(function($query) use($to_user){
                                                $query->where('sent_from', 'like', '%'.$to_user->id.'%')
                                                ->orWhere('sent_to', 'like', '%'.$to_user->id.'%');
                                    })
                                    ->update([
                                        'seen' => 1, 
                                    ]);

        }

        $users = User::all();
        $user_names = array();
        foreach($users as $user)
        {
            $user_names[$user->id] = $user->first_name.' '.$user->last_name;
        }
        
        if(isset($request->from_modal))
        {
            $from_modal = $request->from_modal;
        } else {
            $from_modal = 0;
        }

        $conversation_id = preg_replace("/[^0-9]/i", "", trim($to_user->id));

        return view('dialogs.chat_window', [
                                                'company' => $company, 
                                                'user_names' => $user_names, 
                                                'team_conversation'=>$request->team_conversation, 
                                                'coid' => $request->coid, 
                                                'to_user' => $to_user, 
                                                'conversation_id' => $conversation_id, 
                                                'conversations' => $conversations->reverse(), 
                                                'profile_picture' => $profile_picture, 
                                                'first_name' => $request->first_name, 
                                                'last_name' => $request->last_name, 
                                                'conversations_count' => $conversations_count,
                                                'from_modal' => $from_modal,
                                                'already_optin' => $already_optin, 
                                                'optin_already_sent' => $optin_already_sent, 
                                                'needs_optin' => $needs_optin, 
                                            ]);
    }


    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function loadMoreMessages(Request $request)
    {

        $profile_picture = "default.jpg";

        if(isset($request->team_conversation) && $request->team_conversation == 1)
        {
            $to_user = User::find($request->to_id);
            $conversations_count = ConversationMessage::where(function($query) use($request){
                                            $query->where('from', '=', Auth::id())
                                            ->where('to', '=', $request->to_id);
                                        })->orWhere(function($query) use($request){
                                            $query->where('from', '=', $request->to_id)
                                            ->where('to', '=', Auth::id());
                                        })
                                        ->count();
            $conversations = ConversationMessage::where(function($query) use($request){
                                                $query->where('from', '=', Auth::id())
                                                ->where('to', '=', $request->to_id);
                                            })->orWhere(function($query) use($request){
                                                $query->where('from', '=', $request->to_id)
                                                ->where('to', '=', Auth::id());
                                            })
                                            ->orderBy('id', 'desc')
                                            ->paginate(15);
            ConversationMessage::where('from', $request->to_id)
              ->where('to', Auth::id())
              ->update(['seen' => 1, 'date_seen' => date('Y-m-d H:i:s')]);
        } else {
            $to_user = new \stdClass();
            $to_user->id = $request->to_id;
            $to_user->first_name = $request->first_name;
            $to_user->last_name = $request->last_name;

            $conversation = Conversation::where(function($query) use($to_user){
                                                $query->where('user_identifier', $to_user->id)
                                                ->orWhere('user_identifier', 'like', '%'.$to_user->id.'%');
                                            })
                                            ->where('coid', $request->coid)->first();

            if($conversation) {
                $conversations_count = ConversationMessage::where('conversation_id', $conversation->id)->count();
                $conversations = ConversationMessage::where('conversation_id', $conversation->id)
                                            ->orderBy('id', 'desc')
                                            ->paginate(15);
            } else {
                $conversations_count = 0;
                $conversations = ConversationMessage::where('id', null)->paginate(15);
            }

            
        }

        $users = User::all();
        $user_names = array();
        foreach($users as $user)
        {
            $user_names[$user->id] = $user->first_name.' '.$user->last_name;
        }

        /*$to_user = User::find($request->to_id);
        if($to_user->picture_id != "") {
            $profile_pic = Picture::find($to_user->picture_id);
            $profile_picture = $profile_pic->filename;
        } else {
            $profile_picture = "default.jpg";
        }
        
        $conversations = Conversation::where(function($query) use($request){
                                            $query->where('from_user_id', '=', Auth::id())
                                            ->where('to_user_id', '=', $request->to_id);
                                        })->orWhere(function($query) use($request){
                                            $query->where('from_user_id', '=', $request->to_id)
                                            ->where('to_user_id', '=', Auth::id());
                                        })
                                        ->orderBy('id', 'desc')
                                        ->paginate(15);
        Conversation::where('from_user_id', $request->to_id)
          ->where('to_user_id', Auth::id())
          ->update(['seen' => 1, 'date_seen' => date('Y-m-d H:i:s')]);*/

        //$conversation_id = str_replace('+1', 'replace', $to_user->id);

        $conversation_id = preg_replace("/[^0-9]/i", "", trim($to_user->id));

        return view('dialogs.load_messages', [
                                                'user_names' => $user_names, 
                                                'team_conversation'=>$request->team_conversation, 
                                                'coid' => $request->coid, 
                                                'to_user' => $to_user, 
                                                'conversation_id' => $conversation_id, 
                                                'conversations' => $conversations->reverse(), 
                                                'profile_picture' => $profile_picture, 
                                                'conversations_count' => $conversations->count()
                                            ]);
    }



    public function getChatContacts()
    {
        $admins = array();
        if(Auth::user()->user_type->name == 'Admin') 
        {
            $users = User::where('id', '!=', Auth::id())
                            ->orderBy('user_type_id')
                            ->orderBy('first_name')
                            ->get();

        } else if(Auth::user()->user_type->name == 'Dealer')
        {
            $admins = User::where('id', '!=', Auth::id())
                            ->where('user_type_id', 1)
                            ->orderBy('user_type_id')
                            ->orderBy('first_name')
                            ->get();

            $user_companies_array = array();
            foreach(Auth::user()->dealers as $company)
            {
              $user_companies_array[] = $company->dealer_id;
            }

            /*$users = User::select('users.*')
                            ->join('user_dealer', 'users.id', '=', 'user_dealer.user_id')
                            ->where('users.id', '!=', Auth::id())
                            ->where(function($query) {
                                    $query->where('user_type_id', 2)
                                        ->orWhere('user_type_id', 3);
                                })
                            ->whereRaw('FIND_IN_SET(user_dealer.dealer_id, \''.implode(',',$user_companies_array).'\')')
                            ->groupBy('users.id')
                            ->orderBy('user_type_id')
                            ->orderBy('first_name')
                            ->get();   */

            $users = User::select('users.*')
                            ->join('user_dealer', 'users.id', '=', 'user_dealer.user_id')
                            ->where('users.id', '!=', Auth::id())
                            ->where(function($query) {
                                    $query->where('user_type_id', 2)
                                        ->orWhere('user_type_id', 3);
                                })
                            ->whereRaw('FIND_IN_SET(user_dealer.dealer_id, \''.implode(',',$user_companies_array).'\')')
                            ->groupBy('users.id')
                            ->orderBy('user_type_id')
                            ->orderBy('first_name')
                            ->get();                 

        } else if(Auth::user()->user_type->name == 'Full BDC Caller')
        {

            $admins = User::where('id', '!=', Auth::id())
                            ->where('user_type_id', 1)
                            ->orderBy('user_type_id')
                            ->orderBy('first_name')
                            ->get();

            $user_companies_array = array();
            foreach(Auth::user()->dealers as $company)
            {
              $user_companies_array[] = $company->dealer_id;
            }

            $users = User::select('users.*')
                            ->join('user_dealer', 'users.id', '=', 'user_dealer.user_id')
                            ->where('users.id', '!=', Auth::id())
                            ->whereRaw('FIND_IN_SET(user_dealer.dealer_id, \''.implode(',',$user_companies_array).'\')')
                            ->groupBy('users.id')
                            ->orderBy('user_type_id')
                            ->orderBy('first_name')
                            ->get();     

        } else if(Auth::user()->user_type->name == 'Inbound Caller')
        {
            
            $admins = User::where('id', '!=', Auth::id())
                            ->where('user_type_id', 1)
                            ->orderBy('user_type_id')
                            ->orderBy('first_name')
                            ->get();

            $user_companies_array = array();
            foreach(Auth::user()->dealers as $company)
            {
              $user_companies_array[] = $company->dealer_id;
            }

            $users = User::select('users.*')
                            ->join('user_dealer', 'users.id', '=', 'user_dealer.user_id')
                            ->where('users.id', '!=', Auth::id())
                            ->whereRaw('FIND_IN_SET(user_dealer.dealer_id, \''.implode(',',$user_companies_array).'\')')
                            ->groupBy('users.id')
                            ->orderBy('user_type_id')
                            ->orderBy('first_name')
                            ->get();     

        } else if(Auth::user()->user_type->name == 'Dealer Caller')
        {
            
            $admins = User::where('id', '!=', Auth::id())
                            ->where('user_type_id', 1)
                            ->orderBy('user_type_id')
                            ->orderBy('first_name')
                            ->get();

            $user_companies_array = array();
            foreach(Auth::user()->dealers as $company)
            {
              $user_companies_array[] = $company->dealer_id;
            }

            $users = User::select('users.*')
                            ->join('user_dealer', 'users.id', '=', 'user_dealer.user_id')
                            ->where('users.id', '!=', Auth::id())
                            ->where(function($query) {
                                    $query->where('user_type_id', 2)
                                        ->orWhere('user_type_id', 3);
                                })
                            ->whereRaw('FIND_IN_SET(user_dealer.dealer_id, \''.implode(',',$user_companies_array).'\')')
                            ->groupBy('users.id')
                            ->orderBy('user_type_id')
                            ->orderBy('first_name')
                            ->get();     
                            
        }


        $unseen = ConversationMessage::selectRaw('`from`, count(id) as unseen_count')
                                        ->where('to', Auth::id())
                                        ->where('seen', 0)
                                        ->groupBy('from')
                                        ->get();
        $unseen_array = array();
        foreach ($unseen as $key => $value) {
            $unseen_array[$value->from] = $value->unseen_count;
        }
        
        return view('lists.contacts_list', [
                                            'admins' => $admins,
                                            'users' => $users,
                                            'unseen_array' => $unseen_array,
                                        ]);

    }

    function getCustomerRecordOLD(Request $request)
    {
        $company = DB::connection('mysql2')->table('callcenter_companies')
                                ->select('*')
                                ->where('coid', $request->coid)
                                ->first();

        $db_table = 'livecall_'.$company->co_short;
        $phone = substr($request->phone, -10);
        $record = DB::connection('mysql2')->table($db_table)
                                ->select('*')
                                ->where('phone', $phone)
                                ->orWhere('cellphone', $phone)
                                ->orWhere('best_phone', $phone)
                                ->where(function($query) {
                                                $query->where('livecall_prev_calltype', '!=', 0)
                                                ->orWhere('livecall_next_calltype', '!=', 0);
                                            })
                                ->orderBy('dateadded', 'desc')
                                ->first();

        if($record)
        {
            if($record->livecall_next_calltype != null && $record->livecall_next_calltype != 0)
            {
                $call_type_id = $record->livecall_next_calltype;
            } else {
                $call_type_id = $record->livecall_prev_calltype;
            }

            return response()->json([
                'call_id' => $record->id,
                'call_type_id' => $call_type_id,
            ]);
        } else {
            return response()->json([
                'call_id' => null,
                'call_type_id' => null,
            ]);
        }
    }

    function getCustomerRecord(Request $request)
    {
        $company = DB::connection('mysql2')->table('callcenter_companies')
                                ->select('*')
                                ->where('coid', $request->coid)
                                ->first();

        $db_table = 'livecall_'.$company->co_short;
        $phone = substr($request->phone, -10);
        $records = DB::connection('mysql2')->table($db_table)
                                ->select('*')
                                ->where('phone', $phone)
                                ->orWhere('cellphone', $phone)
                                ->orWhere('best_phone', $phone)
                                ->where(function($query) {
                                                $query->where('livecall_prev_calltype', '!=', 0)
                                                ->orWhere('livecall_next_calltype', '!=', 0);
                                            })
                                ->orderBy('dateadded', 'desc')
                                ->limit(5)
                                ->get();
        $common_functions = new CommonFunctionsController();
        $calltype_texts = $common_functions->getCallTypesTexts();

        return view('dialogs.customer_record_list', [
            'records' => $records,
            'calltype_texts' => $calltype_texts,
            'coid' => $request->coid,
        ]);
    }


}

