<?php

namespace App\Http\Controllers;

use App\CallType;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\DB;

class PendingCallsController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {
        if(Auth::user()->user_type->name == 'Admin') 
        {
            $dealers = DB::connection('mysql2')->table('callcenter_companies')
                                ->select('co_name', 'co_short', 'coid')
                                ->where('is_active', 1)
                                ->orderBy('co_name')
                                ->get();
        } else if(Auth::user()->user_type->name == 'Dealer') {
            $user_companies_array = array();
            foreach(Auth::user()->dealers as $company)
            {
              $user_companies_array[] = $company->dealer_id;
            }

            $dealers = DB::connection('mysql2')->table('callcenter_companies')
                                ->select('co_name', 'co_short', 'coid')
                                ->whereRaw("FIND_IN_SET(coid, '".implode(',', $user_companies_array)."')")
                                ->get();
        } else if(Auth::user()->user_type->name == 'Full BDC Caller') {
            $user_companies_array = array();
            foreach(Auth::user()->dealers as $company)
            {
              $user_companies_array[] = $company->dealer_id;
            }

            $dealers = DB::connection('mysql2')->table('callcenter_companies')
                                ->select('co_name', 'co_short', 'coid')
                                ->whereRaw("FIND_IN_SET(coid, '".implode(',', $user_companies_array)."')")
                                ->get();
        } else if(Auth::user()->user_type->name == 'Inbound Caller') {
            $user_companies_array = array();
            foreach(Auth::user()->dealers as $company)
            {
              $user_companies_array[] = $company->dealer_id;
            }

            $dealers = DB::connection('mysql2')->table('callcenter_companies')
                                ->select('co_name', 'co_short', 'coid')
                                ->whereRaw("FIND_IN_SET(coid, '".implode(',', $user_companies_array)."')")
                                ->get();
        } else if(Auth::user()->user_type->name == 'Dealer Caller') {
            $user_companies_array = array();
            foreach(Auth::user()->dealers as $company)
            {
              $user_companies_array[] = $company->dealer_id;
            }

            $dealers = DB::connection('mysql2')->table('callcenter_companies')
                                ->select('co_name', 'co_short', 'coid')
                                ->whereRaw("FIND_IN_SET(coid, '".implode(',', $user_companies_array)."')")
                                ->get();
        }

        $commonFunctions = new CommonFunctionsController();
        $commonFunctions->modelToSelectOptions($dealers,'co_name', 'coid');
        $dealers_options = $commonFunctions->select_options;

        $commonFunctions->modelToSelectOptions(CallType::orderBy('name')->get(),'name', 'id');
        $calltypes_options = $commonFunctions->select_options;

        $all_dealers = DB::connection('mysql2')->table('callcenter_companies')
                                ->select('coid', 'co_name', 'co_short')
                                ->orderBy('co_name')
                                ->get();
        $dealer_names = array();
        foreach ($all_dealers as $key => $value) {
        	$dealer_names[$value->coid] = $value->co_name;
        }

        return view('pages.pending_calls', [
    										'dealers_options' => $dealers_options,
    										'dealer_names' => $dealer_names,
    										'calltypes_options' => $calltypes_options,
										]);
    }


    /**
     * Display a list of the resource with pagination.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */

    public function pagination(Request $request)

    {
        
        $raw_search = $this->getRawSearch($request);

        //dd($raw_search);

        $search_date = 'remind';  

    	$company = DB::connection('mysql2')->table('callcenter_companies')
                                ->select('*')
                                ->where('coid', $request->coid_search)
                                ->first();

    	$db_table = 'livecall_'.$company->co_short;
        $call_records = DB::connection('mysql2')->table($db_table)
        										->select(
        													'id', 
        													'fname', 
        													'lname', 
        													'phone', 
        													'cellphone', 
        													'email', 
        													'vin', 
        													'service_date', 
        													'livecall_prev_calltype', 
        													'livecall_next_calltype', 
                                                            'caller_comments', 
                                                            'customer_status',
        													'remind'
        												)
        										->whereRaw($raw_search)
        										->orderBy($search_date, 'desc')->get();

        $call_types = CallType::all();
        $call_types_names = array();
        foreach ($call_types as $key => $value) {
        	$call_types_names[$value->id] = $value->name;
        }

        return view('lists.pending_calls_list', [
        										'call_records' => $call_records,
        										'call_types_names' => $call_types_names,
        										'company' => $company,
    										]);
    }

    public function getRawSearch($my_request) {
        $raw_search_array = array();

        $raw_search_array[] = " livecall_count < livecall_times ";
        $raw_search_array[] = " livecall_showtocall = 1 ";
        $raw_search_array[] = " global_dnc != 1 ";
        $raw_search_array[] = " dealer_dnc != 1 ";

        if($my_request->call_type_id_search != '') {
            $raw_search_array[] = " livecall_next_calltype = '".$my_request->call_type_id_search."' ";
        }

        if($my_request->start_date_search != '' && $my_request->end_date_search != '') {
            $start_date_search = date('Y-m-d', strtotime($my_request->start_date_search));
            $end_date_search = date('Y-m-d', strtotime($my_request->end_date_search));
            $raw_search_array[] = " (remind >= '".$start_date_search."' AND remind <= '".$end_date_search."')";
        }

        if(trim($my_request->name_search) != '') {
            $raw_search_array[] = " (
                fname like '%".$my_request->name_search."%' OR 
                lname like '%".$my_request->name_search."%' OR 
                CONCAT(fname, ' ', lname) like '%".$my_request->name_search."%' 
            )";
        }

        if(trim($my_request->phone_search) != '') {
            $raw_search_array[] = " (
                phone like '%".$my_request->phone_search."%' OR 
                cellphone like '%".$my_request->phone_search."%' OR 
                best_phone like '%".$my_request->phone_search."%'
            )";
        }

        if(trim($my_request->email_search) != '') {
            $raw_search_array[] = " (
                email like '%".$my_request->email_search."%' 
            )";
        }

        if(trim($my_request->vin_search) != '') {
            $raw_search_array[] = " (
                vin like '%".$my_request->vin_search."%' 
            )";
        }

        $raw_search = implode(' AND ', $raw_search_array);

        return $raw_search;
    }
}
