<?php

namespace App\Http\Controllers;

use App\CallType;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;

class RecordsImportedReport extends Controller
{
    public function index() {
        $dealers = DB::connection('mysql2')
            ->table('callcenter_companies')
            ->select('co_name', 'co_short', 'coid')
            ->where('is_active', 1)
            ->orderBy('co_name')
            ->get();
        
        $commonFunctions = new CommonFunctionsController();
        $commonFunctions->modelToSelectOptions($dealers,'co_name', 'coid');
        $dealers_options = $commonFunctions->select_options;

        $call_types = CallType::orderBy('name')->get();
        $commonFunctions->modelToSelectOptions($call_types,'name', 'id');
        $calltype_options = $commonFunctions->select_options;

        return view('pages.records_imported', [
            'dealers_options' => $dealers_options,
            'calltype_options' => $calltype_options,
        ]);
    }

    public function getReport(Request $request) {
        $coid = $request->coid_search;

        $dealer = DB::connection('mysql2')
            ->table('callcenter_companies')
            ->select('co_name', 'co_short', 'coid')
            ->where('coid', $coid)
            ->first();

        $start_date = date( 'Y-m-d 00:00:00', strtotime( $request->start_date_search ) );
        $end_date = date( 'Y-m-d 23:59:59', strtotime( $request->end_date_search ) );
        $calltypes = implode(',', $request->calltypes_search);

        $calltypes_where = "dateadded <= '$end_date'";

        if ($calltypes != 'all') {
            $calltypes_where.= ' AND ( FIND_IN_SET(livecall_prev_calltype, "' . $calltypes . '") 
                OR FIND_IN_SET(livecall_next_calltype, "' . $calltypes . '") ) ';
        }

        /* $import_records = DB::connection('mysql2')
            ->table('callcenter_import_log')
            ->selectRaw('SUM(total_records) as records_total, SUM(showtocall) as total_stc')
            ->where('campaign', $dealer->co_short)
            ->where('import_time', '>=', $start_date)
            ->where('import_time', '<=', $end_date)
            ->first(); */

        $import_records = DB::connection('mysql2')
            ->table('livecall_' . $dealer->co_short)
            ->where('dateadded', '>=', $start_date)
            ->whereRaw($calltypes_where)
            ->count();

        $import_records_nstc = DB::connection('mysql2')
            ->table('livecall_' . $dealer->co_short)
            ->where('livecall_next_calltype', '!=', 0)
            ->where('livecall_next_calltype', '!=', '')
            ->where('livecall_showtocall', 0)
            ->where('dateadded', '>=', $start_date)
            ->whereRaw($calltypes_where)
            ->count();

        /* $total_imported = ! empty( $import_records->records_total ) ? $import_records->records_total : 0;
        $total_stc      = ! empty( $import_records->total_stc ) ? $import_records->total_stc : 0;
        $total_nstc     = $total_imported - $total_stc; */

        $total_imported = $import_records;
        $total_stc      = $import_records - $import_records_nstc;
        $total_nstc     = $import_records_nstc;

        $added_records_no_email = DB::connection('mysql2')
            ->table('livecall_' . $dealer->co_short)
            ->where(function($query) {
                $query->where('email', '=', '')
                    ->orWhereRaw('email IS NULL');
            })
            ->where('dateadded', '>=', $start_date)
            ->whereRaw($calltypes_where)
            ->count();

        $added_records_email = DB::connection('mysql2')
            ->table('livecall_' . $dealer->co_short)
            ->where('email', '!=', '')
            ->whereRaw('email IS NOT NULL')
            ->where('dateadded', '>=', $start_date)
            ->whereRaw($calltypes_where)
            ->count();

        $added_records_dnc = DB::connection('mysql2')
            ->table('livecall_' . $dealer->co_short)
            ->where(function($query) {
                $query->where('global_dnc', '=', 1)
                    ->orWhere('dealer_dnc', '=', 1);
            })
            ->where('dateadded', '>=', $start_date)
            ->whereRaw($calltypes_where)
            ->count();

        $records_with_phones = DB::connection('mysql2')
            ->table('livecall_' . $dealer->co_short)
            ->where(function($query) {
                $query->where('phone', '!=', '')
                    ->orWhere('cellphone', '!=', '')
                    ->orWhere('best_phone', '!=', '');
            })
            ->where('dateadded', '>=', $start_date)
            ->whereRaw($calltypes_where)
            ->count();
        
    
        return view('partials.imported_records_totals', [
            'total_imported'         => $total_imported,
            'total_stc'              => $total_stc,
            'total_nstc'             => $total_nstc,
            'added_records_no_email' => $added_records_no_email,
            'added_records_email'    => $added_records_email,
            'added_records_dnc'      => $added_records_dnc,
            'records_with_phones'    => $records_with_phones,
        ]); 
        
    }
}
