<?php

namespace App\Http\Controllers;

use App\CallType;
use Illuminate\Http\Request;
use Maatwebsite\Excel\Facades\Excel;
use Illuminate\Support\Facades\DB;
use App\Exports\SurverReportExport;
use App\Exports\CallStatusExport;
use DateTime;
use DateTimeZone;

class ReportController extends Controller
{
    public function basicDealerReportPanel() {
        $dealers = DB::connection('mysql2')
            ->table('callcenter_companies')
            ->select('co_name', 'co_short', 'coid')
            ->where('is_active', 1)
            ->orderBy('co_name')
            ->get();

        $coid_list = [];
        foreach ($dealers as $dealer) {
            $coid_list[] = $dealer->coid;
        }
        
        $commonFunctions = new CommonFunctionsController();
        $commonFunctions->modelToSelectOptions($dealers,'co_name', 'coid');
        $dealers_options = $commonFunctions->select_options;

        return view('reports.basic_dealer_report_panel', [
            'dealers_options' => $dealers_options,
            'coid_list'       => implode(',', $coid_list),
        ]);
    }

    public function basicDealerReport(Request $request) {

        $coid       = $request->coid_search;
        $start_date = date( 'Y-m-d', strtotime( $request->start_date_search ) ) . ' 00:00:00' ;
        $end_date   = date( 'Y-m-d', strtotime( $request->end_date_search ) ) . ' 23:59:59' ;
        $current_step = $request->current_step;

        if ( $coid == 'all' ) {
            $coid_list = explode( ',', $request->coid_list );
            $coid      = $coid_list[ $current_step ];
        }

        $dealers = DB::connection('mysql2')
                ->table('callcenter_companies')
                ->select('co_name', 'co_short', 'coid')
                ->where('coid', $coid)
                ->get();
        
        $dealer_names = [];
        foreach ($dealers as $key => $value) {
            $co_short              = $value->co_short;
            $co_name               = $value->co_name;
            $coid                  = $value->coid;
            $dealer_names[ $coid ] = $co_name;

            $callcenter_call_log = DB::connection('mysql2')
                ->table('callcenter_call_log')
                ->where('campaign', $co_short)
                ->where('call_time', '>=', $start_date)
                ->where('call_time', '<=', $end_date)
                ->get();
            
            
            $dials[ $coid ]     = [];
            $hr[ $coid ]        = [];
            $emails[ $coid ]    = [];
            $reminders[ $coid ] = [];
            $imported[ $coid ]  = [];
            $callable[ $coid ]  = [];
            $due[ $coid ]       = [];

            $totaldials     = 0;
            $totalhr        = 0;
            $totaldue       = 0;
            $totalemails    = 0;
            $totalreminders = 0;
            $totalimported  = 0;
            $totalcallable  = 0;

            foreach ($callcenter_call_log as $call_log) {
                $call_status = $call_log->call_status;
                
                // Dials.
                if ( ! in_array( $call_status, ['BD', 'CL', 'RPT'] ) ) {
                    if ( empty( $dials[ $coid ][ $call_log->call_type ] ) ) {
                        $dials[ $coid ][ $call_log->call_type ] = 0;
                    }
                    $dials[ $coid ][ $call_log->call_type ]+= 1;
                    $totaldials++;
                }

                // HR.
                if ( in_array( $call_status, ['COMP', 'COMPEF'] ) ) {
                    if ( empty( $hr[ $coid ][ $call_log->call_type ] ) ) {
                        $hr[ $coid ][ $call_log->call_type ] = 0;
                    }
                    $hr[ $coid ][ $call_log->call_type ]+= 1;
                    $totalhr++;

                    // Reminders.
                    if ( $call_log->reminder_scheduled == 1) {
                        if ( empty( $reminders[ $coid ][ $call_log->call_type ] ) ) {
                            $reminders[ $coid ][ $call_log->call_type ] = 0;
                        }
                        $reminders[ $coid ][ $call_log->call_type ]+= 1;
                        $totalreminders++;
                    }
                }

                // Emails.
                if ( in_array( $call_status, ['COMPEF'] ) ) {
                    if ( empty( $emails[ $coid ][ $call_log->call_type ] ) ) {
                        $emails[ $coid ][ $call_log->call_type ] = 0;
                    }
                    $emails[ $coid ][ $call_log->call_type ]+= 1;
                    $totalemails++;
                }
            }

            //HR%
            if ( $totaldials > 0 ) {
                $hrp       = ( $totalhr / $totaldials ) * 100;
                $hrpercent = sprintf( '%1d', $hrp );
            } else {
                $hrpercent = 0;
            }

            // DUE.
            $totaldue_records = DB::connection('mysql2')
                ->table('livecall_' . $co_short)
                ->select('id', 'livecall_next_calltype')
                ->where('remind', '>=', '0000-00-00')
                ->where('remind', '<=', $end_date)
                ->where('livecall_next_calltype', '!=', 3) // service reminders
                ->where('livecall_next_calltype', '!=', 77) // service reminder E
                ->where('livecall_next_calltype', '!=', 51) // 30k service reminder call
                ->where('livecall_next_calltype', '!=', 88) // service reminder C
                ->whereRaw('livecall_count < livecall_times')
                ->where('livecall_showtocall', 1)
                ->get();

            foreach ($totaldue_records as $totaldue_record) {
                // Calltype Due.
                if ( empty( $due[ $coid ][ $totaldue_record->livecall_next_calltype ] ) ) {
                    $due[ $coid ][ $totaldue_record->livecall_next_calltype ] = 0;
                }
                $due[ $coid ][ $totaldue_record->livecall_next_calltype ]+= 1;
                $totaldue++;
            }

            $callcenter_import_log = DB::connection('mysql2')
                ->table('callcenter_import_log')
                ->where('campaign', $co_short)
                ->where('import_time', '>=', $start_date)
                ->where('import_time', '<=', $end_date)
                ->get();

            
            foreach ($callcenter_import_log as $import_log) {
                
                // Imported.
                if ( empty( $imported[ $coid ][ $import_log->calltype ] ) ) {
                    $imported[ $coid ][ $import_log->calltype ] = 0;
                }
                $imported[ $coid ][ $import_log->calltype ]+= $import_log->total_records;
                $totalimported+= $import_log->total_records;
                // Callable.
                if ( empty( $callable[ $coid ][ $import_log->calltype ] ) ) {
                    $callable[ $coid ][ $import_log->calltype ] = 0;
                }
                $callable[ $coid ][ $import_log->calltype ]+= $import_log->showtocall;
                $totalcallable+= $import_log->showtocall;
            }

        }

        $call_types       = CallType::get();
        $call_types_names = [];
        foreach ($call_types as $key => $value) {
            $call_types_names[ $value->id ] = $value->name;
        }

        return view('reports.basic_dealer_report', [
            'dealer_names'     => $dealer_names,
            'dials'            => $dials,
            'hr'               => $hr,
            'emails'           => $emails,
            'reminders'        => $reminders,
            'due'              => $due,
            'imported'         => $imported,
            'callable'         => $callable,
            'totaldials'       => $totaldials,
            'totalhr'          => $totalhr,
            'totaldue'         => $totaldue,
            'totalemails'      => $totalemails,
            'totalreminders'   => $totalreminders,
            'totalimported'    => $totalimported,
            'hrpercent'        => $hrpercent,
            'totalcallable'    => $totalcallable,
            'call_types_names' => $call_types_names,
            'current_step'     => $current_step,
        ]);
    }

    public function emailCapturePanel() {
        $dealers = DB::connection('mysql2')
            ->table('callcenter_companies')
            ->select('co_name', 'co_short', 'coid')
            ->where('is_active', 1)
            ->orderBy('co_name')
            ->get();

        $coid_list = [];
        foreach ($dealers as $dealer) {
            $coid_list[] = $dealer->coid;
        }
        
        $commonFunctions = new CommonFunctionsController();
        $commonFunctions->modelToSelectOptions($dealers,'co_name', 'coid');
        $dealers_options = $commonFunctions->select_options;

        $call_types       = CallType::get();
        $commonFunctions->modelToSelectOptions($call_types,'name', 'id');
        $calltype_options = $commonFunctions->select_options;

        $customer_statuses_db = DB::connection('mysql2')
            ->table('callcenter_dealertalerts')
            ->select('id', 'type', )
            ->orderBy('type')
            ->get();

        $commonFunctions->modelToSelectOptions($customer_statuses_db,'type', 'id');
        $customer_statuses_options = $commonFunctions->select_options;
        
        return view('pages.email_capture_panel', [
            'dealers_options'           => $dealers_options,
            'calltype_options'          => $calltype_options,
            'customer_statuses_options' => $customer_statuses_options,
        ]);
    }

    public function emailCapture(Request $request) {
        $coid               = $request->coid_search;
        $calltype_id        = $request->calltype_search;
        $customer_status_id = $request->customer_status_search;
        $start_date = date( 'Y-m-d', strtotime( $request->start_date_search ) ) . ' 00:00:00' ;
        $end_date   = date( 'Y-m-d', strtotime( $request->end_date_search ) ) . ' 23:59:59' ;

        $dealer = DB::connection('mysql2')
                ->table('callcenter_companies')
                ->select('co_name', 'co_short', 'coid')
                ->where('coid', $coid)
                ->first();

        $db_table = 'livecall_' . $dealer->co_short;

        $extra_where = "";
        $extra_where.= ($customer_status_id != "all")? " AND customer_status = '$customer_status_id' " : "";
        $extra_where.= ($calltype_id != "all")? " AND livecall_prev_calltype = '$calltype_id' " : "";

        $where_sql = " lastupdate >= '$start_date' AND lastupdate <= '$end_date' AND livecall_status = 'COMPEF' " . $extra_where;
        $livecalls = DB::connection('mysql2')
                ->table($db_table)
                ->whereRaw($where_sql)
                ->get();
        $total_emails_farmed_month = $livecalls->count();

        $firstday_of_month = date('Y-m-01');
        $today_date_start = date('Y-m-d');
        $today_date = date('Y-m-d');
        $today_date_start.= ' 00:00:00';
        $firstday_of_month.= ' 00:00:00';
        $today_date.= ' 23:59:59';

        $where_sql = " lastupdate >= '$today_date_start' AND lastupdate <= '$today_date' AND livecall_status = 'COMPEF' " . $extra_where;
        $total_emails_farmed_today = DB::connection('mysql2')
                ->table($db_table)
                ->whereRaw($where_sql)
                ->count();

        $previousmonthdate = date("m", strtotime("first day of previous month"));
        $previousmonthdate = date('Y-'.$previousmonthdate.'-28');
        $previousmonthdate.= ' 00:00:00';
        
        $where_sql = " lastupdate >= '$previousmonthdate' AND lastupdate <= '$today_date' AND livecall_status = 'COMPEF' " . $extra_where;
        $total_emails_farmed_month_to_date = DB::connection('mysql2')
                ->table($db_table)
                ->whereRaw($where_sql)
                ->count();

        $customer_status_icons_db = DB::connection('mysql2')
            ->table('callcenter_dealertalerts')
            ->select('id', 'type', 'icon')
            ->get();
    
        $customer_status_icons = [];
        $customer_status_texts = [];
        foreach ($customer_status_icons_db as $key => $value) {
            $customer_status_icons[$value->id] = $value->icon;
            $customer_status_texts[$value->id] = $value->type;
        }

        $call_types_icons_db = DB::connection('mysql2')
            ->table('callcenter_calltypes')
            ->select('cid', 'calltype_text', 'icon')
            ->get();
    
        $call_types_icons = [];
        $call_types_texts = [];
        foreach ($call_types_icons_db as $key => $value) {
            $call_types_icons[$value->cid] = $value->icon;
            $call_types_texts[$value->cid] = $value->calltype_text;
        }

        return view('reports.email_capture', [
            'coid'                              => $coid,
            'livecalls'                         => $livecalls,
            'total_emails_farmed_month'         => $total_emails_farmed_month,
            'total_emails_farmed_today'         => $total_emails_farmed_today,
            'total_emails_farmed_month_to_date' => $total_emails_farmed_month_to_date,
            'customer_status_icons'             => $customer_status_icons,
            'customer_status_texts'             => $customer_status_texts,
            'call_types_icons'                  => $call_types_icons,
            'call_types_texts'                  => $call_types_texts,
        ]);
    
    }

    public function badDataPanel() {
        $dealers = DB::connection('mysql2')
            ->table('callcenter_companies')
            ->select('co_name', 'co_short', 'coid')
            ->where('is_active', 1)
            ->orderBy('co_name')
            ->get();

        $coid_list = [];
        foreach ($dealers as $dealer) {
            $coid_list[] = $dealer->coid;
        }
        
        $commonFunctions = new CommonFunctionsController();
        $commonFunctions->modelToSelectOptions($dealers,'co_name', 'coid');
        $dealers_options = $commonFunctions->select_options;

        $call_types       = CallType::get();
        $commonFunctions->modelToSelectOptions($call_types,'name', 'id');
        $calltype_options = $commonFunctions->select_options;

        $customer_statuses_db = DB::connection('mysql2')
            ->table('callcenter_dealertalerts')
            ->select('id', 'type', )
            ->orderBy('type')
            ->get();

        $commonFunctions->modelToSelectOptions($customer_statuses_db,'type', 'id');
        $customer_statuses_options = $commonFunctions->select_options;
        
        return view('pages.bad_data', [
            'dealers_options'           => $dealers_options,
            'calltype_options'          => $calltype_options,
            'customer_statuses_options' => $customer_statuses_options,
        ]);
    }

    public function badData(Request $request) {
        $coid               = $request->coid_search;
        $calltype_id        = $request->calltype_search;
        $customer_status_id = $request->customer_status_search;
        $start_date = date( 'Y-m-d', strtotime( $request->start_date_search ) ) . ' 00:00:00' ;
        $end_date   = date( 'Y-m-d', strtotime( $request->end_date_search ) ) . ' 23:59:59' ;

        $dealer = DB::connection('mysql2')
                ->table('callcenter_companies')
                ->select('co_name', 'co_short', 'coid')
                ->where('coid', $coid)
                ->first();

        $db_table = 'livecall_' . $dealer->co_short;

        $extra_where = "";
        $extra_where.= ($customer_status_id != "all")? " AND customer_status = '$customer_status_id' " : "";
        $extra_where.= ($calltype_id != "all")? " AND livecall_prev_calltype = '$calltype_id' " : "";

        $where_sql = " lastupdate >= '$start_date' AND lastupdate <= '$end_date' AND livecall_status = 'BD' " . $extra_where;
        $livecalls = DB::connection('mysql2')
                ->table($db_table)
                ->whereRaw($where_sql)
                ->get();
        $total_emails_farmed_month = $livecalls->count();

        $firstday_of_month = date('Y-m-01');
        $today_date_start = date('Y-m-d');
        $today_date = date('Y-m-d');
        $today_date_start.= ' 00:00:00';
        $firstday_of_month.= ' 00:00:00';
        $today_date.= ' 23:59:59';

        $where_sql = " lastupdate >= '$today_date_start' AND lastupdate <= '$today_date' AND livecall_status = 'BD' " . $extra_where;
        $total_emails_farmed_today = DB::connection('mysql2')
                ->table($db_table)
                ->whereRaw($where_sql)
                ->count();

        $previousmonthdate = date("m", strtotime("first day of previous month"));
        $previousmonthdate = date('Y-'.$previousmonthdate.'-28');
        $previousmonthdate.= ' 00:00:00';
        
        $where_sql = " lastupdate >= '$previousmonthdate' AND lastupdate <= '$today_date' AND livecall_status = 'BD' " . $extra_where;
        $total_emails_farmed_month_to_date = DB::connection('mysql2')
                ->table($db_table)
                ->whereRaw($where_sql)
                ->count();

        $customer_status_icons_db = DB::connection('mysql2')
            ->table('callcenter_dealertalerts')
            ->select('id', 'type', 'icon')
            ->get();
    
        $customer_status_icons = [];
        $customer_status_texts = [];
        foreach ($customer_status_icons_db as $key => $value) {
            $customer_status_icons[$value->id] = $value->icon;
            $customer_status_texts[$value->id] = $value->type;
        }

        $call_types_icons_db = DB::connection('mysql2')
            ->table('callcenter_calltypes')
            ->select('cid', 'calltype_text', 'icon')
            ->get();
    
        $call_types_icons = [];
        $call_types_texts = [];
        foreach ($call_types_icons_db as $key => $value) {
            $call_types_icons[$value->cid] = $value->icon;
            $call_types_texts[$value->cid] = $value->calltype_text;
        }

        return view('reports.bad_data', [
            'coid'                              => $coid,
            'livecalls'                         => $livecalls,
            'total_emails_farmed_month'         => $total_emails_farmed_month,
            'total_emails_farmed_today'         => $total_emails_farmed_today,
            'total_emails_farmed_month_to_date' => $total_emails_farmed_month_to_date,
            'customer_status_icons'             => $customer_status_icons,
            'customer_status_texts'             => $customer_status_texts,
            'call_types_icons'                  => $call_types_icons,
            'call_types_texts'                  => $call_types_texts,
        ]);
    
    }

    public function surveyReportPanel() {
        $dealers = DB::connection('mysql2')
            ->table('callcenter_companies')
            ->select('co_name', 'co_short', 'coid')
            ->where('is_active', 1)
            ->orderBy('co_name')
            ->get();

        $coid_list = [];
        foreach ($dealers as $dealer) {
            $coid_list[] = $dealer->coid;
        }
        
        $commonFunctions = new CommonFunctionsController();
        $commonFunctions->modelToSelectOptions($dealers,'co_name', 'coid');
        $dealers_options = $commonFunctions->select_options;

        $call_types       = CallType::orderBy('name')->get();
        $commonFunctions->modelToSelectOptions($call_types,'name', 'id');
        $calltype_options = $commonFunctions->select_options;
        
        return view('pages.survey_report', [
            'dealers_options'           => $dealers_options,
            'calltype_options'          => $calltype_options,
        ]);
    }

    public function surveyReport(Request $request) {
        $coid               = $request->coid_search;
        $calltype_id        = $request->calltype_search;
        $start_date = date( 'Y-m-d', strtotime( $request->start_date_search ) ) . ' 00:00:00' ;
        $end_date   = date( 'Y-m-d', strtotime( $request->end_date_search ) ) . ' 23:59:59' ;

        $dealer = DB::connection('mysql2')
                ->table('callcenter_companies')
                ->select('co_name', 'co_short', 'coid')
                ->where('coid', $coid)
                ->first();

        $db_table = 'livecall_' . $dealer->co_short;

        $extra_where = "";
        $extra_where.= ($calltype_id != "all")? " AND livecall_prev_calltype = '$calltype_id' " : "";
        
        $questions = DB::connection('mysql2')
                ->table('callcenter_survey_questions')
                ->where('coid', $coid)
                ->where('cid', $calltype_id)
                ->get();
        
        if ( ! $questions ) {
            $questions = DB::connection('mysql2')
                ->table('callcenter_survey_questions')
                ->where('coid', 0)
                ->where('cid', $calltype_id)
                ->get();
        }

        $call_types_icons_db = DB::connection('mysql2')
            ->table('callcenter_calltypes')
            ->select('cid', 'calltype_text', 'icon')
            ->get();
    
        $call_types_icons = [];
        $call_types_texts = [];
        foreach ($call_types_icons_db as $key => $value) {
            $call_types_icons[$value->cid] = $value->icon;
            $call_types_texts[$value->cid] = $value->calltype_text;
        }

        $question_count = [];
        $question_texts = [];
        $qids = [];
        $i = 1;
        $question_number = 0;
        foreach($questions as $q) {
            $qids[] = $q->id;
            $question_texts[$q->id] = $q->question;
            $question_count[$q->id]["Yes"] = 0;
            $question_count[$q->id]["No"] = 0;
            $question_count[$q->id]["N/A"] = 0;
            $i++;
            $question_number++;
        }
        $qids = implode(",",$qids);
        
        $answers_sql = DB::connection('mysql2')
                ->table('callcenter_survey_answers')
                ->where('coid', $coid)
                ->where('answer_date', '>=', $start_date)
                ->where('answer_date', '<=', $end_date)
                ->whereRaw("FIND_IN_SET(qid,'$qids')")
                ->orderBy('answer_date', 'desc')
                ->get();

        $already_checked = [];
        $answers = [];
        foreach($answers_sql as $a) {
            $sid = $a->sid;
            $qid = $a->qid;
            if(!isset($already_checked[$sid."_".$qid])) {
                $already_checked[$sid."_".$qid] = true;
                $answer = $a->answer;
                $answers[$sid][$qid] = $answer;
                //$question_count[$qid][$answer]++;
            }
        }

        $livecalls = DB::connection('mysql2')
                ->table($db_table)
                ->whereRaw("lastupdate >= '$start_date' AND lastupdate <= '$end_date' AND (livecall_status = 'COMP' OR livecall_status = 'COMPEF') $extra_where")
                ->orderBy('lastupdate', 'desc')
                ->get();

        foreach($livecalls as $survey) {
            $sid = $survey->id;
            foreach($questions as $q) {
                if( ! empty( $answers[$sid][$q->id] ) ) {
                    if($answers[$sid][$q->id] == "Yes") {
                        $question_count[$q->id]["Yes"]++;
                    } else if($answers[$sid][$q->id] == "No") {
                        $question_count[$q->id]["No"]++;
                    }
                } 
            }
        }

        $i = 1;
        $grand_total_yes = 0;
        $grand_total_yes_percent = 0;
        $grand_total_no= 0;
        $grand_total_no_percent = 0;
        $total_of_questions = 0;
        $grand_total_na = 0;
        $grand_total_na_percent = 0;
        foreach($questions as $q) {
            $total_yes = $question_count[$q->id]["Yes"];
            $total_no = $question_count[$q->id]["No"];
            $total_na = $question_count[$q->id]["N/A"];
            $question_total = $total_yes+$total_no;
            $question_total = ($question_total==0)? 1 : $question_total;
            $percent_yes = ($total_yes * 100)/$question_total;
            $percent_no = ($total_no * 100)/$question_total;
            $percent_na = ($total_na * 100)/$question_total;
            $grand_total_yes+= $total_yes;
            $grand_total_yes_percent+= $percent_yes;
            $grand_total_no+= $total_no;
            $grand_total_na+= $total_na;
            $grand_total_no_percent+= $percent_no;
            $grand_total_na_percent+= $percent_na;
            $total_of_questions++;
            $i++;
        }

        if ( $request->type == 'excel' ) {
            $file_name = 'Survey_Report_' . str_replace(' ', '_', $dealer->co_name).'.xlsx';
            Excel::store(new SurverReportExport($coid, $livecalls, $questions, $question_texts, $answers, $question_count, $grand_total_yes, $grand_total_yes_percent, $grand_total_no, $grand_total_no_percent, $grand_total_na, $grand_total_na_percent), $file_name, 'temp_files');
            return $file_name;
        } else {
            return view('reports.survey_report', [
                'coid'                    => $coid,
                'livecalls'               => $livecalls,
                'questions'               => $questions,
                'question_texts'          => $question_texts,
                'answers'                 => $answers,
                'question_count'          => $question_count,
                'grand_total_yes'         => $grand_total_yes,
                'grand_total_yes_percent' => $grand_total_yes_percent,
                'grand_total_no'          => $grand_total_no,
                'grand_total_no_percent'  => $grand_total_no_percent,
                'grand_total_na'          => $grand_total_na,
                'grand_total_na_percent'  => $grand_total_na_percent,
            ]);
        }
    }

    public function callStatusReportPanel() {
        $dealers = DB::connection('mysql2')
            ->table('callcenter_companies')
            ->select('co_name', 'co_short', 'coid')
            ->where('is_active', 1)
            ->orderBy('co_name')
            ->get();

        $coid_list = [];
        foreach ($dealers as $dealer) {
            $coid_list[] = $dealer->coid;
        }
        
        $commonFunctions = new CommonFunctionsController();
        $commonFunctions->modelToSelectOptions($dealers,'co_name', 'coid');
        $dealers_options = $commonFunctions->select_options;

        $call_types       = CallType::orderBy('name')->get();
        $commonFunctions->modelToSelectOptions($call_types,'name', 'id');
        $calltype_options = $commonFunctions->select_options;
        
        return view('pages.call_status_report', [
            'dealers_options'           => $dealers_options,
            'calltype_options'          => $calltype_options,
        ]);
    }

    public function callStatusReport(Request $request) {
        $coid               = $request->coid_search;
        $calltype_id        = $request->calltype_search;
        $start_date = date( 'Y-m-d', strtotime( $request->start_date_search ) ) . ' 00:00:00' ;
        $end_date   = date( 'Y-m-d', strtotime( $request->end_date_search ) ) . ' 23:59:59' ;

        $dealer = DB::connection('mysql2')
                ->table('callcenter_companies')
                ->select('co_name', 'co_short', 'coid')
                ->where('coid', $coid)
                ->first();

        $db_table = 'livecall_' . $dealer->co_short;

        $extra_where = "";
        $extra_where.= ($calltype_id != "all")? " AND livecall_prev_calltype = '$calltype_id' " : "";

        

        $customer_statuses = DB::connection('mysql2')
                ->table('callcenter_dealertalerts')
                ->get();

        $customer_status_texts = [];
        foreach( $customer_statuses as $customer_status ) {
            $customer_status_texts[ $customer_status->id ] = $customer_status->type;
        }

        $livecalls = DB::connection('mysql2')
                ->table($db_table)
                ->selectRaw('customer_status, count(id) as status_total')
                ->whereRaw("lastupdate >= '$start_date' AND lastupdate <= '$end_date' $extra_where")
                ->groupBy('customer_status')
                ->orderBy('lastupdate', 'desc')
                ->get();

        return view('reports.call_status', [
            'coid'                  => $coid,
            'livecalls'             => $livecalls,
            'customer_status_texts' => $customer_status_texts,
        ]);
    }
    public function callStatusReportCsv(Request $request) {
        $coid               = $request->coid_search;
        $calltype_id        = $request->calltype_search;
        $customer_status    = $request->customer_status;
        $start_date = date( 'Y-m-d', strtotime( $request->start_date_search ) ) . ' 00:00:00' ;
        $end_date   = date( 'Y-m-d', strtotime( $request->end_date_search ) ) . ' 23:59:59' ;

        $dealer = DB::connection('mysql2')
                ->table('callcenter_companies')
                ->select('co_name', 'co_short', 'coid')
                ->where('coid', $coid)
                ->first();

        $db_table = 'livecall_' . $dealer->co_short;

        $extra_where = " AND customer_status = '$customer_status' ";
        $extra_where.= ($calltype_id != "all")? " AND livecall_prev_calltype = '$calltype_id' " : "";

        

        $customer_statuses = DB::connection('mysql2')
                ->table('callcenter_dealertalerts')
                ->get();

        $customer_status_texts = [];
        foreach( $customer_statuses as $customer_status ) {
            $customer_status_texts[ $customer_status->id ] = $customer_status->type;
        }


        $calltypes = CallType::all();
        $calltypes_names = [];
        foreach ($calltypes as $key => $value) {
            $calltypes_names[$value->id] = $value->name;
        }

        $dealer_dnc_table = "dnc_" . $dealer->co_short;
        $dealer_dne_table = "dne_" . $dealer->co_short;

        $callcenter_dnc_list = DB::connection('mysql2')
                    ->table('callcenter_dnc')
                    ->get();

        $callcenter_dnc_array = [];
        $i=0;
        foreach($callcenter_dnc_list as $phone) {
            $callcenter_dnc_array[$phone->phone] = $i++;
        }

        $dealer_dnc_list = DB::connection('mysql2')
                    ->table($dealer_dnc_table)
                    ->groupBy('phone')
                    ->get();

        $dealer_dnc_array = [];
        $i=0;
        foreach($dealer_dnc_list as $phone) {
            $dealer_dnc_array[$phone->phone] = $i++;
        }

        $dealer_dne_list = DB::connection('mysql2')
                    ->table($dealer_dne_table)
                    ->groupBy('email')
                    ->get();

        $dealer_dne_array = [];
        $i=0;
        foreach($dealer_dne_list as $email) {
            $dealer_dne_array[$email->email] = $i++;
        }


        $fields        = "cust_id,fname,lname,phone,email,livecall_prev_calltype,livecall_next_calltype,address1,city,state,zip,service_date,dateadded,RO,vehicle_year,vehicle_make,vehicle_model,vin,last_miles,salesrep,caller_comments,dealer_comments,customer_status,service_advisor";
        $headers       = "CustomerID,Fname,Lname,Phone,DNC,Email,DNE,PrevCallType,NextCallType,Address,City,State,Zip,ServiceDate,DateAdded,RO,Year,Make,Model,Vin,LastMiles,SalesRep,CallerComments,DealerComments,LastStatus,ServiceAdvisor";
        $fields_array  = explode(",", $fields);
        $headers_array = explode(",", $headers);
        $columns       = [];
        $index         = 0;
        $fields_fake   = explode(",", "cust_id,fname,lname,phone,fake_dnc,email,fake_dne,livecall_prev_calltype,livecall_next_calltype,address1,city,state,zip,service_date,dateadded,RO,vehicle_year,vehicle_make,vehicle_model,vin,last_miles,salesrep,caller_comments,dealer_comments,customer_status,service_advisor");
        foreach( $fields_fake as $field ) {
            $columns[ $field ] = $headers_array[ $index ];
            $index++;
        }

        $livecalls = DB::connection('mysql2')
                ->table($db_table)
                ->selectRaw($fields)
                ->whereRaw("lastupdate >= '$start_date' AND lastupdate <= '$end_date' $extra_where")
                ->orderBy('lastupdate', 'desc')
                ->get();

        $file_name = 'Call_Status_' . str_replace(' ', '_', $dealer->co_name).'.xlsx';
        Excel::store(new CallStatusExport($coid, $livecalls, $columns, $customer_status_texts, $calltypes_names, $callcenter_dnc_array, $dealer_dnc_list, $dealer_dne_list), $file_name, 'temp_files');
        return $file_name;
    }

    public function roiReportPanel() {
        $dealers = DB::connection('mysql2')
            ->table('callcenter_companies')
            ->select('co_name', 'co_short', 'coid')
            ->where('is_active', 1)
            ->orderBy('co_name')
            ->get();

        $coid_list = [];
        foreach ($dealers as $dealer) {
            $coid_list[] = $dealer->coid;
        }
        
        $commonFunctions = new CommonFunctionsController();
        $commonFunctions->modelToSelectOptions($dealers,'co_name', 'coid');
        $dealers_options = $commonFunctions->select_options;

        $call_types       = CallType::get();
        $commonFunctions->modelToSelectOptions($call_types,'name', 'id');
        $calltype_options = $commonFunctions->select_options;

        $customer_statuses_db = DB::connection('mysql2')
            ->table('callcenter_dealertalerts')
            ->select('id', 'type', )
            ->orderBy('type')
            ->get();

        $commonFunctions->modelToSelectOptions($customer_statuses_db,'type', 'id');
        $customer_statuses_options = $commonFunctions->select_options;
        
        return view('pages.roi_report_panel', [
            'dealers_options'           => $dealers_options,
            'calltype_options'          => $calltype_options,
            'customer_statuses_options' => $customer_statuses_options,
        ]);
    }

    public function roiReport(Request $request) {
        $coid                 = $request->coid_search;
        $calltype_for_search  = $request->calltype_for_search;
        $calltype_back_search = $request->calltype_back_search;
        $date_for_search      = $request->date_for_search;
        $date_back_search     = $request->date_back_search;
        $based_on_search      = $request->based_on_search;
        $amount_source_search = $request->amount_source_search;
        $customer_status_id   = $request->customer_status_search;
        $start_date           = date( "Y-m-d", strtotime( $request->start_date_search ) );
        $end_date             = date( "Y-m-d", strtotime( $request->end_date_search ) );
        $start_date_for       = $date_for_search == 'service_date' ? date( "m/d/Y", strtotime( $request->start_date_search ) ) : date( 'Y-m-d', strtotime( $request->start_date_search ) ) . ' 00:00:00' ;
        $end_date_for         = $date_for_search == 'service_date' ? date( "m/d/Y", strtotime( $request->end_date_search ) ) : date( 'Y-m-d', strtotime( $request->end_date_search ) ) . ' 23:59:59' ;
        $start_date_back      = $date_back_search == 'service_date' ? date( "m/d/Y", strtotime( $request->start_date_search ) ) : date( 'Y-m-d', strtotime( $request->start_date_search ) ) . ' 00:00:00' ;
        $end_date_back        = $date_back_search == 'service_date' ? date( "m/d/Y", strtotime( $request->end_date_search ) ) : date( 'Y-m-d', strtotime( $request->end_date_search ) ) . ' 23:59:59' ;
        $customer_status      = ($customer_status_id != "all")? " AND (customer_status = '$customer_status_id') " : "";
        $orderby              = ($date_for_search == "service_date")? " STR_TO_DATE(service_date,'%m/%d/%Y') " : $date_for_search;
        $main_content         = '';
        $result               = [];
        $calltype_content     = '';

        $call_types = DB::connection('mysql2')
            ->table('callcenter_calltypes')
            ->get();
        $call_types_data = [];
        foreach ($call_types as $call_type) {
            foreach ($call_type as $key => $value) {
                $call_types_data[$call_type->cid][$key] = $value;
            }            
        }

        $date_range_for = $date_for_search == 'service_date' ?
            "STR_TO_DATE(service_date,'%m/%d/%Y') >=  STR_TO_DATE('$start_date_for','%m/%d/%Y') AND STR_TO_DATE(service_date,'%m/%d/%Y') <=  STR_TO_DATE('$end_date_for','%m/%d/%Y')" :
            "$date_for_search >= '$start_date_for' AND $date_for_search <= '$end_date_for'";

        $date_range_back = $date_back_search == 'service_date' ?
            " STR_TO_DATE(service_date,'%m/%d/%Y') >=  STR_TO_DATE('$start_date_back','%m/%d/%Y') " :
            " $date_back_search >= '$start_date_back' ";

        $dealer = DB::connection('mysql2')
                ->table('callcenter_companies')
                ->select('co_name', 'co_short', 'coid')
                ->where('coid', $coid)
                ->first();

        $db_table = 'livecall_' . $dealer->co_short;

        $toyota_care_calls = DB::connection('mysql2')
            ->table($db_table)
            ->select($based_on_search, $date_for_search, 'livecall_next_calltype', 'livecall_prev_calltype', 'email', $amount_source_search)
            ->where(function($query) use($calltype_for_search) {
                $query->whereRaw("FIND_IN_SET(livecall_next_calltype, '".implode(',', $calltype_for_search)."')")
                ->orWhereRaw("FIND_IN_SET(livecall_prev_calltype, '".implode(',', $calltype_for_search)."')");
            })
            ->where(function($query) use($based_on_search){
                $query->where($based_on_search, '!=', '')
                ->orWhere('email', '!=', '');
            })
            ->whereRaw($date_range_for . $customer_status)
            ->orderByRaw($orderby . ' ASC')
            ->get();
    
        $find = [];
        $record_date = [];
        $record_sale_amount = [];
        $records_searched = 0;
        foreach ( $toyota_care_calls as $toyocall ) {
            $key_to_use = $based_on_search;
            if ( trim( $toyocall->{$based_on_search} ) == "" ) {
                $key_to_use = "email";
            }
            if ( ! isset( $find[ $toyocall->{$key_to_use} ] ) ) {
                $find[ $toyocall->{$key_to_use} ] = true;
                $record_date[ $toyocall->$key_to_use ] = $toyocall->{$date_for_search};
                $record_sale_amount[ $toyocall->{$key_to_use} ] = $toyocall->{$amount_source_search};
                
                $record_calltype[ $toyocall->{$key_to_use} ] = 
                ( $toyocall->livecall_next_calltype != 0 && $toyocall->livecall_next_calltype != '') ? 
                    $toyocall->livecall_next_calltype : 
                    $toyocall->livecall_prev_calltype;

                $records_searched++;
            }
        }
                
        unset($toyota_care_calls);

        // Coming from reminder emails.
        $toyota_care_calls = DB::connection('mysql2')
            ->table($db_table)
            ->selectRaw("$based_on_search, $date_for_search, livecall_next_calltype, livecall_prev_calltype, $amount_source_search, email_alerts_sent.type as reminder_alert_type, email_alerts_sent.date_sent as reminder_date_sent")
            ->join( 'email_alerts_sent', $db_table. '.id', '=', 'email_alerts_sent.cid'  )
            ->whereRaw("email_alerts_sent.date_sent >= '$start_date' AND email_alerts_sent.date_sent <= '$end_date'")
            ->whereRaw("email_alerts_sent.type = 10")
            ->whereRaw("email_alerts_sent.coid = '$coid'")
            ->whereRaw($date_range_for . $customer_status)
            ->orderByRaw('email_alerts_sent.date_sent ASC')
            ->get();

        $record_date_reminder       = [];
        $coming_from_email_reminder = [];
        foreach ( $toyota_care_calls as $toyocall ) {
            $key_to_use = $based_on_search;
            if ( trim( $toyocall->{$based_on_search} ) == "") {
                $key_to_use = "email";
            }
            if ( ! isset( $find[ $toyocall->{$key_to_use} ] ) ) {
                $find[ $toyocall->{$key_to_use} ] = true;
                $record_date[ $toyocall->{$key_to_use} ] = $toyocall->reminder_date_sent;
                $record_sale_amount[ $toyocall->{$key_to_use} ] = $toyocall->{$amount_source_search};
                $record_calltype[ $toyocall->{$key_to_use} ] = 3; 
                $records_searched++;
            }
            $coming_from_email_reminder[ $toyocall->{$key_to_use} ] = true;
            if( ! isset( $record_date_reminder[ $toyocall->{$key_to_use} ] ) )
                $record_date_reminder[ $toyocall->{$key_to_use} ] = $toyocall->reminder_date_sent;
        }
                
        unset($toyota_care_calls);

        $all_calls = DB::connection('mysql2')
            ->table($db_table)
            ->select('*')
            ->where(function($query) use($calltype_back_search) {
                $query->whereRaw("FIND_IN_SET(livecall_next_calltype, '".implode(',', $calltype_back_search)."')")
                ->orWhereRaw("FIND_IN_SET(livecall_prev_calltype, '".implode(',', $calltype_back_search)."')");
            })
            ->where(function($query) use($based_on_search){
                $query->where($based_on_search, '!=', '')
                ->orWhere('email', '!=', '');
            })
            ->whereRaw($date_range_back . $customer_status)
            ->orderByRaw($orderby . ' DESC')
            ->get();

        $already_found                    = [];
        $already_found_reminder           = [];
        $total_coming_from_email_reminder = 0;

        foreach ( $all_calls as $calls ) {
            $key_to_use = $based_on_search;
            if ( trim( $calls->{$based_on_search} ) == "" ) {
                $key_to_use = "email";
            }
            
            if ( $find[ $calls->{$key_to_use} ] == true ) {
                if ( strtotime( $record_date[ $calls->{$key_to_use} ] ) < strtotime( $calls->{$date_for_search} ) ) {
                    if ( ! isset( $already_found[ $calls->{$key_to_use} ] ) ) {
                        $result[] = $calls;
                    }
                    $already_found[ $calls->{$key_to_use} ] = true;
                }
                if ( isset( $coming_from_email_reminder[ $calls->{$key_to_use} ] ) ) {
                    if ( ! isset( $already_found_reminder[ $calls->{$key_to_use} ] ) ) {
                        if ( strtotime( $record_date_reminder[ $calls->{$key_to_use} ] ) < strtotime( $calls->{$date_for_search} ) ) {
                            $total_coming_from_email_reminder++;
                            $already_found_reminder[ $calls->{$key_to_use} ] = true;
                        }
                    }
                }
            }
        }
        
        unset($all_calls);

        if ( ( $date_for_search == 'service_date' ) ) {
            $today = date('m/d/Y');
            $daysbefore = 210;
            $daysago = date( 'm/d/Y', strtotime( '-'.$daysbefore.' days', strtotime( $today ) ) );
        } else {
            $today = date('Y-m-d');
            $daysbefore = 210;
            $daysago = date( 'Y-m-d', strtotime( '-'.$daysbefore.' days', strtotime( $today ) ) );
            $daysago.= ' 00:00:00';
        }

        $date_based_on_flag = ( $date_for_search == 'service_date' ) ? 
            " STR_TO_DATE(service_date, '%m/%d/%Y') >=  STR_TO_DATE('$daysago','%m/%d/%Y') " : 
        " $date_for_search >= '$daysago' ";    

        $previous_calls = DB::connection('mysql2')
            ->table($db_table)
            ->select('*')
            ->where(function($query) use($calltype_back_search) {
                $query->whereRaw("FIND_IN_SET(livecall_next_calltype, '".implode(',', $calltype_back_search)."')")
                ->orWhereRaw("FIND_IN_SET(livecall_prev_calltype, '".implode(',', $calltype_back_search)."')");
            })
            ->where(function($query) use($based_on_search){
                $query->where($based_on_search, '!=', '')
                ->orWhere('email', '!=', '');
            })
            ->whereRaw($date_based_on_flag)
            ->orderByRaw($orderby . ' DESC')
            ->get();

        $previous_calls_array = [];
        foreach ( $previous_calls as $previous_call ) {
            $key_to_use = $based_on_search;
            if ( trim( $previous_call->{$based_on_search} ) == "" ) {
                $key_to_use = "email";
            }
            if ( ! isset( $previous_calls_array[ $previous_call->{$key_to_use} ] ) ) {
                $previous_calls_array[ $previous_call->{$key_to_use} ] = [];
            }
            $previous_calls_array[ $previous_call->{$key_to_use} ][ $previous_call->id] = $previous_call->{$date_for_search};
        }

        unset( $previous_calls );

        $tc_calltype_count     = [];
        $six_months_count      = 0;
        $twelve_months_count   = 0;
        $total_more_than_five  = 0;
        $total_more_than_seven = 0;
        $sale_amount_total     = 0;

        foreach ( $result as $call ) {
            $id = $call->id;
            $fname = $call->fname;
            $lname = $call->lname;
            $customer_name = "$fname $lname";
            $date = date("m/d/Y", strtotime( $call->{$date_for_search} ) );
            $sale_date = ( $call->sale_date != '' && $call->sale_date != '0000-00-00' ) ? 
                date( 'm/d/Y', strtotime( $call->sale_date ) ) : '';
            $sale_date = ( strpos( $sale_date, "1969" ) === false) ? $sale_date : '';
            $service_date = ( $call->service_date != '' && $call->service_date != '0000-00-00') ? 
                date('m/d/Y', strtotime( $call->service_date ) ) : '';                                 
            $phone = preg_replace( "/(\d{3})(\d{3})(\d{4})/", "\\1-\\2-\\3", $call->phone );
            $vin = $call->vin;
            $ro = $call->RO;
            $sale_amount = $call->{$amount_source_search};
            $sale_amount_total+= $sale_amount;
            $calltype = ( $call->livecall_next_calltype != 0 && $call->livecall_next_calltype != '' ) ? 
                $call->livecall_next_calltype : $call->livecall_prev_calltype;
            
            $lastupdate = $call->lastupdate;
            $lastupdate = ( $lastupdate == "" || $lastupdate == '0000-00-00 00:00:00' ) ? '' : date( "m/d/Y" , strtotime( $lastupdate ) );

            $ctypetext = $call_types_data[$calltype]["calltype_text"];
            $ctypeicon = $call_types_data[$calltype]["icon"];
            $call_type_icon = ($ctypeicon != "")? '<img src="https://login.fullbdc.com/images/'.$ctypeicon.'" title="'.$ctypetext.'">' : $ctypetext;
            
            $tc_calltype = ! empty($record_calltype[ $call->{$based_on_search} ]) ? $record_calltype[ $call->{$based_on_search} ] : 0;
            if ( ! isset( $tc_calltype_count[ $tc_calltype ] ) ) {
                $tc_calltype_count[ $tc_calltype ] = 1;
            } else {
                $tc_calltype_count[ $tc_calltype ]+= 1;
            }

            if ( empty( $tc_calltype_sale_amount[ $tc_calltype ] ) ) {
                $tc_calltype_sale_amount[ $tc_calltype ] = 0;
            }
            $tc_calltype_sale_amount[ $tc_calltype ]+= $sale_amount;
            $tc_ctypetext = ! empty ($call_types_data[$tc_calltype]["calltype_text"] ) ? $call_types_data[$tc_calltype]["calltype_text"] : '';
            $tc_ctypeicon = ! empty( $call_types_data[$tc_calltype]["icon"] ) ? $call_types_data[$tc_calltype]["icon"] : '';
            $tc_call_type_icon = ($tc_ctypeicon != "")? '<img src="https://login.fullbdc.com/images/'.$tc_ctypeicon.'" title="'.$tc_ctypetext.'">' : $tc_ctypetext;
            
            $key_to_use = $based_on_search;
            if ( trim( $call->{$based_on_search} ) == '' ) {
                $key_to_use = "email";
            }
            
            if ( ! isset( $previous_calls_array[ $call->{$key_to_use} ] ) ) {
                $date_diff_style = 'background-color:#F7615E';
                $total_more_than_seven++;            
            } else {
                $date_to_test_1 = '';
                $primary_id_already_found = false;
                $compare_date2 = date( 'Y-m-d', strtotime( $call->{$date_for_search} ) );
                foreach ( $previous_calls_array[ $call->{$key_to_use} ] as $previous_call_id => $previous_call_date ) {
                    if ( $previous_call_id == $id ) {
                        $primary_id_already_found = true;
                    }
                    if ( $previous_call_id != $id && $primary_id_already_found == true ) {
                        $compare_date1 = date( 'Y-m-d', strtotime( $previous_call_date ) );
                        
                        if ( $compare_date1 != $compare_date2 ) {
                            $date_to_test_1 = $compare_date1;
                            break;
                        }
                    }
                }

                if ( $date_to_test_1 == '' ) {
                    $date_diff_style = 'background-color:#F7615E';
                    $total_more_than_seven++;      
                } else {
                    $date_to_test_2 = $compare_date2;
                    $datetime1 = new DateTime( $date_to_test_1 );
                    $datetime2 = new DateTime( $date_to_test_2 );
                    $difference = $datetime1->diff( $datetime2 );
                    if ( $difference->days >= 150 ) {
                        $date_diff_style = 'background-color:#F89F4A';
                        $total_more_than_five++;
                    } else {
                        $date_diff_style='';
                    }       
                }
            }
            
            $main_content.='<tr style="'.$date_diff_style.'">
                                <td>
                                    '.$customer_name.'
                                </td>
                                <td>
                                    '.$phone.'
                                </td>
                                <td>
                                    '.$vin.'
                                </td>
                                <td>
                                    '.$service_date.'
                                </td>
                                <td>
                                    '.$sale_date.'
                                </td>
                                <td>
                                    '.$sale_amount.'
                                </td>
                                <td>
                                    '.$ro.'
                                </td>
                                <td>
                                    '.$tc_call_type_icon.' '.$call_type_icon.'
                                </td>
                                <td>
                                    <button class="btn btn-primary btn-sm" onclick="showCallDialog(event, \''.$id.'\', \''.$coid.'\',\'call_records\', \'\', 0)"><i class="fas fa-search"></i></button>
                                </td>
                            </tr>';
        
        }

        $results_total = count( $result );
        if ( $records_searched != 0 ) {
            $result_percent = ( $results_total * 100 ) / $records_searched;
        } else {
            $result_percent = 0;
        }
        
        $result_percent = number_format($result_percent,2);
        
        $results_label = "<strong>".$results_total. "</strong> records found of <strong>".$records_searched."</strong> total (".$result_percent."%)";

        $tc_calltype_count_new = [];
        foreach($tc_calltype_count as $key=>$value) {
            $tc_label = ! empty( $call_types_data[$key]["calltype_text"] ) ? $call_types_data[$key]["calltype_text"] : '';
            $tc_calltype_count_new[$tc_label] = $value;
            $tc_calltype_sale_amount_new[$tc_label] = $tc_calltype_sale_amount[$key];
        }
        
        ksort($tc_calltype_count_new);

        foreach($tc_calltype_count_new as $key=>$value) {
            if ( empty( $key ) ) {
                continue;
            }
            $tc_label = $key;
            $tc_sale_amount = number_format($tc_calltype_sale_amount_new[$tc_label],2);
            $tc_sale_amount_no_format = $tc_calltype_sale_amount_new[$tc_label];
            $ct_sale_amount_avg = $tc_sale_amount_no_format/$value;
            $calltype_content.= '   
                                <tr>
                                    <td class="usertablerows14px">
                                        <strong>'.$tc_label.':</strong>
                                    </td>
                                    <td class="usertablerows14pxcenter">
                                        '.$value.'
                                    </td>
                                    <td class="usertablerows14pxright">
                                        $'.$tc_sale_amount.'
                                    </td>
                                    <td class="usertablerows14pxright">
                                        $'.number_format( $ct_sale_amount_avg, 2 ).'
                                    </td>
                                </tr>';
        }


    $startdate_label = $request->start_date_search;
    $enddate_label   = $request->end_date_search;
    
    $avg_sale_amount = $sale_amount_total/$results_total;
    
    $header_content = ' <table class="table table-striped" style="width:60%; margin-left:20%; margin-right:20%;">
                            <tr>
                                <td>
                                    <strong>Result:</strong> '.$results_label.'
                                </td>
                            </tr>
                        </table>
                        </table>
                        <table class="table table-striped" style="width:60%; margin-left:20%; margin-right:20%;">
                            <tr>
                                <td>
                                    Type
                                </td>
                                <td>
                                    Records
                                </td>
                                <td>
                                    Sale Amount
                                </td>
                                <td>
                                    Avg
                                </td>
                            </tr>
                            '.$calltype_content.'
                            <tr>
                                <td colspan="3">
                                    <strong>Total Avg RO:</strong>
                                </td>
                                <td>
                                    $' . number_format( $avg_sale_amount, 2 ) . '
                                </td>
                            </tr>
                            <tr>
                                <td colspan="3">
                                    <strong>Total Sale Amount:</strong>
                                </td>
                                <td>
                                    $' . number_format( $sale_amount_total, 2 ) . '
                                </td>
                            </tr>
                        </table>
                        <br>
                        <table class="table table-striped" style="width:60%; margin-left:20%; margin-right:20%;">
                            <tr>
                                <td colspan="3">
                                    Customers who have not been in within 7 or 5 months prior to the new visit
                                </td>
                            </tr>
                            <tr>
                                <td>
                                    7 months
                                </td>
                                <td>
                                    '.$total_more_than_seven.'
                                </td>
                                <td style="background-color:#F7615E">
                                    &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;
                                </td>
                            </tr>
                            <tr>
                                <td>
                                    5 months
                                </td>
                                <td>
                                    '.$total_more_than_five.'
                                </td>
                                <td style="background-color:#F89F4A">
                                    &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;
                                </td>
                            </tr>
                        </table>
                        <br>
                        <table class="table table-striped" style="width:60%; margin-left:20%; margin-right:20%;">
                            <tr>
                                <td>
                                    <strong>Came back from reminder emails:</strong>
                                </td>
                                <td class="usertablerows14px">
                                    '.$total_coming_from_email_reminder.'
                                </td>
                            </tr>
                        </table>';

        return view('reports.roi_report', [
            'main_content'   => $main_content,
            'header_content' => $header_content,
        ]);

    }

    public function callTimeByCallerReportPanel() {
        $dealers = DB::connection('mysql2')
            ->table('callcenter_companies')
            ->select('co_name', 'co_short', 'coid')
            ->where('is_active', 1)
            ->orderBy('co_name')
            ->get();

        $coid_list = [];
        foreach ($dealers as $dealer) {
            $coid_list[] = $dealer->coid;
        }
        
        $commonFunctions = new CommonFunctionsController();
        $commonFunctions->modelToSelectOptions($dealers,'co_name', 'coid');
        $dealers_options = $commonFunctions->select_options;

        $call_types       = CallType::orderBy('name')->get();
        $commonFunctions->modelToSelectOptions($call_types,'name', 'id');
        $calltype_options = $commonFunctions->select_options;
        
        return view('pages.call_time_by_caller_panel', [
            'dealers_options'           => $dealers_options,
            'calltype_options'          => $calltype_options,
        ]);
    }

    public function callTimeByCallerReport(Request $request) {
        $coid       = $request->coid_search;
        $calltype   = $request->calltype_search;
        $start_date = date( 'Y-m-d', strtotime( $request->start_date_search ) ) . ' 00:00:00' ;
        $end_date   = date( 'Y-m-d', strtotime( $request->end_date_search ) ) . ' 23:59:59' ;
        $commonFunctions = new CommonFunctionsController();

        if ($coid == "all" && $calltype == "all") {
            $time_list = DB::connection('mysql4')
                ->table('employee_assignment')
                ->selectRaw('employee_uid, SUM(time_spent) as time_spent, COUNT(employee_uid) as cnt')
                ->where('assignment_date', '>=', $start_date)
                ->where('assignment_date', '<=', $end_date)
                ->where('time_spent', '>', 0)
                ->groupBy('employee_uid')
                ->get();
        } else if ($coid != "all" && $calltype == "all") {
            $time_list = DB::connection('mysql4')
                ->table('employee_assignment')
                ->selectRaw('employee_uid, SUM(time_spent) as time_spent, COUNT(employee_uid) as cnt')
                ->where('companyid', $coid)
                ->where('assignment_date', '>=', $start_date)
                ->where('assignment_date', '<=', $end_date)
                ->where('time_spent', '>', 0)
                ->groupBy('employee_uid')
                ->get();
        } else if ($coid != "all" && $calltype != "all") {
            $time_list = DB::connection('mysql4')
                ->table('employee_assignment')
                ->selectRaw('employee_uid, SUM(time_spent) as time_spent, COUNT(employee_uid) as cnt')
                ->where('companyid', $coid)
                ->where('ctype', $calltype)
                ->where('assignment_date', '>=', $start_date)
                ->where('assignment_date', '<=', $end_date)
                ->where('time_spent', '>', 0)
                ->groupBy('employee_uid')
                ->get();
        } else if ($coid == "all" && $calltype != "all") {
            $time_list = DB::connection('mysql4')
                ->table('employee_assignment')
                ->selectRaw('employee_uid, SUM(time_spent) as time_spent, COUNT(employee_uid) as cnt')
                ->where('ctype', $calltype)
                ->where('assignment_date', '>=', $start_date)
                ->where('assignment_date', '<=', $end_date)
                ->where('time_spent', '>', 0)
                ->groupBy('employee_uid')
                ->get();
        }

        $employees = DB::connection('mysql4')
                ->table('employees')
                ->get();
    
        foreach ( $employees as $employee ) {
            foreach ( $employee as $key => $value ) {
                $employees_data[ $employee->uid ][ $key ] = $value;
            }
        }
        $callers_name = [];
        $caller_data  = [];
        
        if ( $time_list ) {
            foreach($time_list as $time) {
                $employee_uid                                 = $time->employee_uid;
                $seconds                                      = $time->time_spent;
                $seconds                                      = $commonFunctions->seconds_to_hours($seconds);
                $callers_name[ $employee_uid ]                = $employees_data[ $employee_uid ]["first_name"] . " " . $employees_data[ $employee_uid ]["last_name"];
                $caller_data[ $employee_uid ]["calling_time"] = $seconds["hours"].":".$seconds["minutes"].":".$seconds["seconds"];
                
                $hoursresult = DB::connection('mysql4')
                    ->table('employee_timetrack')
                    ->where('employee_uid', $employee_uid)
                    ->where('start_time', '>=', $start_date)
                    ->where('start_time', '<=', $end_date)
                    ->whereNotNull('stop_time')
                    ->get();
    
                $employeeTotal = 0;
    
                foreach ( $hoursresult as $hoursrow ) {
                    $checkValue = strtotime( $hoursrow->stop_time ) - strtotime( $hoursrow->start_time );
                    if ( $checkValue > 86400 ) continue;
                    $employeeTotal += ( strtotime( $hoursrow->stop_time ) - strtotime( $hoursrow->start_time ) );
                }
                $totalHours   = $employeeTotal / 3600;
                $singleHours  = floor( $totalHours );
                $hourFraction = ( $totalHours - $singleHours ) * 100;
                
                if ( $hourFraction > 0 && $hourFraction <= 15 ) {
                    $hourFraction = 0;
                } elseif ( $hourFraction > 15 && $hourFraction <= 35 ) {
                    $hourFraction = 0.25;
                } elseif ( $hourFraction > 35 && $hourFraction <= 65 ) {
                    $hourFraction = 0.50;
                } elseif ( $hourFraction > 65 && $hourFraction <= 85 ) {
                    $hourFraction = 0.75;
                } elseif ( $hourFraction > 85 ) {
                    $hourFraction = 1;
                }
    
                $totalHours = $singleHours + $hourFraction;
                $caller_data[ $employee_uid ]["clocked_in_time"] = $totalHours;
                $payrate_cost = number_format($totalHours * $employees_data[$employee_uid]["payrate"], 2, ".", "");
                $caller_data[ $employee_uid ]["caller_cost"] = $payrate_cost;
                $caller_data[ $employee_uid ]["payrate"] = number_format( $employees_data[$employee_uid]["payrate"], 2, ".", "");
                $caller_data[ $employee_uid ]["assigments_count"] = $time->cnt;
                $caller_data[ $employee_uid ]["avg_calls_per_hour"] = $totalHours > 0 ? number_format($time->cnt / $totalHours, 2, ".", "") : 0;
                $caller_data[ $employee_uid ]["uid"] = $employee_uid;
    
            }
        }
        asort ( $callers_name );
        
        return view('reports.call_time_by_caller', [
            'caller_data'  => $caller_data,
            'callers_name' => $callers_name,
            'coid'         => $coid,
            'calltype'     => $calltype,
        ]);

    }

    public function recordsImportedPanel() {
        $dealers = DB::connection('mysql2')
            ->table('callcenter_companies')
            ->select('co_name', 'co_short', 'coid')
            ->where('is_active', 1)
            ->orderBy('co_name')
            ->get();

        $coid_list = [];
        foreach ($dealers as $dealer) {
            $coid_list[] = $dealer->coid;
        }
        
        $commonFunctions = new CommonFunctionsController();
        $commonFunctions->modelToSelectOptions($dealers,'co_name', 'coid');
        $dealers_options = $commonFunctions->select_options;

        $call_types       = CallType::orderBy('name')->get();
        $commonFunctions->modelToSelectOptions($call_types,'name', 'id');
        $calltype_options = $commonFunctions->select_options;
        
        return view('pages.records_imported_panel', [
            'dealers_options'  => $dealers_options,
            'calltype_options' => $calltype_options,
        ]);
    }

    public function recordsImported(Request $request) {
        $coid        = $request->coid_search;
        $calltype_id = is_array( $request->calltype_search ) ? $request->calltype_search : (array) $request->calltype_search ;
        $date_search = date( 'Y-m-d', strtotime( $request->date_search ) );

        $extra_where = ($calltype_id != ["all"]) ? 
            " AND (FIND_IN_SET(livecall_next_calltype,'".implode(',', $calltype_id)."') OR FIND_IN_SET(livecall_prev_calltype,'".implode(',', $calltype_id)."')) " : 
            "";

        $dealer = DB::connection('mysql2')
            ->table('callcenter_companies')
            ->select('co_name', 'co_short', 'coid')
            ->where('coid', $coid)
            ->first();

        $db_table = 'livecall_' . $dealer->co_short;

        $records = DB::connection('mysql2')
            ->table($db_table)
            ->whereRaw("dateadded >= '$date_search 00:00:00' AND dateadded <= '$date_search 23:59:59' $extra_where")
            ->orderBy('dateadded', 'desc')
            ->get();

        $extra_where = ($calltype_id != "all") ? 
            " AND FIND_IN_SET(calltype,'".implode(',', $calltype_id)."') " : 
            "";

        $import_logs = DB::connection('mysql2')
            ->table('callcenter_import_log')
            ->where('import_time', '>=', "$date_search 00:00:00")
            ->where('import_time', '<=', "$date_search 23:59:59")
            ->whereRaw(" campaign = '$dealer->co_short'" . $extra_where)
            ->orderBy('import_time', 'desc')
            ->get();

        $customer_status_icons_db = DB::connection('mysql2')
            ->table('callcenter_dealertalerts')
            ->select('id', 'type', 'icon')
            ->get();
    
        $customer_status_icons = [];
        $customer_status_texts = [];
        foreach ($customer_status_icons_db as $key => $value) {
            $customer_status_icons[$value->id] = $value->icon;
            $customer_status_texts[$value->id] = $value->type;
        }

        $call_types_icons_db = DB::connection('mysql2')
            ->table('callcenter_calltypes')
            ->select('cid', 'calltype_text', 'icon')
            ->get();
    
        $call_types_icons = [];
        $call_types_texts = [];
        foreach ($call_types_icons_db as $key => $value) {
            $call_types_icons[$value->cid] = $value->icon;
            $call_types_texts[$value->cid] = $value->calltype_text;
        }

        return view('reports.records_imported', [
            'coid'                  => $coid,
            'records'               => $records,
            'import_logs'           => $import_logs,
            'customer_status_icons' => $customer_status_icons,
            'customer_status_texts' => $customer_status_texts,
            'call_types_icons'      => $call_types_icons,
            'call_types_texts'      => $call_types_texts,
        ]);
    }

    public function deleteImportRecords(Request $request) {
        $coid = $request->deleted_coid;

        $dealer = DB::connection('mysql2')
            ->table('callcenter_companies')
            ->select('co_name', 'co_short', 'coid')
            ->where('coid', $coid)
            ->first();
        
        $dbtable = "livecall_".$dealer->co_short;
        
        if ( ! empty( $request->log_list ) ) {
            foreach($request->log_list as $log) {
                DB::connection('mysql2')
                    ->table('callcenter_import_log')
                    ->where('id', $log)
                    ->delete();
            }
        }
        
        if ( ! empty( $request->record_list ) ) {
            foreach($request->record_list as $record) {
                DB::connection('mysql2')
                    ->table($dbtable)
                    ->where('id', $record)
                    ->delete();
            }
        }
        
        return response()->json([
            'message' => 'success'
        ]);
    }

    public function connectRatePanel() {
        $dealers = DB::connection('mysql2')
            ->table('callcenter_companies')
            ->select('co_name', 'co_short', 'coid')
            ->where('is_active', 1)
            ->orderBy('co_name')
            ->get();

        $coid_list = [];
        foreach ($dealers as $dealer) {
            $coid_list[] = $dealer->coid;
        }
        
        $commonFunctions = new CommonFunctionsController();
        $commonFunctions->modelToSelectOptions($dealers,'co_name', 'coid');
        $dealers_options = $commonFunctions->select_options;
        
        return view('pages.connect_rate_panel', [
            'dealers_options'  => $dealers_options,
        ]);
    }

    public function connectRate(Request $request) {

        $coid = $request->coid_search;
        $dealer = DB::connection('mysql2')
            ->table('callcenter_companies')
            ->select('co_name', 'co_short', 'coid')
            ->where('coid', $coid)
            ->first();

        $db_table = 'livecall_' . $dealer->co_short;
        $co_name = $dealer->co_name;
        
        $totalDaysArray = [];
        $start_hour = 9;
        $end_hour = 21;
        
        $current_best_day = 0;
        $current_worst_day = 0;
        $current_max_interval = 0;
        $current_lower_interval = 1000000;
        $current_max_day = 0;
        $current_lower_day = 1000000;
        $current_max_pt = 0;
        $current_lower_pt = 101;
        
        for ($day=2;$day<=7;$day++) {
            $totalDaysArray[$day] = [];

            $dayHours = DB::connection('mysql2')
                ->table($db_table)
                ->selectRaw('HOUR(lastupdate) as hour_interv, COUNT(id) as total')
                ->whereRaw("(livecall_status = 'COMP' OR livecall_status = 'COMPEF') AND DAYOFWEEK(lastupdate) = $day")
                ->groupByRaw('HOUR(lastupdate)')
                ->get();

            $dayTotal = DB::connection('mysql2')
                ->table($db_table)
                ->selectRaw('COUNT(id) as total')
                ->whereRaw("livecall_status != '' AND lastupdate != '00-00-0000 00:00:00' AND DAYOFWEEK(lastupdate) = $day ")
                ->first();
            
            
            $dayTotalConnect = 0;
            $dayArray = [];
            foreach($dayHours as $hours) {
                $hour = $hours->hour_interv;
                $hour_count = $hours->total;
                if ($hour >= $start_hour && $hour <= $end_hour) {
                    $dayArray[$hour] = $hour_count;
                }
                $dayTotalConnect+= $hour_count;
            }
            
            $current_max = 0;
            $current_lower = 1000000;
            $interval = "";
            for ($i=$start_hour;$i<=$end_hour;$i++) {
                $dayOneCount = ( ! empty($dayArray[$i]) ) ? $dayArray[$i] : 0;
                $dayTwoCount = ( ! empty($dayArray[$i+1]) ) ? $dayArray[$i+1] : 0;
                $interval_sum = $dayOneCount + $dayTwoCount;
                if ($current_max < $interval_sum) {
                    $current_max = $interval_sum;
                    $hour_label_start = ($i>12)? $i - 12 : $i;
                    $hour_label_end = (($i+2)>12)? $i - 10 : $i+1;
                    
                    $interval_start = ($i>=12)? $hour_label_start.":00 p.m." : $hour_label_start.":00 a.m.";
                    $interval_end = (($i+2)>=12)? $hour_label_end.":00 p.m." : $hour_label_end.":00 a.m.";
                    $max_interval = $interval_start." - ".$interval_end;
                }
                
                if ($current_lower > $interval_sum && $interval_sum > 0) {
                    $current_lower = $interval_sum;
                    $hour_label_start = ($i>12)? $i - 12 : $i;
                    $hour_label_end = (($i+2)>12)? $i - 10 : $i+1;
                    
                    $interval_start = ($i>=12)? $hour_label_start.":00 p.m." : $hour_label_start.":00 a.m.";
                    $interval_end = (($i+2)>=12)? $hour_label_end.":00 p.m." : $hour_label_end.":00 a.m.";
                    $lower_interval = $interval_start." - ".$interval_end;
                }
                
                if ($current_max_interval < $interval_sum) {
                    $current_max_interval = $interval_sum;
                    $current_best_day_interval = $day;
                    $hour_label_start = ($i>12)? $i - 12 : $i;
                    $hour_label_end = (($i+2)>12)? $i - 10 : $i+1;
                    
                    $global_interval_start = ($i>=12)? $hour_label_start.":00 p.m." : $hour_label_start.":00 a.m.";
                    $global_interval_end = (($i+2)>=12)? $hour_label_end.":00 p.m." : $hour_label_end.":00 a.m.";
                    $global_max_interval = $global_interval_start." - ".$global_interval_end;
                }
                
                if ($current_lower_interval > $interval_sum) {
                    $current_lower_interval = $interval_sum;
                    $current_worst_day_interval = $day;        
                    $hour_label_start = ($i>12)? $i - 12 : $i;
                    $hour_label_end = (($i+2)>12)? $i - 10 : $i+1;
            
                    $global_interval_start = ($i>=12)? $hour_label_start.":00 p.m." : $hour_label_start.":00 a.m.";
                    $global_interval_end = (($i+2)>=12)? $hour_label_end.":00 p.m." : $hour_label_end.":00 a.m.";
                    $global_lower_interval = $global_interval_start." - ".$global_interval_end;
                }  
            }
            
            if ($current_max_day < $dayTotal->total) {
                $current_max_day = $dayTotal->total;
                //$current_best_day = $day;
            }
            if ($current_lower_day > $dayTotal->total) {
                $current_lower_day = $dayTotal->total;
                //$current_worst_day = $day;
            }
            
            $current_max_day_pt = ($dayTotalConnect * 100) / $dayTotal->total;
            if ($current_max_day_pt > $current_max_pt) {
                $current_max_pt = $current_max_day_pt;
                $current_best_day = $day;
            }
            $current_lower_day_pt = ($dayTotalConnect * 100) / $dayTotal->total;
            if ($current_lower_day_pt < $current_lower_pt) {
                $current_lower_pt = $current_lower_day_pt;
                $current_worst_day = $day;
            }
            
            $totalDaysArray[$day]["total"] = $dayTotal->total;
            $totalDaysArray[$day]["total_connect"] = $dayTotalConnect;
            $totalDaysArray[$day]["max_interval"] = $max_interval;
            $totalDaysArray[$day]["lower_interval"] = $lower_interval;
            $totalDaysArray[$day]["max_interval_count"] = $current_max;
            $totalDaysArray[$day]["lower_interval_count"] = $current_lower;  
        }
        
        $day_label = [];
        $day_label[1] = "Sunday";
        $day_label[2] = "Monday";
        $day_label[3] = "Tuesday";
        $day_label[4] = "Wednesday";
        $day_label[5] = "Thursday";
        $day_label[6] = "Friday";
        $day_label[7] = "Saturday";
        
        $best_day = $day_label[$current_best_day];
        $best_day_perc = ($totalDaysArray[$current_best_day]["total_connect"] * 100) / $totalDaysArray[$current_best_day]["total"];
        $best_time = $totalDaysArray[$current_best_day]["max_interval"];
        $best_time_count = $totalDaysArray[$current_best_day]["max_interval_count"];
        $best_time_perc = ($totalDaysArray[$current_best_day]["max_interval_count"] * 100) / $totalDaysArray[$current_best_day]["total"];
        $worst_day = $day_label[$current_worst_day];
        $worst_day_perc = ($totalDaysArray[$current_worst_day]["total_connect"] * 100) / $totalDaysArray[$current_worst_day]["total"];
        $worst_time = $totalDaysArray[$current_worst_day]["lower_interval"];
        $worst_time_count = $totalDaysArray[$current_worst_day]["lower_interval_count"];
        $worst_time_perc = ($totalDaysArray[$current_worst_day]["lower_interval_count"] * 100) / $totalDaysArray[$current_worst_day]["total"];
        
        return view('reports.connect_rate', [
            'co_name'  => $co_name,
            'best_day'  => $best_day,
            'day_label'  => $day_label,
            'best_day_perc'  => $best_day_perc,
            'best_time'  => $best_time,
            'best_time_count'  => $best_time_count,
            'best_time_perc'  => $best_time_perc,
            'worst_day'  => $worst_day,
            'worst_day_perc'  => $worst_day_perc,
            'worst_time'  => $worst_time,
            'worst_time_count'  => $worst_time_count,
            'worst_time_perc'  => $worst_time_perc,
            'totalDaysArray'  => $totalDaysArray,
        ]);
    }
}