<?php

namespace App\Http\Controllers;

use App\ScrubbedFtpFile;
use App\FtpCredential;
use Illuminate\Support\Facades\DB;
use DateTime;
use DateTimeZone;

class ScrubbFtpFileCronController extends Controller
{
    public function scrubbFtpFiles()
    {
        $max_transactions = 5;
        $local_time = new DateTime();
        $local_time->setTimezone(new DateTimeZone('America/New_York'));
        $today = $local_time->format('Y-m-d');
        $today_seconds = $local_time->format('Y-m-d H:i:s');
        $miliseconds_diff = time() - strtotime($today_seconds);
        $five_days_ago = $local_time->modify( '-5 day' )->format('Y-m-d');

        $companies = DB::connection('mysql2')->table('callcenter_companies')
                                ->select('coid', 'co_name', 'co_short')
                                ->where('is_active', 1)
                                ->orderBy('co_name')
                                ->get();

        $ftp_credentials = FtpCredential::all();

        $scrubbed_files = ScrubbedFtpFile::where('last_uploaded', '>=', $five_days_ago.' 00:00:00')
                                            ->where('last_uploaded', '<=', $today.' 23:59:59')
                                            ->get();
        $scrubbed_files_time = [];

        foreach ($scrubbed_files as $key => $value) {
            $scrubbed_files_time[$value->coid][$value->file_name] = strtotime($value->last_uploaded);
        }

        $ftp_data = [];
        foreach ($ftp_credentials as $key => $value) 
        {
            $ftp_data[$value->coid]['coid'] = $value->coid;
            $ftp_data[$value->coid]['host'] = $value->host;
            $ftp_data[$value->coid]['username'] = $value->username;
            $ftp_data[$value->coid]['password'] = $value->password;
            $ftp_data[$value->coid]['delimiter'] = $value->delimiter;
        }

        foreach ($companies as $key => $company) 
        {

            $ftp_path = public_path('callcenteruploads') . "/" . $company->co_short;
            if (!file_exists($ftp_path)) {
                mkdir($ftp_path, 0755, true);
            }
            if(isset($ftp_data[$company->coid]['coid']))
            {

                $ftp_server = $ftp_data[$company->coid]["host"];
                $ftp_username = $ftp_data[$company->coid]["username"];
                $ftp_password = $ftp_data[$company->coid]["password"];
                $delimiter = $ftp_data[$company->coid]["delimiter"];
                $id_ftp = ftp_connect($ftp_server); 
                ftp_login($id_ftp,$ftp_username,$ftp_password); 
                $files = ftp_nlist($id_ftp, ".");
                sort($files);
                
                $two_days_ago = strtotime($five_days_ago);
                foreach($files as $f) {
                    if($max_transactions <= 0)
                    {
                        return false;
                    }
                    $file_time = filemtime(public_path('callcenteruploads') . "/" . $company->co_short."/".$f) - $miliseconds_diff;
                    //if(isset($scrubbed_files_time[$company->coid][$f]))
                        //print($f."last_time: ".$scrubbed_files_time[$company->coid][$f]." - file_time".$file_time."<br>");
                    if($file_time >= $two_days_ago)
                    {
                        $f = str_replace('./', '', $f);
                        if(trim($f) != '.' && trim($f) != '.ftpquota')
                        {
                            if(strpos($f, "_scrubbed.csv") === false)
                            {
                                $scrub_file = true;
                                if(isset($scrubbed_files_time[$company->coid][$f]))
                                {
                                    if($scrubbed_files_time[$company->coid][$f] > $file_time)
                                        $scrub_file = false;
                                } 
                                if($scrub_file == true)
                                {
                                    
                                    try {
                                        $request = new \stdClass();
                                        $request->coid = $company->coid;
                                        $request->filename_ftp = $f;
                                        $request->is_single_file = 1;
                                        $this->applyFiltersToTempFileFTP($request); 
                                        $max_transactions-= 1;
                                    } catch (Throwable $e) {
                                        report($e);
                                        continue;
                                    }
                                }
                            }
                        }
                    }
                }
            } 
        }

        return response()->json([
            'message' => 'Done!',
        ]);
    }

    public function applyFiltersToTempFileFTP($request) 
    { 
        $coid = $request->coid;
        $orig_file_name  = str_replace('./', '', $request->filename_ftp);
        $temp_file_name = str_replace('./', '', $request->filename_ftp);
        $company = DB::connection('mysql2')
                        ->table('callcenter_companies')
                        ->where('coid', $request->coid)
                        ->first();

        $ftp_file_templates = DB::connection('mysql2')
                                ->table('callcenter_ftp_files_templates')
                                ->where(function($query) use($request){
                                    $query->where('ftp_file_name', $request->filename_ftp)
                                        ->orWhere('ftp_file_name', str_replace('./', '', $request->filename_ftp));
                                })
                                ->where('coid', $request->coid)
                                ->first();

        if(!$ftp_file_templates) return false;

        $co_short = $company->co_short;
        $dnc_table = "callcenter_dnc";
        $dealer_dnc_table = "dnc_".$co_short;
        $dealer_dne_table = "dne_".$co_short;
        
        $ftp_file_template = $ftp_file_templates->ftp_file_template;
        $sortby_column = $ftp_file_templates->sortby_column;
        $template_json = json_decode($ftp_file_template,true);
        $added_columns_default_values = json_decode($ftp_file_templates->added_columns_default_values, true);
        $active_columns = explode(",", $ftp_file_templates->active_columns);
        $added_columns = explode(",", $ftp_file_templates->added_columns);
        $file_delimiter = $ftp_file_templates->file_delimiter;
        
        $require_match_column_names = $ftp_file_templates->require_match_column_names;
        $match_column_names = $ftp_file_templates->match_column_names;
        
        $file_delimiter = (trim($file_delimiter)!="")? $file_delimiter : ",";
        $filters_array = array();
        $filters_order_array = array();
        $columns_array = array();
        $filters_params = array();
        $filters_params_2 = array();
        $filters_params_3 = array();
        $phones_columns = array();
        $email_columns = array();
        for($i=0;$i<count($template_json["filter_order"]);$i++) {
            $filters_order_array[$template_json["filter_order"][$i]] = $template_json["filter_order"][$i];
            $filters_array[$template_json["filter_order"][$i]] = $template_json["filter_type"][$i];
            $columns_array[$template_json["filter_order"][$i]] = $template_json["column"][$i];
            $filters_params[$template_json["filter_order"][$i]] = $template_json["params"][$i];
            $filters_params_2[$template_json["filter_order"][$i]] = (isset($template_json["params2"][$i]))? $template_json["params2"][$i] : '';
            $filters_params_3[$template_json["filter_order"][$i]] = (isset($template_json["params3"][$i]))? $template_json["params3"][$i] : '';
            if($template_json["filter_type"][$i] == "phone_column") {
                $phones_columns[] = $template_json["column"][$i];
            }
            if($template_json["filter_type"][$i] == "email_column") {
                $email_columns[] = $template_json["column"][$i];
            }
            
        }


        $added_columns_default_values_array = array();
        for($i=0;$i<count($added_columns_default_values["name"]);$i++) {
            $added_columns_default_values_array[] = $added_columns_default_values["value"][$i];
        }


        //sort($filters_order_array);


        //$upload_dir = 'callcenter_temp_files/';

        $local_file = public_path('callcenteruploads') . "/" . $co_short . "/" .$temp_file_name;
        $gestor = fopen($local_file, 'r');
        $new_file_array = array();
        $first_row = true;
        $first_row_ac = true;


        $sortered_values = array();
        $data_full = array();
        if($sortby_column !== "none" && $sortby_column !== "") {
            while (($row = fgetcsv($gestor,0,$file_delimiter)) !== FALSE) {
                $sortered_values[] = $row[$sortby_column];
                $data_full[] = $row;
            }
            array_multisort($sortered_values, SORT_DESC , $data_full);
        } else {
            while (($row = fgetcsv($gestor,0,$file_delimiter)) !== FALSE) {
                $data_full[] = $row;
            }
        }


        //while (($datos = fgetcsv($gestor)) !== FALSE) {
        
        $calltype_column = false;
        $department_column = false;
        $checked_duplicated_values = array();
        
        $has_one_instance_of_filter = false;
        $has_one_instance_of = false;
        
        $first_row_matching_columns = true;

        $useless_phones_list = DB::connection('mysql2')
                                ->table('callcenter_useless_phones')
                                ->where('phone', '!=', '')
                                ->get();


        $useless_phones_array = array();
        foreach($useless_phones_list as $upl) {
            $useless_phones_array[] = $upl->phone;
        }
        
        foreach ($data_full as $datos) {
            
            if($first_row_matching_columns == true && $require_match_column_names ==  1) {
                //$match_column_names_to_test = implode(",",$datos);
                $match_column_names_array = explode(",",$match_column_names);
                    for($xyz = 0;$xyz<count($match_column_names_array);$xyz++) {
                        
                        
                        $column_to_test_1 = preg_replace('/(?:^[^\p{L}\p{N}]+|[^\p{L}\p{N}]+$)/u', '', strtoupper(trim($match_column_names_array[$xyz])));
                        $column_to_test_2 = preg_replace('/(?:^[^\p{L}\p{N}]+|[^\p{L}\p{N}]+$)/u', '', strtoupper(trim($datos[$xyz])));
                        if($column_to_test_1 != $column_to_test_2) {
                            return response()->json([
                                'message' => "The File did not pass the Matching Column Test! \n".$column_to_test_1." is not equal to ".$column_to_test_2,
                            ]);
                        }
                    }
            }
            $first_row_matching_columns = false;
            //$num = count($datos);
            $insert = true;
            $show_to_call = 1;
            $current_duplicated_value = '';
            $has_duplicated_value_filter = false;
            $calltype_column_value_already_set = false;
            $department_column_value_already_set = false;
            
            $splitted_column = false;
            $splitted_column_array = array();
            
            foreach($filters_order_array as $filter_order) {
                $i = $filter_order;
                
                //$filter_order = $filters_order_array[$i];
                $filter_type = $filters_array[$i];
                $filtered_colum = $columns_array[$i];
                $filter_param = $filters_params[$i];
                $filter_param_2 = $filters_params_2[$i];
                $filter_param_3 = $filters_params_3[$i];
                //$filter_type = $filters_array[$filters_order_array[$i]];
                //$filtered_colum = $columns_array[$filters_order_array[$i]];
                //$filter_param = $filters_params[$filters_order_array[$i]];
                if( ! isset( $datos[ $filtered_colum ] ) ) { // It means the file does not have that column number.
                    continue;
                }
                if($filter_type == "empty") {
                    if(trim($datos[$filtered_colum]) == "") {
                        $insert = false;
                        //break;
                    }
                } else if($filter_type == "exclude_equals_to") {
                    if(strtolower(trim($datos[$filtered_colum])) == strtolower(trim($filter_param))) {
                        $insert = false;
                        //break;
                    }
                } else if($filter_type == "exclude_less_than") {
                    $splitted_test_value = explode(' ', trim( $datos[$filtered_colum] ) );
                    $value_to_test = (int)$splitted_test_value[0] * 1;
                    $value_to_test_against = (int)$filter_param * 1;
                    if($value_to_test < $value_to_test_against) {
                        $insert = false;
                        //break;
                    }
                } else if($filter_type == "exclude_greater_than") {
                    $splitted_test_value = explode(' ', trim( $datos[$filtered_colum] ) );
                    $value_to_test = (int)$splitted_test_value[0] * 1;
                    $value_to_test_against = (int)$filter_param * 1;
                    
                    if($value_to_test > $value_to_test_against) {
                        $insert = false;
                        //break;
                    }
                     /* if($filtered_colum == 19) {
                        if($insert == true)
                        {
                            print('true <br>'); 
                        } else {
                            print('false <br>'); 
                        }
                     } */
                        
                } else if($filter_type == "nstc_less_than") {
                    $value_to_test = (int)$datos[$filtered_colum] * 1;
                    $value_to_test_against = $filter_param * 1;
                    if($value_to_test < $value_to_test_against) {
                        $show_to_call = 0;
                        //break;
                    }
                } else if($filter_type == "nstc_greater_than") {
                    $value_to_test = (int)$datos[$filtered_colum] * 1;
                    $value_to_test_against = $filter_param * 1;
                    if($value_to_test > $value_to_test_against) {
                        $show_to_call = 0;
                        //break;
                    }
                } else if($filter_type == "exclude_if_contains") {
                    if(strpos(strtolower(trim($datos[$filtered_colum])),strtolower(trim($filter_param))) !== false) {
                        $insert = false;
                        //break;
                    }
                } else if($filter_type == "exclude_duplicated") {
                    $has_duplicated_value_filter = true;
                    if(!isset($duplicated_values[$filtered_colum])) {
                        $duplicated_values[$filtered_colum] = [];
                    }
                    $current_duplicated_value = $datos[$filtered_colum];
                    if(!isset($checked_duplicated_values[$filtered_colum][$datos[$filtered_colum]])) {
                        $checked_duplicated_values[$filtered_colum][$datos[$filtered_colum]] = 1;
                    } else {
                        $checked_duplicated_values[$filtered_colum][$datos[$filtered_colum]] = 2;
                    }
                    if(in_array(strtolower(trim($datos[$filtered_colum])),$duplicated_values[$filtered_colum])) {
                        if($checked_duplicated_values[$filtered_colum][$datos[$filtered_colum]] == 2) {
                            $insert = false;
                        }
                        //break;
                    } else {
                        if($insert == true)
                        {
                            $duplicated_values[$filtered_colum][] = strtolower(trim($datos[$filtered_colum]));
                        }
                    }
                } else if($filter_type == "upload_nstc_if_equals_to") {
                    $filter_param_array = explode(",",strtolower(trim($filter_param)));
                    if(in_array(strtolower(trim($datos[$filtered_colum])),$filter_param_array)) {
                        $show_to_call = 0;
                    }
                } else if($filter_type == "upload_nstc_if_not_equals_to") {
                    $filter_param_array = explode(",",strtolower(trim($filter_param)));
                    if(!in_array(strtolower(trim($datos[$filtered_colum])),$filter_param_array)) {
                        $show_to_call = 0;
                    }
                } else if($filter_type == "replace_with") {
                    if(strtolower(trim($datos[$filtered_colum])) == strtolower(trim($filter_param))) {
                        $datos[$filtered_colum] = $filter_param_2;
                    }
                } else if($filter_type == "set_calltype_if_equals_to") {
                    $calltype_column = true;
                    if(strtolower(trim($datos[$filtered_colum])) == strtolower(trim($filter_param))) {
                        //$datos[$filtered_colum] = $filter_param_2;
                        $calltype_column_value = $filter_param_2;
                        $calltype_column_value_already_set = true;
                    } else if($calltype_column_value_already_set == false){
                        $calltype_column_value = "";
                    }
                } else if($filter_type == "set_department_if_equals_to") {
                    $department_column = true;
                    if(strtolower(trim($datos[$filtered_colum])) == strtolower(trim($filter_param))) {
                        //$datos[$filtered_colum] = $filter_param_2;
                        $department_column_value = $filter_param_2;
                        $department_column_value_already_set = true;
                    } else if($department_column_value_already_set == false){
                        $department_column_value = "";
                    }
                } else if($filter_type == "find_replace") {
                    $datos[$filtered_colum] = str_replace($filter_param,$filter_param_2,$datos[$filtered_colum]);
                } else if($filter_type == "replace_with_multiple") {
                    $filter_param1_array = explode(",",strtolower(trim($filter_param)));
                    $filter_param2_array = explode(",",trim($filter_param_2));
                    $param_array = array();
                    for($fi = 0; $fi<count($filter_param1_array); $fi++) {
                        $param_array[$filter_param1_array[$fi]] = $filter_param2_array[$fi];
                    }
                    if(isset($param_array[strtolower(trim($datos[$filtered_colum]))])) {
                        $datos[$filtered_colum] = $param_array[strtolower(trim($datos[$filtered_colum]))];
                    }
                } else if($filter_type == "exclude_if_not_in_date_range") {
                    if(trim($filter_param) != "")
                    {
                        $start_date = strtotime(trim($filter_param));
                    } else {
                        $start_date = '';
                    }

                    if(trim($filter_param_2) != "")
                    {
                        $end_date = strtotime(trim($filter_param_2));
                    } else {
                        $end_date = '';
                    }

                    $test_date = strtotime(trim($datos[$filtered_colum]));

                    if($start_date != '' && $end_date != '')
                    {
                        if($test_date < $start_date || $test_date > $end_date) {
                            $insert = false;
                            //break;
                        }
                    } else if($start_date != '') {
                        if($test_date < $start_date) {
                            $insert = false;
                            //break;
                        }
                    } else if($end_date != '') {
                        if($test_date > $end_date) {
                            $insert = false;
                            //break;
                        }
                    } else {
                        $insert = false;
                    }
                    
                } else if($filter_type == "nstc_by_date_range") {
                    if(trim($filter_param) != "")
                    {
                        $start_date = strtotime(trim($filter_param));
                    } else {
                        $start_date = '';
                    }

                    if(trim($filter_param_2) != "")
                    {
                        $end_date = strtotime(trim($filter_param_2));
                    } else {
                        $end_date = '';
                    }

                    $test_date = strtotime(trim($datos[$filtered_colum]));

                    if($start_date != '' && $end_date != '')
                    {
                        if($test_date < $start_date || $test_date > $end_date) {
                            $show_to_call = 0;
                            //break;
                        }
                    } else if($start_date != '') {
                        if($test_date < $start_date) {
                            $show_to_call = 0;
                            //break;
                        }
                    } else if($end_date != '') {
                        if($test_date > $end_date) {
                            $show_to_call = 0;
                            //break;
                        }
                    } else {
                        $show_to_call = 0;
                    }
                    
                } else if($filter_type == "set_calltype_by_date_range") {
                    $calltype_column = true;
                    if(trim($filter_param) != "")
                    {
                        $start_date = strtotime(trim($filter_param));
                    } else {
                        $start_date = '';
                    }

                    if(trim($filter_param_2) != "")
                    {
                        $end_date = strtotime(trim($filter_param_2));
                    } else {
                        $end_date = '';
                    }

                    $test_date = strtotime(trim($datos[$filtered_colum]));

                    if($start_date != '' && $end_date != '')
                    {
                        if($test_date >= $start_date && $test_date <= $end_date) {
                            
                            $calltype_column_value = $filter_param_3;
                            $calltype_column_value_already_set = true;
                        } else if($calltype_column_value_already_set == false) {
                            $calltype_column_value = "";
                        }
                    } else if($start_date != '') {
                        if($test_date >= $start_date) {
                            $calltype_column_value = $filter_param_3;
                            $calltype_column_value_already_set = true;
                        } else if($calltype_column_value_already_set == false) {
                            $calltype_column_value = "";
                        }
                    } else if($end_date != '') {
                        if($test_date <= $end_date) {
                            $calltype_column_value = $filter_param_3;
                            $calltype_column_value_already_set = true;
                        } else if($calltype_column_value_already_set == false) {
                            $calltype_column_value = "";
                        }
                    } else if($calltype_column_value_already_set == false) {
                        $calltype_column_value = "";
                    }
                    
                } else if($filter_type == "remove_if_older_than_x_days") {
                    $start_date = strtotime(trim($datos[$filtered_colum]));
                    $test_date = strtotime(date('Y-m-d', strtotime("-".trim($filter_param)." days")));
                    if($start_date < $test_date) {
                        $insert = false;
                        //break;
                    }
                } else if($filter_type == "remove_phone_if_empty") {
                    $notags_phone = preg_replace("/[^0-9]/i", "", trim($datos[$filtered_colum]));
                    $test_phone = substr($notags_phone, 3);
                    if($test_phone == "0000000" || $test_phone == "") {
                        $datos[$filtered_colum] = "";
                    }
                } else if($filter_type == "exclude_if_does_not_include") {
                    /*$has_one_of_them = false;
                    $filter_param1_array = explode(",",strtolower(trim($filter_param)));
                    for($fi = 0; $fi<count($filter_param1_array); $fi++) {
                        //$datos[$filtered_colum] = $datos[$filtered_colum];
                        //if(strpos())
                        $original_value = preg_replace('/\s+/', ' ',strtolower(trim($datos[$filtered_colum])));
                        $test_value = preg_replace('/\s+/', ' ',strtolower(trim($filter_param1_array[$fi])));
                        if($original_value == $test_value) {
                            $has_one_of_them = true;
                        }
                    }

                    if($has_one_of_them != true) {
                        $insert = false;
                    }*/
                    $has_one_of_them = false;
                    $filter_param1_array = explode(",",strtolower(trim($filter_param)));
                    for($fi = 0; $fi<count($filter_param1_array); $fi++) {
                        if(strpos(strtolower(trim($datos[$filtered_colum])),strtolower(trim($filter_param1_array[$fi]))) !== false) {
                            $has_one_of_them = true;

                        }
                    }
                    if($has_one_of_them == false) {
                        $insert = false;
                    }


                } else if($filter_type == "exclude_if_does_not_include_tab") {
                    /*$has_one_of_them = false;
                    $filter_param1_array = explode(",",strtolower(trim($filter_param)));
                    for($fi = 0; $fi<count($filter_param1_array); $fi++) {
                        //$datos[$filtered_colum] = $datos[$filtered_colum];
                        //if(strpos())
                        $original_value = preg_replace('/\s+/', ' ',strtolower(trim($datos[$filtered_colum])));
                        $test_value = preg_replace('/\s+/', ' ',strtolower(trim($filter_param1_array[$fi])));
                        if($original_value == $test_value) {
                            $has_one_of_them = true;
                        }
                    }

                    if($has_one_of_them != true) {
                        $insert = false;
                    }*/
                    $has_one_of_them = false;
                    $filter_param1_array = explode(",",strtolower($filter_param));
                    for($fi = 0; $fi<count($filter_param1_array); $fi++) {
                        if(strpos(strtolower($datos[$filtered_colum]),strtolower($filter_param1_array[$fi])) !== false) {
                            $has_one_of_them = true;

                        }
                    }
                    if($has_one_of_them == false) {
                        $insert = false;
                    }
                } else if($filter_type == "exclude_if_begins_with_codes") {
                    $has_one_of_them = false;
                    $filter_param1_array = explode(",",strtolower(trim($filter_param)));
                    $codes_array = explode(" ",strtolower(trim($datos[$filtered_colum])));
                    for($fi = 0; $fi<count($filter_param1_array); $fi++) {
                        for($fx = 0; $fx<count($codes_array); $fx++) {
                            if(substr(trim($codes_array[$fx]), 0, strlen($filter_param1_array[$fi])) === $filter_param1_array[$fi]) {
                                $has_one_of_them = true;
                            }
                        }
                    }
                    if($has_one_of_them == true) {
                        $insert = false;
                    }
                } else if($filter_type == "replace_if_contains") {
                    $filter_param1_array = explode(",",strtolower(trim($filter_param)));
                    $filter_param2_array = explode(",",trim($filter_param_2));
                    for($fi = 0; $fi<count($filter_param1_array); $fi++) {
                        if(strpos(strtolower(trim($datos[$filtered_colum])),strtolower(trim($filter_param1_array[$fi]))) !== false) {
                            $datos[$filtered_colum] = $filter_param2_array[$fi];
                        }
                    }
                } else if($filter_type == "upload_nstc_if_contains") {
                    if(strpos(strtolower(trim($datos[$filtered_colum])),strtolower(trim($filter_param))) !== false) {
                        $show_to_call = 0;
                    }
                } else if($filter_type == "exclude_if_year_less_than") {
                    if((int)$datos[$filtered_colum] * 1 > 0) {
                        if(strlen($datos[$filtered_colum]) > 2) {
                            try {
                                $dt = DateTime::createFromFormat('Y', $datos[$filtered_colum]);
                                $current_year = $dt->format('Y');
                            } catch (Exception $e) {
                                $errorlog = 'Exception: '.  $e->getMessage(). "\n";
                            }
                        } else {
                            try {
                                $dt = DateTime::createFromFormat('y', $datos[$filtered_colum]);
                                $current_year = $dt->format('Y');
                            } catch (Exception $e) {
                                $errorlog = 'Exception: '.  $e->getMessage(). "\n";
                            }
                        }
                        
                        if(strlen($filter_param) > 2) {
                            try {
                                $dt = DateTime::createFromFormat('Y', $filter_param);
                                $filter_param = $dt->format('Y');
                            } catch (Exception $e) {
                                $errorlog = 'Exception: '.  $e->getMessage(). "\n";
                            }
                        } else {
                            try {
                                $dt = DateTime::createFromFormat('y', $filter_param);
                                $filter_param = $dt->format('Y');
                            } catch (Exception $e) {
                                $errorlog = 'Exception: '.  $e->getMessage(). "\n";
                            }
                        }

                        if($filter_param > $current_year) {
                            $show_to_call = 0;
                        }
                    }

                } else if($filter_type == "split_in_two_columns") {
                    $splitter = (trim($filter_param)=="")? ' ' : $filter_param;
                    //$datos[$filtered_colum] = $filter_param2_array[$fi];
                    $splitted_values = explode($splitter,trim($datos[$filtered_colum]));
                    $datos[$filtered_colum] = $splitted_values[0];
                    $splitted_column_value = "";
                    for($sv = 1;$sv < count($splitted_values); $sv++) {
                        if($sv == (count($splitted_values)-1)) {
                            $splitted_column_value.= $splitted_values[$sv];
                        } else {
                            $splitted_column_value.= $splitted_values[$sv].$splitter;
                        }
                        
                    }
                    $splitted_column = true;
                    
                    $splitted_column_array[] = $splitted_column_value;
                } else if($filter_type == "reverse_value") {
                    $splitter = (trim($filter_param)=="")? ' ' : $filter_param;
                    $splitted_values = explode($splitter,trim($datos[$filtered_colum]));
                    $splitted_column_value = "";
                    for($sv = 0;$sv < (count($splitted_values)-1); $sv++) {
                        if($sv == (count($splitted_values)-2)) {
                            $splitted_column_value.= $splitted_values[$sv];
                        } else {
                            $splitted_column_value.= $splitted_values[$sv].$splitter;
                        }
                    }
                    $datos[$filtered_colum] = $splitted_values[count($splitted_values)-1].$splitter.$splitted_column_value;
                } else if($filter_type == "has_negative_numbers") {
                    if(strpos($datos[$filtered_colum],"(") !== false && strpos($datos[$filtered_colum],")") !== false) {
                        $datos[$filtered_colum] = str_replace("(","",$datos[$filtered_colum]);
                        $datos[$filtered_colum] = str_replace(")","",$datos[$filtered_colum]);
                        $datos[$filtered_colum] = str_replace("$","",$datos[$filtered_colum]);
                        $datos[$filtered_colum] = $datos[$filtered_colum] * 1;
                        if($datos[$filtered_colum] >= 0) {
                           $datos[$filtered_colum] = $datos[$filtered_colum] * (-1); 
                        }
                    }
                } else if($filter_type == "file_has_to_contain_instance_of") {
                    $has_one_instance_of_filter = true;
                    $filter_param_array = explode(",",strtolower(trim($filter_param)));
                    if(in_array(strtolower(trim($datos[$filtered_colum])),$filter_param_array)) {
                        $has_one_instance_of = true;
                    }
                } else if($filter_type == "add_to_gdnc_if_equals_to") {
                    
                    if(strpos(strtolower(trim($datos[$filtered_colum])),strtolower(trim($filter_param))) !== false) {
                        foreach($phones_columns as $phones_col) {
                            $phone = $datos[$phones_col];
                            $phone = preg_replace("/[^0-9]/i", "", trim($phone));
                            $phone = $phone*1;
                            if($phone>0) {
                                $timestamp = date("Y-m-d H:i:s");
                                $global_dnc = DB::connection('mysql2')
                                                ->table($dnc_table)
                                                ->where('phone', $phone)
                                                ->first();
                                
                                if(!$global_dnc) {
                                    $dnc_insert = DB::connection('mysql2')
                                                                    ->table($dnc_table)
                                                                    ->insertGetId([
                                                                        'phone' => $phone,
                                                                        'added_by' => 'Admin',
                                                                        'timestamp' => $timestamp,
                                                                        'reason' => 'Added from FTP file filter',
                                                                    ]);
                                }
                            }
                        }
                    }
                } else if($filter_type == "add_to_ddnc_if_equals_to") {
                    if(strpos(strtolower(trim($datos[$filtered_colum])),strtolower(trim($filter_param))) !== false) {
                        foreach($phones_columns as $phones_col) {
                            $phone = $datos[$phones_col];
                            $phone = preg_replace("/[^0-9]/i", "", trim($phone));
                            $phone = (int) $phone * 1;
                            if($phone>0) {
                                $timestamp = date("Y-m-d H:i:s");
                                $dealer_dnc = DB::connection('mysql2')
                                                ->table($dealer_dnc_table)
                                                ->where('phone', $phone)
                                                ->first();
                                if(!$dealer_dnc) {
                                    $dealer_dnc_insert = DB::connection('mysql2')
                                                                    ->table($dealer_dnc_table)
                                                                    ->insertGetId([
                                                                        'phone' => $phone,
                                                                        'added_by' => 'Admin',
                                                                        'timestamp' => $timestamp,
                                                                        'reason' => 'Added from FTP file filter',
                                                                    ]);
                                }
                            }
                        }
                    }
                } else if($filter_type == "add_to_gdnc_ddnc_if_equals_to") {
                    if(strpos(strtolower(trim($datos[$filtered_colum])),strtolower(trim($filter_param))) !== false) {
                        foreach($phones_columns as $phones_col) {
                            $phone = $datos[$phones_col];
                            $phone = preg_replace("/[^0-9]/i", "", trim($phone));
                            $phone = $phone*1;
                            if($phone>0) {
                                $timestamp = date("Y-m-d H:i:s");
                                $global_dnc = DB::connection('mysql2')
                                                ->table($dnc_table)
                                                ->where('phone', $phone)
                                                ->first();
                                if(!$global_dnc) {
                                    $dnc_insert = DB::connection('mysql2')
                                                                    ->table($dnc_table)
                                                                    ->insertGetId([
                                                                        'phone' => $phone,
                                                                        'added_by' => 'Admin',
                                                                        'timestamp' => $timestamp,
                                                                        'reason' => 'Added from FTP file filter',
                                                                    ]);
                                }
                                
                                $dealer_dnc = DB::connection('mysql2')
                                                ->table($dealer_dnc_table)
                                                ->where('phone', $phone)
                                                ->first();
                                if(!$dealer_dnc) {
                                    $dealer_dnc_insert = DB::connection('mysql2')
                                                                    ->table($dealer_dnc_table)
                                                                    ->insertGetId([
                                                                        'phone' => $phone,
                                                                        'added_by' => 'Admin',
                                                                        'timestamp' => $timestamp,
                                                                        'reason' => 'Added from FTP file filter',
                                                                    ]);
                                }
                            }
                        }
                    }
                } else if($filter_type == "add_to_dne_if_equals_to") {
                    
                    if(strpos(strtolower(trim($datos[$filtered_colum])),strtolower(trim($filter_param))) !== false) {
                        foreach($email_columns as $email_col) {
                            $email = trim($datos[$email_col]);
                            if($email!="") {
                                $timestamp = date("Y-m-d H:i:s");
                                $dealer_dne = DB::connection('mysql2')
                                                ->table($dealer_dne_table)
                                                ->where('email', $email)
                                                ->first();
                                if(!$dealer_dne) {
                                    $dealer_dne_insert = DB::connection('mysql2')
                                                                    ->table($dealer_dne_table)
                                                                    ->insertGetId([
                                                                        'email' => $email,
                                                                        'added_by' => 'Admin',
                                                                        'timestamp' => $timestamp,
                                                                        'reason' => 'Added from FTP file filter',
                                                                    ]);
                                }
                            }
                        }
                    }
                } else if($filter_type == "extract_date") {
                    $date_array = date_parse($datos[$filtered_colum]);
                    $year = $date_array["year"];
                    $month = $date_array["month"];
                    $day = $date_array["day"];
                    $datos[$filtered_colum] = $month."/".$day."/".$year;
                } else if($filter_type == "zip_code_column") {
                    if(strlen($datos[$filtered_colum])<5) {
                        $current_zip_num = strlen($datos[$filtered_colum]);
                        while($current_zip_num < 5) {
                            $datos[$filtered_colum] = "0".$datos[$filtered_colum];
                            $current_zip_num++;
                        }
                    }
                } else if($filter_type == "change_to_w_c_code") {
                    $has_one_of_them = false;
                    $codes_array = explode(" ",strtolower(trim($datos[$filtered_colum])));
                    for($fx = 0; $fx<count($codes_array); $fx++) {
                        if(strtolower(substr(trim($codes_array[$fx]), 0, 1)) === "w") {
                            $has_one_of_them = true;
                        }
                    }
                    if($has_one_of_them == true) {
                        $datos[$filtered_colum] = "W";
                    } else {
                        $datos[$filtered_colum] = "C";
                    }
                } else if($filter_type == "remove_useless_phone_number") {
                    if(in_array(trim($datos[$filtered_colum]),$useless_phones_array)) {
                        $datos[$filtered_colum] = "";
                    }
                }
            }
            if($insert == false) {
                continue;
            }
            if($insert == true) {
                
                /*if($has_duplicated_value_filter) {
                    $checked_duplicated_values[$filtered_colum][$current_duplicated_value] = true;
                }*/
                
                $new_row_array = array();
                if($first_row == true) {
                    //$show_to_call = "Show To Call";
                    $first_row = false;
                }
                $new_row_array[] = $show_to_call;
                foreach($active_columns as $column) {
                    if( ! isset( $datos[ $column ] ) ) { // It means the file does not have that column number.
                        continue;
                    }
                    $new_row_array[] = trim($datos[$column]);
                }
                $first_row_ac = false;
                if($first_row_ac == true) {
                    foreach($added_columns as $add_column) {
                        $new_row_array[] = ucwords(strtolower($add_column));
                    }
                    $first_row_ac = false;
                } else {
                    $z = 0;
                    foreach($added_columns as $add_column) {
                        if(isset($added_columns_default_values_array[$z]))
                        {
                            $new_row_array[] = $added_columns_default_values_array[$z];
                            $z++;
                        }
                    }
                }
                
                
                /**
     * if($splitted_column == true) {
     *                 $new_row_array[] = $splitted_column_value;
     *             }
     */
                
                foreach($splitted_column_array as $splitted_column_array_value) {
                    $new_row_array[] = $splitted_column_array_value;
                }
                
                if($calltype_column == true) {
                    $new_row_array[] = $calltype_column_value;
                }
                
                if($department_column == true) {
                    $new_row_array[] = $department_column_value;
                }

                $new_row_array[] = 'stc_flag_'.$show_to_call; //This is the flag to prevent the change of NSTC when reviewing file.

                $new_file_array[] = $new_row_array;
            }
        }
        fclose($gestor);
        
        if($has_one_instance_of_filter == true && $has_one_instance_of == false) {
            return response()->json([
                'message' => 'The "File has to contain one instance of" filter flag did not pass the test!'
            ]);
        }
        
        $new_orig_file_name = str_replace(".csv","", $orig_file_name).'_scrubbed.csv';
        $output_file =  public_path('callcenteruploads') . "/" . $co_short . "/" .$new_orig_file_name;
        $fp = fopen($output_file, 'w');
        foreach($new_file_array as $fields) {
            fputcsv($fp, $fields);
        }
        
        fclose($fp);
        
        $today_date = date("Y-m-d H:i:s");
        $uploaded_file_insert = DB::connection('mysql2')
                                    ->table('callcenter_ftp_uploaded_files')
                                    ->insertGetId([
                                        'filename' => $orig_file_name,
                                        'coid' => $coid,
                                        'date_uploaded' => $today_date,
                                    ]);


        $local_time = new DateTime();
        $local_time->setTimezone(new DateTimeZone('America/New_York'));
        $ftp_scrubbed_last_uploaded = $local_time->format('Y-m-d H:i:s');
        $scrubbed_file = ScrubbedFtpFile::where('coid', $coid)
                                            ->where('file_name', $orig_file_name)
                                            ->first();

        if($scrubbed_file)
        {
            ScrubbedFtpFile::where('coid', $coid)
                            ->where('file_name', $orig_file_name)
                            ->update([
                                        'last_uploaded' => $ftp_scrubbed_last_uploaded
                                    ]);
        } else {
            $scrubbed_ftp_file = new ScrubbedFtpFile;
            $scrubbed_ftp_file->coid = $coid;
            $scrubbed_ftp_file->file_name = $orig_file_name;
            $scrubbed_ftp_file->last_uploaded = $ftp_scrubbed_last_uploaded;
            $scrubbed_ftp_file->save();
        }

        $today_date_to_show = date("m/d/Y H:i:s");
        return response()->json([
            'message' => 'Done!',
            'new_file_name' => $new_orig_file_name,
        ]);
    }

}
