<?php

namespace App\Http\Controllers;

use App\User;
use App\TrainingAlert;
use App\UserTrainingAlert;
use App\UserGroup;
use App\TrainingAlertUserGroup;
use App\TrainingAlertFileType;
use Illuminate\Http\Request;
use App\Mail\GeneralEmailAlert;
use Illuminate\Support\Facades\Mail;
use Illuminate\Support\Facades\Auth;

class TrainingAlertController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index(Request $request)
    {   
        $training_alerts = new \stdClass();
        if(isset($request->redirected))
        {
            $redirected =  1;
        } else {
            $redirected = 0;
        }
        if(Auth::user()->user_type->name == 'Admin') 
        {
            $training_alerts = TrainingAlert::paginate(20);
            $approved_training_alerts = array();
        } else if(Auth::user()->user_type->name == 'Full BDC Caller' || Auth::user()->user_type->name == 'Inbound Caller') {
            //if(count($user_groups_array) > 0)
            //{
                /*$training_alerts = TrainingAlert::selectRaw('training_alerts.*')
                                                  ->join('training_alert_user_groups', 'training_alerts.id', '=', 'training_alert_user_groups.training_alert_id')
                                                  ->where('training_alerts.active', 1)
                                                  ->whereRaw("FIND_IN_SET(training_alert_user_groups.user_group_id, '".implode(',', $user_groups_array)."')")
                                                  ->groupBy('training_alerts.id')
                                                  ->paginate(20);*/
            //}

            $training_alerts = TrainingAlert::select(
                                                      'training_alerts.*', 
                                                      'user_training_alerts.created_at as date_approved', 
                                                      'training_alerts.created_at as training_alert_created_at',
                                                      'user_user_groups.created_at as user_user_group_created_at'
                                                    )
                                                  ->join('training_alert_user_groups', 'training_alerts.id', '=', 'training_alert_user_groups.training_alert_id')
                                                  ->join('user_user_groups', 'training_alert_user_groups.user_group_id', '=', 'user_user_groups.user_group_id')
                                                  ->leftJoin('user_training_alerts', 'training_alerts.id', '=', 'user_training_alerts.training_alert_id')
                                                  ->where('training_alerts.active', 1)
                                                  ->where('user_user_groups.user_id', Auth::id())
                                                  ->groupBy('training_alerts.id')
                                                  ->paginate(20);

            /*$approved_training_alerts = array();

            $approved_alerts = UserTrainingAlert::where('user_id', Auth::id())->get();

            foreach ($approved_alerts as $key => $value) {
                $approved_training_alerts[$value->training_alert_id] = date('m/d/Y', strtotime($value->created_at));
            }*/
        }

        return view('pages.training_alerts', [
                                                'redirected' => $redirected,
                                                'training_alerts' => $training_alerts,
                                                'approved_training_alerts' => ''
                                            ]);
    }

    public function getRawSearch($my_request) {
        $raw_search_array = array();

        /*if($my_request->file_type_id_search != 'all' && $my_request->file_type_id_search != '')
        {
            $raw_search_array[] = " file_type_id = '".$my_request->file_type_id_search."' ";
        }*/

        /*if($my_request->active_search != 'all' && $my_request->active_search != '')
        {
            $raw_search_array[] = " active = '".$my_request->active_search."' ";
        }*/

        if($my_request->start_date_search != '' && $my_request->end_date_search != '')
        {
            if(strtotime($my_request->start_date_search) <= strtotime($my_request->end_date_search))
            {
                $start_date_search = date('Y-m-d 00:00:00', strtotime($my_request->start_date_search));
                $end_date_search = date('Y-m-d 23:59:59', strtotime($my_request->end_date_search));
                $raw_search_array[] = " (training_alerts.created_at  >= '".$start_date_search."' AND training_alerts.created_at  <= '".$end_date_search."')";
            }
        }

        if(trim($my_request->value_search) != '')
        {
            $raw_search_array[] = " (
                                        training_alerts.title like '%".$my_request->value_search."%' OR 
                                        training_alerts.description like '%".$my_request->value_search."%' OR 
                                        training_alerts.content like '%".$my_request->value_search."%' 
                                    )";
        }

        $raw_search = implode(' AND ', $raw_search_array);

        return $raw_search;
    }


    /**
     * Display a list of the resource with pagination.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */

    public function paginationOld(Request $request)

    {
        $current_training_alert_id = 0;
        $no_links = false;
        $raw_search = $this->getRawSearch($request);
        $training_alerts =  new \stdClass();
        if(Auth::user()->user_type->name == 'Admin') 
        {
            if($request->id != "" && $request->id != null) {
                $training_alerts = TrainingAlert::where('id', '=', $request->id)->paginate(20);
                $no_links = false;
                $current_training_alert_id = $request->id;
            } else if($request->showall == "1") {
                $training_alerts = TrainingAlert::all();
                $no_links = true;
            } else {
                if($raw_search != '')
                {
                    $training_alerts = TrainingAlert::whereRaw($raw_search)->paginate(20);
                } else {
                    $training_alerts = TrainingAlert::paginate(20);
                }
                
            }
            $approved_training_alerts = array();
        } else if(Auth::user()->user_type->name == 'Full BDC Caller' || Auth::user()->user_type->name == 'Inbound Caller') {
            $user_groups_array = array();
            foreach (Auth::user()->user_groups as $key => $value) {
                $user_groups_array[] = $value->id;
            }

            if(count($user_groups_array) > 0)
            {
                if($request->id != "" && $request->id != null) {
                    $training_alerts = TrainingAlert::where('id', '=', $request->id)->paginate(20);
                    $no_links = false;
                    $current_training_alert_id = $request->id;
                } else if($request->showall == "1") {
                    $training_alerts = TrainingAlert::selectRaw('training_alerts.*')
                                                      ->join('training_alert_user_groups', 'training_alerts.id', '=', 'training_alert_user_groups.training_alert_id')
                                                      ->where('training_alerts.active', 1)
                                                      ->whereRaw("FIND_IN_SET(training_alert_user_groups.user_group_id, '".implode(',', $user_groups_array)."')")
                                                      ->groupBy('training_alerts.id')
                                                      ->get();
                    $no_links = true;
                } else {
                    if($raw_search != '')
                    {
                        if(isset($request->status_search))
                        {
                            if($request->status_search == 'all')
                            {
                                $training_alerts = TrainingAlert::selectRaw('training_alerts.*')
                                                      ->join('training_alert_user_groups', 'training_alerts.id', '=', 'training_alert_user_groups.training_alert_id')
                                                      ->whereRaw($raw_search)
                                                      ->where('training_alerts.active', 1)
                                                      ->whereRaw("FIND_IN_SET(training_alert_user_groups.user_group_id, '".implode(',', $user_groups_array)."')")
                                                      ->groupBy('training_alerts.id')
                                                      ->paginate(20);
                            } else {
                                if($request->status_search == 'pending')
                                {
                                    $raw_query = "training_alerts.id NOT IN (SELECT training_alert_id FROM user_training_alerts WHERE user_id = '".Auth::id()."')";
                                } elseif($request->status_search == 'approved')
                                {
                                    $raw_query = "training_alerts.id IN (SELECT training_alert_id FROM user_training_alerts WHERE user_id = '".Auth::id()."')";
                                }
                                $training_alerts = TrainingAlert::selectRaw('training_alerts.*')
                                                      ->join('training_alert_user_groups', 'training_alerts.id', '=', 'training_alert_user_groups.training_alert_id')
                                                      ->whereRaw($raw_search)
                                                      ->where('training_alerts.active', 1)
                                                      ->whereRaw("FIND_IN_SET(training_alert_user_groups.user_group_id, '".implode(',', $user_groups_array)."')")
                                                      ->whereRaw($raw_query)
                                                      ->groupBy('training_alerts.id')
                                                      ->paginate(20);
                            }
                        } else {
                            $training_alerts = TrainingAlert::selectRaw('training_alerts.*')
                                                      ->join('training_alert_user_groups', 'training_alerts.id', '=', 'training_alert_user_groups.training_alert_id')
                                                      ->whereRaw($raw_search)
                                                      ->where('training_alerts.active', 1)
                                                      ->whereRaw("FIND_IN_SET(training_alert_user_groups.user_group_id, '".implode(',', $user_groups_array)."')")
                                                      ->groupBy('training_alerts.id')
                                                      ->paginate(20);
                        }
                    } else {

                        if(isset($request->status_search))
                        {
                            if($request->status_search == 'all')
                            {
                                $training_alerts = TrainingAlert::selectRaw('training_alerts.*')
                                                      ->join('training_alert_user_groups', 'training_alerts.id', '=', 'training_alert_user_groups.training_alert_id')
                                                      ->where('training_alerts.active', 1)
                                                      ->whereRaw("FIND_IN_SET(training_alert_user_groups.user_group_id, '".implode(',', $user_groups_array)."')")
                                                      ->groupBy('training_alerts.id')
                                                      ->paginate(20);
                            } else {
                                if($request->status_search == 'pending')
                                {
                                    $raw_query = "training_alerts.id NOT IN (SELECT training_alert_id FROM user_training_alerts WHERE user_id = '".Auth::id()."')";
                                } elseif($request->status_search == 'approved')
                                {
                                    $raw_query = "training_alerts.id IN (SELECT training_alert_id FROM user_training_alerts WHERE user_id = '".Auth::id()."')";
                                }
                                $training_alerts = TrainingAlert::selectRaw('training_alerts.*')
                                                      ->join('training_alert_user_groups', 'training_alerts.id', '=', 'training_alert_user_groups.training_alert_id')
                                                      ->where('training_alerts.active', 1)
                                                      ->whereRaw("FIND_IN_SET(training_alert_user_groups.user_group_id, '".implode(',', $user_groups_array)."')")
                                                      ->whereRaw($raw_query)
                                                      ->groupBy('training_alerts.id')
                                                      ->paginate(20);
                            }
                        } else {
                            $training_alerts = TrainingAlert::selectRaw('training_alerts.*')
                                                      ->join('training_alert_user_groups', 'training_alerts.id', '=', 'training_alert_user_groups.training_alert_id')
                                                      ->where('training_alerts.active', 1)
                                                      ->whereRaw("FIND_IN_SET(training_alert_user_groups.user_group_id, '".implode(',', $user_groups_array)."')")
                                                      ->groupBy('training_alerts.id')
                                                      ->paginate(20);
                        }
                    }
                }
            }

            $approved_training_alerts = array();

            $approved_alerts = UserTrainingAlert::where('user_id', Auth::id())->get();

            foreach ($approved_alerts as $key => $value) {
                $approved_training_alerts[$value->training_alert_id] = date('m/d/Y', strtotime($value->created_at));
            }
        }

        if ($request->ajax()) {
            return view('lists.training_alerts_list', [
                                                'training_alerts' => $training_alerts, 
                                                'approved_training_alerts' => $approved_training_alerts, 
                                                'no_links' => $no_links, 
                                                'current_training_alert_id' => $current_training_alert_id,
                                            ]);
        }
    }


    public function pagination(Request $request)

    {
        $current_training_alert_id = 0;
        $no_links = false;
        $raw_search = $this->getRawSearch($request);
        $training_alerts =  new \stdClass();
        if(Auth::user()->user_type->name == 'Admin') 
        {
            if($request->id != "" && $request->id != null) {
                $training_alerts = TrainingAlert::where('id', '=', $request->id)->paginate(20);
                $no_links = false;
                $current_training_alert_id = $request->id;
            } else if($request->showall == "1") {
                $training_alerts = TrainingAlert::all();
                $no_links = true;
            } else {
                if($raw_search != '')
                {
                    $training_alerts = TrainingAlert::whereRaw($raw_search)->paginate(20);
                } else {
                    $training_alerts = TrainingAlert::paginate(20);
                }
                
            }
            $approved_training_alerts = array();
        } else if(Auth::user()->user_type->name == 'Full BDC Caller' || Auth::user()->user_type->name == 'Inbound Caller') {
            
              if($request->id != "" && $request->id != null) {
                  $training_alerts = TrainingAlert::where('id', '=', $request->id)->paginate(20);
                  $no_links = false;
                  $current_training_alert_id = $request->id;
              } else if($request->showall == "1") {
                  $training_alerts = TrainingAlert::select(
                                                    'training_alerts.*', 
                                                    'user_training_alerts.created_at as date_approved', 
                                                    'training_alerts.created_at as training_alert_created_at',
                                                    'user_user_groups.created_at as user_user_group_created_at'
                                                  )
                                                ->join('training_alert_user_groups', 'training_alerts.id', '=', 'training_alert_user_groups.training_alert_id')
                                                ->join('user_user_groups', 'training_alert_user_groups.user_group_id', '=', 'user_user_groups.user_group_id')
                                                ->leftJoin('user_training_alerts', 'training_alerts.id', '=', 'user_training_alerts.training_alert_id')
                                                ->where('training_alerts.active', 1)
                                                ->where('user_user_groups.user_id', Auth::id())
                                                ->groupBy('training_alerts.id')
                                                ->get();
                  $no_links = true;
              } else {
                  if($raw_search != '')
                  {
                      if(isset($request->status_search))
                      {
                          if($request->status_search == 'all')
                          {
                              $training_alerts = TrainingAlert::select(
                                                    'training_alerts.*', 
                                                    'user_training_alerts.created_at as date_approved', 
                                                    'training_alerts.created_at as training_alert_created_at',
                                                    'user_user_groups.created_at as user_user_group_created_at'
                                                  )
                                                ->join('training_alert_user_groups', 'training_alerts.id', '=', 'training_alert_user_groups.training_alert_id')
                                                ->join('user_user_groups', 'training_alert_user_groups.user_group_id', '=', 'user_user_groups.user_group_id')
                                                ->leftJoin('user_training_alerts', 'training_alerts.id', '=', 'user_training_alerts.training_alert_id')
                                                ->where('training_alerts.active', 1)
                                                ->where('user_user_groups.user_id', Auth::id())
                                                ->whereRaw($raw_search)
                                                ->groupBy('training_alerts.id')
                                                ->paginate(20);

                          } else {
                              if($request->status_search == 'pending')
                              {
                                  $training_alerts = TrainingAlert::select(
                                                    'training_alerts.*', 
                                                    'user_training_alerts.created_at as date_approved', 
                                                    'training_alerts.created_at as training_alert_created_at',
                                                    'user_user_groups.created_at as user_user_group_created_at'
                                                  )
                                                ->join('training_alert_user_groups', 'training_alerts.id', '=', 'training_alert_user_groups.training_alert_id')
                                                ->join('user_user_groups', 'training_alert_user_groups.user_group_id', '=', 'user_user_groups.user_group_id')
                                                ->leftJoin('user_training_alerts', 'training_alerts.id', '=', 'user_training_alerts.training_alert_id')
                                                ->whereRaw('user_training_alerts.training_alert_id IS NULL')
                                                ->where('training_alerts.active', 1)
                                                ->where('user_user_groups.user_id', Auth::id())
                                                ->whereRaw($raw_search)
                                                ->groupBy('training_alerts.id')
                                                ->paginate(20);
                              } elseif($request->status_search == 'approved')
                              {
                                  $training_alerts = TrainingAlert::select(
                                                    'training_alerts.*', 
                                                    'user_training_alerts.created_at as date_approved', 
                                                    'training_alerts.created_at as training_alert_created_at',
                                                    'user_user_groups.created_at as user_user_group_created_at'
                                                  )
                                                ->join('training_alert_user_groups', 'training_alerts.id', '=', 'training_alert_user_groups.training_alert_id')
                                                ->join('user_user_groups', 'training_alert_user_groups.user_group_id', '=', 'user_user_groups.user_group_id')
                                                ->join('user_training_alerts', 'training_alerts.id', '=', 'user_training_alerts.training_alert_id')
                                                ->where('training_alerts.active', 1)
                                                ->where('user_user_groups.user_id', Auth::id())
                                                ->whereRaw($raw_search)
                                                ->groupBy('training_alerts.id')
                                                ->paginate(20);
                              }
                          }
                      } else {
                          $training_alerts = TrainingAlert::select(
                                                    'training_alerts.*', 
                                                    'user_training_alerts.created_at as date_approved', 
                                                    'training_alerts.created_at as training_alert_created_at',
                                                    'user_user_groups.created_at as user_user_group_created_at'
                                                  )
                                                ->join('training_alert_user_groups', 'training_alerts.id', '=', 'training_alert_user_groups.training_alert_id')
                                                ->join('user_user_groups', 'training_alert_user_groups.user_group_id', '=', 'user_user_groups.user_group_id')
                                                ->leftJoin('user_training_alerts', 'training_alerts.id', '=', 'user_training_alerts.training_alert_id')
                                                ->where('training_alerts.active', 1)
                                                ->where('user_user_groups.user_id', Auth::id())
                                                ->whereRaw($raw_search)
                                                ->groupBy('training_alerts.id')
                                                ->paginate(20);
                      }
                  } else {

                      if(isset($request->status_search))
                      {
                          if($request->status_search == 'all')
                          {
                              $training_alerts = TrainingAlert::select(
                                                    'training_alerts.*', 
                                                    'user_training_alerts.created_at as date_approved', 
                                                    'training_alerts.created_at as training_alert_created_at',
                                                    'user_user_groups.created_at as user_user_group_created_at'
                                                  )
                                                ->join('training_alert_user_groups', 'training_alerts.id', '=', 'training_alert_user_groups.training_alert_id')
                                                ->join('user_user_groups', 'training_alert_user_groups.user_group_id', '=', 'user_user_groups.user_group_id')
                                                ->leftJoin('user_training_alerts', 'training_alerts.id', '=', 'user_training_alerts.training_alert_id')
                                                ->where('training_alerts.active', 1)
                                                ->where('user_user_groups.user_id', Auth::id())
                                                ->groupBy('training_alerts.id')
                                                ->paginate(20);
                          } else {
                              if($request->status_search == 'pending')
                              {
                                  $training_alerts = TrainingAlert::select(
                                                    'training_alerts.*', 
                                                    'user_training_alerts.created_at as date_approved', 
                                                    'training_alerts.created_at as training_alert_created_at',
                                                    'user_user_groups.created_at as user_user_group_created_at'
                                                  )
                                                ->join('training_alert_user_groups', 'training_alerts.id', '=', 'training_alert_user_groups.training_alert_id')
                                                ->join('user_user_groups', 'training_alert_user_groups.user_group_id', '=', 'user_user_groups.user_group_id')
                                                ->leftJoin('user_training_alerts', 'training_alerts.id', '=', 'user_training_alerts.training_alert_id')
                                                ->whereRaw('user_training_alerts.training_alert_id IS NULL')
                                                ->where('training_alerts.active', 1)
                                                ->where('user_user_groups.user_id', Auth::id())
                                                ->groupBy('training_alerts.id')
                                                ->paginate(20);
                              } elseif($request->status_search == 'approved')
                              {
                                  $training_alerts = TrainingAlert::select(
                                                    'training_alerts.*', 
                                                    'user_training_alerts.created_at as date_approved', 
                                                    'training_alerts.created_at as training_alert_created_at',
                                                    'user_user_groups.created_at as user_user_group_created_at'
                                                  )
                                                ->join('training_alert_user_groups', 'training_alerts.id', '=', 'training_alert_user_groups.training_alert_id')
                                                ->join('user_user_groups', 'training_alert_user_groups.user_group_id', '=', 'user_user_groups.user_group_id')
                                                ->join('user_training_alerts', 'training_alerts.id', '=', 'user_training_alerts.training_alert_id')
                                                ->where('training_alerts.active', 1)
                                                ->where('user_user_groups.user_id', Auth::id())
                                                ->groupBy('training_alerts.id')
                                                ->paginate(20);
                              }   
                          }
                      } else {
                          $training_alerts = TrainingAlert::select(
                                                    'training_alerts.*', 
                                                    'user_training_alerts.created_at as date_approved', 
                                                    'training_alerts.created_at as training_alert_created_at',
                                                    'user_user_groups.created_at as user_user_group_created_at'
                                                  )
                                                ->join('training_alert_user_groups', 'training_alerts.id', '=', 'training_alert_user_groups.training_alert_id')
                                                ->join('user_user_groups', 'training_alert_user_groups.user_group_id', '=', 'user_user_groups.user_group_id')
                                                ->leftJoin('user_training_alerts', 'training_alerts.id', '=', 'user_training_alerts.training_alert_id')
                                                ->where('training_alerts.active', 1)
                                                ->where('user_user_groups.user_id', Auth::id())
                                                ->groupBy('training_alerts.id')
                                                ->paginate(20);
                      }
                  }
              }
        }

        if ($request->ajax()) {
            return view('lists.training_alerts_list', [
                                                'training_alerts' => $training_alerts, 
                                                'approved_training_alerts' => '', 
                                                'no_links' => $no_links, 
                                                'current_training_alert_id' => $current_training_alert_id,
                                            ]);
        }
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        $commonFunctions = new CommonFunctionsController();
        $commonFunctions->modelToSelectOptions(TrainingAlertFileType::orderBy('name')->get(),'name', 'id');
        $file_types = $commonFunctions->select_options;

        $training_alert = new \stdClass();
        $training_alert->id = "";
        $training_alert->title = "";
        $training_alert->description = "";
        $training_alert->content = "";
        $training_alert->file = "";
        $training_alert->file_type_id = "";
        $training_alert->coid = "";
        $training_alert->user_groups = array();

        return view('dialogs.add_training_alert', [
                                                        'training_alert' => $training_alert, 
                                                        'file_types' => $file_types,
                                                    ]);
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        $validatedData = $request->validate([
            'title' => 'required|max:191',
            'description' => 'nullable',
            'content' => 'nullable',
            'file' => 'nullable',
            'file_type_id' => "required",
            'coid' => "nullable",
        ]);

        $training_alert = new TrainingAlert;
        $training_alert->title = $request->title;
        $training_alert->description = $request->description;
        $training_alert->content = $request->content;
        $training_alert->file = $request->file;
        $training_alert->file_type_id = $request->file_type_id;
        $training_alert->coid = $request->coid;
        $training_alert->save();

        //TrainingAlertUserGroup::where('training_alert_id', $request->training_alert_id)->delete();
        if(trim($request->user_groups) != '')
        {
            $user_groups = explode(',', $request->user_groups);
            foreach ($user_groups as $key => $value) {
                $training_alert_user_group = new TrainingAlertUserGroup;
                $training_alert_user_group->training_alert_id = $training_alert->id;
                $training_alert_user_group->user_group_id = $value;
                $training_alert_user_group->save();
            }
        }

        return response()->json([
            'inserted_id' => $training_alert->id,
            'message' => 'Training alert created.'
        ]);
    }

    /**
     * Display the specified resource.
     *
     * @param  \App\TrainingAlert  $trainingAlert
     * @return \Illuminate\Http\Response
     */
    public function show(Request $request, TrainingAlert $trainingAlert)
    {
        $training_alert = TrainingAlert::find($request->training_alert_id);
        $type = $request->type;

        return view('dialogs.training_alert_content', [
                                                    'type' => $type, 
                                                    'training_alert' => $training_alert, 
                                                ]);
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  \App\TrainingAlert  $trainingAlert
     * @return \Illuminate\Http\Response
     */
    public function edit(Request $request, TrainingAlert $trainingAlert)
    {
        $training_alert = TrainingAlert::find($request->training_alert_id);
        $commonFunctions = new CommonFunctionsController();

        $commonFunctions = new CommonFunctionsController();
        $commonFunctions->modelToSelectOptions(TrainingAlertFileType::orderBy('name')->get(),'name', 'id');
        $file_types = $commonFunctions->select_options;
        
        return view('dialogs.add_training_alert', [
                                                    'training_alert' => $training_alert,  
                                                    'file_types' => $file_types, 
                                                ]);
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  \App\TrainingAlert  $trainingAlert
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, TrainingAlert $trainingAlert)
    {

        $validatedData = $request->validate([
            'title' => 'required|max:191',
            'description' => 'nullable',
            'content' => 'nullable',
            'file' => 'nullable',
            'file_type_id' => "required",
            'coid' => "nullable",
        ]);

        TrainingAlert::where('id', $request->training_alert_id)
                ->update(
                    $request->only( 'title', 
                                    'description',
                                    'content',
                                    'file',
                                    'file_type_id',
                                    'coid'
                                )
                );

        TrainingAlertUserGroup::where('training_alert_id', $request->training_alert_id)->delete();
        if(trim($request->user_groups) != '')
        {
            $user_groups = explode(',', $request->user_groups);
            foreach ($user_groups as $key => $value) {
                $training_alert_user_group = new TrainingAlertUserGroup;
                $training_alert_user_group->training_alert_id = $request->training_alert_id;
                $training_alert_user_group->user_group_id = $value;
                $training_alert_user_group->save();
            }
        }

        return response()->json([
            'inserted_id' => $request->training_alert_id,
            'message' => 'Training alert updated.'
        ]);
    }

    public function uploadFile(Request $request)
    {
        $messages = [
            'upload_file.required' => 'File required!',
            'upload_file.mimes' => 'Invalid file format',
            'upload_file.max' => 'Max file size is 250M.',
        ];

        $validatedData = $request->validate([
            'upload_file' => 'required|mimes:xlsx,xls,csv,txt,jpeg,bmp,png,gif,mp4,mov,ogg,ogx,ogv,qt,webm,mpga,mpeg,wav,doc,docx,pdf,ppt,pptx|max:256000',
        ], $messages);

        $uploaded_file = $request->file('upload_file');

        $orig_file_name = $uploaded_file->getClientOriginalName();
        //$new_name = rand() . '.' . time(). '.' . $orig_file_name . '.' . $uploaded_file->getClientOriginalExtension();
        $new_name = rand() . '' . time(). '_' . $orig_file_name;
        $uploaded_file->move(public_path('uploaded_files'), $new_name);

        $output = array(
                        'error' => '',
                        'success' => 'File successfuly uploaded.',
                        'uploaded_file' => 'uploaded_files/'.$new_name,
                        'orig_file_name' => $orig_file_name,
                        'file_name' => $new_name,
                    );
        return response()->json($output);
    }


    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  \App\User  $user
     * @return \Illuminate\Http\Response
     */
    public function activeTrainingAlert(Request $request, TrainingAlert $trainingAlert)
    {
        TrainingAlert::where('id',$request->training_alert_id)
                ->update([
                            'active' => $request->active
                        ]);

        if($request->active == 1)
        {
            $training_alert = TrainingAlert::find($request->training_alert_id);
            $mail_data = new \stdClass();
            $mail_data->subject = 'New training alert: '.$training_alert->title;
            $mail_data->view = 'general_email';
            $mail_data->email_css = '.label_td {
                                          font-weight: bold;
                                        }

                                        .value_td {
                                          
                                        }

                                        .footer_notes {
                                          font-size: 10px;
                                        }

                                        .align_top {
                                            vertical-align: baseline;
                                        }';
            
            $mail_data->email_content = view('email.training_alert_email')->with(['training_alert' => $training_alert])->render();

            $user_groups_array = array();
            foreach ($training_alert->user_groups as $key => $user_group) {
                $user_groups_array[] = $user_group->id;
            }

            $users = User::selectRaw('users.*')
                          ->join('user_user_groups', 'users.id', '=', 'user_user_groups.user_id')
                          ->whereRaw("FIND_IN_SET(user_user_groups.user_group_id, '".implode(',', $user_groups_array)."')")
                          ->get();

            Mail::to($users)->send(new GeneralEmailAlert($mail_data));
        }

        return response()->json([
            'updated_id' => $request->training_alert_id,
            'message' => 'Training alert updated.'
        ]);
    }

    /**
     * Sets training alert as required for login
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  \App\TrainingAlert  $trainingAlert
     * @return \Illuminate\Http\Response
     */
    public function setRequiredTrainingAlert(Request $request, TrainingAlert $trainingAlert)
    {
        TrainingAlert::where('id',$request->training_alert_id)
                ->update([
                            'required' => $request->required
                        ]);

        return response()->json([
            'updated_id' => $request->training_alert_id,
            'message' => 'Training alert updated.'
        ]);
    }


    /**
     * Sets training alert as required for login
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  \App\TrainingAlert  $trainingAlert
     * @return \Illuminate\Http\Response
     */
    public function approve(Request $request, TrainingAlert $trainingAlert)
    {
        $user_training_alert = new UserTrainingAlert;
        $user_training_alert->user_id = $request->user_id;
        $user_training_alert->training_alert_id = $request->training_alert_id;
        $user_training_alert->save();

        return response()->json([
            'updated_id' => $request->training_alert_id,
            'message' => 'Training alert updated.'
        ]);
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  \App\TrainingAlert  $trainingAlert
     * @return \Illuminate\Http\Response
     */
    public function destroy(Request $request, TrainingAlert $trainingAlert)
    {
        TrainingAlert::where('id', $request->training_alert_id)->delete();
        return response()->json([
            'removed_id' => $request->training_alert_id,
            'message' => 'Training alert removed.'
        ]);
    }
}
