<?php

namespace App\Http\Controllers;

use App\TwilioPhone;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\DB;
use Illuminate\Http\Request;
use Twilio\Rest\Client;

class TwilioPhoneController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {
        $dealers = DB::connection('mysql2')->table('callcenter_companies')
                                ->select('co_name', 'co_short', 'coid', 'automated_reminder')
                                ->where('is_active', 1)
                                ->orderBy('co_name')
                                ->get();
        $twilio_phones = TwilioPhone::all();
        $twilio_phones_array = array();
        foreach ($twilio_phones as $key => $value) {
            $twilio_phones_array[$value->coid] = $value->phone;
        }

        return view('pages.twilio_phones', [
                                                'dealers' => $dealers,
                                                'twilio_phones_array' => $twilio_phones_array
                                            ]);
    }


    /**
     * Display a list of the resource with pagination.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */

    public function twilioPhonesPagination(Request $request)

    {
        $current_coid = 0;
        $no_links = true;
        if(Auth::user()->user_type->name == 'Admin') 
        {
            if($request->id != "" && $request->id != null) {
                $dealers = DB::connection('mysql2')->table('callcenter_companies')
                                ->select('co_name', 'co_short', 'coid', 'automated_reminder')
                                ->where('is_active', 1)
                                ->where('coid', $request->id)
                                ->orderBy('co_name')
                                ->get();
                $no_links = true;
                $current_coid = $request->id;
            } else if($request->showall == "1") {
                $dealers = DB::connection('mysql2')->table('callcenter_companies')
                                ->select('co_name', 'co_short', 'coid', 'automated_reminder')
                                ->where('is_active', 1)
                                ->orderBy('co_name')
                                ->get();
                $no_links = true;
            } else {
                $dealers = DB::connection('mysql2')->table('callcenter_companies')
                                ->select('co_name', 'co_short', 'coid', 'automated_reminder')
                                ->where('is_active', 1)
                                ->orderBy('co_name')
                                ->get();
            }
        }

        $twilio_phones = TwilioPhone::all();
        $twilio_phones_array = array();
        foreach ($twilio_phones as $key => $value) {
            $twilio_phones_array[$value->coid] = $value->phone;
        }
        if ($request->ajax()) {
            return view('lists.twilio_phones_list', [
                                                        'dealers' => $dealers, 'no_links' => $no_links, 
                                                        'current_coid' => $current_coid, 
                                                        'twilio_phones_array' => $twilio_phones_array
                                                    ]);
        }
    }

    public function startConversationDialog()
    {
        if(Auth::user()->user_type->name == 'Admin') 
        {
            $dealers = DB::connection('mysql2')->table('callcenter_companies')
                                ->select('co_name', 'co_short', 'coid')
                                ->where('is_active', 1)
                                ->orderBy('co_name')
                                ->get();
        } else if(Auth::user()->user_type->name == 'Dealer') {
            $user_companies_array = array();
            foreach(Auth::user()->dealers as $company)
            {
              $user_companies_array[] = $company->dealer_id;
            }

            $dealers = DB::connection('mysql2')->table('callcenter_companies')
                                ->select('co_name', 'co_short', 'coid')
                                ->whereRaw("FIND_IN_SET(coid, '".implode(',', $user_companies_array)."')")
                                ->get();
        }

        $dealer_options = '';
        foreach ($dealers as $key => $value) {
            $dealer_options.= '<option value="'.$value->coid.'">'.$value->co_name.'</option>';
        }

        return response()->json([
            'dealer_options' => $dealer_options,
        ]);
    }

    public function searchConversation(Request $request)
    {
        $dealer = DB::connection('mysql2')->table('callcenter_companies')
                                ->select('co_short', 'coid')
                                ->where('is_active', 1)
                                ->where('coid', $request->coid)
                                ->first();

        $db_table = 'livecall_'.$dealer->co_short;
        $search_phone = preg_replace("/[^0-9]/i", "", $request->search);
        $customer = DB::connection('mysql2')->table($db_table)
                                ->select('*')
                                ->where(function($query) use($request){
                                            $query->where('email', $request->search)
                                            ->where(function($query2) use($request){
                                                $query2->whereRaw('phone != "" OR cellphone != "" OR best_phone != ""');
                                            });
                                })
                                ->orWhere('phone', $search_phone)
                                ->orWhere('cellphone', $search_phone)
                                ->orWhere('best_phone', $search_phone)
                                ->first();


        if($customer)
        {
            $phone = $customer->cellphone;
            if($phone == '')
                $phone = $customer->phone;
            if($phone == '')
                $phone = $customer->best_phone;
        } else {
            $db_table = 'inboundcall_'.$dealer->co_short;
            $search_phone = preg_replace("/[^0-9]/i", "", $request->search);
            $customer = DB::connection('mysql3')->table($db_table)
                                    ->select('*')
                                    ->where(function($query) use($request){
                                                $query->where('email', $request->search)
                                                ->where(function($query2) use($request){
                                                    $query2->whereRaw('phone != "" OR cellphone != ""');
                                                });
                                    })
                                    ->orWhere('phone', $search_phone)
                                    ->orWhere('cellphone', $search_phone)
                                    ->first();

            if($customer)
            {
                $phone = $customer->cellphone;
                if($phone == '')
                    $phone = $customer->phone;
                if($phone == '')
                    $phone = $customer->best_phone;
            } else {
                $customer = new \stdClass();
                $customer->fname = '';
                $customer->lname = '';
                $phone = '';
            }            
        }
        
        return response()->json([
            'from_id' => Auth::id(),
            'coid' => $request->coid,
            'first_name' => $customer->fname,
            'last_name' => $customer->lname,
            'phone' => $phone,
        ]);
    }


    /**
     * Display the specified resource in JSON format.
     *
     * @return \Illuminate\Http\Response
     */
    public function search(Request $request)
    {
        if(Auth::user()->user_type->name == 'Admin') 
        {
            $search_value = trim($request->input('search_value'));
            
            $dealers = DB::connection('mysql2')->table('callcenter_companies')
                                ->select('co_name', 'co_short', 'coid')
                                ->where('is_active', 1)
                                ->where('co_name', 'like', '%'.$search_value.'%')
                                ->orderBy('co_name')
                                ->get();
        }
        $search_result = array();
        foreach($dealers as $dealer) {
            $search_result[] = array(
                                        "id"        => $dealer->coid,
                                        "value"  => $dealer->co_name
                                    );
        }
        return response()->json($search_result);
    }


    /**
     * Sends sms to user using Twilio's programmable sms client
     * @param String $message Body of sms
     * @param Number $recipients string or array of phone number of recepient
     */
    private function sendMessage($message, $recipients)
    {
        $account_sid = getenv("TWILIO_SID");
        $auth_token = getenv("TWILIO_AUTH_TOKEN");
        $twilio_number = getenv("TWILIO_NUMBER");
        $client = new Client($account_sid, $auth_token);
        $client->messages->create($recipients, 
                ['from' => $twilio_number, 'body' => $message] );
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        //
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        //
    }

    /**
     * Display the specified resource.
     *
     * @param  \App\TwilioPhone  $twilioPhone
     * @return \Illuminate\Http\Response
     */
    public function show(TwilioPhone $twilioPhone)
    {
        //
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  \App\TwilioPhone  $twilioPhone
     * @return \Illuminate\Http\Response
     */
    public function edit(TwilioPhone $twilioPhone)
    {
        //
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  \App\TwilioPhone  $twilioPhone
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, TwilioPhone $twilioPhone)
    {
        $twilio_number = $request->twilio_phone;

        $current_twilio_number = TwilioPhone::where('coid', $request->coid)->first();

        if($current_twilio_number)
        {
            TwilioPhone::where('coid', $request->coid)->update([
                                                                'phone' => $request->twilio_phone
                                                            ]);
            $message = 'Phone updated';
        } else {
            $current_twilio_number = new TwilioPhone;
            $current_twilio_number->coid = $request->coid;
            $current_twilio_number->phone = $request->twilio_phone;
            $current_twilio_number->save();
            $message = 'Phone inserted';
        }
        
        return response()->json([
            'updated_id' => $current_twilio_number->id,
            'message' => $message
        ]);
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  \App\TwilioPhone  $twilioPhone
     * @return \Illuminate\Http\Response
     */
    public function destroy(TwilioPhone $twilioPhone)
    {
        //
    }
}
