<?php

namespace App\Http\Controllers;

use App\FileType;
use App\CallType;
use App\FtpCredential;
use App\ScrubbedFtpFile;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\DB;
use DateTime;
use DateTimeZone;

class UploadFtpCallFileController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {

        if(Auth::user()->user_type->name == 'Admin') 
        {
            $dealers = DB::connection('mysql2')->table('callcenter_companies')
                                ->select('coid', 'co_name', 'co_short')
                                ->where('is_active', 1)
                                ->orderBy('co_name')
                                ->get();
        } else if(Auth::user()->user_type->name == 'Dealer') {
            $user_companies_array = array();
            foreach(Auth::user()->dealers as $company)
            {
              $user_companies_array[] = $company->dealer_id;
            }

            $dealers = DB::connection('mysql2')->table('callcenter_companies')
                                ->select('coid', 'co_name', 'co_short')
                                ->whereRaw("FIND_IN_SET(coid, '".implode(',', $user_companies_array)."')")
                                ->get();
        }

        

        $commonFunctions = new CommonFunctionsController();
        $commonFunctions->modelToSelectOptions($dealers,'co_name', 'coid');
        $dealers_options = $commonFunctions->select_options;

        $commonFunctions->modelToSelectOptions(FileType::whereRaw('coid IS NULL')->get(),'name', 'id');
        $file_templates = $commonFunctions->select_options;

        $commonFunctions->modelToSelectOptions(CallType::all(),'name', 'id');
        $call_types = $commonFunctions->select_options;

        return view('pages.upload_call_file', [
                                            'dealers' => $dealers_options,
                                            'file_templates' => $file_templates,
                                            'call_types' => $call_types,
                                        ]);
    }

    /**
     * Display the specified resource in JSON format.
     *
     * @return \Illuminate\Http\Response
     */
    public function ftpSection(Request $request)
    {
        $commonFunctions = new CommonFunctionsController();
        $coid = $request->coid;
        $company = DB::connection('mysql2')->table('callcenter_companies')->where('coid', $coid)->first();
        $department_access = $company->department_access;
        $ftp_data = FtpCredential::where('coid', $coid)->first();
        $ftp_file_select = '';
        $uploaded_file_date = "No upload date data for this file!";
        $ftp_file_select_options = '<option value="">Select file...</option>';
        $scrubbed_file_select_options = '<option value="">Select file...</option>';
        
        $ftp_path = public_path('callcenteruploads') . "/" . $company->co_short;
        if (!file_exists($ftp_path)) {
            mkdir($ftp_path, 0755, true);
        }
        if($ftp_data)
        {
            $ftp_server = $ftp_data->host;
            $ftp_username = $ftp_data->username;
            $ftp_password = $ftp_data->password;
            $delimiter = $ftp_data->delimiter;
            $id_ftp=ftp_connect($ftp_server); 
            ftp_login($id_ftp,$ftp_username,$ftp_password); 
            
            $files = ftp_nlist($id_ftp, ".");
            sort($files);
            $ftp_file_select_options = '<option value="">Select file...</option>';
            $scrubbed_file_select_options = '<option value="">Select file...</option>';
            $two_days_ago = strtotime('-5 days');
            $is_first_file = true;
            foreach($files as $f) {
                $file_time = filemtime(public_path('callcenteruploads') . "/" . $company->co_short."/".$f);
                if($file_time >= $two_days_ago)
                {
                    $f = str_replace('./', '', $f);
                    if(trim($f) != '.' && trim($f) != '.ftpquota')
                    {
                        if(strpos($f,"_scrubbed.csv") !== false)
                        {
                            $scrubbed_file_select_options.= '<option value="'.$f.'">'.$f.'</option>';

                            if($is_first_file == true) {
                                $uploaded_file = DB::connection('mysql2')
                                            ->table('callcenter_uploaded_files')
                                            ->where('coid', $coid)
                                            ->where('filename', $f)
                                            ->orderBy('date_uploaded', 'desc')
                                            ->first();

                                $is_first_file = false;
                                if($uploaded_file) {
                                    $uploaded_file_date = 'Last date uploaded: '.date("m/d/Y H:i:s", strtotime($uploaded_file->date_uploaded));
                                }
                            }
                        } else {
                            $ftp_file_select_options.= '<option value="'.$f.'">'.$f.'</option>';
                        }
                    }
                }
            }
        } else {
            $ftp_data = new \stdClass();
            $ftp_data->id = '';
            $ftp_data->host = '';
            $ftp_data->username = '';
            $ftp_data->password = '';
            $ftp_data->delimiter = '';
        }

        $departments_options = $commonFunctions->getDepartmentsSelect($department_access, $selected = "");
        $calltypes_options = $commonFunctions->getCalltypesSelect("", "");

        return view('partials.ftp_files', [
                                            'ftp_file_select_options' => $ftp_file_select_options,
                                            'scrubbed_file_select_options' => $scrubbed_file_select_options,
                                            'departments_options' => $departments_options,
                                            'calltypes_options' => $calltypes_options,
                                            'coid' => $coid,
                                            'ftp_data' => $ftp_data,
                                            'company' => $company,
                                        ]);
    }


    public function ftpSectionAll(Request $request)
    {

        $commonFunctions = new CommonFunctionsController();

        if(Auth::user()->user_type->name == 'Admin') 
        {
            $dealers = DB::connection('mysql2')->table('callcenter_companies')
                                ->where('is_active', 1)
                                ->orderBy('co_name')
                                ->get();
            $ftp_credentials = FtpCredential::get();
        } else if(Auth::user()->user_type->name == 'Dealer') {
            $user_companies_array = array();
            foreach(Auth::user()->dealers as $company)
            {
              $user_companies_array[] = $company->dealer_id;
            }

            $dealers = DB::connection('mysql2')->table('callcenter_companies')
                                ->whereRaw("FIND_IN_SET(coid, '".implode(',', $user_companies_array)."')")
                                ->get();
            $ftp_credentials = FtpCredential::whereRaw("FIND_IN_SET(coid, '".implode(',', $user_companies_array)."')")->get();
        }
        $dealer_names = [];
        $ftp_host = [];
        $ftp_username = [];
        $ftp_password = [];
        $ftp_delimiter = [];
        $departments_options = [];

        foreach($ftp_credentials as $ftp_data)
        {
            $ftp_host[$ftp_data->coid] = $ftp_data->host;
            $ftp_username[$ftp_data->coid] = $ftp_data->username;
            $ftp_password[$ftp_data->coid] = $ftp_data->password;
            $ftp_delimiter[$ftp_data->coid] = $ftp_data->delimiter;
        }

        foreach ($dealers as $key => $company) 
        {
            $dealer_names[$company->coid] = $company->co_name;
            $coid = $company->coid;
            $ftp_files[$coid] = [];
            $scrubbed_files[$coid] = [];
            //$company = DB::connection('mysql2')->table('callcenter_companies')->where('coid', $coid)->first();
            $department_access = $company->department_access;
            //$ftp_data = FtpCredential::where('coid', $coid)->first();
            $ftp_file_select = '';
            $uploaded_file_date = "No upload date data for this file!";
            $ftp_file_select_options = '<option value="">Select file...</option>';
            $scrubbed_file_select_options = '<option value="">Select file...</option>';
            
            $ftp_path = public_path('callcenteruploads') . "/" . $company->co_short;
            if (!file_exists($ftp_path)) {
                mkdir($ftp_path, 0755, true);
            }
            if(isset($ftp_host[$coid]))
            {
                $ftp_server = $ftp_host[$coid];
                $ftp_username_text = $ftp_username[$coid];
                $ftp_password_text = $ftp_password[$coid];
                $delimiter = $ftp_delimiter[$coid];
                $id_ftp = ftp_connect($ftp_server); 
                ftp_login($id_ftp, $ftp_username_text, $ftp_password_text); 
                $files = ftp_nlist($id_ftp, ".");
                sort($files);
                $two_days_ago = strtotime('-5 days');
                $is_first_file = true;
                foreach($files as $f) 
                {
                    $file_time = filemtime(public_path('callcenteruploads') . "/" . $company->co_short."/".$f);
                    if($file_time >= $two_days_ago)
                    {
                        $f = str_replace('./', '', $f);
                        if(trim($f) != '.' && trim($f) != '.ftpquota')
                        {
                            if(strpos($f, "_scrubbed.csv") !== false)
                            {
                                $scrubbed_files[$coid][] = $f;   
                            } else {
                                $ftp_files[$coid][] = $f;
                            }
                        }
                    }
                }
                $file_templates = DB::connection('mysql2')
                                            ->table('file_templates')
                                            ->where('coid', $coid)
                                            ->get();
                $file_separator[$coid] = [];
                $file_department[$coid] = [];
                $file_date_range_param[$coid] = [];
                $file_calltype[$coid] = [];
                foreach ($file_templates as $file_template) 
                {
                    $file_separator[$file_template->coid][$file_template->filename] = (trim($file_template->file_separator) == "")? "," : $file_template->file_separator;
                    $file_department[$file_template->coid][$file_template->filename] = $file_template->file_department;
                    $file_date_range_param[$file_template->coid][$file_template->filename] = $file_template->date_range_param;
                    $file_calltype[$file_template->coid][$file_template->filename] = $file_template->file_calltype;
                }
            }
            $departments_options[$coid] = $commonFunctions->getDepartmentsSelectOptions($department_access);

            
        }
        $uploaded_files = [];
        foreach ($dealers as $key => $company) 
        {
            if(isset($scrubbed_files[$company->coid]))
            {
                $uploaded_files = DB::connection('mysql2')
                                    ->table('callcenter_uploaded_files')
                                    ->where('coid', $company->coid)
                                    ->whereIn('filename', $scrubbed_files[$company->coid])
                                    ->orderBy('date_uploaded', 'desc')
                                    ->groupBy('filename')
                                    ->get();
            }
        }
        $file_uploaded_dates = [];
        foreach ($uploaded_files as $key => $value) 
        {
            $file_uploaded_dates[$value->coid][$value->filename] = date('m/d/Y, H:i:s', strtotime($value->date_uploaded));
        }

        

        $calltypes_options = $commonFunctions->getCalltypesSelectOptions();
        
        return view('pages.call_files', [
                                            'dealers' => $dealers,
                                            'scrubbed_files' => $scrubbed_files,
                                            'ftp_files' => $ftp_files,
                                            'departments_options' => $departments_options,
                                            'calltypes_options' => $calltypes_options,
                                            'ftp_host' => $ftp_host,
                                            'ftp_username' => $ftp_username,
                                            'ftp_password' => $ftp_password,
                                            'ftp_delimiter' => $ftp_delimiter,
                                            'file_separator' => $file_separator,
                                            'file_department' => $file_department,
                                            'file_date_range_param' => $file_date_range_param,
                                            'file_calltype' => $file_calltype,
                                        ]);
    }
    /**
     * Display the specified resource in JSON format.
     *
     * @return \Illuminate\Http\Response
     */
    public function ftpCredentials(Request $request)
    {
        $host = $request->host;
        $username = $request->username;
        $password = $request->password;
        $coid = $request->dealer_coid;
        $delimiter = $request->delimiter;
        $ftp_data = FtpCredential::where('coid', $coid)->first();
        if($ftp_data)
        {
            FtpCredential::where('coid', $coid)
                        ->update([
                            'host' => $host,
                            'username' => $username,
                            'password' => $password,
                            'delimiter' => $delimiter,
                        ]);
        } else {
            $ftp_data = new FtpCredential;
            $ftp_data->host = $host;
            $ftp_data->username = $username;
            $ftp_data->password = $password;
            $ftp_data->delimiter = $delimiter;
            $ftp_data->coid = $coid;
            $ftp_data->save();
        }
        
        return response()->json([
            'updated_id' => $ftp_data->id,
            'message' => 'Ftp credentials updated.'
        ]);
    }

    public function openFTPFile($data_equest) {

        $company = DB::connection('mysql2')->table('callcenter_companies')->where('coid', $data_equest->coid)->first();
        $coid = $data_equest->coid;
        $co_name = $company->co_name;
        $co_short = $company->co_short;

        $ftp_server = $data_equest->host;
        $ftp_username = $data_equest->username;
        $ftp_password = $data_equest->password;
        $file_name = $data_equest->file_name;
        $file_name_strip = str_replace('./', '', $file_name);
        // ruta al archivo remoto
        $remote_file = $file_name;
        //$local_file = public_path('callcenteruploads') . "/" . 'ftp_temp.csv';

        $local_file = public_path('callcenteruploads') . "/" . $co_short . "/" . $file_name_strip;

        $handle = fopen($local_file, 'r');

        return $handle;

    /*// abrir un archivo para escribir
        $handle = fopen($local_file, 'w');

    // establecer una conexión básica
        $conn_id = ftp_connect($ftp_server);

    // iniciar sesión con nombre de usuario y contraseña
        $login_result = ftp_login($conn_id, $ftp_username, $ftp_password);

    // intenta descargar un $remote_file y guardarlo en $handle
        if (ftp_fget($conn_id, $handle, $remote_file, FTP_ASCII, 0)) {
            //echo "Se ha escrito satisfactoriamente sobre $local_file\n";
        } else {
            echo "Ha habido un problema durante la descarga de $remote_file en $local_file\n";
        }

    // cerrar la conexión ftp y el gestor de archivo
        ftp_close($conn_id);
        fclose($handle);
        $handle = fopen($local_file, 'r');

        return $handle;*/

    }


    public function ftpTemplateDialog(Request $request)
    {   
        $company = DB::connection('mysql2')->table('callcenter_companies')->where('coid', $request->coid)->first();

        $coid = $request->coid;
        $co_name = $company->co_name;
        $co_short = $company->co_short;
        $file_name = $request->file_name;
        $file_name_strip = str_replace('./', '', $file_name);
        $delimiter = ($request->delimiter != "")? $request->delimiter : ",";

        $commonFunctions = new CommonFunctionsController();

        $ftp_file_templates = DB::connection('mysql2')
                            ->table('callcenter_ftp_files_templates')
                            ->where(function($query) use($request){
                                $query->where('ftp_file_name', $request->file_name)
                                    ->orWhere('ftp_file_name', str_replace('./', '', $request->file_name));
                            })
                            ->where('coid', $coid)
                            ->first();
        
        $call_types_texts = $commonFunctions->getCalltypesKeysText();

        if($ftp_file_templates) {
            $ftp_file_template = $ftp_file_templates->ftp_file_template;
            $ftp_file_uid = $ftp_file_templates->uid;
            $template_json = json_decode($ftp_file_template, true);
            $active_columns = explode(",", $ftp_file_templates->active_columns);
            $added_columns = $ftp_file_templates->added_columns;
            $sortby_column = $ftp_file_templates->sortby_column;
            $require_match_column_names = $ftp_file_templates->require_match_column_names;
            $added_columns_array = explode(",",$added_columns);
            $added_columns_default_values = $ftp_file_templates->added_columns_default_values;
            $added_columns_default_values = json_decode($added_columns_default_values, true);
            $acdv_content = '';
            if(!$added_columns_default_values) {
                $added_columns_default_values = array();
            }

            for($i=0;$i<count($added_columns_array);$i++) {
                $input_id = strtolower($added_columns_array[$i]);
                $input_name = ucfirst($input_id);
                if(isset($added_columns_default_values["value"][$i]))
                {
                    $input_value = $added_columns_default_values["value"][$i];
                    $acdv_content.= ' <strong>'.$input_name.':</strong> <input type="text" id="'.$input_id.'" name="'.$input_id.'" value="'.$input_value.'" class="added_columns_default_values form-control" style="font-size: 12px"> ';
                }
            }
        } else {
            $ftp_file_uid = '';
            $require_match_column_names = '';
            $added_columns = '';
            $acdv_content = '';
            $sortby_column = '';
            $template_json = array();
            $active_columns = array();
            $template_json["filter_order"] = array();
        }

        $filters_array = array();
        $filters_order_array = array();
        $columns_array = array();
        $filters_params = array();
        $filters_params_2 = array();
        $filters_params_3 = array();
        for($i=0;$i<count($template_json["filter_order"]);$i++) {
            $filters_order_array[$template_json["filter_order"][$i]] = $template_json["filter_order"][$i];
            $filters_array[$template_json["filter_order"][$i]] = $template_json["filter_type"][$i];
            $columns_array[$template_json["filter_order"][$i]] = $template_json["column"][$i];
            $filters_params[$template_json["filter_order"][$i]] = $template_json["params"][$i];
            $filters_params_2[$template_json["filter_order"][$i]] = (isset($template_json["params2"][$i]))? $template_json["params2"][$i] : '';
            $filters_params_3[$template_json["filter_order"][$i]] = (isset($template_json["params3"][$i]))? $template_json["params3"][$i] : '';
        }

        $gestor = $this->openFTPFile($request);
        $maxcolumns = 0;
        $i=0;
        while (($datos = fgetcsv($gestor,0,$delimiter)) !== FALSE) {
            $num = count($datos);
            $maxcolumns = ($num>$maxcolumns)? $num : $maxcolumns;
            $i++;
            if($i>25) break;
        }

        fclose($gestor);

        $contentheader = '<tr>';
        for($i=0;$i<$maxcolumns;$i++) {
            $checked = (in_array($i,$active_columns))? "checked" : "";
            $contentheader.= '
                    <td class="headertags14px" nowrap>
                        # '.$i.' <input type="checkbox" name="'.$i.'_use" id="'.$i.'_use" class="columns_to_use" '.$checked.' title="Import this column">
                    </td>';
        }
        $contentheader.= '</tr>';


        $content = "";
        $filters_content = '<table class="table table-striped" style="max-width:400px;font-size: 12px">
                                <tr>
                                    <th colspan="2">
                                        Match Column Names
                                    </th>
                                </tr>
                                <tr>
                                    <td>
                                        <button class="btn btn-primary" onclick="updateCurrentColumnNamesMatch(\''.$ftp_file_uid.'\')">Update Current Column Names</button>
                                    </td>
                                    <td>
                                        <input type="checkbox" value="1" id="require_match_column_names" name="require_match_column_names" '.$commonFunctions->setRadioValue($require_match_column_names,1).'>
                                    </td>
                                </tr>
                            </table>

                            <table class="table table-striped" style="font-size: 12px">
                                <thead>
                                    <tr>
                                        <th>
                                            Added Columns
                                        </th>
                                        <th>
                                            Default Values
                                        </th>
                                    </tr>
                                </thead>
                                <tbody>
                                    <tr>
                                        <td>
                                            <input type="text" id="added_columns" name="added_columns" value="'.$added_columns.'" class="form-control" placeholder="Column names separated by comma" onchange="changeDefaultValuesForAddedColumns()" style="font-size: 12px">
                                        </td>
                                        <td>
                                            <div id="added_columns_default_values_div">'.$acdv_content.'</div>
                                        </td>
                                    </tr>
                                </tbody>
                            </table>';

        $selected = ($sortby_column == "none" || $sortby_column == "")? 'selected' : '';
        $already_selected = ($sortby_column == "none" || $sortby_column == "")? true : false;
        $sortby_column_select = '<select id="sortby_column_select" name="sortby_column_select" class="form-control" style="font-size: 12px">
                                    <option value="none" '.$selected.'>None</option>';
        for($x=0;$x<$maxcolumns;$x++) {
            $selected = ($sortby_column == $x && $already_selected == false)? 'selected' : '';
            $sortby_column_select.= '<option value="'.$x.'" '.$selected.'>'.$x.'</option>';
        }
        $sortby_column_select.= '</select>';
        $filters_content.= '<table class="table table-striped" style="font-size: 12px">
                                <tr>
                                    <td>
                                        <span class="add cursor btn btn-primary">Add new filter</span>
                                    </td>
                                    <td>
                                        Sort by this column: '.$sortby_column_select.'
                                    </td>
                                </tr>
                            </table>
                            <table class="table table-striped" id="filters_table" style="font-size: 12px">
                            <thead>
                                <tr>
                                    <th>
                                        &nbsp;
                                    </th>
                                    <th>
                                        Filter Priority
                                    </th>
                                    <th>
                                        Column
                                    </th>
                                    <th>
                                        Filter
                                    </th>
                                    <th>
                                        Params
                                    </th>
                                </tr>
                            </thead>
                            <tbody>';

        foreach($filters_order_array as $filter_order) {
            $i = $filter_order;
            //$filter_order = $filters_order_array[$i];
            $filter_type = $filters_array[$i];
            $column_num = $columns_array[$i];
            $filter_param = $filters_params[$i];
            $filter_param_2 = $filters_params_2[$i];
            $filter_param_3 = $filters_params_3[$i];
            $column_select = '<select class="column_select form-control" style="font-size: 12px">';
            for($x=0;$x<$maxcolumns;$x++) {
                $selected = ($column_num == $x)? 'selected' : '';
                $column_select.= '<option value="'.$x.'" '.$selected.'>'.$x.'</option>';
            }
            $column_select.= '</select>';

            $filter_type_select = '
                                <select class="filter_type form-control" style="font-size: 12px">
                                    
                                    <option value="exclude_equals_to" '.$commonFunctions->setSelectValue($filter_type, 'exclude_equals_to').'>Exclude if equals to</option>
                                    <option value="upload_nstc_if_equals_to" '.$commonFunctions->setSelectValue($filter_type, 'upload_nstc_if_equals_to').'>Mark NSTC if equals to</option>
                                    <option value="upload_nstc_if_not_equals_to" '.$commonFunctions->setSelectValue($filter_type, 'upload_nstc_if_not_equals_to').'>Mark NSTC if NOT equals to</option>
                                    <option value="empty" '.$commonFunctions->setSelectValue($filter_type, 'empty').'>Remove if empty</option>
                                    <option value="upload_nstc_if_empty" '.$commonFunctions->setSelectValue($filter_type, 'upload_nstc_if_empty').'>Mark NSTC if empty</option>
                                    <option value="exclude_if_contains" '.$commonFunctions->setSelectValue($filter_type, 'exclude_if_contains').'>Exclude if contains</option>
                                    <option value="exclude_duplicated" '.$commonFunctions->setSelectValue($filter_type, 'exclude_duplicated').'>Exclude duplicates</option>
                                    <option value="find_replace" '.$commonFunctions->setSelectValue($filter_type, 'find_replace').'>Find and Replace (All)</option>
                                    <option value="replace_with" '.$commonFunctions->setSelectValue($filter_type, 'replace_with').'>Replace with</option>
                                    <option value="replace_with_multiple" '.$commonFunctions->setSelectValue($filter_type, 'replace_with_multiple').'>Replace with (Multiple)</option>
                                    <option value="exclude_if_not_in_date_range" '.$commonFunctions->setSelectValue($filter_type, 'exclude_if_not_in_date_range').'>Exclude if NOT in date range</option>
                                    <option value="exclude_if_does_not_include" '.$commonFunctions->setSelectValue($filter_type, 'exclude_if_does_not_include').'>Exclude if DOES NOT contains</option>
                                    <option value="exclude_if_does_not_include_tab" '.$commonFunctions->setSelectValue($filter_type, 'exclude_if_does_not_include_tab').'>Exclude if DOES NOT contains (Tab/Space)</option>
                                    
                                    <option value="replace_if_contains" '.$commonFunctions->setSelectValue($filter_type, 'replace_if_contains').'>Replace if contains</option>
                                    <option value="upload_nstc_if_contains" '.$commonFunctions->setSelectValue($filter_type, 'upload_nstc_if_contains').'>Mark NSTC if contains</option>
                                    <option value="exclude_if_year_less_than" '.$commonFunctions->setSelectValue($filter_type, 'exclude_if_year_less_than').'>Mark NSTC if year less than</option>
                                    <option value="split_in_two_columns" '.$commonFunctions->setSelectValue($filter_type, 'split_in_two_columns').'>Split in two columns</option>
                                    <option value="reverse_value" '.$commonFunctions->setSelectValue($filter_type, 'reverse_value').'>Reverse column value</option>
                                    <option value="exclude_if_begins_with_codes" '.$commonFunctions->setSelectValue($filter_type, 'exclude_if_begins_with_codes').'>Exclude if any code begins with</option>
                                    
                                    <option value="set_calltype_if_equals_to" '.$commonFunctions->setSelectValue($filter_type, 'set_calltype_if_equals_to').'>Set call type if equals to</option>
                                    <option value="set_department_if_equals_to" '.$commonFunctions->setSelectValue($filter_type, 'set_department_if_equals_to').'>Set department if equals to</option>
                                    
                                    <option value="has_negative_numbers" '.$commonFunctions->setSelectValue($filter_type, 'has_negative_numbers').'>Has (xxxx) negative numbers</option>
                                    <option value="find_vins" '.$commonFunctions->setSelectValue($filter_type, 'find_vins').'>Find Vin by last digits</option>
                                    <option value="find_customers_by_vin" '.$commonFunctions->setSelectValue($filter_type, 'find_customers_by_vin').'>Find Customers by VIN</option>
                                    <option value="exclude_less_than" '.$commonFunctions->setSelectValue($filter_type, 'exclude_less_than').'>Exclude if Less than</option>
                                    <option value="exclude_greater_than" '.$commonFunctions->setSelectValue($filter_type, 'exclude_greater_than').'>Exclude if Greater than</option>
                                    <option value="nstc_less_than" '.$commonFunctions->setSelectValue($filter_type, 'nstc_less_than').'>Mark NSTC if Less than</option>
                                    <option value="nstc_greater_than" '.$commonFunctions->setSelectValue($filter_type, 'nstc_greater_than').'>Mark NSTC if Greater than</option>
                                    
                                    <option value="file_has_to_contain_instance_of" '.$commonFunctions->setSelectValue($filter_type, 'file_has_to_contain_instance_of').'>File has to contain one instance of</option>
                                    
                                    
                                    <option value="add_to_gdnc_if_equals_to" '.$commonFunctions->setSelectValue($filter_type, 'add_to_gdnc_if_equals_to').'>Add to GDNC if equals to</option>
                                    <option value="add_to_ddnc_if_equals_to" '.$commonFunctions->setSelectValue($filter_type, 'add_to_ddnc_if_equals_to').'>Add to DDNC if equals to</option>
                                    <option value="add_to_gdnc_ddnc_if_equals_to" '.$commonFunctions->setSelectValue($filter_type, 'add_to_gdnc_ddnc_if_equals_to').'>Add to GDNC & DDNC if equals to</option>
                                    <option value="add_to_dne_if_equals_to" '.$commonFunctions->setSelectValue($filter_type, 'add_to_dne_if_equals_to').'>Add to DNE if equals to</option>
                                    <option value="phone_column" '.$commonFunctions->setSelectValue($filter_type, 'phone_column').'>Phone Column</option>
                                    <option value="email_column" '.$commonFunctions->setSelectValue($filter_type, 'email_column').'>Email Column</option>
                                    
                                    <option value="remove_phone_if_empty" '.$commonFunctions->setSelectValue($filter_type, 'remove_phone_if_empty').'>Remove phone if empty (000-0000)</option>
                                    
                                    <option value="remove_if_older_than_x_days" '.$commonFunctions->setSelectValue($filter_type, 'remove_if_older_than_x_days').'>Remove if older than X days</option>
                                    <option value="extract_date" '.$commonFunctions->setSelectValue($filter_type, 'extract_date').'>Extract date from string</option>
                                    <option value="zip_code_column" '.$commonFunctions->setSelectValue($filter_type, 'zip_code_column').'>Zip code column</option>
                                    <option value="change_to_w_c_code" '.$commonFunctions->setSelectValue($filter_type, 'change_to_w_c_code').'>Check W and C codes</option>
                                    <option value="remove_useless_phone_number" '.$commonFunctions->setSelectValue($filter_type, 'remove_useless_phone_number').'>Remove Useless Phone Number</option>

                                    <option value="nstc_by_date_range" '.$commonFunctions->setSelectValue($filter_type, 'nstc_by_date_range').'>Mark NSTC if NOT in date range</option>
                                    <option value="set_calltype_by_date_range" '.$commonFunctions->setSelectValue($filter_type, 'set_calltype_by_date_range').'>Set CallType by date range</option>

                                </select>';


    //if($filter_type != 'replace_with' && $filter_type != 'exclude_if_not_in_date_range' && $filter_type != 'replace_with_multiple' && $filter_type != 'replace_if_contains' && $filter_type != 'set_calltype_if_equals_to' && $filter_type != 'set_department_if_equals_to') {
      //          $filter_input = '<input type="text" class="filter_param" value="'.$filter_param.'">';
        //    } else

            if ($filter_type == 'replace_with' || $filter_type == 'replace_with_multiple' || $filter_type == 'find_replace' || $filter_type == 'replace_if_contains'){
                $filter_input = '<input type="text" class="filter_param form-control" value="'.$filter_param.'" placeholder="Find..." style="font-size: 12px"> <input type="text" class="filter_param2" value="'.$filter_param_2.'" placeholder="Replace with..." style="font-size: 12px">';
            } elseif ($filter_type == 'exclude_if_not_in_date_range' || $filter_type == 'nstc_by_date_range'){
                $filter_input = '<input type="text" class="filter_param ftp_dateinputs form-control" value="'.$filter_param.'" placeholder="Start Date" style="font-size: 12px"> <input type="text" class="filter_param2 ftp_dateinputs form-control" value="'.$filter_param_2.'" placeholder="End Date" style="font-size: 12px">';
            } elseif ($filter_type == 'set_calltype_if_equals_to'){
                
                
                $second_input = '<select class="filter_param2 form-control" style="font-size: 12px">';
                foreach($call_types_texts as $key=>$ctype) {
            
                    $second_input.= '<option value="'.$key.'" '.$commonFunctions->setSelectValue($filter_param_2, $key).'>'.$ctype.'</option>';
                }
                $second_input.= '</select>';
                
                $filter_input = '<input type="text" class="filter_param form-control" value="'.$filter_param.'" placeholder="Find..." style="font-size: 12px"> '.$second_input;
            } elseif ($filter_type == 'set_department_if_equals_to'){
                
                
                $second_input = '<select class="filter_param2 form-control" style="font-size: 12px">';
                $second_input.= $commonFunctions->getDepartmentsSelectId("", $filter_param_2);
                $second_input.= '</select>';
                
                $filter_input = '<input type="text" class="filter_param form-control" value="'.$filter_param.'" placeholder="Find..." style="font-size: 12px"> '.$second_input;
            } elseif ($filter_type == 'remove_if_older_than_x_days'){
                $filter_input = '<input type="text" class="filter_param form-control" value="'.$filter_param.'" placeholder="Number of Days..." style="font-size: 12px">';
            } elseif ($filter_type == 'set_calltype_by_date_range'){

                $third_input = '<select class="filter_param3 form-control" style="font-size: 12px">';
                foreach($call_types_texts as $key=>$ctype) {
            
                    $third_input.= '<option value="'.$key.'" '.$commonFunctions->setSelectValue($filter_param_3, $key).'>'.$ctype.'</option>';
                }
                $third_input.= '</select>';

                $filter_input = '<input type="text" class="filter_param ftp_dateinputs form-control" value="'.$filter_param.'" placeholder="Start Date" style="font-size: 12px"> <input type="text" class="filter_param2 ftp_dateinputs form-control" value="'.$filter_param_2.'" placeholder="End Date" style="font-size: 12px"> '.$third_input;
            } else {
                $filter_input = '<input type="text" class="filter_param form-control" value="'.$filter_param.'" style="font-size: 12px">';
            }

            $filters_content.= '<tr>
                                    <td class="filter_order">'.$filter_order.'</td>
                                    <td>
                                        <span class="up cursorpointer"><i class="fas fa-chevron-up"></i></span> |
                                        <span class="down cursorpointer"><i class="fas fa-chevron-down"></i></span> |
                                        <span class="top cursorpointer">Top</span> |
                                        <span class="bottom cursorpointer">Bottom</span> |
                                        <span class="delete cursorpointer"><i class="fas fa-trash-alt"></i></span>
                                    </td>
                                    <td>
                                        '.$column_select.'
                                    </td>
                                    <td>
                                        '.$filter_type_select.'
                                    </td>
                                    <td>
                                        '.$filter_input.'
                                    </td>
                                </tr>';

        }

        $filters_content.= '</tbody>
                            </table>
        <input type="hidden" id="columns_total" name="columns_total" value="'.$maxcolumns.'">
        <input type="hidden" id="filename_ftp" name="filename_ftp" value="'.$file_name.'">';
        $table_preview_content = '';
        $local_file = public_path('callcenteruploads') . "/" . $co_short . "/" . $file_name_strip;
        $gestor = fopen($local_file, 'r');
        $i = 0;
        $first_row = true;
        while (($datos = fgetcsv($gestor, 0, $delimiter)) !== FALSE) {
            $num = count($datos);
            $table_preview_content.= '<tr>';
            for ($c=0; $c < $num; $c++) {
                $newValue = trim($datos[$c]);
                if($first_row == true) {
                    $table_preview_content.= '<td>
                                <span class="first_row_ftp">'.$newValue.'</span>
                            </td>';
                } else {
                    $table_preview_content.= '<td>
                                '.$newValue.'
                            </td>';
                }
            }
            for($j=$c;$j<$maxcolumns;$j++) {
                $table_preview_content.= '<td>
                                &nbsp;
                            </td>';
            }
            $first_row = false;
            $table_preview_content.= '</tr>';
            $i++;
            if($i>25) break;
        }
        fclose($gestor);


        $table_preview = '<br/>
                                <table class="table table-striped" style="font-size: 12px">
                                    '.$contentheader.$table_preview_content.'
                                </table>';

        
        
        $call_types_select_content = '';
        foreach($call_types_texts as $key=>$ctype) {
            
            $call_types_select_content.= '<option value="'.$key.'">'.$ctype.'</option>';
        }
            
        $department_select_content = $commonFunctions->getDepartmentsSelect();
        $textarea_content = '<textarea id="calltype_select_temp" style="display:none">'.$call_types_select_content.'</textarea>';
        $textarea_content.= '<textarea id="department_select_temp" style="display:none">'.$department_select_content.'</textarea>';
        $content.= $filters_content.$table_preview.$textarea_content;
        $content.= '<script>
                        moveTableRows();
                        $(".ftp_dateinputs").each(function(){
                            $(this).datepicker();
                        });
                    </script>';
                
        /*return response()->json([
            'content' => $content,
        ]);*/

        return view('dialogs.ftp_file_template', [
                                            'content' => $content,
                                            'coid' => $coid,
                                            'file_name' => $file_name_strip,
                                        ]);
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  \App\Company  $company
     * @return \Illuminate\Http\Response
     */
    public function updateCurrentColumnNamesMatch(Request $request)
    {
        DB::connection('mysql2')
            ->table('callcenter_ftp_files_templates')
            ->where('uid', $request->uid)
            ->update([
                        'match_column_names' => $request->column_names, 
                    ]);

        return response()->json([
            'updated_id' => $request->uid,
            'message' => 'Updated!'
        ]);
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  \App\Company  $company
     * @return \Illuminate\Http\Response
     */
    public function saveFtpFileTemplate(Request $request)
    {
        $coid = $request->coid;
        $active_columns = $request->active_columns;
        $jsonstring = $request->jsonstring;
        $jsonstring2 = $request->jsonstring2;
        $file_name = str_replace('./', '', $request->filename_ftp);

        $ftp_file_template = DB::connection('mysql2')
                                ->table('callcenter_ftp_files_templates')
                                ->where('ftp_file_name', $file_name)
                                ->where('coid', $coid)
                                ->first();

        $ftp_server = $request->ftp_server;
        $ftp_username = $request->ftp_username;
        $ftp_password = $request->ftp_password;
        $added_columns = ($request->added_columns != null)? $request->added_columns : '';
        $ftp_delimiter = $request->ftp_delimiter;
        $require_match_column_names = $request->require_match_column_names;
        $sortby_column = $request->sortby_column;

        if($ftp_file_template) {
            DB::connection('mysql2')
                ->table('callcenter_ftp_files_templates')
                ->where('ftp_file_name', $file_name)
                ->where('coid', $coid)
                ->update([
                            'ftp_file_template' => $jsonstring, 
                            'added_columns_default_values' => $jsonstring2, 
                            'active_columns' => $active_columns, 
                            'added_columns' => $added_columns, 
                            'sortby_column' => $sortby_column,  
                            'file_delimiter' => $ftp_delimiter, 
                            'require_match_column_names' => $require_match_column_names, 
                        ]);
            $inserted_id = $ftp_file_template->uid;
        } else {
            $inserted_id = DB::connection('mysql2')
                                    ->table('callcenter_ftp_files_templates')
                                    ->insertGetId([
                                        'coid' => $coid,
                                        'ftp_file_name' => $file_name,
                                        'ftp_file_template' => $jsonstring, 
                                        'added_columns_default_values' => $jsonstring2, 
                                        'active_columns' => $active_columns, 
                                        'added_columns' => $added_columns, 
                                        'sortby_column' => $sortby_column,  
                                        'file_delimiter' => $ftp_delimiter, 
                                        'require_match_column_names' => $require_match_column_names, 
                                    ]);
        }

        return response()->json([
            'inserted_id' => $inserted_id,
            'message' => 'Template saved!'
        ]);
    }



    public function applyFiltersToTempFileFTP(Request $request) 
    { 
        $coid = $request->coid;
        $orig_file_name  = str_replace('./', '', $request->filename_ftp);
        $temp_file_name = str_replace('./', '', $request->filename_ftp);
        $company = DB::connection('mysql2')
                        ->table('callcenter_companies')
                        ->where('coid', $request->coid)
                        ->first();

        $ftp_file_templates = DB::connection('mysql2')
                                ->table('callcenter_ftp_files_templates')
                                ->where(function($query) use($request){
                                    $query->where('ftp_file_name', $request->filename_ftp)
                                        ->orWhere('ftp_file_name', str_replace('./', '', $request->filename_ftp));
                                })
                                ->where('coid', $request->coid)
                                ->first();

        $co_short = $company->co_short;
        $dnc_table = "callcenter_dnc";
        $dealer_dnc_table = "dnc_".$co_short;
        $dealer_dne_table = "dne_".$co_short;
        
        $ftp_file_template = $ftp_file_templates->ftp_file_template;
        $sortby_column = $ftp_file_templates->sortby_column;
        $template_json = json_decode($ftp_file_template,true);
        $added_columns_default_values = json_decode($ftp_file_templates->added_columns_default_values, true);
        $active_columns = explode(",", $ftp_file_templates->active_columns);
        $added_columns = explode(",", $ftp_file_templates->added_columns);
        $file_delimiter = $ftp_file_templates->file_delimiter;
        
        $require_match_column_names = $ftp_file_templates->require_match_column_names;
        $match_column_names = $ftp_file_templates->match_column_names;
        
        $file_delimiter = (trim($file_delimiter)!="")? $file_delimiter : ",";
        $filters_array = array();
        $filters_order_array = array();
        $columns_array = array();
        $filters_params = array();
        $filters_params_2 = array();
        $filters_params_3 = array();
        $phones_columns = array();
        $email_columns = array();
        $already_searched_vins = false;
        $already_searched_customer = false;
        $has_filter_find_customer_by_vin = false;
        $find_customers_vins = [];

        for($i=0;$i<count($template_json["filter_order"]);$i++) {
            $filters_order_array[$template_json["filter_order"][$i]] = $template_json["filter_order"][$i];
            $filters_array[$template_json["filter_order"][$i]] = $template_json["filter_type"][$i];
            $columns_array[$template_json["filter_order"][$i]] = $template_json["column"][$i];
            $filters_params[$template_json["filter_order"][$i]] = $template_json["params"][$i];
            $filters_params_2[$template_json["filter_order"][$i]] = (isset($template_json["params2"][$i]))? $template_json["params2"][$i] : '';
            $filters_params_3[$template_json["filter_order"][$i]] = (isset($template_json["params3"][$i]))? $template_json["params3"][$i] : '';
            if($template_json["filter_type"][$i] == "phone_column") {
                $phones_columns[] = $template_json["column"][$i];
            }
            if($template_json["filter_type"][$i] == "email_column") {
                $email_columns[] = $template_json["column"][$i];
            }
            
        }


        $added_columns_default_values_array = array();
        for($i=0;$i<count($added_columns_default_values["name"]);$i++) {
            $added_columns_default_values_array[] = $added_columns_default_values["value"][$i];
        }


        //sort($filters_order_array);


        //$upload_dir = 'callcenter_temp_files/';

        $local_file = public_path('callcenteruploads') . "/" . $co_short . "/" .$temp_file_name;
        $gestor = fopen($local_file, 'r');
        $new_file_array = array();
        $first_row = true;
        $first_row_ac = true;


        $sortered_values = array();
        $data_full = array();
        if($sortby_column !== "none" && $sortby_column !== "") {
            while (($row = fgetcsv($gestor,0,$file_delimiter)) !== FALSE) {
                $sortered_values[] = $row[$sortby_column];
                $data_full[] = $row;
            }
            array_multisort($sortered_values, SORT_DESC , $data_full);
        } else {
            while (($row = fgetcsv($gestor,0,$file_delimiter)) !== FALSE) {
                $data_full[] = $row;
            }
        }


        //while (($datos = fgetcsv($gestor)) !== FALSE) {
        
        $calltype_column = false;
        $department_column = false;
        $checked_duplicated_values = array();
        
        $has_one_instance_of_filter = false;
        $has_one_instance_of = false;
        
        $first_row_matching_columns = true;

        $useless_phones_list = DB::connection('mysql2')
                                ->table('callcenter_useless_phones')
                                ->where('phone', '!=', '')
                                ->get();


        $useless_phones_array = array();
        foreach($useless_phones_list as $upl) {
            $useless_phones_array[] = $upl->phone;
        }
        
        foreach ($data_full as $datos) {
            
            if($first_row_matching_columns == true && $require_match_column_names ==  1) {
                //$match_column_names_to_test = implode(",",$datos);
                $match_column_names_array = explode(",",$match_column_names);
                    for($xyz = 0;$xyz<count($match_column_names_array);$xyz++) {
                        
                        
                        $column_to_test_1 = preg_replace('/(?:^[^\p{L}\p{N}]+|[^\p{L}\p{N}]+$)/u', '', strtoupper(trim($match_column_names_array[$xyz])));
                        $column_to_test_2 = preg_replace('/(?:^[^\p{L}\p{N}]+|[^\p{L}\p{N}]+$)/u', '', strtoupper(trim($datos[$xyz])));
                        if($column_to_test_1 != $column_to_test_2) {
                            return response()->json([
                                'message' => "The File did not pass the Matching Column Test! \n".$column_to_test_1." is not equal to ".$column_to_test_2,
                            ]);
                        }
                    }
            }
            $first_row_matching_columns = false;
            //$num = count($datos);
            $insert = true;
            $show_to_call = 1;
            $current_duplicated_value = '';
            $has_duplicated_value_filter = false;
            $calltype_column_value_already_set = false;
            $department_column_value_already_set = false;
            
            $splitted_column = false;
            $splitted_column_array = array();
            
            foreach($filters_order_array as $filter_order) {
                $i = $filter_order;
                //$filter_order = $filters_order_array[$i];
                $filter_type = $filters_array[$i];
                $filtered_colum = $columns_array[$i];
                $filter_param = $filters_params[$i];
                $filter_param_2 = $filters_params_2[$i];
                $filter_param_3 = $filters_params_3[$i];
                //$filter_type = $filters_array[$filters_order_array[$i]];
                //$filtered_colum = $columns_array[$filters_order_array[$i]];
                //$filter_param = $filters_params[$filters_order_array[$i]];
                if( ! isset( $datos[ $filtered_colum ] ) ) { // It means the file does not have that column number.
                    continue;
                }
                if($filter_type == "empty") {
                    if(trim($datos[$filtered_colum]) == "") {
                        $insert = false;
                        //break;
                    }
                } else if($filter_type == "upload_nstc_if_empty") {
                    if(trim($datos[$filtered_colum]) == "") {
                        $show_to_call = 0;
                        //break;
                    }
                } else if($filter_type == "exclude_equals_to") {
                    if(strtolower(trim($datos[$filtered_colum])) == strtolower(trim($filter_param))) {
                        $insert = false;
                        //break;
                    }
                } else if($filter_type == "exclude_less_than") {
                    $splitted_test_value = explode(' ', trim( $datos[$filtered_colum] ) );
                    $value_to_test = (int)$splitted_test_value[0] * 1;
                    $value_to_test_against = (int)$filter_param * 1;
                    if($value_to_test < $value_to_test_against) {
                        $insert = false;
                        //break;
                    }
                } else if($filter_type == "exclude_greater_than") {
                    $splitted_test_value = explode(' ', trim( $datos[$filtered_colum] ) );
                    $value_to_test = (int)$splitted_test_value[0] * 1;
                    $value_to_test_against = (int)$filter_param * 1;
                    
                    if($value_to_test > $value_to_test_against) {
                        $insert = false;
                        //break;
                    }
                     /* if($filtered_colum == 19) {
                        if($insert == true)
                        {
                            print('true <br>'); 
                        } else {
                            print('false <br>'); 
                        }
                     } */
                        
                } else if($filter_type == "nstc_less_than") {
                    $value_to_test = (int)$datos[$filtered_colum] * 1;
                    $value_to_test_against = $filter_param * 1;
                    if($value_to_test < $value_to_test_against) {
                        $show_to_call = 0;
                        //break;
                    }
                } else if($filter_type == "nstc_greater_than") {
                    $value_to_test = (int)$datos[$filtered_colum] * 1;
                    $value_to_test_against = $filter_param * 1;
                    if($value_to_test > $value_to_test_against) {
                        $show_to_call = 0;
                        //break;
                    }
                } else if($filter_type == "exclude_if_contains") {
                    if(strpos(strtolower(trim($datos[$filtered_colum])),strtolower(trim($filter_param))) !== false) {
                        $insert = false;
                        //break;
                    }
                } else if($filter_type == "exclude_duplicated") {
                    $has_duplicated_value_filter = true;
                    if(!isset($duplicated_values[$filtered_colum])) {
                        $duplicated_values[$filtered_colum] = [];
                    }
                    $current_duplicated_value = $datos[$filtered_colum];
                    if(!isset($checked_duplicated_values[$filtered_colum][$datos[$filtered_colum]])) {
                        $checked_duplicated_values[$filtered_colum][$datos[$filtered_colum]] = 1;
                    } else {
                        $checked_duplicated_values[$filtered_colum][$datos[$filtered_colum]] = 2;
                    }
                    if(in_array(strtolower(trim($datos[$filtered_colum])),$duplicated_values[$filtered_colum])) {
                        if($checked_duplicated_values[$filtered_colum][$datos[$filtered_colum]] == 2) {
                            $insert = false;
                        }
                        //break;
                    } else {
                        if($insert == true)
                        {
                            $duplicated_values[$filtered_colum][] = strtolower(trim($datos[$filtered_colum]));
                        }
                    }
                } else if($filter_type == "upload_nstc_if_equals_to") {
                    $filter_param_array = explode(",",strtolower(trim($filter_param)));
                    if(in_array(strtolower(trim($datos[$filtered_colum])),$filter_param_array)) {
                        $show_to_call = 0;
                    }
                } else if($filter_type == "upload_nstc_if_not_equals_to") {
                    $filter_param_array = explode(",",strtolower(trim($filter_param)));
                    if(!in_array(strtolower(trim($datos[$filtered_colum])),$filter_param_array)) {
                        $show_to_call = 0;
                    }
                } else if($filter_type == "replace_with") {
                    if(strtolower(trim($datos[$filtered_colum])) == strtolower(trim($filter_param))) {
                        $datos[$filtered_colum] = $filter_param_2;
                    }
                } else if($filter_type == "set_calltype_if_equals_to") {
                    $calltype_column = true;
                    if(strtolower(trim($datos[$filtered_colum])) == strtolower(trim($filter_param))) {
                        //$datos[$filtered_colum] = $filter_param_2;
                        $calltype_column_value = $filter_param_2;
                        $calltype_column_value_already_set = true;
                    } else if($calltype_column_value_already_set == false){
                        $calltype_column_value = "";
                    }
                } else if($filter_type == "set_department_if_equals_to") {
                    $department_column = true;
                    if(strtolower(trim($datos[$filtered_colum])) == strtolower(trim($filter_param))) {
                        //$datos[$filtered_colum] = $filter_param_2;
                        $department_column_value = $filter_param_2;
                        $department_column_value_already_set = true;
                    } else if($department_column_value_already_set == false){
                        $department_column_value = "";
                    }
                } else if($filter_type == "find_replace") {
                    $datos[$filtered_colum] = str_replace($filter_param,$filter_param_2,$datos[$filtered_colum]);
                } else if($filter_type == "replace_with_multiple") {
                    $filter_param1_array = explode(",",strtolower(trim($filter_param)));
                    $filter_param2_array = explode(",",trim($filter_param_2));
                    $param_array = array();
                    for($fi = 0; $fi<count($filter_param1_array); $fi++) {
                        $param_array[$filter_param1_array[$fi]] = $filter_param2_array[$fi];
                    }
                    if(isset($param_array[strtolower(trim($datos[$filtered_colum]))])) {
                        $datos[$filtered_colum] = $param_array[strtolower(trim($datos[$filtered_colum]))];
                    }
                } else if($filter_type == "exclude_if_not_in_date_range") {
                    if(trim($filter_param) != "")
                    {
                        $start_date = strtotime(trim($filter_param));
                    } else {
                        $start_date = '';
                    }

                    if(trim($filter_param_2) != "")
                    {
                        $end_date = strtotime(trim($filter_param_2));
                    } else {
                        $end_date = '';
                    }

                    $test_date = strtotime(trim($datos[$filtered_colum]));

                    if($start_date != '' && $end_date != '')
                    {
                        if($test_date < $start_date || $test_date > $end_date) {
                            $insert = false;
                            //break;
                        }
                    } else if($start_date != '') {
                        if($test_date < $start_date) {
                            $insert = false;
                            //break;
                        }
                    } else if($end_date != '') {
                        if($test_date > $end_date) {
                            $insert = false;
                            //break;
                        }
                    } else {
                        $insert = false;
                    }
                    
                } else if($filter_type == "nstc_by_date_range") {
                    if(trim($filter_param) != "")
                    {
                        $start_date = strtotime(trim($filter_param));
                    } else {
                        $start_date = '';
                    }

                    if(trim($filter_param_2) != "")
                    {
                        $end_date = strtotime(trim($filter_param_2));
                    } else {
                        $end_date = '';
                    }

                    $test_date = strtotime(trim($datos[$filtered_colum]));

                    if($start_date != '' && $end_date != '')
                    {
                        if($test_date < $start_date || $test_date > $end_date) {
                            $show_to_call = 0;
                            //break;
                        }
                    } else if($start_date != '') {
                        if($test_date < $start_date) {
                            $show_to_call = 0;
                            //break;
                        }
                    } else if($end_date != '') {
                        if($test_date > $end_date) {
                            $show_to_call = 0;
                            //break;
                        }
                    } else {
                        $show_to_call = 0;
                    }
                    
                } else if($filter_type == "set_calltype_by_date_range") {
                    $calltype_column = true;
                    if(trim($filter_param) != "")
                    {
                        $start_date = strtotime(trim($filter_param));
                    } else {
                        $start_date = '';
                    }

                    if(trim($filter_param_2) != "")
                    {
                        $end_date = strtotime(trim($filter_param_2));
                    } else {
                        $end_date = '';
                    }

                    $test_date = strtotime(trim($datos[$filtered_colum]));

                    if($start_date != '' && $end_date != '')
                    {
                        if($test_date >= $start_date && $test_date <= $end_date) {
                            
                            $calltype_column_value = $filter_param_3;
                            $calltype_column_value_already_set = true;
                        } else if($calltype_column_value_already_set == false) {
                            $calltype_column_value = "";
                        }
                    } else if($start_date != '') {
                        if($test_date >= $start_date) {
                            $calltype_column_value = $filter_param_3;
                            $calltype_column_value_already_set = true;
                        } else if($calltype_column_value_already_set == false) {
                            $calltype_column_value = "";
                        }
                    } else if($end_date != '') {
                        if($test_date <= $end_date) {
                            $calltype_column_value = $filter_param_3;
                            $calltype_column_value_already_set = true;
                        } else if($calltype_column_value_already_set == false) {
                            $calltype_column_value = "";
                        }
                    } else if($calltype_column_value_already_set == false) {
                        $calltype_column_value = "";
                    }
                    
                } else if($filter_type == "remove_if_older_than_x_days") {
                    $start_date = strtotime(trim($datos[$filtered_colum]));
                    $test_date = strtotime(date('Y-m-d', strtotime("-".trim($filter_param)." days")));
                    if($start_date < $test_date) {
                        $insert = false;
                        //break;
                    }
                } else if($filter_type == "remove_phone_if_empty") {
                    $notags_phone = preg_replace("/[^0-9]/i", "", trim($datos[$filtered_colum]));
                    $test_phone = substr($notags_phone, 3);
                    if($test_phone == "0000000" || $test_phone == "") {
                        $datos[$filtered_colum] = "";
                    }
                } else if($filter_type == "exclude_if_does_not_include") {
                    /*$has_one_of_them = false;
                    $filter_param1_array = explode(",",strtolower(trim($filter_param)));
                    for($fi = 0; $fi<count($filter_param1_array); $fi++) {
                        //$datos[$filtered_colum] = $datos[$filtered_colum];
                        //if(strpos())
                        $original_value = preg_replace('/\s+/', ' ',strtolower(trim($datos[$filtered_colum])));
                        $test_value = preg_replace('/\s+/', ' ',strtolower(trim($filter_param1_array[$fi])));
                        if($original_value == $test_value) {
                            $has_one_of_them = true;
                        }
                    }

                    if($has_one_of_them != true) {
                        $insert = false;
                    }*/
                    $has_one_of_them = false;
                    $filter_param1_array = explode(",",strtolower(trim($filter_param)));
                    for($fi = 0; $fi<count($filter_param1_array); $fi++) {
                        if(strpos(strtolower(trim($datos[$filtered_colum])),strtolower(trim($filter_param1_array[$fi]))) !== false) {
                            $has_one_of_them = true;

                        }
                    }
                    if($has_one_of_them == false) {
                        $insert = false;
                    }


                } else if($filter_type == "exclude_if_does_not_include_tab") {
                    /*$has_one_of_them = false;
                    $filter_param1_array = explode(",",strtolower(trim($filter_param)));
                    for($fi = 0; $fi<count($filter_param1_array); $fi++) {
                        //$datos[$filtered_colum] = $datos[$filtered_colum];
                        //if(strpos())
                        $original_value = preg_replace('/\s+/', ' ',strtolower(trim($datos[$filtered_colum])));
                        $test_value = preg_replace('/\s+/', ' ',strtolower(trim($filter_param1_array[$fi])));
                        if($original_value == $test_value) {
                            $has_one_of_them = true;
                        }
                    }

                    if($has_one_of_them != true) {
                        $insert = false;
                    }*/
                    $has_one_of_them = false;
                    $filter_param1_array = explode(",",strtolower($filter_param));
                    for($fi = 0; $fi<count($filter_param1_array); $fi++) {
                        if(strpos(strtolower($datos[$filtered_colum]),strtolower($filter_param1_array[$fi])) !== false) {
                            $has_one_of_them = true;

                        }
                    }
                    if($has_one_of_them == false) {
                        $insert = false;
                    }
                } else if($filter_type == "exclude_if_begins_with_codes") {
                    $has_one_of_them = false;
                    $filter_param1_array = explode(",",strtolower(trim($filter_param)));
                    $codes_array = explode(" ",strtolower(trim($datos[$filtered_colum])));
                    for($fi = 0; $fi<count($filter_param1_array); $fi++) {
                        for($fx = 0; $fx<count($codes_array); $fx++) {
                            if(substr(trim($codes_array[$fx]), 0, strlen($filter_param1_array[$fi])) === $filter_param1_array[$fi]) {
                                $has_one_of_them = true;
                            }
                        }
                    }
                    if($has_one_of_them == true) {
                        $insert = false;
                    }
                } else if($filter_type == "replace_if_contains") {
                    $filter_param1_array = explode(",",strtolower(trim($filter_param)));
                    $filter_param2_array = explode(",",trim($filter_param_2));
                    for($fi = 0; $fi<count($filter_param1_array); $fi++) {
                        if(strpos(strtolower(trim($datos[$filtered_colum])),strtolower(trim($filter_param1_array[$fi]))) !== false) {
                            $datos[$filtered_colum] = $filter_param2_array[$fi];
                        }
                    }
                } else if($filter_type == "upload_nstc_if_contains") {
                    if(strpos(strtolower(trim($datos[$filtered_colum])),strtolower(trim($filter_param))) !== false) {
                        $show_to_call = 0;
                    }
                } else if($filter_type == "exclude_if_year_less_than") {
                    if((int)$datos[$filtered_colum] * 1 > 0) {
                        if(strlen((int)$datos[$filtered_colum]) > 2) {
                            try {
                                $dt = DateTime::createFromFormat('Y', (int)$datos[$filtered_colum]);
                                $current_year = $dt->format('Y');
                            } catch (Exception $e) {
                                $errorlog = 'Exception: '.  $e->getMessage(). "\n";
                            }
                        } else {
                            try {
                                $dt = DateTime::createFromFormat('y', (int)$datos[$filtered_colum]);
                                $current_year = $dt->format('Y');
                            } catch (Exception $e) {
                                $errorlog = 'Exception: '.  $e->getMessage(). "\n";
                            }
                        }
                        
                        if(strlen($filter_param) > 2) {
                            try {
                                $dt = DateTime::createFromFormat('Y', $filter_param);
                                $filter_param = $dt->format('Y');
                            } catch (Exception $e) {
                                $errorlog = 'Exception: '.  $e->getMessage(). "\n";
                            }
                        } else {
                            try {
                                $dt = DateTime::createFromFormat('y', $filter_param);
                                $filter_param = $dt->format('Y');
                            } catch (Exception $e) {
                                $errorlog = 'Exception: '.  $e->getMessage(). "\n";
                            }
                        }

                        if($filter_param > $current_year) {
                            $show_to_call = 0;
                        }
                    }  else {
                        $test_year_array = explode(' ', $datos[$filtered_colum]);
                        foreach ($test_year_array as $test_year) {
                            if ( is_numeric($test_year) ) {
                                if(strlen($test_year) > 2) {
                                    try {
                                        $dt = DateTime::createFromFormat('Y', $test_year);
                                        $current_year = $dt->format('Y');
                                    } catch (Exception $e) {
                                        $errorlog = 'Exception: '.  $e->getMessage(). "\n";
                                    }
                                } else {
                                    try {
                                        $dt = DateTime::createFromFormat('y', $test_year);
                                        $current_year = $dt->format('Y');
                                    } catch (Exception $e) {
                                        $errorlog = 'Exception: '.  $e->getMessage(). "\n";
                                    }
                                }
                                
                                if(strlen($filter_param) > 2) {
                                    try {
                                        $dt = DateTime::createFromFormat('Y', $filter_param);
                                        $filter_param = $dt->format('Y');
                                    } catch (Exception $e) {
                                        $errorlog = 'Exception: '.  $e->getMessage(). "\n";
                                    }
                                } else {
                                    try {
                                        $dt = DateTime::createFromFormat('y', $filter_param);
                                        $filter_param = $dt->format('Y');
                                    } catch (Exception $e) {
                                        $errorlog = 'Exception: '.  $e->getMessage(). "\n";
                                    }
                                }
        
                                if($filter_param > $current_year) {
                                    $show_to_call = 0;
                                }
                            }
                        }
                    }

                } else if($filter_type == "split_in_two_columns") {
                    $splitter = (trim($filter_param)=="")? ' ' : $filter_param;
                    //$datos[$filtered_colum] = $filter_param2_array[$fi];
                    $splitted_values = explode($splitter,trim($datos[$filtered_colum]));
                    $datos[$filtered_colum] = $splitted_values[0];
                    $splitted_column_value = "";
                    for($sv = 1;$sv < count($splitted_values); $sv++) {
                        if($sv == (count($splitted_values)-1)) {
                            $splitted_column_value.= $splitted_values[$sv];
                        } else {
                            $splitted_column_value.= $splitted_values[$sv].$splitter;
                        }
                        
                    }
                    $splitted_column = true;
                    
                    $splitted_column_array[] = $splitted_column_value;
                } else if($filter_type == "reverse_value") {
                    $splitter = (trim($filter_param)=="")? ' ' : $filter_param;
                    $splitted_values = explode($splitter,trim($datos[$filtered_colum]));
                    $splitted_column_value = "";
                    for($sv = 0;$sv < (count($splitted_values)-1); $sv++) {
                        if($sv == (count($splitted_values)-2)) {
                            $splitted_column_value.= $splitted_values[$sv];
                        } else {
                            $splitted_column_value.= $splitted_values[$sv].$splitter;
                        }
                    }
                    $datos[$filtered_colum] = $splitted_values[count($splitted_values)-1].$splitter.$splitted_column_value;
                } else if($filter_type == "has_negative_numbers") {
                    if(strpos($datos[$filtered_colum],"(") !== false && strpos($datos[$filtered_colum],")") !== false) {
                        $datos[$filtered_colum] = str_replace("(","",$datos[$filtered_colum]);
                        $datos[$filtered_colum] = str_replace(")","",$datos[$filtered_colum]);
                        $datos[$filtered_colum] = str_replace("$","",$datos[$filtered_colum]);
                        $datos[$filtered_colum] = str_replace(",","",$datos[$filtered_colum]);
                        if ( is_numeric( $datos[$filtered_colum] ) ) {
                            $datos[$filtered_colum] = $datos[$filtered_colum] * 1;
                        } else {
                            $datos[$filtered_colum] = 0;
                        }
                        
                        $datos[$filtered_colum] = $datos[$filtered_colum] * (-1);
                    }
                } else if($filter_type == "find_vins") {
                    if ( $already_searched_vins === false ) {
                        $searched_vins = DB::connection('mysql2')
                                ->table('livecall_' . $company->co_short)
                                ->select('vin')
                                ->where('vin', '!=', '')
                                ->whereNotNull('vin')
                                ->groupBy('vin')
                                ->get();

                        $searched_vins_array = [];
                        foreach ( $searched_vins as $searched_vin ) {
                            $vin_portion = substr($searched_vin->vin, -6);
                            $searched_vins_array[$vin_portion] = $searched_vin->vin;
                        }
                        $already_searched_vins = true;
                    }

                    if ( ! empty($searched_vins_array[ $datos[$filtered_colum] ]) ) {
                        $datos[$filtered_colum] = $searched_vins_array[ $datos[$filtered_colum] ];
                    }

                } else if($filter_type == "find_customers_by_vin") {
                    $has_filter_find_customer_by_vin = true;
                    if ( $already_searched_customer === false ) {
                        $searched_customers = DB::connection('mysql2')
                                ->table('livecall_' . $company->co_short)
                                ->selectRaw('fname, lname, phone, cellphone, email, vin')
                                ->where('vin', '!=', '')
                                ->whereNotNull('vin')
                                ->groupBy('vin')
                                ->orderBy('dateadded', 'desc')
                                ->get();

                        $searched_customers_array = [];
                        foreach ( $searched_customers as $searched_cus ) {
                            foreach ( $searched_cus as $cus_key => $cus_value ) {
                                $searched_customers_array[$searched_cus->vin][$cus_key] = $cus_value;
                            }
                        }
                        $already_searched_customer = true;
                    }

                    $customer_info_to_add = [];
                    if ( ! empty($searched_customers_array[ $datos[$filtered_colum] ]) ) {
                        $customer_info_to_add = [
                            'fname' => $searched_customers_array[ $datos[$filtered_colum] ]['fname'],
                            'lname' => $searched_customers_array[ $datos[$filtered_colum] ]['lname'],
                            'phone' => $searched_customers_array[ $datos[$filtered_colum] ]['phone'],
                            'cellphone' => $searched_customers_array[ $datos[$filtered_colum] ]['cellphone'],
                            'email' => $searched_customers_array[ $datos[$filtered_colum] ]['email'],
                            'vin' => $searched_customers_array[ $datos[$filtered_colum] ]['vin'],
                        ];
                    } else {
                        $customer_info_to_add = [
                            'fname' => '',
                            'lname' => '',
                            'phone' => '',
                            'cellphone' => '',
                            'email' => '',
                            'vin' => '',
                        ];
                    }

                } else if($filter_type == "file_has_to_contain_instance_of") {
                    $has_one_instance_of_filter = true;
                    $filter_param_array = explode(",",strtolower(trim($filter_param)));
                    if(in_array(strtolower(trim($datos[$filtered_colum])),$filter_param_array)) {
                        $has_one_instance_of = true;
                    }
                } else if($filter_type == "add_to_gdnc_if_equals_to") {
                    
                    if(strpos(strtolower(trim($datos[$filtered_colum])),strtolower(trim($filter_param))) !== false) {
                        foreach($phones_columns as $phones_col) {
                            $phone = $datos[$phones_col];
                            $phone = preg_replace("/[^0-9]/i", "", trim($phone));
                            $phone = is_numeric($phone)? $phone * 1 : 0;
                            if($phone>0) {
                                $timestamp = date("Y-m-d H:i:s");
                                $global_dnc = DB::connection('mysql2')
                                                ->table($dnc_table)
                                                ->where('phone', $phone)
                                                ->first();
                                
                                if(!$global_dnc) {
                                    $dnc_insert = DB::connection('mysql2')
                                                                    ->table($dnc_table)
                                                                    ->insertGetId([
                                                                        'phone' => $phone,
                                                                        'added_by' => 'Admin',
                                                                        'timestamp' => $timestamp,
                                                                        'reason' => 'Added from FTP file filter',
                                                                    ]);
                                }
                            }
                        }
                    }
                } else if($filter_type == "add_to_ddnc_if_equals_to") {
                    if(strpos(strtolower(trim($datos[$filtered_colum])),strtolower(trim($filter_param))) !== false) {
                        foreach($phones_columns as $phones_col) {
                            $phone = $datos[$phones_col];
                            $phone = preg_replace("/[^0-9]/i", "", trim($phone));
                            $phone = (int) $phone * 1;
                            if($phone>0) {
                                $timestamp = date("Y-m-d H:i:s");
                                $dealer_dnc = DB::connection('mysql2')
                                                ->table($dealer_dnc_table)
                                                ->where('phone', $phone)
                                                ->first();
                                if(!$dealer_dnc) {
                                    $dealer_dnc_insert = DB::connection('mysql2')
                                                                    ->table($dealer_dnc_table)
                                                                    ->insertGetId([
                                                                        'phone' => $phone,
                                                                        'added_by' => 'Admin',
                                                                        'timestamp' => $timestamp,
                                                                        'reason' => 'Added from FTP file filter',
                                                                    ]);
                                }
                            }
                        }
                    }
                } else if($filter_type == "add_to_gdnc_ddnc_if_equals_to") {
                    if(strpos(strtolower(trim($datos[$filtered_colum])),strtolower(trim($filter_param))) !== false) {
                        foreach($phones_columns as $phones_col) {
                            $phone = $datos[$phones_col];
                            $phone = preg_replace("/[^0-9]/i", "", trim($phone));
                            $phone = intval($phone)*1;
                            if($phone>0) {
                                $timestamp = date("Y-m-d H:i:s");
                                $global_dnc = DB::connection('mysql2')
                                                ->table($dnc_table)
                                                ->where('phone', $phone)
                                                ->first();
                                if(!$global_dnc) {
                                    $dnc_insert = DB::connection('mysql2')
                                                                    ->table($dnc_table)
                                                                    ->insertGetId([
                                                                        'phone' => $phone,
                                                                        'added_by' => 'Admin',
                                                                        'timestamp' => $timestamp,
                                                                        'reason' => 'Added from FTP file filter',
                                                                    ]);
                                }
                                
                                $dealer_dnc = DB::connection('mysql2')
                                                ->table($dealer_dnc_table)
                                                ->where('phone', $phone)
                                                ->first();
                                if(!$dealer_dnc) {
                                    $dealer_dnc_insert = DB::connection('mysql2')
                                                                    ->table($dealer_dnc_table)
                                                                    ->insertGetId([
                                                                        'phone' => $phone,
                                                                        'added_by' => 'Admin',
                                                                        'timestamp' => $timestamp,
                                                                        'reason' => 'Added from FTP file filter',
                                                                    ]);
                                }
                            }
                        }
                    }
                } else if($filter_type == "add_to_dne_if_equals_to") {
                    
                    if(strpos(strtolower(trim($datos[$filtered_colum])),strtolower(trim($filter_param))) !== false) {
                        foreach($email_columns as $email_col) {
                            $email = trim($datos[$email_col]);
                            if($email!="") {
                                $timestamp = date("Y-m-d H:i:s");
                                $dealer_dne = DB::connection('mysql2')
                                                ->table($dealer_dne_table)
                                                ->where('email', $email)
                                                ->first();
                                if(!$dealer_dne) {
                                    $dealer_dne_insert = DB::connection('mysql2')
                                                                    ->table($dealer_dne_table)
                                                                    ->insertGetId([
                                                                        'email' => $email,
                                                                        'added_by' => 'Admin',
                                                                        'timestamp' => $timestamp,
                                                                        'reason' => 'Added from FTP file filter',
                                                                    ]);
                                }
                            }
                        }
                    }
                } else if($filter_type == "extract_date") {
                    $date_array = date_parse($datos[$filtered_colum]);
                    $year = $date_array["year"];
                    $month = $date_array["month"];
                    $day = $date_array["day"];
                    $datos[$filtered_colum] = $month."/".$day."/".$year;
                } else if($filter_type == "zip_code_column") {
                    if(strlen($datos[$filtered_colum])<5) {
                        $current_zip_num = strlen($datos[$filtered_colum]);
                        while($current_zip_num < 5) {
                            $datos[$filtered_colum] = "0".$datos[$filtered_colum];
                            $current_zip_num++;
                        }
                    }
                } else if($filter_type == "change_to_w_c_code") {
                    $has_one_of_them = false;
                    $codes_array = explode(" ",strtolower(trim($datos[$filtered_colum])));
                    for($fx = 0; $fx<count($codes_array); $fx++) {
                        if(strtolower(substr(trim($codes_array[$fx]), 0, 1)) === "w") {
                            $has_one_of_them = true;
                        }
                    }
                    if($has_one_of_them == true) {
                        $datos[$filtered_colum] = "W";
                    } else {
                        $datos[$filtered_colum] = "C";
                    }
                } else if($filter_type == "remove_useless_phone_number") {
                    if(in_array(trim($datos[$filtered_colum]),$useless_phones_array)) {
                        $datos[$filtered_colum] = "";
                    }
                }
            }
            if($insert == false) {
                continue;
            }
            if($insert == true) {
                
                if ( $has_filter_find_customer_by_vin === true && ! empty ( $find_customers_vins[$customer_info_to_add['vin']] ) ) {
                    continue;
                }
                /*if($has_duplicated_value_filter) {
                    $checked_duplicated_values[$filtered_colum][$current_duplicated_value] = true;
                }*/
                
                $new_row_array = array();
                
                $new_row_array[] = $show_to_call;
                foreach($active_columns as $column) {
                    if( ! isset( $datos[ $column ] ) ) { // It means the file does not have that column number.
                        continue;
                    }
                    $new_row_array[] = trim($datos[$column]);
                }

                if ($first_row == true) {
                    //$show_to_call = "Show To Call";
                    $first_row = false;

                    if ( $has_filter_find_customer_by_vin === true ) {
                        $new_row_array[] = 'FirstName';
                        $new_row_array[] = 'LastName';
                        $new_row_array[] = 'Phone';
                        $new_row_array[] = 'CellPhone';
                        $new_row_array[] = 'Email';
                    }

                }

                if ( $has_filter_find_customer_by_vin === true && $first_row === false ) {
                    if ( empty ( $find_customers_vins[$customer_info_to_add['vin']] )) {
                        foreach ( $customer_info_to_add as $customer_info_key => $customer_info_value ) {
                            if ( $customer_info_key != 'vin' ) {
                                $new_row_array[] = $customer_info_value;
                            }
                        }
                        $find_customers_vins[$customer_info_to_add['vin']] = true;
                    }
                }

                $first_row_ac = false;
                if($first_row_ac == true) {
                    foreach($added_columns as $add_column) {
                        $new_row_array[] = ucwords(strtolower($add_column));
                    }
                    $first_row_ac = false;
                } else {
                    $z = 0;
                    foreach($added_columns as $add_column) {
                        if(isset($added_columns_default_values_array[$z]))
                        {
                            $new_row_array[] = $added_columns_default_values_array[$z];
                            $z++;
                        }
                    }
                }
                
                
                /**
     * if($splitted_column == true) {
     *                 $new_row_array[] = $splitted_column_value;
     *             }
     */
                
                foreach($splitted_column_array as $splitted_column_array_value) {
                    $new_row_array[] = $splitted_column_array_value;
                }
                
                if($calltype_column == true) {
                    $new_row_array[] = $calltype_column_value;
                }
                
                if($department_column == true) {
                    $new_row_array[] = $department_column_value;
                }

                $new_row_array[] = 'stc_flag_'.$show_to_call; //This is the flag to prevent the change of NSTC when reviewing file.

                $new_file_array[] = $new_row_array;
            }
        }
        fclose($gestor);
        
        if($has_one_instance_of_filter == true && $has_one_instance_of == false) {
            return response()->json([
                'message' => 'The "File has to contain one instance of" filter flag did not pass the test!'
            ]);
        }
        
        $new_orig_file_name = str_replace(".csv","", $orig_file_name).'_scrubbed.csv';
        $output_file =  public_path('callcenteruploads') . "/" . $co_short . "/" .$new_orig_file_name;
        $fp = fopen($output_file, 'w');
        foreach($new_file_array as $fields) {
            fputcsv($fp, $fields);
        }
        
        fclose($fp);
        
        $today_date = date("Y-m-d H:i:s");
        $uploaded_file_insert = DB::connection('mysql2')
                                    ->table('callcenter_ftp_uploaded_files')
                                    ->insertGetId([
                                        'filename' => $orig_file_name,
                                        'coid' => $coid,
                                        'date_uploaded' => $today_date,
                                    ]);


        $local_time = new DateTime();
        $local_time->setTimezone(new DateTimeZone('America/New_York'));
        $ftp_scrubbed_last_uploaded = $local_time->format('Y-m-d H:i:s');
        $scrubbed_file = ScrubbedFtpFile::where('coid', $coid)
                                            ->where('file_name', $orig_file_name)
                                            ->first();

        if($scrubbed_file)
        {
            ScrubbedFtpFile::where('coid', $coid)
                            ->where('file_name', $orig_file_name)
                            ->update([
                                        'last_uploaded' => $ftp_scrubbed_last_uploaded
                                    ]);
        } else {
            $scrubbed_ftp_file = new ScrubbedFtpFile;
            $scrubbed_ftp_file->coid = $coid;
            $scrubbed_ftp_file->file_name = $orig_file_name;
            $scrubbed_ftp_file->last_uploaded = $ftp_scrubbed_last_uploaded;
            $scrubbed_ftp_file->save();
        }

        if($request->is_single_file == 1)
        {
            $today_date_to_show = date("m/d/Y H:i:s");
            return response()->json([
                'message' => 'Done!',
                'new_file_name' => $new_orig_file_name,
            ]);
        } else {
            $commonFunctions = new CommonFunctionsController();
            $filetemplate = DB::connection('mysql2')
                                            ->table('file_templates')
                                            ->where('coid', $coid)
                                            ->where('filename', $new_orig_file_name)
                                            ->first();

            $departments_options = $commonFunctions->getDepartmentsSelectOptions($company->department_access);
            $calltypes_options = $commonFunctions->getCalltypesSelectOptions();

            $content = view('partials.scrubbed_file')->with([
                                                                'file_name' => $new_orig_file_name,
                                                                'company' => $company,
                                                                'filetemplate' => $filetemplate,
                                                                'departments_options' => $departments_options,
                                                                'calltypes_options' => $calltypes_options,
                                                            ])
                                                            ->render();
                //exit;
            return response()->json([
                'message' => 'Done!',
                'file_name' => $new_orig_file_name,
                'form_id' => str_replace('.','', $new_orig_file_name),
                'content' => $content,
            ]);
        }
    }
}
