<?php

namespace App\Http\Controllers;

use App\UploadedFile;
use App\FileType;
use App\CallType;
use App\UserDealer;
use App\Imports\VinsImport;
use App\Imports\FileTemplateImport;
use App\Imports\CallRecordsImport;
use Maatwebsite\Excel\Facades\Excel;
use Illuminate\Support\Facades\Auth;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;

class UploadedFileController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {

        if(Auth::user()->user_type->name == 'Admin') 
        {
            $dealers = DB::connection('mysql2')->table('callcenter_companies')
                                ->select('co_name', 'co_short')
                                ->where('is_active', 1)
                                ->orderBy('co_name')
                                ->get();
        } else if(Auth::user()->user_type->name == 'Dealer') {
            $user_companies_array = array();
            foreach(Auth::user()->dealers as $company)
            {
              $user_companies_array[] = $company->dealer_id;
            }

            $dealers = DB::connection('mysql2')->table('callcenter_companies')
                                ->select('co_name', 'co_short')
                                ->whereRaw("FIND_IN_SET(coid, '".implode(',', $user_companies_array)."')")
                                ->get();
        }

        

        $commonFunctions = new CommonFunctionsController();
        $commonFunctions->modelToSelectOptions($dealers,'co_name', 'co_short');
        $dealers_options = $commonFunctions->select_options;

        $commonFunctions->modelToSelectOptions(FileType::whereRaw('coid IS NULL')->get(),'name', 'id');
        $file_templates = $commonFunctions->select_options;

        $commonFunctions->modelToSelectOptions(CallType::all(),'name', 'id');
        $call_types = $commonFunctions->select_options;

        return view('uploads.fileupload', [
                                            'dealers' => $dealers_options,
                                            'file_templates' => $file_templates,
                                            'call_types' => $call_types,
                                        ]);
    }


    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function search(Request $request)
    {
        $file_templates = FileType::whereRaw('coid IS NULL')->get();
        $file_templates_options = '<option value="">Select</option>';
        foreach ($file_templates as $key => $value) {
            $file_templates_options.= '<option value="'.$value->id.'">'.$value->name.'</option>';
        }

        if($request->coid != '')
        {
            $company = DB::connection('mysql2')->table('callcenter_companies')
                                ->select('*')
                                ->where('co_short', $request->coid)
                                ->first();

            $saved_file_template = FileType::where('coid', $company->coid)
                                    ->get();

            foreach ($saved_file_template as $key => $value) {
                $file_templates_options.= '<option value="'.$value->id.'">'.$value->name.'</option>';
            }
        }

        $output = array(
            'file_templates_options' => $file_templates_options,
        );

        return response()->json($output);
    }

    public function editFileTemplateOld(Request $request)
    {
        if($request->file_template != '')
        {
            $file_template = FileType::find($request->file_template);
        } else {
            $file_template = '';
        }

        
        $output = array(
            'file_template' => $file_template,
        );

        return response()->json($output);
    }

    public function editFileTemplate(Request $request)
    {
        if($request->coid != '')
        {
            $company = DB::connection('mysql2')->table('callcenter_companies')
                                ->select('*')
                                ->where('co_short', $request->coid)
                                ->first();
            $request->merge([
                'coid' => $company->coid,
            ]);
        }
        
        if($request->coid != '')
        {
            $file_type = FileType::where('name', $request->orig_filename_template)
                                ->where('coid', $request->coid)
                                ->first();
        } elseif($request->file_template != '') {
            $file_type = FileType::find($request->file_template);
        }


        $added_columns = explode(',', $request->added_columns);
        $added_values = explode(',', $request->added_values);

        $i=0;
        $added_columns_array = array();
        foreach ($added_columns as $key => $value) {
            $added_columns_array[$value] = $added_values[$i];
            $i++;
        }

        $added_columns = json_encode($added_columns_array);

        $request->merge([
                'added_columns' => $added_columns,
            ]);
        
        if(!$file_type)
        {
            $file_type = new FileType;
            if($request->coid != '')
            {
                $file_type->coid = $request->coid;
            }
            $file_type->name = $request->orig_filename_template;
            $file_type->template = $request->template;
            if($request->call_type_id != '')
            {
                $file_type->call_type_id = $request->call_type_id;
            }
            $file_type->added_columns = $request->added_columns;
            $file_type->save();
            $type = "added";
        } else {
            FileType::where('id',$file_type->id)
                        ->update(
                            $request->only( 'template', 
                                            'call_type_id',
                                            'added_columns',
                                        )
                        );
            $type = "updated";
        }

        $template = explode(',', $file_type->template);
        $has_calltype_column = (array_search('call_type', $template) !== false)? 1 : '';
        
        $output = array(
            'file_template_id' => $file_type->id,
            'file_template_name' => $file_type->name,
            'has_calltype_column' => $has_calltype_column,
            'type' => $type,
        );

        return response()->json($output);
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        //
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        $messages = [
            'upload_file.required' => 'File required!',
            'upload_file.mimes' => 'Invalid file format',
            'upload_file.max' => 'Max file size is 25M.',
        ];

        $validatedData = $request->validate([
            'upload_file' => 'required|mimes:xlsx,xls,csv,txt|max:25600‬',
        ], $messages);

        $uploaded_file = $request->file('upload_file');

         
        $orig_file_name = $uploaded_file->getClientOriginalName();
        $new_name = rand() . '.' . time(). '.' . $orig_file_name . '.' . $uploaded_file->getClientOriginalExtension();
        $uploaded_file->move(public_path('uploaded_files'), $new_name);

        if($request->upload_type != 'template')
        {
            $file_upload_record = new UploadedFile;
            $file_upload_record->file_name = $new_name;
            $file_upload_record->user_id = Auth::id();
            $file_upload_record->save();

            if($request->file_type == 'leo')
            {
                $db_table = $request->db_table.'_master_vins';
                DB::connection('mysql')->table($db_table)->delete();
                Excel::import($excel_import = new VinsImport($db_table), 'uploaded_files/'.$new_name);
            } else {
                if($request->file_type == 'toyota_care')
                {
                   $db_table = 'toyota_care_'.$request->db_table; 
                } else if($request->file_type == 'recall') {
                    $db_table = 'recall_records';
                } else if($request->file_type == 'call_records'){
                    $db_table = 'call_records_'.$request->db_table;
                }
                
                $file_template = FileType::find($request->file_template);
                $template = explode(',', $file_template->template);
                $calltypes = CallType::all();
                $calltypes_keys = array();
                foreach ($calltypes as $key => $value) {
                    $calltypes_keys[strtoupper($value->key)] = $value->id;
                }

                $company = DB::connection('mysql2')->table('callcenter_companies')
                                ->select('*')
                                ->where('co_short', $request->db_table)
                                ->first();

                $upload_options = new \stdClass();
                $upload_options->calltypes_keys = $calltypes_keys;
                $upload_options->file_type = $request->file_type;
                $upload_options->coid = $company->coid;
                $upload_options->added_columns = $file_template->added_columns;
                $upload_options->use_calltype_column = isset($request->use_calltype_column)? $request->use_calltype_column : 0;

                Excel::import($excel_import = new CallRecordsImport($db_table, array_flip($template), $file_template, $request->call_type, $file_upload_record->id, $upload_options), 'uploaded_files/'.$new_name);
            }

            
            $output = array(
                'error' => $excel_import->getErrorMessage(),
                'row_count' => $excel_import->getRowCount(),  
                'success' => 'File successfuly uploaded.',
                'upload_type' => $request->upload_type,  
                'download_link'  => $excel_import->getRowCount().' rows added.'
            );
        } elseif($request->upload_type == 'template')
        {
            $output = array(
                'error' => '',
                'success' => 'File successfuly uploaded.',
                'uploaded_file' => 'uploaded_files/'.$new_name,
                'orig_file_name' => $orig_file_name,
                'upload_type' => $request->upload_type,  
            );
        }

        

        return response()->json($output);
    }

    public function showTemplateDialog(Request $request)
    {   
        if($request->file_template != '')
        {
            $file_template = FileType::find($request->file_template);
            $template = explode(',', $file_template->template);
            $added_columns = json_decode($file_template->added_columns, true);
            if(!$added_columns)
            {
                $added_columns = array();
            }
        } else {
            $template = array();
            $added_columns = array();
        }

        $columns = array(
                            'dealer_id' => 'Dealer ID', 
                            'vin' => 'VIN', 
                            'pma' => 'PMA',
                            'fullname' => 'Full Name',
                            'first_name' => 'First Name',
                            'last_name' => 'Last Name',
                            'phone' => 'Phone',
                            'cellphone' => 'Cellphone',
                            'address' => 'Address',
                            'city' => 'City',
                            'state' => 'State',
                            'zip' => 'Zip',
                            'email' => 'Email',
                            'vehicle_year' => 'Vehicle Year',
                            'vehicle_make' => 'Vehicle Make',
                            'vehicle_model' => 'Vehicle Model',
                            'effective_date' => 'Effective Date',
                            'expiration_date' => 'Expiration Date',
                            'oil_type' => 'Oil Type',
                            'days_last_ro' => 'Days Last RO',
                            'days_last_tc' => 'Days Last TC',
                            'days_last_exp' => 'Days Last Exp',
                            'last_service_advisor' => 'Last Service Advisor',
                            'estimated_mileage' => 'Estimated Mileage',
                            'actual_mileage' => 'Actual Mileage',
                            'distance_from_dealer' => 'Distance From Dealer',
                            'call_type' => 'Call Type',
                            'priority_group' => 'Priority Group',
                            'notes' => 'Notes',
                            'new_owner_date' => 'New Owner Date',
                            'recall_code' => 'Recall Code',
                            'category' => 'Category',
                        );
        asort($columns);
        Excel::import($excel_import = new FileTemplateImport(), $request->file);
        
        
        $num_columns = 0;
        foreach ($excel_import->rows as $row) {
            $num_columns = count($row);
            break;
        }

        $column_templates = array();
        for ($i=0; $i < $num_columns; $i++) { 
            $column_templates[$i] = '<select id="'.$i.'_file_template_column_value" name="'.$i.'_file_template_column_value" class="file_template_select">
                                        <option value="ignore">Ignore</option>';
            foreach ($columns as $key => $value) {
                $template[$i] = (isset($template[$i]))? $template[$i] : '';
                $selected = ($key == $template[$i])? 'selected' : '';
                $column_templates[$i].= '<option value="'.$key.'" '.$selected.'>'.$value.'</option>';
            }
            $column_templates[$i].= '</select>';
        }

        return view('dialogs.file_template', [
                                            'file_rows' => $excel_import->rows,
                                            'column_templates' => $column_templates,
                                            'file_template' => $request->file_template,
                                            'orig_filename' => $request->orig_filename,
                                            'added_columns' => $added_columns,
                                        ]);
    }

    public function checkFileTemplate(Request $request)
    {
        $file_template = FileType::find($request->file_template);
        $template = explode(',', $file_template->template);
        $has_calltype_column = (array_search('call_type', $template) !== false)? 1 : '';
        $output = array(
                            'has_calltype_column' => $has_calltype_column,
                        );
        return response()->json($output);
    }

    /**
     * Display the specified resource.
     *
     * @param  \App\UploadedFile  $uploadedFile
     * @return \Illuminate\Http\Response
     */
    public function show(UploadedFile $uploadedFile)
    {
        //
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  \App\UploadedFile  $uploadedFile
     * @return \Illuminate\Http\Response
     */
    public function edit(UploadedFile $uploadedFile)
    {
        //
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  \App\UploadedFile  $uploadedFile
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, UploadedFile $uploadedFile)
    {
        //
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  \App\UploadedFile  $uploadedFile
     * @return \Illuminate\Http\Response
     */
    public function destroy(UploadedFile $uploadedFile)
    {
        //
    }
}
