<?php

namespace App\Http\Controllers;

use App\User;
use App\UserType;
use App\UserDealer;
use App\UserUserGroup;
use App\UserCalltype;
use App\Events\UserAlert;
use App\Imports\UsersImport;
use Illuminate\Validation\Rule;
use Illuminate\Http\Request;
use App\Events\GeneralAlertEvent;
use Maatwebsite\Excel\Facades\Excel;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Facades\DB;

class UserController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {
        if(Auth::user()->user_type->name == 'Admin') 
        {
            $users = User::paginate(20);
        }

        $commonFunctions = new CommonFunctionsController();
        $dealer_names = $commonFunctions->getDealersNames();

        return view('pages.users', [
                                        'users' => $users,
                                        'dealer_names' => $dealer_names
                                    ]);
    }

    /**
     * Display a list of the resource with pagination.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */

    public function pagination(Request $request)

    {
        $current_user_id = 0;
        $no_links = false;
        if(Auth::user()->user_type->name == 'Admin') 
        {
            if($request->id != "" && $request->id != null) {
                $users = User::where('id', '=', $request->id)->paginate(20);
                $no_links = false;
                $current_user_id = $request->id;
            } else if($request->showall == "1") {
                $users = User::all();
                $no_links = true;
            } else {
                $users = User::paginate(20);
            }
        }

        $commonFunctions = new CommonFunctionsController();
        $dealer_names = $commonFunctions->getDealersNames();

        if ($request->ajax()) {
            return view('lists.users_list', [
                                                'users' => $users, 
                                                'no_links' => $no_links, 
                                                'current_user_id' => $current_user_id,
                                                'dealer_names' => $dealer_names
                                            ]);
        }
    }


    /**
     * Display the specified resource in JSON format.
     *
     * @param  \App\Company  $company
     * @return \Illuminate\Http\Response
     */
    public function search(Request $request)
    {
        if(Auth::user()->user_type->name == 'Admin') 
        {
            $search_value = trim($request->input('search_value'));
            $users = User::where('first_name', 'like', '%'.$search_value.'%')
            ->orWhere('last_name', 'like', '%'.$search_value.'%')
            ->orWhereRaw('CONCAT(first_name," ",last_name) = "'.$search_value.'"')
            ->orWhere('email', 'like', '%'.$search_value.'%')
            ->get();
        }
        $users_result = array();
        foreach($users as $user) {
            $users_result[] = array(
                                        "id"        => $user->id,
                                        "value"  => $user->first_name.' '.$user->last_name
                                    );
        }
        return response()->json($users_result);
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        $commonFunctions = new CommonFunctionsController();
        $commonFunctions->modelToSelectOptions(UserType::all(),'name', 'id');
        $user_types = $commonFunctions->select_options;

        $user = new \stdClass();
        $user->id = "";
        $user->first_name = "";
        $user->last_name = "";
        $user->reference_id = '';
        $user->email = "";
        $user->phone = "";
        $user->cellphone = "";
        $user->user_name = "";
        $user->user_type_id = "";
        $user->country_id = "";
        $user->coordinator = "";
        $user->sms_access = "";
        $user->calls_per_day_avg = "";
        $user->assignments_priority = "";
        //$user->clients = new \stdClass();
        $user->user_groups = new \stdClass();
        $user->dealers = new \stdClass();
        $dealer_names = array();
        $user_calltypes = array();

        $call_types = DB::connection('mysql2')->table('callcenter_calltypes')
                                ->select('calltype_text as name', 'cid as id')
                                ->orderBy('calltype_text')
                                ->get();

        return view('dialogs.add_user_dialog', [
                                                        'user' =>$user, 
                                                        'user_types' => $user_types,
                                                        'dealer_names' => $dealer_names,
                                                        'call_types' => $call_types,
                                                        'user_calltypes' => $user_calltypes,
                                                    ]);
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        /*$messages = [
            'phone.regex' => 'El formato del número de teléfono no es válido.',
            'cellphone.regex' => 'El formato del número de celular no es válido.',
            'email.unique' => 'Ya existe otro usuario con el mismo correo electrónico',
            'user_name.unique' => 'Ya existe el mismo nombre de usuario, por favor elija otro.',
            'user_name.min' => 'El nombre de usuario debe tener al menos 5 caracteres.',
            'password.confirmed' => 'Las contraseñas no coinciden.',
            'user_type_id.required' => 'Por favor elija el tipo de usuario.',
            'country_id.required' => 'Por favor elija un país.',
            'password.min' => 'La contraseña debe tener al menos 6 dígitos',
        ];*/

        $validatedData = $request->validate([
            'first_name' => 'required|max:191',
            'last_name' => 'max:191|nullable',
            'phone' => 'regex:/^[0-9\-\(\)\/\+\s]*$/|max:50|nullable',
            'cellphone' => 'regex:/^[0-9\-\(\)\/\+\s]*$/|max:50|nullable',
            'email' => "required|email|unique:users,email,NULL,id,deleted_at,NULL",
            'user_type_id' => 'required',
            'password' => 'required|confirmed|min:6',
        ]);

        $user = new User;
        $user->first_name = $request->first_name;
        $user->last_name = $request->last_name;
        $user->reference_id = $request->reference_id;
        $user->phone = $request->phone;
        $user->cellphone = $request->cellphone;
        $user->email = $request->email;
        $user->sms_access = $request->sms_access;
        $user->calls_per_day_avg = $request->calls_per_day_avg;
        $user->assignments_priority = $request->assignments_priority;
        $user->user_type_id = $request->user_type_id;
        
        /*if(isset($request->coordinator))
        {
            $user->coordinator = $request->coordinator;
        } else {
            $user->coordinator = 0;
        }*/
        $user->password = Hash::make($request->password);
        $user->save();

        if(trim($request->dealer_access) != '')
        {
            $dealer_access = explode(',', $request->dealer_access);
            foreach ($dealer_access as $key => $value) {
                $user_acces_level = new UserDealer;
                $user_acces_level->user_id = $user->id;
                $user_acces_level->dealer_id = $value;
                $user_acces_level->save();
            }
        }

        if(trim($request->user_groups) != '')
        {
            $user_groups = explode(',', $request->user_groups);
            foreach ($user_groups as $key => $value) {
                $user_user_group = new UserUserGroup;
                $user_user_group->user_id = $user->id;
                $user_user_group->user_group_id = $value;
                $user_user_group->save();
            }
        }

        if(isset($request->user_calltypes))
        {
            foreach ($request->user_calltypes as $key => $value) {
                $user_calltype = new UserCalltype;
                $user_calltype->user_id = $user->id;
                $user_calltype->call_type_id = $value;
                $user_calltype->save();
            }
        }

        return response()->json([
            'inserted_id' => $user->id,
            'message' => 'User created.'
        ]);
    }

    /**
     * Display the specified resource.
     *
     * @param  \App\User  $user
     * @return \Illuminate\Http\Response
     */
    public function show(User $user)
    {
        //
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  \App\User  $user
     * @return \Illuminate\Http\Response
     */
    public function edit(Request $request)
    {
        $user = User::find($request->user_id);
        $commonFunctions = new CommonFunctionsController();

        $commonFunctions->modelToSelectOptions(UserType::all(),'name', 'id');
        $user_types = $commonFunctions->select_options;

        //$user->dealers = new \stdClass();
        $dealer_names = $commonFunctions->getDealersNames();

        $user_calltypes =  array();
        foreach ($user->user_calltypes as $key => $value) {
            $user_calltypes[] = $value->call_type_id;
        }

        $call_types = DB::connection('mysql2')->table('callcenter_calltypes')
                                ->select('calltype_text as name', 'cid as id')
                                ->orderBy('calltype_text')
                                ->get();
        
        return view('dialogs.add_user_dialog', [
                                                    'user' =>$user,  
                                                    'user_types' => $user_types, 
                                                    'dealer_names' => $dealer_names, 
                                                    'call_types' => $call_types, 
                                                    'user_calltypes' => $user_calltypes, 
                                                ]);
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  \App\User  $user
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, User $user)
    {
        /*$messages = [
            'phone.regex' => 'El formato del número de teléfono no es válido.',
            'cellphone.regex' => 'El formato del número de celular no es válido.',
            'email.unique' => 'Ya existe otro usuario con el mismo correo electrónico',
            'user_name.unique' => 'Ya existe el mismo nombre de usuario, por favor elija otro.',
            'user_name.min' => 'El nombre de usuario debe tener al menos 5 caracteres.',
            'password.confirmed' => 'Las contraseñas no coinciden.',
            'user_type_id.required' => 'Por favor elija el tipo de usuario.',
            'country_id.required' => 'Por favor elija un país.',
        ];*/

        $validatedData = $request->validate([
            'first_name' => 'required|max:191',
            'last_name' => 'max:191|nullable',
            'phone' => 'regex:/^[0-9\-\(\)\/\+\s]*$/|max:50|nullable',
            'cellphone' => 'regex:/^[0-9\-\(\)\/\+\s]*$/|max:50|nullable',
            'email' => "required|email|unique:users,email,".$request->user_id.",id,deleted_at,NULL",
            'user_type_id' => 'required',
            'password' => 'confirmed',
        ]);

        $user = User::find($request->user_id);
        if(trim($request->password) == '')
        {
            $request->merge([
                'password' => $user->password,
            ]);
        } else 
        {
            $request->merge([
                'password' => Hash::make($request->password),
            ]);
        }
        
        /*if(!isset($request->coordinator))
        {
            $request->merge([
                'coordinator' => 0,
            ]);
        }*/

        User::where('id',$request->user_id)
                ->update(
                    $request->only( 'first_name', 
                                    'last_name',
                                    'phone',
                                    'reference_id',
                                    'email',
                                    'sms_access',
                                    'cellphone',
                                    'password',
                                    'calls_per_day_avg',
                                    'assignments_priority',
                                    'user_type_id'
                                )
                );

        User::where('id',$request->user_id)
        ->update([
            'sms_access' => $request->sms_access? : 0,
        ]);
        
        UserDealer::where('user_id', $user->id)->delete();
        if(trim($request->dealer_access) != '')
        {
            $dealer_access = explode(',', $request->dealer_access);
            foreach ($dealer_access as $key => $value) {
                $user_acces_level = new UserDealer;
                $user_acces_level->user_id = $user->id;
                $user_acces_level->dealer_id = $value;
                $user_acces_level->save();
            }
        }

        if(trim($request->user_groups) != '')
        {
            UserUserGroup::where('user_id', $user->id)
                        ->whereRaw('user_group_id NOT IN ('.$request->user_groups.')')
                        ->delete();

            $user_user_groups = UserUserGroup::where('user_id', $user->id)->get();
            $user_groups_array = array();
            foreach ($user_user_groups as $key => $value) {
                $user_groups_array[$value->user_group_id] = $value->user_group_id;
            }
            
            $user_groups = explode(',', $request->user_groups);
            foreach ($user_groups as $key => $value) {
                if(!isset($user_groups_array[$value]))
                {
                    $user_user_group = new UserUserGroup;
                    $user_user_group->user_id = $user->id;
                    $user_user_group->user_group_id = $value;
                    $user_user_group->save();
                } 
            }
        } else {
            UserUserGroup::where('user_id', $user->id)->delete();
        }

        UserCalltype::where('user_id', $user->id)->delete();
        if(isset($request->user_calltypes))
        {
            foreach ($request->user_calltypes as $key => $value) {
                $user_calltype = new UserCalltype;
                $user_calltype->user_id = $user->id;
                $user_calltype->call_type_id = $value;
                $user_calltype->save();
            }
        }

        return response()->json([
            'inserted_id' => $user->id,
            'message' => 'User updated.'
        ]);
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  \App\User  $user
     * @return \Illuminate\Http\Response
     */
    public function destroy(Request $request, User $user)
    {
        User::where('id', $request->user_id)->delete();
        return response()->json([
            'removed_id' => $request->user_id,
            'message' => 'User removed.'
        ]);
    }


    public function accountSettings()
    {
        return view('pages.my_account', ['user' => Auth::user()]);
    }

    public function changeAccountSettings(Request $request)
    {
        /*$messages = [
            'password.confirmed' => 'Las contraseñas no coinciden.',
        ];
*/
        $validatedData = $request->validate([
            'password' => 'confirmed',
        ]);

        $user = User::find($request->user_id);
        if(trim($request->password) == '')
        {
            $request->merge([
                'password' => $user->password,
            ]);
        } else 
        {
            if(strlen(trim($request->password)) < 6)
            {
                return response()->json([
                    'error' => 'Password must be at least 6 characters long'
                ]);
            }
            $request->merge([
                'password' => Hash::make($request->password),
            ]);
        }

        $phone = preg_replace("/[^0-9]/i", "", $request->phone);
        $cell_phone = preg_replace("/[^0-9]/i", "", $request->cell_phone);

        $request->merge([
                'phone' => $phone,
                'cellphone' => $cell_phone,
            ]);
        
        User::where('id',$request->user_id)
                ->update(
                    $request->only( 
                                    'phone',
                                    'cellphone',
                                    'password',
                                )
                );

        return response()->json([
            'inserted_id' => $user->id,
            'error' => '',
            'message' => 'Your data was updated.'
        ]);
    }



    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function LoginOnTheFly(Request $request)
    {
         $validatedData = $request->validate([
            'email' => "required|email",
            'password' => ['required'],
        ]);
        
        if(Auth::guard('web')->attempt(['email' => $request->email, 'password' => $request->password]))
        {       
            $alert_data = new \stdClass();
            $alert_data->type = 'login';
            $alert_data->token = csrf_token();
            $channel_id = '.'.Auth::id();

            broadcast(new UserAlert($alert_data, $channel_id));

            return response()->json([
                    'status' => 'success',
                    'message' => 'You have succesfully logged in!',
                    'token' => $alert_data->token,
                    'id' => Auth::user()->id
                ]);
        } else {
            return response()->json([
                    'errors' => [
                                'email' =>'Wrong Email or Password'
                                ]
                    ], 500);
        }
    }


    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  \App\User  $user
     * @return \Illuminate\Http\Response
     */
    public function blockUser(Request $request, User $user)
    {
        User::where('id',$request->user_id)
                ->update([
                            'blocked' => $request->blocked
                        ]);

        $alert_data = new \stdClass();
        $alert_data->type = 'blocked';
        $alert_data->reference = $request->user_id;
        $alert_data->blocked = $request->blocked;
        // broadcast(new UserAlert($alert_data, '.'.$request->user_id));

        return response()->json([
            'inserted_id' => $user->id,
            'message' => 'User updated.'
        ]);
    }

    public function importFullBdcUsers() 
    {
        Excel::import(new UsersImport, 'uploaded_files/users_import.xlsx');
    }

    /**
     * Gets the callers calls per hour avg.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function getCallerCallsPerHourAvg(Request $request)
    {
        $user = User::find($request->user_id);
        $time_on_calls = DB::connection('mysql4')->table('employee_assignment')
                                                ->selectRaw('SUM(time_spent) as total_time, COUNT(callid) as total_count')
                                                ->where('employee_uid', $user->reference_id)
                                                ->where('time_spent', '>', 0)
                                                ->whereNotNull('time_spent')
                                                ->first();
        $avg_calls_per_hour = 0;
        if($time_on_calls)
        {
            $hours = $time_on_calls->total_time/60/60;
            $avg_calls_per_hour = $time_on_calls->total_count/$hours;
        }
         
        return response()->json([
            'avg_calls_per_hour' => floor($avg_calls_per_hour),
        ]);
    }

    /**
     * Logs user out.
     *
     * @return void
     */
    public function LogOut() 
    {
        Auth::logout();
        return view('auth.login');
    }

    public function keepSessionAlive() 
    {
        return response()->json([
            'message' => 'session alive!',
        ]);
    }
}
