<?php

namespace App\Http\Controllers;

use App\UserGroup;
use App\UserGroupDealer;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\DB;


class UserGroupController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {
        if(Auth::user()->user_type->name == 'Admin') 
        {
            $user_groups = UserGroup::orderBy('name')->paginate(20);
            $dealers = DB::connection('mysql2')->table('callcenter_companies')
                                ->select('co_name', 'co_short', 'coid')
                                ->where('is_active', 1)
                                ->orderBy('co_name')
                                ->get();
        } else if(Auth::user()->user_type->name == 'Dealer') {
            $user_dealers_array = array();
            foreach (Auth::user()->dealers as $key => $value) {
                $user_dealers_array[] =  $value->dealer_id;
            }

            $user_groups = UserGroup::whereRaw("FIND_IN_SET(coid, '".implode(',', $user_dealers_array)."')")
                                    ->orderBy('name')
                                    ->paginate(20);

            $dealers = DB::connection('mysql2')->table('callcenter_companies')
                                ->select('co_name', 'co_short', 'coid')
                                ->whereRaw("FIND_IN_SET(coid, '".implode(',', $user_dealers_array)."')")
                                ->get();
        }

        $dealer_names = array();
        foreach ($dealers as $key => $value) {
            $dealer_names[$value->coid] = $value->co_name;
        }


        $commonFunctions = new CommonFunctionsController();
        $commonFunctions->modelToSelectOptions($dealers,'co_name', 'coid');
        $dealers_options = $commonFunctions->select_options;

        return view('pages.user_groups', [
                                                'user_groups' => $user_groups,
                                                'dealers_options' => $dealers_options,
                                                'dealer_names' => $dealer_names,
                                            ]);
    }


    public function getRawSearch($my_request) {
        $raw_search_array = array();

        /*if($my_request->file_type_id_search != 'all' && $my_request->file_type_id_search != '')
        {
            $raw_search_array[] = " file_type_id = '".$my_request->file_type_id_search."' ";
        }*/

        if($my_request->coid_search != 'all' && $my_request->coid_search != '')
        {
            $raw_search_array[] = " coid = '".$my_request->coid_search."' ";
        }

        if(trim($my_request->value_search) != '')
        {
            $raw_search_array[] = " (
                                        name like '%".$my_request->value_search."%'  
                                    )";
        }

        $raw_search = implode(' AND ', $raw_search_array);

        return $raw_search;
    }


    /**
     * Display a list of the resource with pagination.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */

    public function pagination(Request $request)

    {
        $current_user_group_id = 0;
        $no_links = false;
        $raw_search = $this->getRawSearch($request);

        if(Auth::user()->user_type->name == 'Admin') 
        {
            if($request->id != "" && $request->id != null) {
                $user_groups = UserGroup::where('id', '=', $request->id)->paginate(20);
                $no_links = false;
                $current_user_group_id = $request->id;
            } else if($request->showall == "1") {
                $user_groups = UserGroup::orderBy('name')->get();
                $no_links = true;
            } else {
                if($raw_search != '')
                {
                    $user_groups = UserGroup::whereRaw($raw_search)->orderBy('name')->paginate(20);
                } else {
                    $user_groups = UserGroup::orderBy('name')->paginate(20);
                }
            }

            $dealers = DB::connection('mysql2')->table('callcenter_companies')
                                ->select('co_name', 'co_short', 'coid')
                                ->where('is_active', 1)
                                ->orderBy('co_name')
                                ->get();
        } else if(Auth::user()->user_type->name == 'Dealer') {
            $user_dealers_array = array();
            foreach (Auth::user()->dealers as $key => $value) {
                $user_dealers_array[] =  $value->dealer_id;
            }

            if($request->id != "" && $request->id != null) {
                $training_alerts = TrainingAlert::where('id', '=', $request->id)->paginate(20);
                $no_links = false;
                $current_user_group_id = $request->id;
            } else if($request->showall == "1") {
                $user_groups = UserGroup::whereRaw("FIND_IN_SET(coid, '".implode(',', $user_dealers_array)."')")
                                    ->orderBy('name')
                                    ->get();
                $no_links = true;
            } else {
                if($raw_search != '')
                {
                    $user_groups = UserGroup::whereRaw("FIND_IN_SET(coid, '".implode(',', $user_dealers_array)."')")
                                    ->whereRaw($raw_search)
                                    ->orderBy('name')
                                    ->paginate(20);
                } else {
                    $user_groups = UserGroup::whereRaw("FIND_IN_SET(coid, '".implode(',', $user_dealers_array)."')")
                                    ->orderBy('name')
                                    ->paginate(20);
                }
            }

            $dealers = DB::connection('mysql2')->table('callcenter_companies')
                                ->select('co_name', 'co_short', 'coid')
                                ->whereRaw("FIND_IN_SET(coid, '".implode(',', $user_dealers_array)."')")
                                ->get();
        }


        $dealer_names = array();
        foreach ($dealers as $key => $value) {
            $dealer_names[$value->coid] = $value->co_name;
        }

        if ($request->ajax()) {
            return view('lists.user_groups_list', [
                                                'user_groups' => $user_groups, 
                                                'no_links' => $no_links, 
                                                'dealer_names' => $dealer_names, 
                                                'current_user_group_id' => $current_user_group_id,
                                            ]);
        }
    }

    public function search(Request $request)
    {
        $search_value = trim($request->input('search_value'));
        $user_groups = UserGroup::where('name', 'like', '%'.$search_value.'%')->orderBy('name')->get();
        $user_groups_result = array();
        foreach($user_groups as $user_group) {
            $user_groups_result[] = array(
                                        "id"        => $user_group->id,
                                        "value"  => $user_group->name
                                    );
        }
        return response()->json($user_groups_result);
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {

        $user_group = new \stdClass();
        $user_group->id = "";
        $user_group->name = "";
        $user_group->coid = "";

        if(Auth::user()->user_type->name == 'Admin') 
        {
            $dealers = DB::connection('mysql2')->table('callcenter_companies')
                                ->select('co_name', 'co_short', 'coid')
                                ->where('is_active', 1)
                                ->orderBy('co_name')
                                ->get();
        } else if(Auth::user()->user_type->name == 'Dealer') {
            $user_dealers_array = array();
            foreach (Auth::user()->dealers as $key => $value) {
                $user_dealers_array[] =  $value->dealer_id;
            }

            $dealers = DB::connection('mysql2')->table('callcenter_companies')
                                ->select('co_name', 'co_short', 'coid')
                                ->whereRaw("FIND_IN_SET(coid, '".implode(',', $user_dealers_array)."')")
                                ->get();
        }

        $commonFunctions = new CommonFunctionsController();
        $commonFunctions->modelToSelectOptions($dealers,'co_name', 'coid');
        $dealers_options = $commonFunctions->select_options;

        $dealer_names = $commonFunctions->getDealersNames();

        $user_group->dealers = new \stdClass();

        return view('dialogs.add_user_group', [
                                                        'user_group'      => $user_group, 
                                                        'dealers_options' => $dealers_options,
                                                        'dealer_names'    => $dealer_names,
                                                    ]);
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        $validatedData = $request->validate([
            'name' => 'required|max:191',
            'coid' => 'nullable'
        ]);

        $user_group = new UserGroup;
        $user_group->name = $request->name;
        $user_group->coid = $request->coid;
        $user_group->save();

        if(trim($request->dealer_access) != '')
        {
            $dealer_access = explode(',', $request->dealer_access);
            foreach ($dealer_access as $key => $value) {
                $user_acces_level = new UserGroupDealer;
                $user_acces_level->user_group_id = $user_group->id;
                $user_acces_level->dealer_id = $value;
                $user_acces_level->save();
            }
        }

        return response()->json([
            'inserted_id' => $user_group->id,
            'message' => 'User group created.'
        ]);
    }

    /**
     * Display the specified resource.
     *
     * @param  \App\UserGroup  $userGroup
     * @return \Illuminate\Http\Response
     */
    public function show(UserGroup $userGroup)
    {
        //
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  \App\UserGroup  $userGroup
     * @return \Illuminate\Http\Response
     */
    public function edit(Request $request, UserGroup $userGroup)
    {
        $user_group = UserGroup::find($request->user_group_id);
        
        if(Auth::user()->user_type->name == 'Admin') 
        {
            $dealers = DB::connection('mysql2')->table('callcenter_companies')
                                ->select('co_name', 'co_short', 'coid')
                                ->where('is_active', 1)
                                ->orderBy('co_name')
                                ->get();
        } else if(Auth::user()->user_type->name == 'Dealer') {
            $user_dealers_array = array();
            foreach (Auth::user()->dealers as $key => $value) {
                $user_dealers_array[] =  $value->dealer_id;
            }

            $dealers = DB::connection('mysql2')->table('callcenter_companies')
                                ->select('co_name', 'co_short', 'coid')
                                ->whereRaw("FIND_IN_SET(coid, '".implode(',', $user_dealers_array)."')")
                                ->get();
        }

        $commonFunctions = new CommonFunctionsController();
        $commonFunctions->modelToSelectOptions($dealers,'co_name', 'coid');
        $dealers_options = $commonFunctions->select_options;

        $dealer_names = $commonFunctions->getDealersNames();

        return view('dialogs.add_user_group', [
                                                        'user_group'      => $user_group, 
                                                        'dealers_options' => $dealers_options,
                                                        'dealer_names'    => $dealer_names,
                                                    ]);
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  \App\UserGroup  $userGroup
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, UserGroup $userGroup)
    {

        $validatedData = $request->validate([
            'name' => 'required|max:191',
            'coid' => 'nullable'
        ]);

        UserGroup::where('id', $request->user_group_id)
                ->update(
                    $request->only( 'name', 
                                    'coid'
                                )
                );

        UserGroupDealer::where('user_group_id', $request->user_group_id)->delete();
        if(trim($request->dealer_access) != '')
        {
            $dealer_access = explode(',', $request->dealer_access);
            foreach ($dealer_access as $key => $value) {
                $user_acces_level = new UserGroupDealer;
                $user_acces_level->user_group_id = $request->user_group_id;
                $user_acces_level->dealer_id = $value;
                $user_acces_level->save();
            }
        }

        return response()->json([
            'inserted_id' => $request->user_group_id,
            'message' => 'User group updated.'
        ]);
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  \App\UserGroup  $userGroup
     * @return \Illuminate\Http\Response
     */
    public function destroy(Request $request, UserGroup $userGroup)
    {
        UserGroup::where('id', $request->user_group_id)->delete();
        UserGroupDealer::where('user_group_id', $request->user_group_id)->delete();
        return response()->json([
            'removed_id' => $request->user_group_id,
            'message' => 'User group removed.'
        ]);
    }
}
