<?php

namespace App\Http\Controllers;

use DateTime;
use App\User;
use App\Company;
use App\UserTimeTrack;
use App\Events\GeneralAlertEvent;
use App\Events\UserAlert;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\DB;
use DateTimeZone;

class UserTimeTrackController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index(Request $request)
    {
        if(Auth::user()->user_type->name == 'Admin') 
        {
            $users = User::where('user_type_id', 4)->orWhere('user_type_id', 5)->get();
            /*$commonFunctions = new CommonFunctionsController();
            $commonFunctions->modelToSelectOptions(Company::orderBy('name')->get(),'name', 'id');
            $companies_options = $commonFunctions->select_options;
            $commonFunctions->modelToSelectOptions(UserType::orderBy('name')->get(),'name', 'id');
            $user_type_options = $commonFunctions->select_options;*/
        } else if(Auth::user()->user_type->name == 'Dealer') {
            $user_companies_array = array();
            foreach(Auth::user()->dealers as $value)
            {
              $user_companies_array[] = $value->dealer_id;
            }

            $users = User::join('user_dealer', 'users.id', '=', 'user_dealer.user_id')
                            ->select('users.*')
                            ->where('users.user_type_id', '=', 3)
                            ->whereRaw("FIND_IN_SET(user_dealer.dealer_id, '".implode(',', $user_companies_array)."')")
                            ->get();
            /*$commonFunctions = new CommonFunctionsController();
            $commonFunctions->modelToSelectOptions(Auth::user()->companies,'name', 'id');
            $companies_options = $commonFunctions->select_options;
            $commonFunctions->modelToSelectOptions(UserType::where('id', '!=', 1)->orderBy('name')->get(),'name', 'id');
            $user_type_options = $commonFunctions->select_options;*/
        }
        /*$first_company = '';
        foreach(Auth::user()->dealers as $value)
        {
          $first_company = $value->dealer_id;
          break;
        }*/
        $user_totals = array();
        $user_timers = array();
        $user_timers_running = array();
        $commonFunctions = new CommonFunctionsController();
        foreach ($users as $key => $user) {
            $date_start = date('Y-m-d 00:00:00');
            $date_end = date('Y-m-d 23:59:59');

            $running_timers = UserTimeTrack::where('user_id', $user->id)
                                            ->whereBetween('start_time', [$date_start, $date_end])
                                            ->whereNotNull('end_time')
                                            ->where('end_time', '!=', '') 
                                            ->orderBy('start_time', 'desc')
                                            ->get();
            $user_totals[$user->id] = 0;
            foreach ($running_timers as $key => $value) {
                $user_totals[$user->id] += (strtotime($value->end_time) - strtotime($value->start_time));
            }

            $running_timer = UserTimeTrack::where('user_id', $user->id)
                                            ->whereBetween('start_time', [$date_start, $date_end])
                                            ->where(function($query)
                                                    {
                                                       $query->whereNull('end_time')
                                                       ->orWhere('end_time', ''); 
                                                    })
                                            ->orderBy('start_time', 'desc')
                                            ->first();
            $user_timers_running[$user->id] = 0;
            if($running_timer)
            {
                $user_timers_running[$user->id] = 1;
                $user_totals[$user->id] += (time() - strtotime($running_timer->start_time));
            } 

            if($user_totals[$user->id] > 0)
            {
                $total_hours = $user_totals[$user->id]/3600;
                $dec_time = $total_hours;
                $hour = floor($dec_time);
                $min = floor(60 * ($dec_time - $hour));
                $sec = (60 * ($dec_time - $hour)) - (floor(60 * ($dec_time - $hour)));
                $sec = floor(60*$sec);

                //$current_timer = $commonFunctions->fillWithZeros($hour).":".$commonFunctions->fillWithZeros($min).":".$commonFunctions->fillWithZeros($sec);
                $user_timers[$user->id] = $commonFunctions->fillWithZeros($hour).":".$commonFunctions->fillWithZeros($min).":".$commonFunctions->fillWithZeros($sec);
            } else {
                $user_timers[$user->id] = '00:00:00';
            }
        }

        return view('pages.users_timetracks', [
                                        'users' => $users,
                                        'user_timers' => $user_timers,
                                        'user_timers_running' => $user_timers_running
                                    ]);
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        //
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        if(isset($request->fbdc))
        {
          $user = User::where('reference_id', $request->reference_id)->first();
          if($user)
          {
            $user_time_track = new UserTimeTrack;
            $user_time_track->user_id = $user->id;
            $user_time_track->start_time = date('Y-m-d H:i:s');
            $user_time_track->save();

            $alert_data = new \stdClass();
            $alert_data->type = 'clocked_out';
            $alert_data->running_timer = 1;
            $alert_data->reference = $user->id;
            //broadcast(new GeneralAlertEvent($alert_data));
          }

        } else {
          $user_time_track = new UserTimeTrack;
          $user_time_track->user_id = Auth::id();
          $user_time_track->start_time = date('Y-m-d H:i:s');
          $user_time_track->save();

          if(Auth::user()->reference_id != null && Auth::user()->reference_id != '')
          {
            //$date_start = date('Y-m-d H:i:s', time() - 60 * 60 * 5);
            $date_start = new DateTime();
            $date_start->setTimezone(new DateTimeZone('America/New_York'));
            $employee_timetrack_id = DB::connection('mysql4')->table('employee_timetrack')->insertGetId([
                          'employee_uid' => Auth::user()->reference_id, 
                          'start_time' => $date_start->format('Y-m-d H:i:s'), 
                      ]);
          }
          

          $alert_data = new \stdClass();
          $alert_data->type = 'clocked_out';
          $alert_data->running_timer = 1;
          $alert_data->reference = Auth::id();
          //broadcast(new GeneralAlertEvent($alert_data));

          return response()->json([
              'message' => 'timer started.',
              'inserted_id' => $user_time_track->id
          ]);
        }
    }

    /**
     * Display the specified resource.
     *
     * @param  \App\UserTimeTrack  $userTimeTrack
     * @return \Illuminate\Http\Response
     */
    public function show(UserTimeTrack $userTimeTrack)
    {
        //
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  \App\UserTimeTrack  $userTimeTrack
     * @return \Illuminate\Http\Response
     */
    public function edit(UserTimeTrack $userTimeTrack)
    {
        //
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  \App\UserTimeTrack  $userTimeTrack
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, UserTimeTrack $userTimeTrack)
    {
      if(isset($request->fbdc))
      {
        $user = User::where('reference_id', $request->reference_id)->first();
        if($user)
        {
          UserTimeTrack::where('user_id', $user->id)
                    ->where('start_time', 'like', date('Y-m-d').'%')
                    ->whereNull('end_time')
                    ->update([
                        'end_time' => date('Y-m-d H:i:s')
                    ]);

          $alert_data = new \stdClass();
          $alert_data->type = 'clocked_out';
          $alert_data->running_timer = 0;
          $alert_data->reference = $user->id;
          //broadcast(new GeneralAlertEvent($alert_data));
            
          $alert_data = new \stdClass();
          $alert_data->type = 'clocked_out';
          $alert_data->reference = $user->id;
          //broadcast(new UserAlert($alert_data, '.'.$user->id));
        }
      } else {
        if(isset($request->user_id))
        {
            $user_id = $request->user_id;
        } else {
            $user_id = Auth::id();
        }

        $user = User::find($user_id);

        UserTimeTrack::where('user_id', $user_id)
                    ->where('start_time', 'like', date('Y-m-d').'%')
                    ->whereNull('end_time')
                    ->update([
                        'end_time' => date('Y-m-d H:i:s')
                    ]);

        if($user->reference_id != null && $user->reference_id != '')
        {
          //$stop_time = date('Y-m-d H:i:s', time() - 60 * 60 * 5);
          $stop_time = new DateTime();
          $stop_time->setTimezone(new DateTimeZone('America/New_York'));

          $today_date = date('Y-m-d');
          $updated_employee_timetrack = DB::connection('mysql4')->table('employee_timetrack')
                                                ->whereRaw("start_time LIKE '%$today_date%' ")
                                                ->where('employee_uid', $user->reference_id)
                                                ->whereNull('stop_time')
                                                ->update([
                                                    'stop_time' => $stop_time->format('Y-m-d H:i:s')
                                                ]);
        }
        
        if($user_id == Auth::id())
        {
            $alert_data = new \stdClass();
            $alert_data->type = 'clocked_out';
            $alert_data->running_timer = 0;
            $alert_data->reference = $user_id;
            //broadcast(new GeneralAlertEvent($alert_data));
        } else {
            $alert_data = new \stdClass();
            $alert_data->type = 'clocked_out';
            $alert_data->reference = $user_id;
            //broadcast(new UserAlert($alert_data, '.'.$user_id));
        }

        return response()->json([
            'message' => 'timer stoped.'
        ]);
      }
        
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  \App\UserTimeTrack  $userTimeTrack
     * @return \Illuminate\Http\Response
     */
    public function destroy(UserTimeTrack $userTimeTrack)
    {
        //
    }
}
