<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Auth;

class InboundController extends Controller
{
    public function dealerMessage() {
        $dealers = DB::connection('mysql2')->table('callcenter_companies')
                                ->select('co_name', 'coid')
                                ->where('is_active', 1)
                                ->orderBy('co_name')
                                ->get();

        $commonFunctions = new CommonFunctionsController();
        $commonFunctions->modelToSelectOptions($dealers,'co_name', 'coid');
        $dealers_options = $commonFunctions->select_options;

        $call_types = DB::connection('mysql2')->table('callcenter_calltypes_inbound')->orderBy('calltype_text')->get();
        $commonFunctions->modelToSelectOptions($call_types, 'calltype_text', 'cid');
        $calltypes_options = $commonFunctions->select_options;

        return view('pages.dealer_message', [
            'dealers_options' => $dealers_options,
            'calltypes_options' => $calltypes_options,
        ]);
    }

    public function getDealerMessage(Request $request) {
        $coid = $request->coid;
        $call_type_id = $request->call_type_id;

        // Get the completion note.
        $completion_note = DB::connection('mysql2')->table('callcenter_inbound_completion_notes')
                                ->select('note')
                                ->where('coid', $coid)
                                ->where('cid', $call_type_id)
                                ->first();

        $dealer_note = $completion_note->note ?? '';

    
        // Get the dealer message.
        $dealer_message_sql = DB::connection('mysql2')->table('callcenter_daily_dealer_message')
                                ->select('message')
                                ->where('coid', $coid)
                                ->where('cid', $call_type_id)
                                ->orderBy('set_time', 'desc')
                                ->first();

        if( ! $dealer_message_sql ) {
            $dealer_message_sql = DB::connection('mysql2')->table('callcenter_daily_dealer_message')
                                ->select('message')
                                ->where('coid', $coid)
                                ->whereNull('cid')
                                ->orderBy('set_time', 'desc')
                                ->first();
        }

        $dealer_message = $dealer_message_sql->message ?? '';
        
        return view('partials.dealer_message_editor', [
            'dealer_note' => $dealer_note,
            'dealer_message' => $dealer_message,
        ]);

    }

    public function update(Request $request) {
        $coid = $request->coid;
        $call_type_id = $request->call_type_id;
        $completion_note = $request->completion_note ?? '';
        $dealer_message = $request->dealer_message ?? '';

        //Delete completion note.
        DB::connection('mysql2')->table('callcenter_inbound_completion_notes')
            ->where('coid', $coid)
            ->where('cid', $call_type_id)
            ->limit(1)
            ->delete();
        
        //Insert completion note.
        DB::connection('mysql2')->table('callcenter_inbound_completion_notes')->insert([
            'coid' => $coid,
            'cid' => $call_type_id,
            'note' => $completion_note
        ]);


        DB::connection('mysql2')->table('callcenter_daily_dealer_message')->insert([
            'coid' => $coid,
            'cid' => $call_type_id,
            'message' => $dealer_message,
            'set_time' => date('Y-m-d H:i:s'),
            'created_by' => Auth::user()->first_name . ' ' . Auth::user()->last_name
        ]);

        return response()->json([
            'message' => 'Dealer message and Completion note successfully updated.',
        ]);
    }

    public function lastCallsPanel() {
        $dealers = DB::connection('mysql2')->table('callcenter_companies')
                                ->select('co_name', 'coid')
                                ->where('is_active', 1)
                                ->orderBy('co_name')
                                ->get();

        $commonFunctions = new CommonFunctionsController();
        $commonFunctions->modelToSelectOptions($dealers,'co_name', 'coid');
        $dealers_options = $commonFunctions->select_options;

        return view('pages.last_calls', [
            'dealers_options' => $dealers_options,
        ]);
    }

    public function lastCalls(Request $request) {
        $coid = $request->coid;
        $range = $request->search_type;
    
        // Icons.
        $icons_sql = DB::connection('mysql2')->table('callcenter_ibcallstatus')
                                ->select('type', 'icon')
                                ->get();
        $icons = [];
        foreach ($icons_sql as $key => $value) {
            $icons[ $value->type ] = $value->icon;
        }
        
        // Sales persons data.
        $sales_person_sql = DB::connection('mysql2')->table('callcenter_inbound_sales_person')
                                ->select('id', 'fname', 'lname')
                                ->get();
        
        $sales_person_names = [];    
        foreach($sales_person_sql as $key => $value) {
            $sales_person_names[ $value->id ] = $value->fname . ' ' . $value->lname;
        }

        $dealer = DB::connection('mysql2')->table('callcenter_companies')
                                ->where('coid', $coid) 
                                ->first();

        $inbound_table = 'inboundcall_' . $dealer->co_short;

        date_default_timezone_set('America/New_York');
        $where_ib = '';
        if($range == "24") {
            $test_date = date('Y-m-d h:i:s', strtotime("-1 day"));
            $where_ib.= " lastupdate > '$test_date' ";
        } else if($range == "48") {
            $test_date = date('Y-m-d h:i:s', strtotime("-2 day"));
            $where_ib.= " lastupdate > '$test_date' ";
        } else if($range == "na") {
            $where_ib.= " lastupdate > '2020-11-06 00:00:00' AND call_to_action = 'Urgent' ";
        } else if($range == "72") {
            $test_date = date('Y-m-d h:i:s', strtotime("-3 day"));
            $where_ib.= " lastupdate > '$test_date' ";
        } else if($range == "5") {
            $test_date = date('Y-m-d h:i:s', strtotime("-5 day"));
            $where_ib.= " lastupdate > '$test_date' ";
        }
        
        
        $last_calls_sql = DB::connection('mysql3')->table($inbound_table)
                                ->select('*')
                                ->whereRaw($where_ib)
                                ->get();

        $call_times_6_9 = DB::connection('mysql3')->table($inbound_table)
            ->selectRaw('uid, DAYOFWEEK(lastupdate) as day_of_week')
            ->whereRaw("(lastupdate LIKE '% 06:%' OR lastupdate LIKE '% 07:%' OR lastupdate LIKE '% 08:%' OR lastupdate LIKE '% 09:00:%') AND " . $where_ib)
            ->get();
        
        $sunday    = [];
        $monday    = [];
        $tuesday   = [];
        $wednesday = [];
        $thursday  = [];
        $friday    = [];
        $saturday  = [];
        
        $sunday["6_9"]    = 0;
        $monday["6_9"]    = 0;
        $tuesday["6_9"]   = 0;
        $wednesday["6_9"] = 0;
        $thursday["6_9"]  = 0;
        $friday["6_9"]    = 0;
        $saturday["6_9"]  = 0;
        
        foreach($call_times_6_9 as $call_time) {
            $day_of_week = $call_time->day_of_week;
            switch( $day_of_week ) {
                case 1 : $sunday["6_9"]+=1; break;
                case 2 : $monday["6_9"]+=1; break;
                case 3 : $tuesday["6_9"]+=1; break;
                case 4 : $wednesday["6_9"]+=1; break;
                case 5 : $thursday["6_9"]+=1; break;
                case 6 : $friday["6_9"]+=1; break;
                case 7 : $saturday["6_9"]+=1; break;    
            }
        }

        $call_times_9_12 = DB::connection('mysql3')->table($inbound_table)
            ->selectRaw('uid, DAYOFWEEK(lastupdate) as day_of_week')
            ->whereRaw("(lastupdate LIKE '% 09:%' OR lastupdate LIKE '% 10:%' OR lastupdate LIKE '% 11:%' OR lastupdate LIKE '% 12:00:%') AND lastupdate NOT LIKE '% 09:00:%'  AND " . $where_ib)
            ->get();
        
        $sunday["9_12"]    = 0;
        $monday["9_12"]    = 0;
        $tuesday["9_12"]   = 0;
        $wednesday["9_12"] = 0;
        $thursday["9_12"]  = 0;
        $friday["9_12"]    = 0;
        $saturday["9_12"]  = 0;
        
        foreach($call_times_9_12 as $call_time) {
            $day_of_week = $call_time->day_of_week;
            switch($day_of_week) {
                case 1 : $sunday["9_12"]+=1; break;
                case 2 : $monday["9_12"]+=1; break;
                case 3 : $tuesday["9_12"]+=1; break;
                case 4 : $wednesday["9_12"]+=1; break;
                case 5 : $thursday["9_12"]+=1; break;
                case 6 : $friday["9_12"]+=1; break;
                case 7 : $saturday["9_12"]+=1; break;    
            }
        }
        
        $call_times_12_15 = DB::connection('mysql3')->table($inbound_table)
            ->selectRaw('uid, DAYOFWEEK(lastupdate) as day_of_week')
            ->whereRaw("(lastupdate LIKE '% 12:%' OR lastupdate LIKE '% 13:%' OR lastupdate LIKE '% 14:%' OR lastupdate LIKE '% 15:00:%') AND lastupdate NOT LIKE '% 12:00:%'  AND " . $where_ib)
            ->get();
        
        $sunday["12_15"]    = 0;
        $monday["12_15"]    = 0;
        $tuesday["12_15"]   = 0;
        $wednesday["12_15"] = 0;
        $thursday["12_15"]  = 0;
        $friday["12_15"]    = 0;
        $saturday["12_15"]  = 0;
        
        foreach($call_times_12_15 as $call_time) {
            $day_of_week = $call_time->day_of_week;
            switch($day_of_week) {
                case 1 : $sunday["12_15"]+=1; break;
                case 2 : $monday["12_15"]+=1; break;
                case 3 : $tuesday["12_15"]+=1; break;
                case 4 : $wednesday["12_15"]+=1; break;
                case 5 : $thursday["12_15"]+=1; break;
                case 6 : $friday["12_15"]+=1; break;
                case 7 : $saturday["12_15"]+=1; break;    
            }
        }
        
        $call_times_15_18 = DB::connection('mysql3')->table($inbound_table)
            ->selectRaw('uid, DAYOFWEEK(lastupdate) as day_of_week')
            ->whereRaw("(lastupdate LIKE '% 15:%' OR lastupdate LIKE '% 16:%' OR lastupdate LIKE '% 17:%' OR lastupdate LIKE '% 18:00:%') AND lastupdate NOT LIKE '% 15:00:%'  AND " . $where_ib)
            ->get();
        
        $sunday["15_18"]    = 0;
        $monday["15_18"]    = 0;
        $tuesday["15_18"]   = 0;
        $wednesday["15_18"] = 0;
        $thursday["15_18"]  = 0;
        $friday["15_18"]    = 0;
        $saturday["15_18"]  = 0;
        
        foreach($call_times_15_18 as $call_time) {
            $day_of_week = $call_time->day_of_week;
            switch($day_of_week) {
                case 1 : $sunday["15_18"]+=1; break;
                case 2 : $monday["15_18"]+=1; break;
                case 3 : $tuesday["15_18"]+=1; break;
                case 4 : $wednesday["15_18"]+=1; break;
                case 5 : $thursday["15_18"]+=1; break;
                case 6 : $friday["15_18"]+=1; break;
                case 7 : $saturday["15_18"]+=1; break;    
            }
        }
        
        $call_times_18_21 = DB::connection('mysql3')->table($inbound_table)
            ->selectRaw('uid, DAYOFWEEK(lastupdate) as day_of_week')
            ->whereRaw("(lastupdate LIKE '% 18:%' OR lastupdate LIKE '% 19:%' OR lastupdate LIKE '% 20:%' OR lastupdate LIKE '% 21:00:%') AND lastupdate NOT LIKE '% 18:00:%'  AND " . $where_ib)
            ->get();
        
        $sunday["18_21"] = 0;
        $monday["18_21"] = 0;
        $tuesday["18_21"] = 0;
        $wednesday["18_21"] = 0;
        $thursday["18_21"] = 0;
        $friday["18_21"] = 0;
        $saturday["18_21"] = 0;
        
        foreach($call_times_18_21 as $call_time) {
            $day_of_week = $call_time->day_of_week;
            switch($day_of_week) {
                case 1 : $sunday["18_21"]+=1; break;
                case 2 : $monday["18_21"]+=1; break;
                case 3 : $tuesday["18_21"]+=1; break;
                case 4 : $wednesday["18_21"]+=1; break;
                case 5 : $thursday["18_21"]+=1; break;
                case 6 : $friday["18_21"]+=1; break;
                case 7 : $saturday["18_21"]+=1; break;    
            }
        }

        return view('partials.last_calls', [
            'coid' => $coid,
            'dealer' => $dealer,
            'last_calls_sql' => $last_calls_sql,
            'sunday' => $sunday,
            'monday' => $monday,
            'tuesday' => $tuesday,
            'wednesday' => $wednesday,
            'thursday' => $thursday,
            'friday' => $friday,
            'saturday' => $saturday,
            'sales_person_names' => $sales_person_names,
        ]);
    }

    public function editSalesReps(Request $request) {
        $coid = $request->coid;
        $sales_reps = DB::connection('mysql2')
            ->table('callcenter_inbound_sales_person')
            ->where('coid', $coid)
            ->get();

        $sales_reps_system_access = DB::connection('mysql2')
            ->table('callcenter_users')
            ->where('inbound_access', 1)
            ->get();

        $sr_system_access_array = [];
        foreach ($sales_reps_system_access as $key => $value) {
            $sr_system_access_array[ $value->email ] = $value->email;
        }

        return view('dialogs.inbound_sales_reps', [
            'coid'                   => $coid,
            'sales_reps'             => $sales_reps,
            'sr_system_access_array' => $sr_system_access_array,
        ]);
    }

    public function updateSalesReps(Request $request) {
        foreach ( $request->sales_rep_ids as $index => $sales_rep_id ) {
            DB::connection('mysql2')->table('callcenter_inbound_sales_person')
                ->where('id', $sales_rep_id)
                ->update([
                    'fname'         => $request->sales_rep_first_name[ $index ],
                    'lname'         => $request->sales_rep_last_name[ $index ],
                    'email'         => $request->sales_rep_email[ $index ],
                    'phone'         => $request->sales_rep_phone[ $index ],
                    'cellphone'     => $request->sales_rep_cellphone[ $index ],
                    'on_alert_list' => $request->sales_rep_on_alert_list[ $index ] ?? 0,
                ]);
        }

        return response()->json([
            'message' => 'Sales Reps updated.',
        ]);
    }

    public function addSalesReps(Request $request) {
        $coid = $request->coid;
        $rep_id = DB::connection('mysql2')->table('callcenter_inbound_sales_person')
            ->insertGetId([
                'fname'         => '',
                'lname'         => '',
                'email'         => '',
                'phone'         => '',
                'cellphone'     => '',
                'on_alert_list' => 0,
                'coid' => $coid,
            ]);

            return view('partials.sales_rep_row', [
                'id'   => $rep_id,
                'coid' => $coid,
            ]);
    }


    public function deleteSalesReps(Request $request) {
        DB::connection('mysql2')->table('callcenter_inbound_sales_person')
            ->where('id', $request->id)
            ->delete();

            return response()->json([
                'message' => 'Deleted!.',
                'removed_id' => $request->id,
            ]);
    }

    public function editServices(Request $request) {
        $coid = $request->coid;
        $services = DB::connection('mysql2')
            ->table('dealer_services')
            ->where('coid', $coid)
            ->orderBy('did')
            ->get();

        $departments = DB::connection('mysql2')
            ->table('callcenter_companies_departments')
            ->orderBy('name')
            ->get();

        $commonFunctions = new CommonFunctionsController();
        $commonFunctions->modelToSelectOptions($departments,'name', 'id');
        $departments_options = $commonFunctions->select_options;

        return view('dialogs.inbound_services', [
            'coid'                => $coid,
            'services'            => $services,
            'departments_options' => $departments_options,
        ]);
    }

    public function updateServices(Request $request) {
        foreach ( $request->services_ids as $index => $service_id ) {
            DB::connection('mysql2')->table('dealer_services')
                ->where('uid', $service_id)
                ->update([
                    'did'         => $request->department[ $index ],
                    'title'       => $request->title[ $index ],
                    'subtitle'    => $request->subtitle[ $index ],
                    'price'       => $request->price[ $index ],
                    'description' => $request->description[ $index ],
                    'disclaimer'  => $request->disclaimer[ $index ],
                    'startdate'   => ! empty($request->startdate[ $index ]) ?  date('Y-m-d', strtotime( $request->startdate[ $index ] ) ) : null,
                    'expiredate'  => ! empty($request->expiredate[ $index ]) ?  date('Y-m-d', strtotime( $request->expiredate[ $index ] ) ) : null,
                    'image'       => $request->image[ $index ] ?? '',
                    'comments'    => $request->comments[ $index ],
                    'url_link'    => $request->url_link[ $index ],
                ]);
        }

        return response()->json([
            'message' => 'Service updated.',
        ]);
    }

    public function editLinks(Request $request) {
        $coid = $request->coid;
        $links = DB::connection('mysql2')
            ->table('callcenter_dealer_links')
            ->where('coid', $coid)
            ->get();

        return view('dialogs.inbound_links', [
            'coid'  => $coid,
            'links' => $links,
        ]);
    }

    public function updateLinks(Request $request) {
        foreach ( $request->links_ids as $index => $link_id ) {
            DB::connection('mysql2')->table('callcenter_dealer_links')
                ->where('uid', $link_id)
                ->update([
                    'title'    => $request->title[ $index ],
                    'url'      => $request->url[ $index ],
                    'comments' => $request->comments[ $index ],
                    'icon'     => $request->icon[ $index ],
                ]);
        }

        return response()->json([
            'message' => 'Link updated.',
        ]);
    }
}
