<?php

namespace App\Imports;

use Illuminate\Support\Collection;
use Maatwebsite\Excel\Concerns\ToCollection;
use Illuminate\Support\Facades\DB;

class CallRecordsImport implements ToCollection
{
    public function __construct($db_table, $template, $file_template, $call_type_id, $upload_file_id, $upload_options)
    {
        $this->db_table = $db_table;
        $this->template = $template;
        $this->file_template = $file_template;
        $this->call_type_id = $call_type_id;
        $this->upload_file_id = $upload_file_id;
        $this->upload_options = $upload_options;
        $this->error_message = '';
        $this->row_count = 0;
        $this->added_columns = json_decode($upload_options->added_columns, true);
    }

    public function getErrorMessage()
    {
        return $this->error_message;
    }

    public function getRowCount()
    {
        return $this->row_count;
    }

    public function getCallTypeByName()
    {

    }

    /**
    * @param Collection $collection
    */
    public function collection(Collection $collection)
    {

    	foreach ($collection as $row) 
        {
        	$vin = (array_key_exists('vin', $this->template) === true)? $row[$this->template['vin']] : null;
        	if(strtoupper(trim($vin)) != 'VIN')
        	{
	        	if(array_key_exists('fullname', $this->template) === true)
	        	{
	        		$full_name = explode(' ', $row[$this->template['fullname']]);
	        		$first_name = $full_name[0];
	        		$last_name = '';
	        		for ($i=1; $i < count($full_name); $i++) { 
	        			$last_name.= ' '.$full_name[$i];
	        		}
	        			
	        		$last_name = trim($last_name);
	        	} else {
	        		$first_name = (array_key_exists('first_name', $this->template) === true)? $row[$this->template['first_name']] : null;
	        		$last_name = (array_key_exists('last_name', $this->template) === true)? $row[$this->template['last_name']] : null;
	        	}
	        	
	        	$file_type_id = $this->file_template->id;
	        	
	        	$upload_file_id = $this->upload_file_id;

	        	if($this->upload_options->use_calltype_column == 1)
	        	{
	        		if(array_key_exists('call_type', $this->template) === true)
	        		{
	        			if(array_key_exists(strtoupper(trim($row[$this->template['call_type']])), $this->upload_options->calltypes_keys) === true)
	        			{
	        				$call_type_id = $this->upload_options->calltypes_keys[strtoupper(trim($row[$this->template['call_type']]))];
	        			} else {
	        				$call_type_id = $this->call_type_id;
	        			}
	        		} else {
	        			$call_type_id = $this->call_type_id;
	        		}
	        	} else {
	        		$call_type_id = $this->call_type_id;
	        	}
	        	
	        	$pma = (array_key_exists('pma', $this->template) === true)? $row[$this->template['pma']] : null;
	        	$category = (array_key_exists('category', $this->template) === true)? $row[$this->template['category']] : null;
	        	$phone = (array_key_exists('phone', $this->template) === true)? preg_replace("/[^0-9]/i", "", $row[$this->template['phone']]) : null;
	        	$cellphone = (array_key_exists('cellphone', $this->template) === true)? preg_replace("/[^0-9]/i", "", $row[$this->template['cellphone']]) : null;
	        	$address = (array_key_exists('address', $this->template) === true)? $row[$this->template['address']] : null;
	        	$city = (array_key_exists('city', $this->template) === true)? $row[$this->template['city']] : null;
	        	$state = (array_key_exists('state', $this->template) === true)? $row[$this->template['state']] : null;
	        	$zip = (array_key_exists('zip', $this->template) === true)? $row[$this->template['zip']] : null;
	        	$email = (array_key_exists('email', $this->template) === true)? $row[$this->template['email']] : null;

	        	if(array_key_exists('vehicle_year', $this->template))
	        	{
	        		$vehicle_year = (strlen($row[$this->template['vehicle_year']]) <= 4)? $row[$this->template['vehicle_year']] : null;
	        	}

	        	$vehicle_make = (array_key_exists('vehicle_make', $this->template) === true)? $row[$this->template['vehicle_make']] : null;
	        	$vehicle_model = (array_key_exists('vehicle_model', $this->template) === true)? $row[$this->template['vehicle_model']] : null;

	        	$recall_code = (array_key_exists('recall_code', $this->template) === true)? $row[$this->template['recall_code']] : null;

	        	if(array_key_exists('effective_date', $this->template) === true)
	        	{
	        		if(trim($row[$this->template['effective_date']]) != '')
	        		{
	        			$effective_date = date('Y-m-d',strtotime($row[$this->template['effective_date']]));
	        		} else {
	        			$effective_date = null;
	        		}
	        	} else {
	        		$effective_date = null;
	        	}

	        	if(array_key_exists('expiration_date', $this->template) === true)
	        	{
	        		if(trim($row[$this->template['expiration_date']]) != '')
	        		{
	        			$expiration_date = date('Y-m-d',strtotime($row[$this->template['expiration_date']]));
	        		} else {
	        			$expiration_date = null;
	        		}
	        	} else {
	        		$expiration_date = null;
	        	}


	        	if(array_key_exists('new_owner_date', $this->template) === true)
	        	{
	        		if(trim($row[$this->template['new_owner_date']]) != '')
	        		{
	        			$new_owner_date = date('Y-m-d',strtotime($row[$this->template['new_owner_date']]));
	        		} else {
	        			$new_owner_date = null;
	        		}
	        	} else {
	        		$new_owner_date = null;
	        	}


	        	

	        	$oil_type = (array_key_exists('oil_type', $this->template) === true)? $row[$this->template['oil_type']] : null;
	        	$days_last_ro = (array_key_exists('days_last_ro', $this->template) === true)? $row[$this->template['days_last_ro']] : null;
	        	$days_last_tc = (array_key_exists('days_last_tc', $this->template) === true)? $row[$this->template['days_last_tc']] : null;
	        	$days_last_exp = (array_key_exists('days_last_exp', $this->template) === true)? $row[$this->template['days_last_exp']] : null;
	        	$last_service_advisor = (array_key_exists('last_service_advisor', $this->template) === true)? $row[$this->template['last_service_advisor']] : null;
	        	$estimated_mileage = (array_key_exists('estimated_mileage', $this->template) === true)? $row[$this->template['estimated_mileage']] : null;
	        	$actual_mileage = (array_key_exists('actual_mileage', $this->template) === true)? $row[$this->template['actual_mileage']] : null;
	        	$distance_from_dealer = (array_key_exists('distance_from_dealer', $this->template) === true)? $row[$this->template['distance_from_dealer']] : null;

	        	$priority_group = (array_key_exists('priority_group', $this->template) === true)? $row[$this->template['priority_group']] : null;
	        	$notes = (array_key_exists('notes', $this->template) === true)? $row[$this->template['notes']] : null;


	        	if($this->added_columns)
	        	{
	        		$vehicle_make = (array_key_exists('vehicle_make', $this->added_columns) === true)? $this->added_columns['vehicle_make'] : null;
	        		$recall_code = (array_key_exists('recall_code', $this->added_columns) === true)? $this->added_columns['recall_code'] : null;
	        	}

	        	if(strpos($this->db_table, 'toyota_care_') !== false)
	        	{
	        		DB::connection('mysql')->table($this->db_table)->insert([
		        		'vin' => $vin,
		        		'pma' => $pma,
		        		'file_type_id' => $file_type_id,
		        		'upload_file_id' => $upload_file_id,
		        		'category' => $category,
		        		'first_name' => $first_name,
		        		'last_name' => $last_name,
		        		'phone' => $phone,
		        		'cellphone' => $cellphone,
		        		'address' => $address,
		        		'city' => $city,
		        		'state' => $state,
		        		'zip' => $zip,
		        		'email' => $email,
		        		'vehicle_year' => $vehicle_year,
		        		'vehicle_make' => $vehicle_make,
		        		'vehicle_model' => $vehicle_model,
		        		'effective_date' => $effective_date,
		        		'expiration_date' => $expiration_date,
		        		'oil_type' => $oil_type,
		        		'days_last_ro' => $days_last_ro,
		        		'days_last_tc' => $days_last_tc,
		        		'days_last_exp' => $days_last_exp,
		        		'last_service_advisor' => $last_service_advisor,
		        		'estimated_mileage' => $estimated_mileage,
		        		'actual_mileage' => $actual_mileage,
		        		'created_at' => date('Y-m-d H:i:s'),
		        		'updated_at' => date('Y-m-d H:i:s'),
		        	]);
	        	} else if(strpos($this->db_table, 'recall_records') !== false) {
	        		DB::connection('mysql')->table($this->db_table)->insert([
		        		'vin' => $vin,
		        		'pma' => $pma,
		        		'coid' => $this->upload_options->coid,
		        		'file_type_id' => $file_type_id,
		        		'upload_file_id' => $upload_file_id,
		        		'recall_status_id' => 1,
		        		'first_name' => $first_name,
		        		'last_name' => $last_name,
		        		'phone' => $phone,
		        		'cellphone' => $cellphone,
		        		'address' => $address,
		        		'city' => $city,
		        		'state' => $state,
		        		'zip' => $zip,
		        		'email' => $email,
		        		'vehicle_year' => $vehicle_year,
		        		'vehicle_make' => $vehicle_make,
		        		'vehicle_model' => $vehicle_model,
		        		'distance_from_dealer' => $distance_from_dealer,
		        		'priority_group' => $priority_group,
		        		'notes' => $notes,
		        		'new_owner_date' => $new_owner_date,
		        		'recall_code' => $recall_code,
		        		'created_at' => date('Y-m-d H:i:s'),
		        		'updated_at' => date('Y-m-d H:i:s'),
		        	]);
	        	} else if(strpos($this->db_table, 'call_records_') !== false) {
	        		DB::connection('mysql')->table($this->db_table)->insert([
		        		'vin' => $vin,
		        		'pma' => $pma,
		        		'file_type_id' => $file_type_id,
		        		'call_type_id' => $call_type_id,
		        		'upload_file_id' => $upload_file_id,
		        		'first_name' => $first_name,
		        		'last_name' => $last_name,
		        		'phone' => $phone,
		        		'cellphone' => $cellphone,
		        		'address' => $address,
		        		'city' => $city,
		        		'state' => $state,
		        		'zip' => $zip,
		        		'email' => $email,
		        		'vehicle_year' => $vehicle_year,
		        		'vehicle_make' => $vehicle_make,
		        		'vehicle_model' => $vehicle_model,
		        		'effective_date' => $effective_date,
		        		'expiration_date' => $expiration_date,
		        		'oil_type' => $oil_type,
		        		'days_last_ro' => $days_last_ro,
		        		'days_last_tc' => $days_last_tc,
		        		'days_last_exp' => $days_last_exp,
		        		'last_service_advisor' => $last_service_advisor,
		        		'estimated_mileage' => $estimated_mileage,
		        		'actual_mileage' => $actual_mileage,
		        		'distance_from_dealer' => $distance_from_dealer,
		        		'created_at' => date('Y-m-d H:i:s'),
		        		'updated_at' => date('Y-m-d H:i:s'),
		        	]);
	        	}
        		
	            $this->row_count++;
        	}
            
        }
    }
}
