<?php

namespace App\Providers;

use App\AssignmentNote;
use App\CallType;
use App\Assignment;
use Illuminate\Support\ServiceProvider;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Auth;

class LockedAssignmentProvider extends ServiceProvider
{
    /**
     * Register services.
     *
     * @return void
     */
    public function register()
    {
        //
    }

    /**
     * Bootstrap services.
     *
     * @return void
     */
    public function boot()
    {
        view()->composer('pages.caller_assignments', function($view)
        {
            if (Auth::check())
            {
                $today = date('Y-m-d');
                $locked_assignment = AssignmentNote::where('user_id', Auth::id())
                                                    ->where('assigned_date', $today)
                                                    ->where('is_locked', 1)
                                                    ->where('approved', 0)
                                                    ->first();

                if($locked_assignment)
                {
                    $calltypes = CallType::all();
                    $calltypes_array = array();
                    foreach ($calltypes as $key => $value) {
                        $calltypes_array[$value->id] = $value->name;
                    }

                    $dealers = DB::connection('mysql2')->table('callcenter_companies')
                                            ->select('co_name', 'co_short', 'coid')
                                            ->where('is_active', 1)
                                            ->orderBy('co_name')
                                            ->get();

                    $dealers_array = array();
                    foreach ($dealers as $key => $value) {
                        $dealers_array[$value->coid] = $value->co_name;
                    }

                    $caller_assignments = Assignment::selectRaw('COUNT(id) as call_count, coid, call_type_id')
                                                    ->where('user_id', Auth::id())
                                                    ->where('assigned_date', $today)
                                                    ->groupBy('coid')
                                                    ->groupBy('call_type_id')
                                                    ->get();

                    $assigments_array = array();
                    foreach ($caller_assignments as $key => $value) {
                        $assigments_array[$value->coid][$value->call_type_id] = $value->call_count;
                    }

                    $view->with('has_locked_assignment', '1')
                            ->with('locked_assignment', $locked_assignment)
                            ->with('calltypes_array', $calltypes_array)
                            ->with('dealers_array', $dealers_array)
                            ->with('assigments_array', $assigments_array);
                } else {
                    $view->with('has_locked_assignment', '0');
                }
            } else {
                $view->with('has_locked_assignment', '0');
            }
        });
    }
}
