<?php

namespace App\Providers;

use Illuminate\Support\ServiceProvider;
use Illuminate\Support\Facades\Auth;
use App\UserTimeTrack;

class UserTimeTrackProvider extends ServiceProvider
{
    /**
     * Register services.
     *
     * @return void
     */
    public function register()
    {
        //
    }

    /**
     * Bootstrap services.
     *
     * @return void
     */
    public function boot()
    {
        view()->composer('layouts.app', function($view)
        {
            if (Auth::check())
            {
                if(Auth::user()->user_type->name == "Dealer Caller" || Auth::user()->user_type->name == "Inbound Caller" || Auth::user()->user_type->name == "Full BDC Caller")
                {

                    $date_start = date('Y-m-d 00:00:00');
                    $date_end = date('Y-m-d 23:59:59');

                    $running_timers = UserTimeTrack::where('user_id', Auth::id())
                                                    ->whereBetween('start_time', [$date_start, $date_end])
                                                    ->whereNotNull('end_time')
                                                    ->where('end_time', '!=', '') 
                                                    ->orderBy('start_time', 'desc')
                                                    ->get();
                    $today_total = 0;
                    foreach ($running_timers as $key => $value) {
                        $today_total += (strtotime($value->end_time) - strtotime($value->start_time));
                    }

                    $running_timer = UserTimeTrack::where('user_id', Auth::id())
                                                    ->whereBetween('start_time', [$date_start, $date_end])
                                                    ->where(function($query)
                                                            {
                                                               $query->whereNull('end_time')
                                                               ->orWhere('end_time', ''); 
                                                            })
                                                    ->orderBy('start_time', 'desc')
                                                    ->first();
                    $timer_running = 0;
                    if($running_timer)
                    {
                        $timer_running = 1;
                        $today_total += (time() - strtotime($running_timer->start_time));
                    } 

                    if($today_total > 0)
                    {

                        $total_hours = $today_total/3600;
                        $dec_time = $total_hours;
                        $hour = floor($dec_time);
                        $min = floor(60 * ($dec_time - $hour));
                        $sec = (60 * ($dec_time - $hour)) - (floor(60 * ($dec_time - $hour)));
                        $sec = floor(60*$sec);

                        $current_timer = $this->fillWithZeros($hour).":".$this->fillWithZeros($min).":".$this->fillWithZeros($sec);

                        $view->with('current_timer', $current_timer)->with('timer_running', $timer_running);

                    } else {
                        $view->with('current_timer', '00:00:00')->with('timer_running', 0);
                    }
                } else {
                    $view->with('current_timer', '00:00:00')->with('timer_running', 0);
                }
            } else {
                $view->with('current_timer', '00:00:00')->with('timer_running', 0);
            }
        });
    }


    public function fillWithZeros($t){
        if(strlen($t)==1) {
            return "0".$t;
        }
        else {
            return $t;
        }
    }
}
