@extends('layouts.app')
@section('style_files')
    <link rel="stylesheet" href="{{ asset('css/jquery.typeahead.min.css') }}">
@endsection
@section('content')
<div class="container">
    <div class="row justify-content-center">
        <div class="col-md-12">
            <div class="card">
                <div class="card-header">Training Alerts</div>
                <div class="card-body">
                    <form id="training_alert_search_form" name="training_alert_search_form">
                        <div class="form-row">
                            <div class="form-group col-md-3">
                                <label for="start_date_search">{{ __('Start Date') }}</label>
                                <input type="text" class="form-control search_datepicker_inputs" name="start_date_search" id="start_date_search">
                                <div class="invalid-feedback">
                                    Provide start date.
                                </div>
                            </div>
                            <div class="form-group col-md-3">
                                <label for="end_date_search">{{ __('End Date') }}</label>
                                <input type="text" class="form-control search_datepicker_inputs" name="end_date_search" id="end_date_search">
                                <div class="invalid-feedback">
                                    Provide end date
                                </div>
                            </div>


                            @if(Auth::user()->user_type->name == "Dealer Caller" || Auth::user()->user_type->name == "Inbound Caller" || Auth::user()->user_type->name == "Full BDC Caller")
                                <div class="form-group col-md-2">
                                    <label for="record_type_search">{{ __('Status') }}</label>
                                    <select class="form-control" id="status_search" name="status_search">
                                        <option value="all">All</option>
                                        <option value="pending">Pending</option>
                                        <option value="approved">Approved</option>
                                    </select>
                                    <div class="invalid-feedback">
                                        Provide the status.
                                    </div>
                                </div>
                            @endif
                            <div class="form-group col-md-4">
                                <label for="value_search">{{ __('Search') }}</label>
                                <div class="input-group">
                                    <input type="text" id="value_search" name="value_search" class="form-control" placeholder="Search">
                                    <div class="input-group-append">
                                        <button type="submit" class="btn btn-secondary" id="submit_search" onclick="searchTrainingAlerts(event)"><i class="fas fa-search"></i></button>
                                        @if(Auth::user()->user_type->name == 'Admin') 
                                            <button type="submit" class="btn btn-primary typeahead-button" title="Add training alert" id="add_new_ta_button" onclick="addNewTrainingAlertDialog(event)">
                                                <i class="far fa-plus-square"></i>
                                            </button>
                                        @endif
                                    </div>
                                </div>
                            </div>
                            <div class="form-group col-md-2" style="display: flex;align-items: end;justify-content: center;">
                                <div id="total_records_found"></div>
                            </div>
                        </div>
                    </form>
                    @if($redirected == 1)
                        <div class="alert alert-danger" role="alert">
                            You have training alerts waiting for approval.
                        </div>
                    @endif
                    <br>
                    <div class="alert alert-success" role="alert" id="success-alert-message" style="display: none;">
                    </div>
                    <div id="training_alert_container">
                        <table class="table table-striped">
                            <thead>
                                <tr>
                                    <th>Title</th>
                                    <th class="table_cell_content">Type</th>
                                    @if(Auth::user()->user_type->name == 'Admin') 
                                        <th style="text-align: center;">Active</th>
                                        <th style="text-align: center;">Required</th>
                                    @endif
                                    <th>Date</th>
                                    <th style="text-align: center;">Show</th>
                                    @if(Auth::user()->user_type->name == 'Admin') 
                                        <th style="text-align: center;">Edit</th>
                                        <th style="text-align: center;">Delete</th>
                                    @elseif(Auth::user()->user_type->name == "Dealer Caller" || Auth::user()->user_type->name == "Inbound Caller" || Auth::user()->user_type->name == "Full BDC Caller")
                                        <th style="text-align: center;">Approve</th>
                                    @endif
                                </tr>
                            </thead>
                            <tbody>
                                @foreach ($training_alerts as $training_alert)
                                    @if((strtotime($training_alert->training_alert_created_at) > strtotime($training_alert->user_user_group_created_at)) || Auth::user()->user_type->name == 'Admin')
                                        <tr id="ta_row_{{ $training_alert->id }}">
                                            <td id="ta_name_{{ $training_alert->id }}" data-toggle="tooltip" data-placement="top" title="{{ $training_alert->description }}">{{ $training_alert->title }}</td>
                                            <td class="table_cell_content">{{ $training_alert->file_type->name }}</td>
                                            @if(Auth::user()->user_type->name == 'Admin') 
                                                <td align="center">
                                                    <div class="custom-control custom-checkbox">
                                                        <input type="checkbox" class="custom-control-input" id="active_{{ $training_alert->id }}" name="active_{{ $training_alert->id }}" 
                                                        @include('inputs.checkbox_radio', ['input_value' => 1, 'value' => $training_alert->active]) onclick="activeTrainingAlert(event, '{{ $training_alert->id }}')">
                                                        <label class="custom-control-label" for="active_{{ $training_alert->id }}"></label>
                                                    </div>
                                                </td>
                                                <td align="center">
                                                    <div class="custom-control custom-checkbox">
                                                        <input type="checkbox" class="custom-control-input" id="required_{{ $training_alert->id }}" name="required_{{ $training_alert->id }}" 
                                                        @include('inputs.checkbox_radio', ['input_value' => 1, 'value' => $training_alert->required]) onclick="setRequiredTrainingAlert(event, '{{ $training_alert->id }}')">
                                                        <label class="custom-control-label" for="required_{{ $training_alert->id }}"></label>
                                                    </div>
                                                </td>
                                            @endif
                                            <td class="table_cell_content">{{ date('m/d/Y H:i', strtotime($training_alert->created_at)) }}</td>
                                            <td align="center" class="cursorpointer" data-toggle="tooltip" data-placement="top" title="{{ $training_alert->content }}" onclick="showTrainingAlertDialog(event, {{ $training_alert->id }}, '{{ $training_alert->file_type->name }}')">
                                                <i class="fas fa-search"></i>
                                            </td>
                                            @if(Auth::user()->user_type->name == 'Admin') 
                                                <td align="center" class="cursorpointer" onclick="editTrainingAlertDialog(event, {{ $training_alert->id }})">
                                                    <i class="fas fa-edit"></i>
                                                </td>
                                                <td align="center" class="cursorpointer" onclick="deleteTrainingAlertDialog(event, {{ $training_alert->id }})">
                                                    <i class="fas fa-trash-alt"></i>
                                                </td>
                                            @elseif(Auth::user()->user_type->name == "Dealer Caller" || Auth::user()->user_type->name == "Inbound Caller" || Auth::user()->user_type->name == "Full BDC Caller")
                                                @if($training_alert->date_approved == null)
                                                    <td align="center" class="cursorpointer" onclick="approveTrainingAlertDialog(event, {{ $training_alert->id }}, {{ Auth::id() }})" id="approved_{{ $training_alert->id }}">
                                                        <i class="fas fa-thumbs-up"></i>
                                                    </td>
                                                @else 
                                                    <td align="center">
                                                        {{ date('m/d/Y', strtotime($training_alert->date_approved)) }}
                                                    </td>
                                                @endif    
                                            @endif
                                        </tr>
                                    @endif
                                @endforeach
                            </tbody>
                        </table>
                        {{ $training_alerts->links() }} 
                    </div>

                </div>
            </div>
        </div>
    </div>
</div>
@endsection


@section('scripts_extra')
    <script src="{{ asset('js/bloodhound.min.js') }}" type="application/javascript"></script>
    <script src="{{ asset('js/typeahead.bundle.min.js') }}" type="application/javascript"></script>

    <script type="text/javascript">
        $(document).ready(function()
        {
            $(document).on('click', '.pagination a',function(event)
            {
                event.preventDefault();
                $('li').removeClass('active');

                $(this).parent('li').addClass('active');
                var myurl = $(this).attr('href');
                var page=$(this).attr('href').split('page=')[1];
                trainingAlertPaginationData(page, '', 0);
            });
            $('[data-toggle="tooltip"]').tooltip();
        });

        function trainingAlertPaginationData(page,id, showall)
        {
            var html = getLoadingContent();
            $("#training_alert_container").empty().html(html);
            var my_form = $('#training_alert_search_form');
            var form_id = "training_alert_search_form";
            var form_values = my_form.serialize();
            form_values+='&page='+page+'&id='+id+'&showall='+showall;
            
            $.ajax(
            {

                url: $('meta[name="main-url"]').attr('content')+'/training_alerts/pagination',
                type: "post",
                headers: {
                    'X-CSRF-TOKEN': $('meta[name="csrf-token"]').attr('content')
                },
                datatype: "json",
                data: form_values

            }).done(function(data){

                $("#training_alert_container").empty().html(data);

                $(function () {
                  $('[data-toggle="popover"]').popover({
                        html: true,
                        trigger: 'hover'
                    })
                });

                location.hash = page;
                $('[data-toggle="tooltip"]').tooltip();
            }).fail(function(jqXHR, ajaxOptions, thrownError){

                  alert('No response from server');
            });
        }

        $('#start_date_search').datepicker({
            language: "en",
            autoclose: true,
            orientation: "bottom",
        });

        $('#end_date_search').datepicker({
            language: "en",
            autoclose: true,
            orientation: "bottom",
        });

    </script>
@endsection   