@extends('layouts.app')
@section('style_files')
    <link rel="stylesheet" href="{{ asset('css/jquery.typeahead.min.css') }}">
@endsection
@section('content')
<div class="container">
    <div class="row justify-content-center">
        <div class="col-md-12">
            <div class="card">
                <div class="card-header">User Groups</div>
                <div class="card-body">
                    <form id="user_group_search_form" name="user_group_search_form">
                        <div class="form-row">
                            <div class="form-group col-md-2">
                                <label for="coid_search">{{ __('Dealer') }}</label>
                                <select class="form-control" id="coid_search" name="coid_search" required>
                                    @if(Auth::user()->user_type->name == 'Admin') 
                                        <option value="">All</option>
                                    @endif
                                    @include('inputs.select_options', ['options' => $dealers_options, 'value_selected' => ''])
                                </select>
                                <div class="invalid-feedback">
                                    Provide a dealer.
                                </div>
                            </div>
                            <div class="form-group col-md-4">
                                <label for="value_search">{{ __('Search') }}</label>
                                <div class="input-group">
                                    <input type="text" id="value_search" name="value_search" class="form-control" placeholder="Search">
                                    <div class="input-group-append">
                                        <button type="submit" class="btn btn-secondary" id="submit_search" onclick="searchUserGroups(event)"><i class="fas fa-search"></i></button>
                                        <button type="submit" class="btn btn-primary typeahead-button" title="Add training alert" id="add_new_ta_button" onclick="addNewUserGroupDialog(event)">
                                            <i class="far fa-plus-square"></i>
                                        </button>
                                    </div>
                                </div>
                            </div>
                            <div class="form-group col-md-2" style="display: flex;align-items: end;justify-content: center;">
                                <div id="total_records_found"></div>
                            </div>
                        </div>
                    </form>

                    <br>
                    <div class="alert alert-success" role="alert" id="success-alert-message" style="display: none;">
                    </div>
                    <div id="user_group_container">
                        <table class="table table-striped">
                            <thead>
                                <tr>
                                    <th>Name</th>
                                    <th class="table_cell_content">Dealer</th>
                                    <th style="text-align: center;">Edit</th>
                                    <th style="text-align: center;">Delete</th>
                                </tr>
                            </thead>
                            <tbody>
                                @foreach ($user_groups as $user_group)
                                    <tr id="ta_row_{{ $user_group->id }}">
                                        <td id="user_group_name_{{ $user_group->id }}">{{ $user_group->name }}</td>
                                        <td class="table_cell_content">
                                            @if(isset($dealer_names[$user_group->coid]))
                                                {{ $dealer_names[$user_group->coid] }}
                                            @else 
                                                Global
                                            @endif
                                        </td>
                                        <td align="center" class="cursorpointer" onclick="editUserGroupDialog(event, {{ $user_group->id }})">
                                            <i class="fas fa-edit"></i>
                                        </td>
                                        <td align="center" class="cursorpointer" onclick="deleteUserGroupDialog(event, {{ $user_group->id }})">
                                            <i class="fas fa-trash-alt"></i>
                                        </td>
                                    </tr>
                                @endforeach
                            </tbody>
                        </table>
                        {{ $user_groups->links() }} 
                    </div>

                </div>
            </div>
        </div>
    </div>
</div>
@endsection


@section('scripts_extra')
    <script src="{{ asset('js/bloodhound.min.js') }}" type="application/javascript"></script>
    <script src="{{ asset('js/typeahead.bundle.min.js') }}" type="application/javascript"></script>

    <script type="text/javascript">
        $(document).ready(function()
        {
            $(document).on('click', '.pagination a',function(event)
            {
                event.preventDefault();
                $('li').removeClass('active');

                $(this).parent('li').addClass('active');
                var myurl = $(this).attr('href');
                var page=$(this).attr('href').split('page=')[1];
                userGroupPaginationData(page, '', 0);
            });
            $('[data-toggle="tooltip"]').tooltip();
        });

        function userGroupPaginationData(page,id, showall)
        {
            var html = getLoadingContent();
            $("#user_group_container").empty().html(html);
            var my_form = $('#user_group_search_form');
            var form_id = "user_group_search_form";
            var form_values = my_form.serialize();
            form_values+='&page='+page+'&id='+id+'&showall='+showall;
            
            $.ajax(
            {

                url: $('meta[name="main-url"]').attr('content')+'/usergroups/pagination',
                type: "post",
                headers: {
                    'X-CSRF-TOKEN': $('meta[name="csrf-token"]').attr('content')
                },
                datatype: "json",
                data: form_values

            }).done(function(data){

                $("#user_group_container").empty().html(data);

                $(function () {
                  $('[data-toggle="popover"]').popover({
                        html: true,
                        trigger: 'hover'
                    })
                });

                location.hash = page;
                $('[data-toggle="tooltip"]').tooltip();
            }).fail(function(jqXHR, ajaxOptions, thrownError){

                  alert('No response from server');
            });
        }

        $('#start_date_search').datepicker({
            language: "en",
            autoclose: true,
            orientation: "bottom",
        });

        $('#end_date_search').datepicker({
            language: "en",
            autoclose: true,
            orientation: "bottom",
        });

    </script>
@endsection   