<?php

/*
|--------------------------------------------------------------------------
| Web Routes
|--------------------------------------------------------------------------
|
| Here is where you can register web routes for your application. These
| routes are loaded by the RouteServiceProvider within a group which
| contains the "web" middleware group. Now create something great!
|
*/



Auth::routes();

//Users routes
Route::get('/users', 'UserController@index')->middleware('auth');
Route::post('/users/pagination', 'UserController@pagination');
Route::post('/users/search', 'UserController@search');
Route::post('/users/add_dialog', 'UserController@create');
Route::post('/users/add', 'UserController@store');
Route::post('/users/edit_dialog', 'UserController@edit');
Route::post('/users/edit', 'UserController@update');
Route::post('/users/delete', 'UserController@destroy');
Route::get('/myaccount', 'UserController@accountSettings')->name('MyAccount')->middleware('auth');
Route::post('/users/myaccount/update', 'UserController@changeAccountSettings');
Route::post('/users/block', 'UserController@blockUser');
Route::get('/users/import', 'UserController@importFullBdcUsers');
Route::post('/users/avg/calls/hours', 'UserController@getCallerCallsPerHourAvg');


Route::get('/user/logout', 'UserController@LogOut');
Route::post('/users/keep/session', 'UserController@keepSessionAlive');
Route::get('/user/login/prompt', 'UserController@LoginOnTheFly');

Route::get('/session/check', 'CommonFunctionsController@checkSession');

Route::get('/dealers/search', 'DealerController@search');
Route::post('/dealers/inbound/reps/data', 'DealerController@getInboundRepsData');

Route::get('/home', 'MasterVinController@index')->middleware('auth');

Route::get('/', 'MasterVinController@index')->middleware('auth');

//Upload File Routes
Route::get('uploadfile', 'UploadedFileController@index')->middleware('auth');
Route::post('fileupload/upload', 'UploadedFileController@store');
Route::post('/file/template/search', 'UploadedFileController@search');
Route::post('/file/template/edit', 'UploadedFileController@editFileTemplate');
Route::post('/file/template/edit/dialog', 'UploadedFileController@showTemplateDialog');
Route::post('/file/template/check', 'UploadedFileController@checkFileTemplate');


//Search Customers Routes
Route::get('search_customers', 'SearchCustomerController@index')->middleware('auth');
Route::post('search_customers/get', 'SearchCustomerController@searchCustomers');
Route::post('search_customers/export', 'SearchCustomerController@getVinsDialog');
Route::post('/search/customer', 'SearchCustomerController@search');


//Report Routes
Route::get('report', 'MasterVinController@index')->middleware('auth');
Route::post('report/get', 'MasterVinController@getReport');
Route::post('report/get/second', 'MasterVinController@getReportSecond');
Route::post('report/get/third', 'MasterVinController@getReporThird');
Route::post('report/vin_list', 'MasterVinController@getVinsDialog');

Route::post('report/inbound_call_list', 'MasterVinController@getInboundListDialog');


Route::post('report/schedule_call_dialog', 'MasterVinController@scheduleCallDialog');
Route::post('report/schedule_email_dialog', 'MasterVinController@scheduleEmailDialog');
Route::post('report/search_phones_and_emails', 'MasterVinController@findPhonesAndEmails');

Route::post('call/take_inbound_call', 'CallController@takeCall');
Route::post('call/get_ghost_vin_count', 'CallController@getGhostVinCount');
Route::post('call/send/email/dialog', 'CallController@sendCustomerEmailDialog');
Route::post('call/send/email/send', 'CallController@sendCustomerEmail');
Route::post('call/take/save', 'CallController@storeInboundCall');
Route::post('call/make/call/dialog', 'CallController@makeCallDialog');
Route::post('call/make/call/show', 'CallController@makeCall');
Route::post('/call/make/save', 'CallController@storeCall');
Route::post('/call/get/call/times', 'CallController@getCallTimes');


Route::post('call/check/email', 'CallController@checkEmail');

Route::post('vin/data', 'CallController@getVinInfoToolTipData');

Route::post('report/search_for_new_active_customers', 'MasterVinController@searchActiveCustomers');

Route::get('twilio_phones', 'TwilioPhoneController@index')->middleware('auth');
Route::post('/twilio/pagination', 'TwilioPhoneController@twilioPhonesPagination');
Route::post('/twilio/search', 'TwilioPhoneController@search');
Route::post('/twilio/update', 'TwilioPhoneController@update');
Route::post('/twilio/search_dialog', 'TwilioPhoneController@startConversationDialog');
Route::post('/twilio/send', 'TwilioPhoneController@sendMessage');
Route::post('/twilio/search_conversation', 'TwilioPhoneController@searchConversation');


Route::post('conversations/conversation_window','ConversationController@conversationWindow');
Route::post('conversations/load_more_messages','ConversationController@loadMoreMessages');
Route::post('conversations/send_message','ConversationController@store');
Route::get('conversations/send_message','ConversationController@store');
Route::get('conversations/upcoming_sms_message','ConversationController@storeFromTwilio');
Route::post('/chat/get/contacts','ConversationController@getChatContacts');
Route::post('/conversation/update/seen','ConversationController@update');
Route::post('/get/customer/record', 'ConversationController@getCustomerRecord');

Route::get('/sent/sms/notification','SentSmsNotificationController@SendEmailAlert');

Route::get('/communication/{type}', 'CommunicationController@index')->middleware('auth');
Route::post('/communication/pagination', 'CommunicationController@pagination');

//Inbound Email Routes
Route::get('inbound/email','InboundEmailController@index');
Route::post('/service_connect/pagination', 'ServiceConnectController@serviceConnectPagination');
Route::post('/service_connect/get_service_connect_info', 'ServiceConnectController@getServiceConnectData');

Route::post('/vehicle/make', 'VehicleYearMakeModelController@getVehicleMake');
Route::post('/vehicle/model', 'VehicleYearMakeModelController@getVehicleModel');
Route::post('/vehicle/get_user_vehicle_data', 'VehicleYearMakeModelController@getUserVehicleData');

//Call Types Routes
Route::get('/calltype','CallTypeController@index');
Route::post('/calltype/script', 'CallTypeController@getScriptData');
Route::post('/calltype/inbound/script', 'CallTypeController@getInboundScriptData');
Route::post('/calltype/chage/script', 'CallTypeController@getScriptDataRaw');
Route::post('/calltype/chage/script/update', 'CallTypeController@updateCallScript');
Route::post('/ib/calltype/chage/script', 'CallTypeController@getIBScriptDataRaw');
Route::post('/ib/calltype/chage/script/update', 'CallTypeController@updateIBCallScript');
Route::post('/calltype/value/update', 'CallTypeController@updateCallTypeValue');
Route::post('/calltype/chage/questions', 'CallTypeController@surveyQuestionsDialog');
Route::post('/calltype/question/delete', 'CallTypeController@surveyQuestionsDelete');
Route::post('/calltype/question/add', 'CallTypeController@surveyQuestionsInsert');
Route::post('/calltype/question/update', 'CallTypeController@updateSurveyQuestions');

Route::post('/email/template/get', 'CallerEmailController@getTemplate');

Route::get('report/create_master_tables', 'MasterVinController@createMasterTables');

//DB modifiers
Route::get('report/add_columns_to_master_tables', 'MasterVinController@addColumnsToMasterTables')->middleware('auth');
Route::get('report/create_call_records_tables', 'MasterVinController@createCallRecordsTables')->middleware('auth');
Route::get('report/drop_call_records_tables', 'MasterVinController@dropCallRecordsTables')->middleware('auth');


Route::get('/myiframe', 'HomeController@iframe');


Route::get('/signup', 'HomeController@signup');

//Session valiables
Route::get('settimezone/{minutes}', function ($minutes) {
	$seconds = $minutes * 60 * -1;
    session(['timezone_difference' => $seconds]);
    return session('timezone_difference');
});

//Companies routes
Route::get('/companies', 'CompanyController@index')->middleware('auth');
Route::post('/companies/pagination', 'CompanyController@pagination');
Route::post('/companies/search', 'CompanyController@search');
Route::post('/companies/add_dialog', 'CompanyController@create');
Route::post('/companies/add', 'CompanyController@store');
Route::post('/companies/edit_dialog', 'CompanyController@edit');
Route::post('/companies/edit', 'CompanyController@update');
Route::post('/companies/delete', 'CompanyController@destroy');
Route::post('/reminders/active', 'CompanyController@activeAutomatedReminder');

//Training Alerts routes
Route::get('/training_alerts', 'TrainingAlertController@index')->middleware('auth');
Route::post('/training_alerts/pagination', 'TrainingAlertController@pagination');
Route::post('/training_alerts/search', 'TrainingAlertController@search');
Route::post('/training_alerts/add_dialog', 'TrainingAlertController@create');
Route::post('/training_alerts/add', 'TrainingAlertController@store');
Route::post('/training_alerts/edit_dialog', 'TrainingAlertController@edit');
Route::post('/training_alerts/edit', 'TrainingAlertController@update');
Route::post('/training_alerts/delete', 'TrainingAlertController@destroy');
Route::post('/training_alerts/file/upload', 'TrainingAlertController@uploadFile');
Route::post('/training_alerts/active', 'TrainingAlertController@activeTrainingAlert');
Route::post('/training_alerts/required', 'TrainingAlertController@setRequiredTrainingAlert');
Route::post('/training_alerts/show', 'TrainingAlertController@show');
Route::post('/training_alerts/approve', 'TrainingAlertController@approve');

//Call Reports
Route::get('callsreport', 'CallReportController@index')->middleware('auth');
Route::post('/calls/report/pagination', 'CallReportController@pagination');

Route::get('pendingcalls', 'PendingCallsController@index')->middleware('auth');
Route::post('/pending/calls/pagination', 'PendingCallsController@pagination');

Route::post('/inbound/get/sale_amount', 'InboundReportController@saleAmountReport');


//Caller Schedule
Route::get('/caller/schedule', 'CallerCalendarController@index')->middleware('auth');
Route::get('/caller/assignment', 'AssignmentController@getCallerAssignments')->middleware('auth')->name('callerAssignments');;
Route::post('/assignments/agree', 'AssignmentController@agreeToAssignment');
Route::get('/admin/caller/hours', 'AssignmentController@assignHoursToCallerPanel')->middleware('auth');

Route::post('/caller/schedule/edit/dialog', 'CallerScheduleController@edit');
Route::post('/caller/schedule/edit', 'CallerScheduleController@update');
Route::post('/caller/schedule/events', 'CallerScheduleController@events');
Route::post('/caller/schedule/change/dialog', 'CallerScheduleController@requestChangeDialog');
Route::post('/caller/schedule/change/request', 'CallerScheduleController@requestChange');

Route::post('/caller/schedule/range/edit/dialog', 'CallerScheduleController@editRange');
Route::post('/caller/schedule/range/edit', 'CallerScheduleController@updateRange');


Route::post('/user/timetrack/start', 'UserTimeTrackController@store');
Route::post('/user/timetrack/stop', 'UserTimeTrackController@update');
Route::get('/user/timetrack/start', 'UserTimeTrackController@store');
Route::get('/user/timetrack/stop', 'UserTimeTrackController@update');
Route::get('/users/timetrack', 'UserTimeTrackController@index')->middleware('auth');

Route::get('/usergroups/search', 'UserGroupController@search');
Route::get('/usergroups', 'UserGroupController@index')->middleware('auth');
Route::post('/usergroups/pagination', 'UserGroupController@pagination');
Route::post('/usergroups/search', 'UserGroupController@search');
Route::post('/usergroups/add_dialog', 'UserGroupController@create');
Route::post('/usergroups/add', 'UserGroupController@store');
Route::post('/usergroups/edit_dialog', 'UserGroupController@edit');
Route::post('/usergroups/edit', 'UserGroupController@update');
Route::post('/usergroups/delete', 'UserGroupController@destroy');

Route::get('/admin/assignments', 'AssignmentController@index')->middleware('auth');
Route::post('/admin/assignments/search', 'AssignmentController@search');
Route::post('/assignments/preview', 'AssignmentController@getAssignmentsNotes');
Route::post('/assignments/preview/side/panel', 'AssignmentController@getAssignmentsPreview');
Route::post('/assignments/save', 'AssignmentController@store');
Route::post('/assignments/send/email', 'AssignmentController@sendAssignmentsEmail');
Route::post('/caller/hours', 'AssignmentController@getCallerHours');
Route::post('/caller/hours/save', 'AssignmentController@saveCallerHours');

Route::get('/reminders', 'ReminderController@index')->middleware('auth');
Route::post('/get/reminders/report', 'ReminderController@getRemindersReport');
Route::get('/send/reminders', 'ReminderController@getPendingReminders');
Route::get('/send/reminder/report', 'ReminderController@getRemindersEmailReport');

Route::get('/upload/call/file', 'UploadFtpCallFileController@index')->middleware('auth');
Route::post('/get/ftp/section', 'UploadFtpCallFileController@ftpSection');
Route::post('/edit/ftp/credentials', 'UploadFtpCallFileController@ftpCredentials');
Route::post('/ftp/edit/dialog', 'UploadFtpCallFileController@ftpTemplateDialog');
Route::post('/ftp/update/column/names', 'UploadFtpCallFileController@updateCurrentColumnNamesMatch');
Route::post('/ftp/save/template', 'UploadFtpCallFileController@saveFtpFileTemplate');
Route::post('/ftp/scrub', 'UploadFtpCallFileController@applyFiltersToTempFileFTP');

Route::post('/scrub/file/options', 'ReviewCallFileController@getScrubbedFileOptions');
Route::post('/scrubbed/edit/dialog', 'ReviewCallFileController@scrubbedFileTemplate');
Route::post('/scrubbed/save/template', 'ReviewCallFileController@saveReviewFileEditor');
Route::post('/scrubbed/file/review', 'ReviewCallFileController@reviewFile');
Route::post('/export/recent/called', 'ReviewCallFileController@exportRecentCalled');

Route::post('/review/search/customers', 'SearchCustomerReviewFileController@searchRecentCalled');

Route::post('/import/call/file', 'ImportCallFileController@importCallFile');
Route::post('/review/file/options/save', 'ImportCallFileController@saveReviewFileOptions');

Route::post('/ghosted/vin/history/dialog', 'GhostedVinController@getGhostedVinHistoryDialog');
Route::post('/remove/ghosted/vin/single', 'GhostedVinController@removeGhostedVinSingle');

Route::post('/future/calls/dialog', 'FutureCallController@getFutureCallsPanel');

Route::post('/close/call', 'UpdateCallController@closeCall');
Route::post('/close/call/save', 'UpdateCallController@closeAllFutureCalls');

Route::get('/send/import/emails', 'SendImportEmailController@sendImpotEmails');

Route::get('/upload/calls/files', 'UploadFtpCallFileController@ftpSectionAll')->middleware('auth');

Route::post('/import/email/template/dialog', 'ImportEmailTemplateController@getImportEmailTemplateDialog');
Route::post('/import/email/template/get', 'ImportEmailTemplateController@getImportEmailTemplateContent');
Route::post('/import/email/template/update', 'ImportEmailTemplateController@update');

Route::post('/get/useless/emails/dialog', 'UselessEmailController@getUselessEmailDialog');
Route::post('/update/useless/email', 'UselessEmailController@updateUselessEmail');
Route::post('/get/useless/emails', 'UselessEmailController@getUselessEmails');
Route::post('/remove/useless/email', 'UselessEmailController@removeUselessEmail');
Route::post('/add/useless/email', 'UselessEmailController@addUselessEmail');
Route::post('/add/useless/email/single', 'UselessEmailController@addUselessEmailSingle');

Route::get('/scrub/ftp/files', 'ScrubbFtpFileCronController@scrubbFtpFiles');

Route::post('/call/information/dialog', 'SearchCustomerReviewFileController@getCallInfo');

Route::post('/save/file/options', 'SaveFileOptionController@saveFileOptions');

Route::get('/find/phone', 'UtilController@findDealerPhone');
Route::get('/set/nstc/{call_type}', 'UtilController@setNotShowToCallRecords');

//Inbound section
Route::get('/inbound/dealer/message', 'InboundController@dealerMessage')->middleware('auth');
Route::post('/inbound/dealer/message/get', 'InboundController@getDealerMessage');
Route::post('/inbound/dealer/message/update', 'InboundController@update');
Route::get('/inbound/recent/calls', 'InboundController@recentCalls')->middleware('auth');
Route::post('/inbound/report', 'InboundController@report')->middleware('auth');
Route::get('/inbound/last/call', 'InboundController@lastCall')->middleware('auth');

Route::get('/inbound/last/calls', 'InboundController@lastCallsPanel')->middleware('auth');
Route::post('/inbound/last/calls/get', 'InboundController@lastCalls');

Route::post('/edit/email/template', 'EmailTemplateController@editEmailTemplate');
Route::post('/update/email/template', 'EmailTemplateController@updateEmailTemplate');

Route::post('/inbound/sales/reps/edit', 'InboundController@editSalesReps');
Route::post('/inbound/sales/reps/update', 'InboundController@updateSalesReps');
Route::post('/inbound/sales/reps/add', 'InboundController@addSalesReps');
Route::post('/inbound/sales/reps/delete', 'InboundController@deleteSalesReps');

Route::post('/inbound/services/edit', 'InboundController@editServices');
Route::post('/inbound/services/update', 'InboundController@updateServices');

Route::post('/inbound/links/edit', 'InboundController@editLinks');
Route::post('/inbound/links/update', 'InboundController@updateLinks');

Route::post('/edit/inbound/salesrep/access/dialog', 'InboundController@updatSalesRepAccessDialog');
Route::post('/edit/inbound/salesrep/access', 'InboundController@updateSalesRepAccess');

Route::post('/complete/call/message/edit', 'CompleteCallMessageController@edit');
Route::post('/complete/call/message/update', 'CompleteCallMessageController@update');

Route::post('/outbound/dealer/message/edit', 'OutBoundDealerMessageController@edit');
Route::post('/outbound/dealer/message/update', 'OutBoundDealerMessageController@update');

Route::get('/records/imported', 'RecordsImportedReport@index')->middleware('auth');
Route::post('/records/imported/get', 'RecordsImportedReport@getReport');


Route::get('/settings/global', 'GeneralSettingsController@index')->middleware('auth');

Route::get('/reports/basic/dealer/report/panel', 'ReportController@basicDealerReportPanel')->middleware('auth');
Route::post('/reports/basic/dealer/report', 'ReportController@basicDealerReport');

Route::get('/reports/email/capture/panel', 'ReportController@emailCapturePanel')->middleware('auth');
Route::post('/reports/email/capture', 'ReportController@emailCapture');

Route::get('/search/customer/phone/{phone}', 'SearchPhoneController@searchByPhone')->middleware('auth');

Route::get('/reports/bad/data/panel', 'ReportController@badDataPanel')->middleware('auth');
Route::post('/reports/bad/data', 'ReportController@badData');

Route::get('/reports/survey/report/panel', 'ReportController@surveyReportPanel')->middleware('auth');
Route::post('/reports/survey/report/{type}', 'ReportController@surveyReport');

Route::get('/reports/call/status/panel', 'ReportController@callStatusReportPanel')->middleware('auth');
Route::post('/reports/call/status', 'ReportController@callStatusReport');
Route::post('/reports/call/status/excel', 'ReportController@callStatusReportCsv');

Route::get('/reports/roi/panel', 'ReportController@roiReportPanel')->middleware('auth');
Route::post('/reports/roi', 'ReportController@roiReport');

Route::get('/reports/calltime/caller/panel', 'ReportController@callTimeByCallerReportPanel')->middleware('auth');
Route::post('/reports/calltime/caller', 'ReportController@callTimeByCallerReport');

Route::get('/reports/records/imported/panel', 'ReportController@recordsImportedPanel')->middleware('auth');
Route::post('/reports/records/imported', 'ReportController@recordsImported');
Route::post('/delete/records/imported', 'ReportController@deleteImportRecords');

Route::get('/reports/connect/rate/panel', 'ReportController@connectRatePanel')->middleware('auth');
Route::post('/reports/connect/rate', 'ReportController@connectRate');




